
local function ScoreFlashTag(matchID)
	return string.format("ScoreFlashForMatch%d", matchID)
end

local function SpecialGemFlashTag(matchID)
	return string.format("SpecialGemFlashForMatch%d", matchID)
end

local balloonsNumber = 1
local coconutNumber = 1

local safeDialTime = 300
local safeOpenTime = 1000
local switchToggleTime = 200

function Scripts.PieceMatchAnimationStart(rootGameView, score, matchID, matchedPieces_count, pieceView, destroyAtEnd, specialCount)
	--logprint("PieceMatchAnimationStart "..matchedPieces_count)
    -- Make sure a match color has been set.
	local centerPoint = pieceView:GetGlobalPoint("Center")

	local lockedCovered = false
	if(pieceView.coveredPiece ~= nil) then
		if(pieceView.coveredPiece.isLocked) then
			lockedCovered = true
			if(pieceView.coveredPiece.isGiftBox) then
				--Match onto a one-hit locked gift.
				--Play gift box animation
				PlaySound("bwv_sand_paper_drop_01.ogg")
				PlaySound("bag_paper_small_open_02.ogg")
				StartParticleEmitterFromFile {
					file = "ColorBurst1_green",
					position = centerPoint,
				}
				StartParticleEmitterFromFile {
					file = "ColorBurst1_purple",
					position = centerPoint,
				}
				StartParticleEmitterFromFile {
					file = "ColorBurst1_yellow",
					position = centerPoint,
				}
				--Play lock opened animation
				Scripts.RemoveLockHalves(rootGameView,pieceView)
				--Hide covering gem to be destroyed later
				pieceView.alpha = 0
				pieceView.visible = true
				--Destroy covered gem
				pieceView.coveredPiece:Destroy()
			end
			if(pieceView.coveredPiece.isGiftBox2) then
				--Match onto a two-hit locked gift.
				--Play gift box animation
				PlaySound("bwv_sand_paper_set_down_01.ogg")
				PlaySound("magicwand.ogg")
				StartParticleEmitterFromFile {
					file = "ColorBurst1_purple",
					position = centerPoint,
				}
				--Play lock opened animation
				Scripts.RemoveLockHalves(rootGameView,pieceView)
				--Check if covered gem had a splash icon. If so, save for later
				Scripts.CopySplashIcons(pieceView.coveredPiece,pieceViews)
				--Destroy covered gem
				pieceView.coveredPiece:Destroy()
				--Make covering gem look like a giftBox1
				local savedCenter = pieceView:GetGlobalPoint(Center)
				pieceView:ApplyProperties(Resources.Gems_LockedGift1GamePieceView)
				pieceView:SetGlobalPoint(Center,savedCenter)
				pieceView.extraScale = pieceView.coveredPiece.extraScale
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			end 
			if(pieceView.coveredPiece.isGiftBox3) then
				--Match onto a three-hit locked gift.
				--Play gift box animation
				PlaySound("bwv_sand_paper_set_down_01.ogg")
				PlaySound("magicwand_down5.ogg")
				StartParticleEmitterFromFile {
					file = "ColorBurst1_purple",
					position = centerPoint,
				}
				--Play lock opened animation
				Scripts.RemoveLockHalves(rootGameView,pieceView)
				--Check if covered gem had a splash icon. If so, save for later
				Scripts.CopySplashIcons(pieceView.coveredPiece,pieceViews)
				--Destroy covered gem
				pieceView.coveredPiece:Destroy()
				--Make covering gem look like a giftBox2
				local savedCenter = pieceView:GetGlobalPoint(Center)
				pieceView:ApplyProperties(Resources.Gems_LockedGift2GamePieceView)
				pieceView:SetGlobalPoint(Center,savedCenter)
				pieceView.extraScale = pieceView.coveredPiece.extraScale
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			end
		end
		if(pieceView.isBoomerang) then
			--Determine the location of the destination of the stolen piece
			local allSlotsCells = rootGameView:FindChild("AllSlotsCells",true)
			local cellName = "BoardCell_" .. pieceView.coveredPiece.returnColumn .. "_0"
			local destinationAligner = allSlotsCells:FindChild(cellName,true)
			local positionSave = pieceView.coveredPiece.position

			--Fly displaced piece view down to the target location in an arc
			pieceView.coveredPiece:SetGlobalPoint(Center,destinationAligner:GetGlobalPoint(Center))
			pieceView.layer = 5
			local destination = pieceView.coveredPiece:GetGlobalPoint(TopLeft)
			pieceView.coveredPiece.position = positionSave
			local deltaY = math.abs(positionSave.y - destination.y)
			local flightTime = deltaY / 1800.0 * 1000 * 2

			RunScriptWithParams {
				target = pieceView.coveredPiece,
				script = "SlideToArc",
				params = {destination, flightTime, },
				tags = {},
			}

			--At the same time, fly the boomerang to a position below that that is off the screen
			local belowDestination = {x = destination.x, y = destination.y + 100}
			pieceView.pulseDuration = flightTime
			pieceView:AddScript("SpinBoomerang","")
			Scripts.SlideToArc(pieceView, belowDestination, flightTime)
		end
	end
	
	if(pieceView.isSafe) then
		--Prevent overlap with safe dial when it's turning
		centerPoint.y = centerPoint.y - 20
	end
	local matchColor = pieceView.matchColor

	if not IsColor(matchColor) then
		matchColor = "White"
	end
	
	-- Show a score flash
	if not (GameUITweaks and GameUITweaks.enable_score_flashes == false) then
		-- Convert the score to a string to prevent a comma from being added in.
		-- This should help keep the score flash from getting to be too large,
		-- and overlapping adjacent score flashes.
		RunScriptWithParams {
			target = rootGameView.AllScoreFlashes,
			script = "CascadeScoreFlash",
			params = { centerPoint, tostring(score), matchColor, },
			tags = ScoreFlashTag(matchID),
		}
	end
	
	if(specialCount > 0 and destroyAtEnd) then
		RunScriptWithParams {
			target = rootGameView.AllScoreFlashes,
			script = "CascadeSpecialGemFlash",
			params = { centerPoint, specialCount, matchColor, pieceView.specialType,},
			tags = ScoreFlashTag(matchID),
		}
	end

	if(pieceView.isJailed)then
		PlaySound("jail_cell_door_metal_impact_09.ogg")
		StartParticleEmitterFromFile {
			file = "JailBreak",
			position = centerPoint,
		}
		pieceView.isJailed = false
		pieceView.JailedGemDecoration:Destroy()
	end

	if(pieceView.isInedible)then
		PlaySound("jail_cell_door_metal_impact_09.ogg")
		StartParticleEmitterFromFile {
			file = "JailBreak",
			position = centerPoint,
		}
		pieceView.isInedible = false
		pieceView.InedibleBarDecoration:Destroy()
	end

	if(pieceView.isJailed2)then
		PlaySound("jail_cell_door_metal_impact_09.ogg")
		StartParticleEmitterFromFile {
			file = "JailBreak",
			position = centerPoint,
		}
		pieceView.isJailed2 = false
		pieceView.isJailed = true
		pieceView.DoubleJailedGemDecoration:ApplyProperties(Resources.JailedGemDecoration)
	end

	if(pieceView.isVines)then
		PlaySound("Magic_Respawn38_short.ogg")
		PlaySound("large_wood_break_04.ogg")
		StartParticleEmitterFromFile {
			file = "VineBreak1",
			position = centerPoint,
		}
		StartParticleEmitterFromFile {
			file = "Break_vine1",
			position = centerPoint,
		}
		pieceView.isVines = false
		pieceView.VinesGemDecoration:Destroy()
	end

	if(pieceView.isSuper)then
		StartParticleEmitterFromFile {
			file = "Zap1",
			position = centerPoint,
		}
		PlaySound("Magic_CrystalBlast7.ogg")
	end

	if(destroyAtEnd) then
        --Marked Gems (Rings)
        if(pieceView.isMarkedGem) then
			RunScriptWithParams {
				target = pieceView,
				script = "RingToss",
				params = {},
				tags = {},
			}
        end

		if(pieceView.isDiamond)then
			StartParticleEmitterFromFile {
				file = "DiamondGone1",
				position = centerPoint,
			}
			PlaySound("MagicSpell_Appear.ogg")
		elseif(pieceView.isKey)then
			StartParticleEmitterFromFile {
				file = "StarToss1.smaller",
				position = centerPoint,
			}
			PlaySound("MagicSpell_Cast1.ogg")
			PlaySound("Magic_SpeedPickup.ogg")
		elseif(pieceView.isCrate)then
			PlaySound("crate_smash_01.ogg")
			StartParticleEmitterFromFile {
				file = "Break",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "WoodBreak4",
				position = centerPoint,
			}
		elseif(pieceView.isBarrel or pieceView.isMarsBarrel)then
			PlaySound("cave_rocks_crashing_05.ogg")
			StartParticleEmitterFromFile {
				file = "BoulderBreak3",
				position = centerPoint,
			}
		elseif(pieceView.isRealBarrel)then
			pieceView.visible = false
			PlaySound("crate_smash_01.ogg")
			StartParticleEmitterFromFile {
				file = "MagicPoof1",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "WoodBreak4",
				position = centerPoint,
			}
		elseif(pieceView.isGiftBox or pieceView.isGiftBox2 or pieceView.isGiftBox3)then
			pieceView.visible = false
			PlaySound("bwv_sand_paper_drop_01.ogg")
			PlaySound("bag_paper_small_open_02.ogg")
			StartParticleEmitterFromFile {
				file = "ColorBurst1_green",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "ColorBurst1_purple",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "ColorBurst1_yellow",
				position = centerPoint,
			}
		elseif(pieceView.isSafe)then

            --spin as indicator of progress
            if(pieceView.totalSpawns > 0) then
                --Determine rotation amount
                local spawnsLeft = pieceView.extraSpawns
                if(spawnsLeft == 0) then
                    spawnsLeft = 1
                end
                
                --Rotate the dial
                if(pieceView.isLastSafe) then
                    --Rotate to final spot 2x quicker, to show final state before opening
                    RotateTo(pieceView.Dial, {degrees=360}, safeDialTime/2)
                    RotateTo(pieceView.GemHolder.IconDecoration, {degrees=-360},safeDialTime/2)
                else
                    local rotationAmount = 360 * spawnsLeft/pieceView.totalSpawns
                    RotateTo(pieceView.Dial, {degrees=rotationAmount}, safeDialTime)
                    RotateTo(pieceView.GemHolder.IconDecoration, {degrees=-rotationAmount},safeDialTime)
                end

                PlaySound("safe_dial_turn.ogg")
                PlaySound("bwv_safe_dial_turn_01_quick.ogg")
            end

            ScriptDelay(safeDialTime)

            --open the safe
			if(pieceView.isLastSafe) then
				PlaySound("slot_machine_win_19.ogg")
				PlaySound("prison_door_open.ogg")
                PlaySound("latch_open.ogg")

                --OPEN safe
                pieceView.Dial.visible = false
                pieceView.GemHolder.visible = false

                --fire poker chips
                FireParticle(pieceView,"StarBurst1.smaller.white","Center")

                StartParticleEmitterFromFile {
                    file = "StarBurst2",
                    position = centerPoint,
                }
				StartParticleEmitterFromFile {
					file = "PokerChipToss1",
					position = centerPoint,
				}
                StartParticleEmitterFromFile {
                    file = "PokerChipToss2",
                    position = centerPoint,
                }
                StartParticleEmitterFromFile {
                    file = "PokerChipToss3",
                    position = centerPoint,
                }
                StartParticleEmitterFromFile {
                    file = "PokerChipToss4",
                    position = centerPoint,
                }

                pieceView.resizeOnSetImage = true
                pieceView.image = "Safe_Open.png"

                --Took this out as it was not working well with image size
                --CorrectedPop(pieceView,  {expandTime=160, shrinkTime=160, expandTo={1.1,1.1}})
                
                --keep the safe open just enough to see, and then fade out so it is more smooth than just disappearing
                pieceView:FadeOut(safeOpenTime,false,true)
			end
		elseif(pieceView.isSwitch) then
			PlaySound("Click6.ogg")
			PlaySound("Hit_SoftLargeStoneImpact2.ogg")

            if(pieceView.isToggle) then
                pieceView:AddScript("FlexOut","FlexOut")
                if(pieceView.isToggle) then
                    if (pieceView.isDungeonSwitch) then
                        pieceView:ApplyProperties(Resources.Gems_DungeonSwitchOnGamePieceView)
                        ScriptDelay(switchToggleTime)
                        pieceView:ApplyProperties(Resources.Gems_DungeonSwitchToggleGamePieceView)
                    elseif (pieceView.isEgyptSwitch) then
                        pieceView:ApplyProperties(Resources.Gems_EgyptSwitchOnGamePieceView)
                        ScriptDelay(switchToggleTime)
                        pieceView:ApplyProperties(Resources.Gems_EgyptSwitchToggleGamePieceView)
                    else
                        pieceView:ApplyProperties(Resources.Gems_SwitchOnGamePieceView)
                        ScriptDelay(switchToggleTime)
                        pieceView:ApplyProperties(Resources.Gems_SwitchToggleGamePieceView)
                    end
                end
            end
		elseif(pieceView.isStoneHit1 or pieceView.isMarsStoneHit1)then
			PlaySound("cave_rocks_crashing_07.ogg")
			StartParticleEmitterFromFile {
				file = "StoneBreak4",
				position = centerPoint,
			}
		elseif(pieceView.isChest or pieceView.isMarsChest) then
			PlaySound("slot_machine_win_19.ogg")
			PlaySound("Pickup_Key12_short.ogg")
			PlaySound("Cascade_Chimes.ogg")
			PlaySound("Cascade_Ooo.ogg")
			StartParticleEmitterFromFile {
				file = "StarBurst2",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "DiamondToss",
				position = centerPoint,
			}
			if (pieceView.isChest) then
				pieceView.image = "Chest_opened.png"
			else
				pieceView.image = "Chest_opened_25.png"
			end 
		elseif(pieceView.isCoconut) then
			pieceView.visible = false
			StartParticleEmitterFromFile {
				file = "StarBurst2",
				position = centerPoint,
			}
		elseif(pieceView.isHorizontalSplash) then
			PlaySound("slot_machine_jackpot_win_hit1.ogg")
		elseif(pieceView.isVerticalSplash) then
			PlaySound("slot_machine_jackpot_win_hit1.ogg")
		elseif(pieceView.isAreaSplash) then
			StartParticleEmitterFromFile {file = "Shoot1_burst",position = centerPoint,	}
			PlaySound("Hit_MetalRobotPunch1.ogg")
			PlaySound("slot_machine_win_05.ogg")
		elseif(pieceView.isEmptyBalloon) then
			local soundNumber = math.random(1,6)
			PlaySound("balloon_pop_1"..soundNumber..".ogg")
			local centerPoint = pieceView:GetGlobalPoint("Center")
			if (pieceView.isRedBalloon) then
				StartParticleEmitterFromFile {
					file = "BalloonRed",
					position = centerPoint,
				}
			elseif (pieceView.isBlueBalloon) then
				StartParticleEmitterFromFile {
					file = "BalloonBlue",
					position = centerPoint,
				}
			elseif (pieceView.isGreenBalloon) then
				StartParticleEmitterFromFile {
					file = "BalloonGreen",
					position = centerPoint,
				}
			elseif (pieceView.isPurpleBalloon) then
				StartParticleEmitterFromFile {
					file = "BalloonPurple",
					position = centerPoint,
				}
			elseif (pieceView.isYellowBalloon) then
				StartParticleEmitterFromFile {
					file = "BalloonYellow",
					position = centerPoint,
				}
			elseif (pieceView.isOrangeBalloon) then
				StartParticleEmitterFromFile {
					file = "BalloonOrange",
					position = centerPoint,
				}
			elseif (pieceView.isRainbowBalloon) then
				StartParticleEmitterFromFile {
					file = "BalloonClear",
					position = centerPoint,
				}
			else
				StartParticleEmitterFromFile {
					file = "BalloonClear",
					position = centerPoint,
				}
			end
			pieceView:FadeOut(1000,false,false)
		elseif(pieceView.isLava)then
			PlaySound("boulder_impact_on_stones_14.ogg")
			FireParticle(pieceView, "Steam1", "Center")
			StartParticleEmitterFromFile {
				file = "StoneBreak4_dark",
				position = centerPoint,
			}
		else
			StartParticleEmitterFromFile {
				file = "StarBurst2",
				position = centerPoint,
			}
		end
		if(pieceView.isBomb) then
			StartParticleEmitterFromFile {file = "Poof1",position = centerPoint,	}
			PlaySound("Hit_MetalRobotPunch1.ogg")
			PlaySound("slot_machine_win_01_crop2.ogg")
		end

		if(pieceView.isPickAxe)then
			StartParticleEmitterFromFile {file = "Poof1",position = centerPoint,	}
			--logprint("PickAxe")
		end

	else
		if(pieceView.isBomb) then
			-- Bomb on amber, shake but don't boom
			--logprint("shakeslide because bomb " .. pieceView.name)
			pieceView:AddScript("ShakeSlide","ShakeSlide")
		end
		if(pieceView.isStoneHit4 or pieceView.isMarsStoneHit4)then
			PlaySound("cave_rocks_crashing_06.ogg")
			StartParticleEmitterFromFile {
				file = "StoneBreak4_dark",
				position = centerPoint,
			}
			if (pieceView.isMarsStoneHit4) then
				pieceView.isMarsStoneHit4 = false
				pieceView:ApplyProperties(Resources.Gems_MarsStoneHit3_GamePieceView)
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			else 
				pieceView.isStoneHit4 = false
				pieceView:ApplyProperties(Resources.Gems_StoneHit3_GamePieceView)
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			end
		elseif(pieceView.isStoneHit3 or pieceView.isMarsStoneHit3)then
			PlaySound("cave_rocks_crashing_06.ogg")
			StartParticleEmitterFromFile {
				file = "StoneBreak4_dark",
				position = centerPoint,
			}
			if (pieceView.isMarsStoneHit3) then
				pieceView.isMarsStoneHit3 = false
				pieceView:ApplyProperties(Resources.Gems_MarsStoneHit2_GamePieceView)
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			else 
				pieceView.isStoneHit3 = false
				pieceView:ApplyProperties(Resources.Gems_StoneHit2_GamePieceView)
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			end
		elseif(pieceView.isStoneHit2 or pieceView.isMarsStoneHit2)then
			PlaySound("cave_rocks_crashing_06.ogg")
			StartParticleEmitterFromFile {
				file = "StoneBreak4_dark",
				position = centerPoint,
			}
			if (pieceView.isMarsStoneHit2) then
				pieceView.isMarsStoneHit2 = false
				pieceView:ApplyProperties(Resources.Gems_MarsStoneHit1_GamePieceView)
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			else 
				pieceView.isStoneHit2 = false
				pieceView:ApplyProperties(Resources.Gems_StoneHit1_GamePieceView)
				pieceView:AddScript("ShakeSlide","ShakeSlide")
			end
		elseif(pieceView.isBarrel2 or pieceView.isMarsBarrel2)then
			PlaySound("Hit_SoftLargeStoneImpact2.ogg")
			StartParticleEmitterFromFile {
				file = "BoulderBreak3",
				position = centerPoint,
			}
			if (pieceView.isMarsBarrel2) then
				pieceView.isMarsBarrel2 = false
				pieceView:ApplyProperties(Resources.Gems_MarsBoulderGamePieceView)
			else
				pieceView.isBarrel2 = false
				pieceView:ApplyProperties(Resources.Gems_BarrelGamePieceView)
			end
			pieceView:AddScript("ShakeSlide","ShakeSlide")
		elseif(pieceView.isRealBarrel2)then
			PlaySound("Hit_ChainFence7.ogg")
			StartParticleEmitterFromFile {
				file = "MagicPoof1",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "WoodBreak4",
				position = centerPoint,
			}
			pieceView.isRealBarrel2 = false
			pieceView.isRealBarrel = true
			pieceView:ApplyProperties(Resources.Gems_RealBarrelGamePieceView)
		elseif(pieceView.isGiftBox3 and not lockedCovered)then
			PlaySound("bwv_sand_paper_set_down_01.ogg")
			PlaySound("magicwand_down5.ogg")
			StartParticleEmitterFromFile {
				file = "ColorBurst1_purple",
				position = centerPoint,
			}
			pieceView.isGiftBox3 = false
			pieceView:ApplyProperties(Resources.Gems_GiftBox2GamePieceView)
			pieceView:AddScript("ShakeSlide","ShakeSlide")
		elseif(pieceView.isGiftBox2 and not lockedCovered)then
			PlaySound("bwv_sand_paper_set_down_01.ogg")
			PlaySound("magicwand.ogg")
			StartParticleEmitterFromFile {
				file = "ColorBurst1_purple",
				position = centerPoint,
			}
			pieceView.isGiftBox2 = false
			pieceView:ApplyProperties(Resources.Gems_GiftBoxGamePieceView)
			pieceView:AddScript("ShakeSlide","ShakeSlide")
		end
		if(pieceView.isAmber)then
			pieceView.isAmber = false
			StartParticleEmitterFromFile {
				file = "AmberBurst1",
				position = centerPoint,
			}

			PlaySound("Hit_KickPad2.ogg")
			CorrectedPop(pieceView.AmberGemDecoration,  {expandTime=100, shrinkTime=60, expandTo={0.75, 0.75}})

			PlaySound("Smash_DistantBottle.ogg")
			ScaleOut(pieceView.AmberGemDecoration,{1.5,1.5},100,"nowait")
			FadeOut(pieceView.AmberGemDecoration,100)
			ScriptDelay(120)
			StopSound("boulder_impact_on_stones_10.ogg")
			pieceView.AmberGemDecoration:Destroy()
		end
		if(pieceView.isIce)then
			pieceView.isIce = false
			PlaySound("ice_clink05_hit.ogg")
			CorrectedPop(pieceView.IceGemDecoration,  {expandTime=32*1, shrinkTime=32*2, expandTo={1.10, 1.10}})
			PlaySound("vase_smash_05.ogg")
			pieceView.IceGemDecoration:Destroy()
			StartParticleEmitterFromFile {
				file = "SparkleBurst1",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "IceBreak1",
				position = centerPoint,
			}
			StartParticleEmitterFromFile {
				file = "Boom1_small",
				position = centerPoint,
			}
			--ScaleOut(pieceView.IceGemDecoration,{1.5,1.5},100,"nowait")
			--FadeOut(pieceView.IceGemDecoration,100)
			--StopSound("boulder_impact_on_stones_10.ogg")

		end
		if(pieceView.isBalloons)then
			pieceView.isBalloons = false
			local soundNumber = math.random(1,6)
			PlaySound("balloon_pop_1"..soundNumber..".ogg")
			local balloons = pieceView.BalloonsGemDecoration
			local savedPosition = balloons:GetGlobalPoint("Center")
			pieceView:RemoveChild(balloons)
			balloons.autoFillParent = false
			balloons.name = "Balloons" .. balloonsNumber
			balloonsNumber = balloonsNumber + 1
			pieceView.parent:AddChild(balloons)
			balloons:SetGlobalPoint("Center",savedPosition)

			-- Scale up piece view
			--ScaleIn(pieceView,{1.0,1.0},300,false)
			--Pop(pieceView, {expandTo={1.2, 1.2}, expandTime=200, shrinkTime=100})

			balloons:AddScript("PopBalloons","")

			--Scale up piece view and scale down balloons
			local gemScale = pieceView.extraScale
			gemScale.x = gemScale.x / 0.7
			gemScale.y = gemScale.y / 0.7
			--pieceView.extraScale = gemScale
			ScaleTo(pieceView, {gemScale.x,gemScale.y}, 100,false)

			--Scale down balloons
			local balloonScale = balloons.extraScale
			balloonScale.x = balloonScale.x * 0.7 * gemScale.x
			balloonScale.y = balloonScale.y * 0.7 * gemScale.y
			balloons.extraScale = balloonScale
		end
	end

	if(specialCount > 0) then
		ScriptDelay(600)
	end
end

Scripts.PopBalloons = function(balloons)
	balloons:FadeOut(1000,false,false)
	local centerPoint = balloons:GetGlobalPoint("Center")
	if (balloons.isRedBalloon) then
		StartParticleEmitterFromFile {
			file = "BalloonRed",
			position = centerPoint,
		}
	elseif (balloons.isBlueBalloon) then
		StartParticleEmitterFromFile {
			file = "BalloonBlue",
			position = centerPoint,
		}
	elseif (balloons.isGreenBalloon) then
		StartParticleEmitterFromFile {
			file = "BalloonGreen",
			position = centerPoint,
		}
	elseif (balloons.isPurpleBalloon) then
		StartParticleEmitterFromFile {
			file = "BalloonPurple",
			position = centerPoint,
		}
	elseif (balloons.isYellowBalloon) then
		StartParticleEmitterFromFile {
			file = "BalloonYellow",
			position = centerPoint,
		}
	elseif (balloons.isOrangeBalloon) then
		StartParticleEmitterFromFile {
			file = "BalloonOrange",
			position = centerPoint,
		}
	elseif (balloons.isRainbowBalloon) then
		StartParticleEmitterFromFile {
			file = "BalloonClear",
			position = centerPoint,
		}
	else
		StartParticleEmitterFromFile {
			file = "BalloonClear",
			position = centerPoint,
		}
	end
	--PopOut(balloons, {expandTime=100, shrinkTime=0, expandTo={1.2, 1.2}})
	balloons:Destroy()
end

local function SortMatchedPiecesLeftToRightThenTopToBottom(a, b)
	if a.boardCoordinate.y == b.boardCoordinate.y then
		return a.boardCoordinate.x < b.boardCoordinate.x
	else
		return a.boardCoordinate.y < b.boardCoordinate.y
	end
end

local NextMatchID = 1

function Scripts.RemoveMatchedPieceAnimation(rootGameView, removePiece)
	removePiece.zorder = -1
	if(removePiece.isSafe) then
        if(removePiece.isLastSafe) then
            ScriptDelay(safeDialTime+safeOpenTime)
        else
            ScriptDelay(safeDialTime)
        end
	else
		if(removePiece.isStoneHit1 or removePiece.isMarsStoneHit1 or removePiece.isCrate or removePiece.isBarrel or removePiece.isRealBarrel or removePiece.isRealBarrel2) then
			removePiece.visible = false
        elseif(removePiece.isToggle) then
            ScriptDelay(switchToggleTime)
            ScaleOut(removePiece,{0.0,0.0},500 * DemoAnimationSlowingFactor,"wait")
		else
			ScaleOut(removePiece,{0.0,0.0},500 * DemoAnimationSlowingFactor,"wait")
		end
	end
	removePiece.parent:DestroyChild(removePiece)
end

function Scripts.RemovePieceAnimation(rootGameView, removePiece)
	removePiece.zorder = -1
	ScaleOut(removePiece,{0.0,0.0},250 * DemoAnimationSlowingFactor,"wait")
	removePiece.visible = false
	ScriptDelay(2000) --Keep piece from being deleted, so piece covering it can check its properties
	removePiece.parent:DestroyChild(removePiece)
end

function Scripts.RemoveLockedGiftAnimation(rootGameView, removePiece)
	--Leave the piece intact, and let the matching animation take care of it.
end

function Scripts.GemEater(rootGameView, removePiece)
	GlobalDropDelay = 300
	removePiece.zorder = -1

	PlaySound("jail_cell_door_metal_impact_14.ogg")
	
	local isSplash = removePiece.isHorizontalSplash or removePiece.isVerticalSplash or removePiece.isAreaSplash
	local isBomb = removePiece.isBomb
	local notGem = not removePiece.isGem
	local centerPoint = removePiece:GetGlobalPoint("Center")
	
	if(notGem or isBomb or isSplash) then
		--logprint("Not Gem")
		PlaySound("cave_rocks_crashing_06.ogg")
	end

	removePiece.GemEaterOverlay = {
		class = "Actor",
		image = "GemEater_Closed.png",
		alignment = Center,
		layer = 2,
        visible = false,

	}
	removePiece.GemEaterOverlay.resizeOnSetImage = false
	removePiece.GemEaterOverlay.size = removePiece.size
	-- adjust this if it's in a balloon
	if (removePiece.isBalloons) then
		removePiece.GemEaterOverlay.extraScale = {removePiece.GemEaterOverlay.extraScale.x / 0.7, removePiece.GemEaterOverlay.extraScale.y / 0.7}
	end

	local unused = removePiece.GemEaterOverlay.fadeState

    removePiece.GemEaterOverlay.visible = true

    if(notGem or isBomb or isSplash) then
        StartParticleEmitterFromFile {file = "Boom1",position = centerPoint,}
        StartParticleEmitterFromFile {file = "Break",position = centerPoint,}
        StartParticleEmitterFromFile {file = "WoodBreak4",position = centerPoint,}
    else
        StartParticleEmitterFromFile {file = "Boom1_small",position = centerPoint,}
    end

	ScriptDelay(40)
	ScriptDelay(220)
	ScriptDelay(40)

	removePiece.parent:DestroyChild(removePiece)
end

function Scripts.GemEaterUpdate(rootGameView,gemEater,turnsLeft)
	ScriptBreak()

	WaitForTaggedScriptCompletion(rootGameView,"GemEaterAttempt")
	WaitForTaggedScriptCompletion(rootGameView,"GemEater")

	if(turnsLeft == 0) then
		gemEater.image = "GemEater_Open.png"
		gemEater.layer = 2
	else
		gemEater.image = "GemEater_Closed.png"
		gemEater:ClearLayer()
	end
end

function Scripts.GemEaterAttempt(rootGameView,gemEater,isEmpty)
	PlaySound("cave_rocks_crashing_06.ogg")
	if(isEmpty) then
		gemEater.image = "GemEater_Closed.png"
	else
		gemEater.image = "GemEater_Midway.png"
	end
	ScriptDelay(40)
	ScriptDelay(220)
	ScriptDelay(40)
	gemEater.image = "GemEater_Open.png"
end

function Scripts.RemoveLockHalves(rootGameView,removePiece)
	-- Set up pieces
	removePiece.topLeftHalf = Resources.Actor:new {
		resource = "Actor",
		image = "Lock_chest_target2_1.png",
		layer = 2,
	}
	removePiece.topRightHalf = Resources.Actor:new {
		resource = "Actor",
		image = "Lock_chest_target2_2.png",
		layer = 2,
	}
	removePiece.bottomRightHalf = Resources.Actor:new {
		resource = "Actor",
		image = "Lock_chest_target2_3.png",
		layer = 2,
	}
	removePiece.bottomLeftHalf = Resources.Actor:new {
		resource = "Actor",
		image = "Lock_chest_target2_4.png",
		layer = 2,
	}
		
	-- Set up destination location of fly off
	local topLeftHalf_dest		= {-550,-550}
	local topRightHalf_dest	= {550,-550}
	local bottomRightHalf_dest		= {550,550}
	local bottomLeftHalf_dest		= {-550,550}

	-- Play crack sound and start pieces flyint out
	AddScriptWithParams {
		target = removePiece.topLeftHalf,
		script = "Interpolate",
		params = { "position", topLeftHalf_dest, 1250 },
		tags = {},
	}
	AddScriptWithParams {
		target = removePiece.topRightHalf,
		script = "Interpolate",
		params = { "position", topRightHalf_dest, 1250 },
		tags = {},
	}
	AddScriptWithParams {
		target = removePiece.bottomRightHalf,
		script = "Interpolate",
		params = { "position", bottomRightHalf_dest, 1250 },
		tags = {},
	}
	AddScriptWithParams {
		target = removePiece.bottomLeftHalf,
		script = "Interpolate",
		params = { "position", bottomLeftHalf_dest, 1250 },
		tags = {},
	}

	-- Start poof
	StartParticleEmitterFromFile {
		file = "MagicPoof1",
		position = removePiece.center,
	}
end

function Scripts.RemoveLockAnimation(rootGameView, removePiece)
	ScriptDelay(50)
	PlaySound("jail_cell_door_metal_impact_14.ogg")
	ScriptDelay(200)

	-- Make piece appear, but with alpha 0 to children can appear but not actor yet
	removePiece.alpha = 0
	removePiece.visible = true
	
	Scripts.RemoveLockHalves(rootGameView,removePiece)

	removePiece.zorder = 1
	local centerPoint = removePiece:GetGlobalPoint("Center")
	PlaySound("slot_machine_win_19.ogg")
	--PlaySound("Pickup_Key12_short.ogg")
	PlaySound("Cascade_Chimes.ogg")
	StartParticleEmitterFromFile {
		file = "StarBurst2",
		position = centerPoint,
	}
	StartParticleEmitterFromFile {
		file = "DiamondToss",
		position = centerPoint,
	}
	--ScaleOut(removePiece,{0.0,0.0},250 * DemoAnimationSlowingFactor,"wait")
	ScriptDelay(1250)
	
	removePiece.parent:DestroyChild(removePiece)
end

function Scripts.DoPatternCompletedAnimation(rootGameView,patternScore,matchID)
	--Flash good job word
	--PlaySound("slot_machine_win_19.ogg")

	--Make big score flash
	local allUnmatchedSpaces = rootGameView:FindChild("AllUnmatchedSpaces",true)

	FireParticle( rootGameView.AllScoreFlashes,"SparkleBurst1","Center")

	RunScriptWithParams {
		target = rootGameView.AllScoreFlashes,
		script = "CascadeScoreFlash",
		params = { GetGlobalCenterPoint(allUnmatchedSpaces), patternScore, "White", "AppScoreFlash42" },
		tags = ScoreFlashTag(matchID),
	}

	PlaySound("slot_machine_win_07.ogg")

	--Flash the pattern
   -- ScriptDelay(100)

	--By now, all piece animations involving patterns should be done.
	--Set alpha of all components of the pattern to 255
	local children = allUnmatchedSpaces.children
	for unused,actor in pairs(children) do
		actor.alpha = 150
	end

	FireParticle(rootGameView, "Sparks1", "Center")

	for i = 1,2 do
		PlaySound("slot_machine_win_21_short.ogg")
		allUnmatchedSpaces.visible = true
		ScriptDelay(100)
		allUnmatchedSpaces.visible = false
		ScriptDelay(100)
	end	
	
	for unused,actor in pairs(children) do
		actor.alpha = 0
        actor.visible = true
	end
	
	local scoreLabel = rootGameView.ScoreContainer.ScoreLabel
	scoreLabel:SetScore(scoreLabel.score + patternScore,false)

	ScriptDelay(250)

	--PlaySound("slot_machine_win_06.ogg")

end

function Scripts.BurstMakingAnimation(rootGameView,matchedPieces,matchStart)
	--logprint("MATCHES: " .. SerializeToString(matchedPieces,true))
	--logprint("MATCH START: " .. SerializeToString(matchStart,true))
	--logprint("MATCHES: " .. #matchedPieces)

	local coreMatchCount = 0
	local min_x = -1
	local max_x = -1
	local min_y = -1
	local max_y = -1

	-- Count up core matches -- TODO: Get from game code
	for i,v in ipairs(matchedPieces) do
		local matchedPiece = v
		if(matchedPiece.extraData.matchReason1=="core")then
			coreMatchCount = coreMatchCount + 1
		end
		if((matchedPiece.boardCoordinate.x < min_x) or (min_x == -1))then
			min_x = matchedPiece.boardCoordinate.x
		end
		if((matchedPiece.boardCoordinate.y < min_y) or (min_y == -1))then
			min_y = matchedPiece.boardCoordinate.y
		end
		if((matchedPiece.boardCoordinate.x > max_x) or (max_x == -1))then
			max_x = matchedPiece.boardCoordinate.x
		end
		if((matchedPiece.boardCoordinate.y > max_y) or (max_y == -1))then
			max_y = matchedPiece.boardCoordinate.y
		end
	end
	
	--logprint("CORE MATCHES: " .. coreMatchCount)
	--logprint("min_x: " .. min_x)
	--logprint("max_x: " .. max_x)
	--logprint("min_y: " .. min_y)
	--logprint("max_y: " .. max_y)

	local burstMakingMatch = 7 -- TODO: Get from game code
	if(coreMatchCount>burstMakingMatch)then
		-- Put up highlight on big matches
		for i,v in ipairs(matchedPieces) do
			local matchedPiece = v
			if(matchedPiece.pieceView)then
				local piecePosition = GetGlobalCenterPoint(matchedPiece.pieceView)
				local pieceWidth = matchedPiece.pieceView.width
				local pieceHeight = matchedPiece.pieceView.height

				-- Show outer most cells
				local showHighlight = (matchedPiece.extraData.matchReason1=="core") and 
										(	(matchedPiece.boardCoordinate.x == min_x) or 
											(matchedPiece.boardCoordinate.y == min_y) or 
											(matchedPiece.boardCoordinate.x == max_x) or 
											(matchedPiece.boardCoordinate.y == max_y))

				-- Show outer most cells based on shape
				local matchWidth = max_x - min_x
				local matchHeight = max_y - min_y
				local isSquare	= (matchWidth == matchHeight)
				local isWide	= (matchWidth > matchHeight)
				local isTall	= (matchWidth < matchHeight)

				if(isSquare)then
					--logprint("isSquare")
					showHighlight = (matchedPiece.extraData.matchReason1=="core")
				end
				if(isWide)then
					--logprint("isWide")
					showHighlight = (matchedPiece.extraData.matchReason1=="core") and 
								(	(matchedPiece.boardCoordinate.x == min_x) or 
									(matchedPiece.boardCoordinate.x == max_x)	) 
				end
				if(isTall)then
					--logprint("isTall")
					showHighlight = (matchedPiece.extraData.matchReason1=="core") and 
								(	(matchedPiece.boardCoordinate.y == min_y) or 
									(matchedPiece.boardCoordinate.y == max_y)	)
				end

				-- But for now, show all shape cells
				local showHighlight = (matchedPiece.extraData.matchReason1=="core") 

				if(showHighlight or showWhiteHighlight)then
					local colorName = "White"
					--[[
					if(string.find(matchedPiece.pieceView.image,"Blue")) then colorName = "Blue" end
					if(string.find(matchedPiece.pieceView.image,"Green")) then colorName = "Green" end
					if(string.find(matchedPiece.pieceView.image,"Purple")) then colorName = "Purple" end
					if(string.find(matchedPiece.pieceView.image,"Red")) then colorName = "Red" end
					if(string.find(matchedPiece.pieceView.image,"Yellow")) then colorName = "Yellow" end
					if(string.find(matchedPiece.pieceView.image,"Orange")) then colorName = "Orange" end
					--]]
					local imageName = "slot_highlight_"..colorName..".png"
					local name = "matchedHighlight"..i
					local piecePosition = GetGlobalCenterPoint(matchedPiece.pieceView)
					piecePosition.x = piecePosition.x-(pieceWidth/2)
					piecePosition.y = piecePosition.y-(pieceHeight/2)
					rootGameView[name] = Resources.Actor:new {
						resource = "Actor",
						image = imageName,
						position = piecePosition,
						visible = true,
						size = {pieceWidth,pieceHeight},
						zorder = 3,
						alpha = 128,
					}

					FadeOutAndDestroy(rootGameView[name],750) 
				end
			end
		
			-- Line code for possible later use
			--[[
			local piecePosition = GetGlobalCenterPoint(matchedPiece.pieceView)
			local topName = "topline"..i
			local topLineWidth = pieceWidth
			local topLineHeight = 1
			local topPiecePosition = piecePosition
			topPiecePosition.x = topPiecePosition.x-(topLineWidth/2)
			topPiecePosition.y = topPiecePosition.y-(topLineHeight/2)-(pieceHeight/2)
			rootGameView[topName] = Resources.Actor:new {
				resource = "Actor",
				size = {topLineWidth,topLineHeight},
				color = "White",
				position = topPiecePosition,
			}

			local piecePosition = GetGlobalCenterPoint(matchedPiece.pieceView)
			local bottomName = "bottomline"..i
			local bottomLineWidth = pieceWidth
			local bottomLineHeight = 1
			local bottomPiecePosition = piecePosition
			bottomPiecePosition.x = bottomPiecePosition.x-(bottomLineWidth/2)
			bottomPiecePosition.y = bottomPiecePosition.y+(bottomLineHeight/2)+(pieceHeight/2)
			rootGameView[bottomName] = Resources.Actor:new {
				resource = "Actor",
				size = {bottomLineWidth,bottomLineHeight},
				color = "White",
				position = bottomPiecePosition,
			}

			local piecePosition = GetGlobalCenterPoint(matchedPiece.pieceView)
			local leftName = "leftline"..i
			local leftLineWidth = 1
			local leftLineHeight = pieceWidth
			local leftPiecePosition = piecePosition
			leftPiecePosition.x = leftPiecePosition.x-(leftLineWidth/2)-(pieceWidth/2)
			leftPiecePosition.y = leftPiecePosition.y-(leftLineHeight/2)
			rootGameView[leftName] = Resources.Actor:new {
				resource = "Actor",
				size = {leftLineWidth,leftLineHeight},
				color = "White",
				position = leftPiecePosition ,
			}

			local piecePosition = GetGlobalCenterPoint(matchedPiece.pieceView)
			local rightName = "rightline"..i
			local rightLineWidth = 1
			local rightLineHeight = pieceWidth
			local rightPiecePosition = piecePosition
			rightPiecePosition.x = rightPiecePosition.x+(rightLineWidth/2)+(pieceWidth/2)
			rightPiecePosition.y = rightPiecePosition.y-(rightLineHeight/2)
			rootGameView[rightName] = Resources.Actor:new {
				resource = "Actor",
				size = {rightLineWidth,rightLineHeight},
				color = "White",
				position = rightPiecePosition ,
			}

			FadeOutAndDestroy(rootGameView[topName],1000)
			FadeOutAndDestroy(rootGameView[bottomName],1000)
			FadeOutAndDestroy(rootGameView[leftName],1000)
			FadeOutAndDestroy(rootGameView[rightName],1000)

			--]]
		end
	end
end

function Scripts.MasterMatchAnimation(rootGameView, matchedPieces, matchPatterns, matchStart, numSlotsProcessed, isDiamondMatch, isSuperWildGemMatch, isKeyGemMatch, isCrateMatch, currentMultiplier, patternScore, matchScore)
	--logprint("MATCHES: " .. SerializeToString(matchedPieces,true))
	--logprint("MATCH START: " .. SerializeToString(matchStart,true))

	-- TBD: Add a special animation on making big matches
	-- AddScriptWithParams(rootGameView,"BurstMakingAnimation", matchedPieces, matchStart)

	-- isKeyGemMatch = true
	-- isSuperWildGemMatch = true

	if(isKeyGemMatch)then
		DelayOfDropPiece = 1500 * DemoAnimationSlowingFactor  --HACK: Give extra time for key match animation
		ScriptDelay(50) -- give a little bit more time for key to get there
		
		-- Add disco ball effect
		local pick = math.random(1,5)
		local fileName = "Disco"..pick..".oneHit"
		FireParticle(rootGameView, fileName, "Center")

		-- Turn key
		PlaySound("Pickup_Key12_short.ogg")
		local keyTurnTime = 165 * DemoAnimationSlowingFactor
		for i,v in ipairs(matchedPieces) do
			local matchedPiece = v
		    if(matchedPiece.pieceView.isKey)then
				RotateTo(matchedPiece.pieceView, {degrees=60}, keyTurnTime)
			end           
		end
		ScriptDelay(keyTurnTime)
		for i,v in ipairs(matchedPieces) do
			local matchedPiece = v
		    if(matchedPiece.pieceView.isKey)then
				RotateTo(matchedPiece.pieceView, {degrees=0}, keyTurnTime/2)
			end
		end
		ScriptDelay(keyTurnTime/2)
		PlaySound("slot_machine_bet_03_alt.ogg")

		AddScriptWithParams(rootGameView,"GoodJobWord","levelup.png",STRINGS("Level Up!"),2000,"top")
		PlaySound("Cascade_LevelUp.ogg")
		local allBoardCells = rootGameView:FindChild("AllBoardCells",true)
		allBoardCells:AddScript("BoardShine","")
	end

	-- Play a match-made sound:
	local soundIndex = 1
	if IsNumber(numSlotsProcessed) then
		local minSoundFXIndex = 1
		local maxSoundFXIndex = 10
		soundIndex = math.min(math.max(numSlotsProcessed, minSoundFXIndex), maxSoundFXIndex)
	end
    StopSound("Move2_up.ogg")
	local hasBoomerang = false
	for i,v in ipairs(matchedPieces) do
		local matchedPiece = v
		if(matchedPiece.pieceView ~= nil and matchedPiece.pieceView.isBoomerang)then
			hasBoomerang = true
		end
	end
	
	if(not hasBoomerang) then
		ScriptDelay(50)
	end
	local soundFileName = "score" .. soundIndex .. ".ogg"
	PlaySound(soundFileName)

	-- Make sure matches are sorted.  This is to make sure that score flashes
	-- get displayed in the correct order.
	table.sort(matchedPieces, SortMatchedPiecesLeftToRightThenTopToBottom)
	
	-- Choose a unique id for the match animation.  Please note, this id
	-- is only guaranteed to be unique among matches in a single game.
	local matchID = NextMatchID
	NextMatchID = NextMatchID + 1

	DoStagedAnimation(rootGameView, matchedPieces, matchPatterns, matchID, matchStart)

	if(patternScore > 0) then
		RunScriptWithParams {
			target = rootGameView,
			script = "DoPatternCompletedAnimation",
			params = { patternScore, matchID,},
			tags = { "AnimationShouldBlockGameEnd" },
		}
	end

	local scoreLabel = rootGameView.ScoreContainer.ScoreLabel
	scoreLabel:SetScore(scoreLabel.score + matchScore,false)
end

function SortByFirstElement(table1, table2)
	return (table1[1] < table2[1])
end

function ReverseSortByFirstElement(table1, table2)
	return (table1[1] > table2[1])
end

local patternFlightDuration = 200

local linkAnimationTime = 2000
local linkAnimationIndex = 1

function DoStagedAnimation(rootGameView, matchedPieces, matchPatterns, matchID, matchStart)
	local patternApplicationTime = 300
	if(matchStart == 0) then
		matchStart = {x = 0, y = 0}
	end
	--To figure out: Candy mixing, Particles beyond the basic match
	
	--Determine some things that were in the match for later sound playing.
	local hasDiamond = false
	local hasSuper = false
	local hasKey = false
	local pieceViewParent = nil
	--Reaction sounds
	for i,v in ipairs(matchedPieces) do
		if(v.pieceView ~= nil and v.pieceView.isDiamond)then
			hasDiamond = true
			break
		end
		if(v.pieceView ~= nil) then
			pieceViewParent = v.pieceView.parent
		end
    end
	for i,v in ipairs(matchedPieces) do
		if(v.pieceView ~= nil and v.pieceView.isSuper)then
			hasSuper = true
			break
		end
	end
	for i,v in ipairs(matchedPieces) do
		if(v.pieceView ~= nil and v.pieceView.isKey)then
			hasKey = true
			break
		end
	end   
	--Locate every piece in the match
	for i,v in ipairs(matchedPieces) do
		if(v.pieceView ~= nil)then
			v.extraData.globalCenter = v.pieceView:GetGlobalPoint(Center)
		end
	end   

	--A table of all pickaxe gems.
	--The format is patternName = {pieceView, pieceView, pieceView}
	local pickaxeGems = {}

	--A table of all fully-matched links. They are indexed by link number
	local matchedLinks = {}

	--Make a table of all piece locations that will trigger further reactions. Format is [1] = {position of trigger, thing the piece will cause},
	local triggerPieces = { }

	local triggerIndex = 1

	for i,v in ipairs(matchedPieces) do
		if(v.extraData.splashType ~= nil) then
			triggerPieces[triggerIndex] = { v.boardCoordinate,v.extraData.splashType}
			triggerIndex = triggerIndex + 1
			--logprint("Piece at " .. v.boardCoordinate.x .. "," .. v.boardCoordinate.y .. " has splash type " .. v.extraData.splashType )
			if(v.extraData.secondSplashType ~= nil) then
				--logprint("Piece at " .. v.boardCoordinate.x .. "," .. v.boardCoordinate.y .. " has secondSplashType " .. v.extraData.secondSplashType )
				if(v.extraData.secondSplashType ~= v.extraData.splashType) then
					triggerPieces[triggerIndex] = { v.boardCoordinate,v.extraData.secondSplashType}
					triggerIndex = triggerIndex + 1
				end
			end
		end
	end

	--Initialize pending secondary explosions to empty
	local activeSecondarySquares = { } -- Format is [1] = {position, piece}

	--Initialize match reason pool to Core, GemSwipe, Hammer, or Drop.
	local matchReasonPool = {} -- Format is [1] = {reason, position},
	local nextMatchReasonPool = {}
    local matchingLastSafe = false

	for i,v in ipairs(matchedPieces) do
		if(v.extraData.matchReason1 == "gemswipe") then
			matchReasonPool = {[1] = {{x = 0,y = 0},"gemswipe"}}
			break
		end
		if(v.extraData.matchReason1 == "pieceDrop") then
			matchReasonPool = {[1] = {{x = 0,y = 0},"pieceDrop"}}
			break
		end
		if(v.extraData.matchReason1 == "core") then
			matchReasonPool = {[1] = {{x = 0,y = 0},"core"}}
			break
		end
		if(v.extraData.matchReason1 == "hammer") then
			matchReasonPool = {[1] = {{x = 0,y = 0},"hammer"}}
			break
		end
		if(v.extraData.matchReason1 == "stealPiece") then
			matchReasonPool = {[1] = {{x = 0,y = 0},"stealPiece"}}
			break
		end
	end
	if(next(matchReasonPool) == nil) then
		logprint("Warning, match reason pool empty at match animation start.")
	end

	local cumulativeDelay = 0
	local stepNumber = 0

	--Repeat until match reason pool is empty
	while(next(matchReasonPool) ~= nil) do
		stepNumber = stepNumber + 1

		--Candy mixing
		if(stepNumber == 2 and #matchReasonPool > 0) then
			--Verify that the match reason pool contains a candy-mixed splash type.
			local hasMixedType = false
			for i,v in ipairs(matchReasonPool) do
				if(v[2] == "plusSplash" or
					v[2] == "wideVerticalSplash" or
					v[2] == "wideHorizontalSplash" or
					v[2] == "widePlusSplash") then
					hasMixedType = true
					break
				end
			end
			if(hasMixedType) then
				--rootGameView, delay, allMatches, matchReasonPool
				RunScriptWithParams {
					target = rootGameView,
					script = "QueueCandyMixAnimation",
					params = {cumulativeDelay * DemoAnimationSlowingFactor, matchedPieces, matchReasonPool},
					tags = { matchAnimationTag },
				}
				cumulativeDelay = cumulativeDelay + 500
			end
		end

        --reset special gem counter popping
        local specialGemPanel = Screens.currentScreen.RootActor.AllGameViews.SpecialGemPanel
        specialGemPanel:ResetPopping()

		--For each reason in the set of reasons,
		local maxDelay = 0
		local maxSequence = 0
		local delays = {} --Format is [1] = {delay, piece, sstep, match reason}
		for index, currentMatchReason in ipairs(matchReasonPool) do
			--logprint("Current match reason is: " .. currentMatchReason[2])
			local piecesForThisReason = {} --Just a list of pieces
			local reasonPiecesIndex = 1

			--Find all matches with that reason
			--For all pieces
			for i,v in ipairs(matchedPieces) do
				--For all match reasons for a piece
				for i = 1,56 do
					local matchReasonOfPiece = v.extraData["matchReason" ..i]
					if(matchReasonOfPiece == nil) then
						break
					end
					--logprint("matchReason" .. i .. " = " .. matchReasonOfPiece)

					if(matchReasonOfPiece == currentMatchReason[2]) then
						local matches = true
						if(matchReasonOfPiece ~= "core" and
							matchReasonOfPiece ~= "gemswipe" and
							matchReasonOfPiece ~= "pieceDrop" and
							matchReasonOfPiece ~= "hammer" and
							matchReasonOfPiece ~= "stealPiece") then
							if(v.extraData["sourcePiece" .. i].x ~= currentMatchReason[1].x) then
								--logprint("Doesn't match due to X mismatch")
								matches = false
							end
							if(v.extraData["sourcePiece" .. i].y ~= currentMatchReason[1].y) then
								--logprint("Doesn't match due to Y mismatch")
								matches = false
							end
						end
						if(matches) then
							piecesForThisReason[reasonPiecesIndex] = v
							reasonPiecesIndex = reasonPiecesIndex + 1
							if(matchReasonOfPiece == "gemswipe" or matchReasonOfPiece == "hammer" or matchReasonOfPiece == "stealPiece") then
								if(v.extraData.gemswipeStart == true) then
									matchStart = v.boardCoordinate
								end
							end
							break --A piece cannot match twice for the same reason
						end
					end
				end
			end

			--Set delays on those matches based on the reason
			if(currentMatchReason[2] == "core" or currentMatchReason[2] == "gemswipe" or currentMatchReason[2] == "hammer" or currentMatchReason[2] == "stealPiece") then
				--Sort by Manhattan distance from the origin point.
				local sortedPieces = {} --Format = [1] = {distance, piece}
				for i,v in ipairs(piecesForThisReason) do
					--logprint("Sorting piece " .. i .. " for core match")
					local deltaX = math.abs(v.boardCoordinate.x - matchStart.x)
					local deltaY = math.abs(v.boardCoordinate.y - matchStart.y)
					local manhattanDistance = deltaX + deltaY
					sortedPieces[i] = {manhattanDistance,v}
				end
				table.sort(sortedPieces,SortByFirstElement)

				--Delay between each element of the match is 1 tick
				local totalDelay = 0
				for i,v in ipairs(sortedPieces) do
					delays[#delays + 1] = {totalDelay, v[2],i,currentMatchReason[2]}
					totalDelay = totalDelay + 32
				end
			end
			if(currentMatchReason[2] == "pieceDrop") then
				--Sort from bottom to top in each column
				local columnMaxY = {[0] = 0,[1] = 0,[2] = 0,[3] = 0,[4] = 0,[5] = 0,[6] = 0,[7] = 0,[8] = 0,[9] = 0,[10] = 0}
				--Find the max Y values of each column
				for i,v in ipairs(piecesForThisReason) do
					if(columnMaxY[v.boardCoordinate.x] < v.boardCoordinate.y) then
						columnMaxY[v.boardCoordinate.x] = v.boardCoordinate.y
					end
				end
				--0 ms per layer of chests
				for i,v in ipairs(piecesForThisReason) do
					local delay = (columnMaxY[v.boardCoordinate.x] - v.boardCoordinate.y) * 0
					delays[#delays + 1] = {delay, v, (columnMaxY[v.boardCoordinate.x] - v.boardCoordinate.y) + 1,currentMatchReason[2]}
				end
			end

			if(currentMatchReason[2] == "verticalSplash" or currentMatchReason[2] == "wideVerticalSplash") then
				--Delay by vertical distance from origin point
				for i,v in ipairs(piecesForThisReason) do
					local delay = math.abs(v.boardCoordinate.y - currentMatchReason[1].y) * 51
					delays[#delays + 1] = {delay+50, v, delay/51 + 1,currentMatchReason[2]}
				end
			end

			if(currentMatchReason[2] == "horizontalSplash" or currentMatchReason[2] == "wideHorizontalSplash") then
				--Delay by horizontal distance from origin point
				for i,v in ipairs(piecesForThisReason) do
					local delay = math.abs(v.boardCoordinate.x - currentMatchReason[1].x) * 51
					delays[#delays + 1] = {delay+50, v, delay/51 + 1,currentMatchReason[2]}
				end
			end

			if(currentMatchReason[2] == "plusSplash" or currentMatchReason[2] == "widePlusSplash") then
				--Delay by minimum of horizontal and vertical distance from origin point
				for i,v in ipairs(piecesForThisReason) do
					local horizontalDelay = math.abs(v.boardCoordinate.x - currentMatchReason[1].x) * 51
					local verticalDelay =  math.abs(v.boardCoordinate.y - currentMatchReason[1].y) * 51
					if(verticalDelay < horizontalDelay) then
						verticalDelay = horizontalDelay
					end
					delays[#delays + 1] = {verticalDelay+50, v, verticalDelay/51 + 1,currentMatchReason[2]}
				end
			end

			if(currentMatchReason[2] == "superWild") then
				--Constant delay for all pieces
				for i,v in ipairs(piecesForThisReason) do
					delays[#delays + 1] = {200, v, 1,currentMatchReason[2]}
				end
			end

			if(currentMatchReason[2] == "areaSplash") then
				--Constant delay for all pieces
				for i,v in ipairs(piecesForThisReason) do
					delays[#delays + 1] = {100, v, 1,currentMatchReason[2]}
				end
			end

			for i,v in ipairs(delays) do
				if(v[3] > maxSequence) then
					maxSequence = v[3]
				end
			end
			
			--Run the script for the base reason
			local sourcePiece = 0
			for i,v in ipairs(matchedPieces) do
				if(currentMatchReason[1].x == v.boardCoordinate.x and
					currentMatchReason[1].y == v.boardCoordinate.y and
					currentMatchReason[2] ~= "core" and
					currentMatchReason[2] ~= "pieceDrop" and
					currentMatchReason[2] ~= "gemswipe" and
					currentMatchReason[2] ~= "hammer" and
					currentMatchReason[2] ~= "stealPiece") then
					sourcePiece = v
					break
				end
			end

			if(sourcePiece ~= 0) then
				RunScriptWithParams {
					target = rootGameView,
					script = "QueuePowerupEndAnimation",
					params = {sourcePiece,matchID, cumulativeDelay * DemoAnimationSlowingFactor, currentMatchReason[2], piecesForThisReason,sourcePiece.extraData.hits},
					tags = { matchAnimationTag },
				}
			end
		end
		
		table.sort(delays,SortByFirstElement)

		--Animate the matches with the delays.
		for i,v in ipairs(delays) do
			if(v[1] > maxDelay) then
				maxDelay = v[1]
			end

			--Mark the piece as having been hit. This is used to time when the bursts activate
			if(v[2].extraData.hits == nil) then
				v[2].extraData.hits = 1
			else
				v[2].extraData.hits = v[2].extraData.hits + 1
			end
			if(v[4] == "hammer") then --Special case code to do all the hammer hits at once
				--logprint("Hammered gem")
				v[2].extraData.hits = v[2].numberOfHits
				if(v[2].extraData.hits > v[2].extraData.hitsRequired) then
					-- A hammer does not overkill even if bursts will do so later
					v[2].extraData.hits = v[2].extraData.hitsRequired
				end
			end
			
			--If any specials are hit, queue them in next round's match reason pool.
			local someSpecialHits = false
			for j,possibleReason in ipairs(triggerPieces) do
				--logprint("Checking against trigger piece " .. j)
				--logprint("possible reason: " .. possibleReason[1].x .. "," .. possibleReason[1].y .. " " .. possibleReason[2])
				local thisSpecialHits = true
				if(v[2].boardCoordinate.x ~= possibleReason[1].x) then
					thisSpecialHits = false
				end
				if(v[2].boardCoordinate.y ~= possibleReason[1].y) then
					thisSpecialHits = false
				end
				if(thisSpecialHits) then
					--Check to see if the piece has matched yet. If not, it wasn't really hit.
					if(v[2].extraData.hits ~= v[2].extraData.hitsRequired) then
						--logprint("Failed hits required test: " .. v[2].extraData.hits .. " vs " .. v[2].extraData.hitsRequired)
						thisSpecialHits = false
					end
				end
				if(thisSpecialHits) then
					nextMatchReasonPool[#nextMatchReasonPool + 1] = possibleReason
					someSpecialHits = true
					--Don't break because some pieces hit for multiple reasons
				end
			end
			local pieceIsPickaxe = false
			if(v[2].pieceView ~= nil and v[2].pieceView.isPickAxe) then
				local patternName = v[2].extraData.pattern1
				if(patternName ~= nil) then
					pieceIsPickaxe = true
					local thisPatternTable = pickaxeGems[patternName]
					if(thisPatternTable == nil) then
						thisPatternTable = {}
						pickaxeGems[patternName] = thisPatternTable
					end
					table.insert(thisPatternTable,v[2])
				end
			end
			local pieceIsLink = false
			if(v[2].pieceView ~= nil and (v[2].pieceView.isLink or v[2].pieceView.isJailedLink)) then
				if(v[2].extraData.spawnedPieceView == nil) then
					pieceIsLink = true
					if(matchedLinks[v[2].pieceView.linkNumber] == nil) then
						matchedLinks[v[2].pieceView.linkNumber] = {}
					end
					table.insert(matchedLinks[v[2].pieceView.linkNumber],v[2])
				end
			end
            if(v[2].pieceView ~= nil and v[2].pieceView.isLastSafe) then
                matchingLastSafe = true
            end
			if(someSpecialHits or pieceIsPickaxe or pieceIsLink) then
				RunScriptWithParams {
					target = rootGameView,
					script = "QueuePowerupStartAnimation",
					params = {v[2],(v[1] + cumulativeDelay) * DemoAnimationSlowingFactor},
					tags = { matchAnimationTag },
				}
			else
				--logprint("Queueing piece animation")
				RunScriptWithParams {
					target = rootGameView,
					script = "QueuePieceAnimation",
					params = {v[2],matchID,(v[1] + cumulativeDelay)  * DemoAnimationSlowingFactor,v[3],maxSequence,v[4],v[2].extraData.hits,v[2].extraData.hitsRequired},
					tags = { matchAnimationTag },
				}
			end
		end

		--Make next round's match reasons into this round's match reasons
		matchReasonPool = nextMatchReasonPool
		nextMatchReasonPool = { }
		
		--Wait for all animations to terminate
		cumulativeDelay = cumulativeDelay + maxDelay + 250 --250 ms break between cycles
	end

	cumulativeDelay = cumulativeDelay - 50 --Don't need as much of a break after last cycle

    --special exception for safes
    if(matchingLastSafe) then
        cumulativeDelay = cumulativeDelay + safeDialTime + safeOpenTime - 250
    end

	GlobalDropDelay = cumulativeDelay * DemoAnimationSlowingFactor  --tells piece drop scripts how long to wait
	if(#matchPatterns > 0) then
		GlobalDropDelay = GlobalDropDelay + patternApplicationTime * DemoAnimationSlowingFactor --Drop scripts also need to wait for pattern applications
	end

	--Drop scripts also need to wait for pattern applications
	local totalLinkAnimationTime = (GetTableCount(matchedLinks) * linkAnimationTime * DemoAnimationSlowingFactor)
	GlobalDropDelay = GlobalDropDelay + totalLinkAnimationTime

	ScriptDelay(cumulativeDelay * DemoAnimationSlowingFactor)
	
	--For each link
	--logprint(SerializeToString(matchedLinks, true))
	for linkIndex, linkList in pairs(matchedLinks) do
		local linkColor = "Red"
		if(linkIndex == 1) then
			linkColor = "Blue"
		end
		if(linkIndex == 2) then
			linkColor = "Green"
		end
		if(linkIndex == 3) then
			linkColor = "Purple"
		end
		if(linkIndex == 4) then
			linkColor = "Red"
		end
		if(linkIndex == 5) then
			linkColor = "Yellow"
		end
		if(linkIndex == 6) then
			linkColor = "Orange"
		end

		--Extend an appropriately-colored line between each link and the other links
		local connectionActors = {}
		--Reshow old connectors
		for oldConnectorIndex = 1,linkAnimationIndex do
			local connector = pieceViewParent["connector_" .. oldConnectorIndex]
			if(connector ~= nil) then
				connector.alpha = 255
				FadeIn(connector,150)
			end
		end

		for sourceIndex, sourcePiece in ipairs(linkList) do
			for targetIndex, targetPiece in ipairs(linkList) do
				if(sourceIndex >= targetIndex) then
					--Ensures that only one image is created per pair of actors
					continue
				end
				local sourcePoint = sourcePiece.pieceView:GetGlobalPoint("Center")
				local destPoint = targetPiece.pieceView:GetGlobalPoint("Center")
				local center = {x = (sourcePoint.x + destPoint.x)/2,y = (sourcePoint.y + destPoint.y)/2}
				local deltaX = sourcePoint.x - destPoint.x
				local deltaY = sourcePoint.y - destPoint.y
				local distance = math.sqrt(deltaX * deltaX + deltaY * deltaY)
				local rotationRadians = math.atan2(deltaY,deltaX)
				
				sourcePiece.pieceView.layer = 2
				targetPiece.pieceView.layer = 2

				sourcePiece.pieceView.parent["connector_" .. linkAnimationIndex] = {
					class = "Actor",
					image = "Link" .. linkColor .. "_Connector.png",
					size = {distance,5},
					rotationAngle = {radians = rotationRadians},
					particleColor = linkColor,
					layer = 1,
				}
				local connector = sourcePiece.pieceView.parent["connector_" .. linkAnimationIndex]
				table.insert(connectionActors,connector)
				connector:SetGlobalPoint("Center",center)
				FadeIn(connector,150)
				linkAnimationIndex = linkAnimationIndex + 1
			end
		end

		--Pulse the lines
		for unused, linkLine in ipairs(connectionActors) do
			linkLine:AddScript("PulseLinkLine","")
		end

		--Pulse old connectors
		for oldConnectorIndex = 1,linkAnimationIndex do
			local connector = pieceViewParent["connector_" .. oldConnectorIndex]
			if(connector ~= nil) then
				connector:AddScript("PulseLinkLine","")
			end
		end

		ScriptDelay(1000 * DemoAnimationSlowingFactor)

		--Explode all the lines with a YEAH!
		--Add particles to each link and connector.
		PlaySound("Cascade_Yeah.ogg")
		PlaySound("Magic_SpiritBlast03.ogg")
		for unused, linkLine in ipairs(connectionActors) do
			linkLine:AddScript("ExplodeLinkLine","")
		end

		--Fade out old connectors
		for oldConnectorIndex = 1,linkAnimationIndex do
			local connector = pieceViewParent["connector_" .. oldConnectorIndex]
			if(connector ~= nil and not connector:HasTaggedScript("ExplodeLinkLine")) then
				FadeOut(connector,150)
			end
		end
		
		ScriptDelay((linkAnimationTime - 1000) * DemoAnimationSlowingFactor / 2)

		--Remove excitedness of the links
		for unused, piece in ipairs(linkList) do
			--End the animations of how excited this pickaxe is to do its thing.
			StopSound("paddleChargeUpLoop.ogg")
			piece.pieceView:KillTaggedScripts("PulseContinuously")
			piece.pieceView.pieceCharge:Pause()

			--Make the link go away with normal matching animations.
			local specialCount = 0
			if(IsNumber(piece.extraData.specialGemAmount)) then
				specialCount = piece.extraData.specialGemAmount
			end

			RunScriptWithParams {
				target = rootGameView,
				script = "PieceMatchAnimationStart",
				params = { 
					piece.extraData.score, 
					matchID, 
					0,
					piece.pieceView,
					true,
					specialCount},
				tags = { matchAnimationTag },
			}

			if(piece.extraData.matchProgress) then
				local hits = piece.extraData.hits
				RunScriptWithParams {
					target = rootGameView,
					script = "UpdateMatchProgress",
					params = {piece,hits},
					tags = { matchAnimationTag },
				}
			end

			if(piece.extraData.spawnedPieceView ~= nil) then
				RunScriptWithParams {
					target = rootGameView,
					script = "SpawnedPieceAnimation",
					params = { 
						piece.pieceView, 
						piece.extraData.spawnedPieceView
					},
					tags = { matchAnimationTag },
				}
			end
	
			AddScriptWithParams(rootGameView,"RemoveMatchedPieceAnimation", piece.pieceView)
		end
		
		ScriptDelay((linkAnimationTime - 1000) * DemoAnimationSlowingFactor / 2)
	end

	--Reaction sounds
	if(hasDiamond)then
		PlaySound("Cascade_Ooo.ogg")
	end
	if(hasSuper)then
		--PlaySound("Cascade_Awesome.ogg")
	end
	if(hasKey)then
		PlaySound("Cascade_Money.ogg")
	end

    PlaySound("swish05.ogg")
	
	--Animate application of patterns
	local hasBedRock = false
	for unused,patternData in ipairs(matchPatterns) do
		local gemsWithThisPattern = {} -- A list of global centers since the gems themselves have been destroyed by now

		--Search for all gems that apply this pattern
		for i,matchedPiece in ipairs(matchedPieces) do
			if(matchedPiece.extraData.numberOfPatterns ~= nil and matchedPiece.extraData.numberOfPatterns > 0) then
				for patternIndex = 1,matchedPiece.extraData.numberOfPatterns do
					local piecePatternName = matchedPiece.extraData["pattern" .. patternIndex]
					if(piecePatternName == patternData.name) then
						gemsWithThisPattern[#gemsWithThisPattern + 1] = matchedPiece.extraData.globalCenter
					end
				end
			end
		end

		local allSparkles = {}
		local sparkleNumber = 0
		
		--For each gem that applies this pattern
		for unused,sourceSpot in ipairs(gemsWithThisPattern) do
			--For each destination gem
			for index,gemData in ipairs(patternData.removedPieces) do
				if(gemData ~= "Removed") then
					--Schedule a particle stream to fly from source to destination over a fixed time period, deleting itself on arrival
					sparkleNumber = sparkleNumber + 1
					local destination = GetGlobalCenterPoint(gemData.pieceView)
					allSparkles[sparkleNumber] = CreateInstance {
						name = "",
						class = "hgeParticleSystemActor",
						file = "SuperWildTrail3",
						position = sourceSpot,
					}
					allSparkles[sparkleNumber].destination  = destination
					allSparkles[sparkleNumber].duration = patternFlightDuration
					rootGameView:AddChild(allSparkles[sparkleNumber])
					allSparkles[sparkleNumber]:AddScript("FlyToDestination","")
					if(gemData.pieceView.isBedRock)then
						AddScriptWithParams(allSparkles[sparkleNumber],"BoomAtDestination_bedrock", gemData.pieceView.IconDecoration.image)
						hasBedRock = true
					else 
						allSparkles[sparkleNumber]:AddScript("BoomAtDestination_pattern","")
					end
				else 
					local gemData = patternData.addedPieces[index]
					if(gemData ~= "Removed") then
						--Schedule a particle stream to fly from source to destination over a fixed time period, deleting itself on arrival
						sparkleNumber = sparkleNumber + 1
						local destination = GetGlobalCenterPoint(gemData.pieceView)
						allSparkles[sparkleNumber] = CreateInstance {
							name = "",
							class = "hgeParticleSystemActor",
							file = "SuperWildTrail3",
							position = sourceSpot,
						}
						allSparkles[sparkleNumber].destination  = destination
						allSparkles[sparkleNumber].duration = patternFlightDuration
						rootGameView:AddChild(allSparkles[sparkleNumber])
						allSparkles[sparkleNumber]:AddScript("FlyToDestination","")
						if(gemData.pieceView.isBedRock) then
							AddScriptWithParams(allSparkles[sparkleNumber],"BoomAtDestination_bedrock", gemData.pieceView.IconDecoration.image)
							hasBedRock = true
						else 
							allSparkles[sparkleNumber]:AddScript("BoomAtDestination_pattern","")
						end
					end
				end
			end
		end
	end
	
	if(#matchPatterns > 0) then
		if(hasBedRock)then
			PlaySound("fdn_convolution_pulse_07.ogg")
		else
			--PlaySound("laserShot1.ogg")
			PlaySound("Cascade_Open_SFX_10.ogg")
		end
		ScriptDelay(patternFlightDuration * DemoAnimationSlowingFactor)
	end
	
	for unused,patternData in ipairs(matchPatterns) do
		local gemsForThisPattern = pickaxeGems[patternData.name]
		if(gemsForThisPattern ~= nil) then
			--Undo the excitedness of the pickaxes that launched this pattern
			for unused, piece in ipairs(gemsForThisPattern) do
				--Prevent the pickaxe from being destroyed twice if it applied two different patterns
				if(piece.extraData.pickaxeUsed == true) then
					continue
				end
				piece.extraData.pickaxeUsed = true

				--End the animations of how excited this pickaxe is to do its thing.
				StopSound("paddleChargeUpLoop.ogg")
				piece.pieceView:KillTaggedScripts("PulseContinuously")
				piece.pieceView.pieceCharge:Pause()

				--Make the pickaxe go away with normal matching animations.
				local specialCount = 0
				if(IsNumber(piece.extraData.specialGemAmount)) then
					specialCount = piece.extraData.specialGemAmount
				end

				RunScriptWithParams {
					target = rootGameView,
					script = "PieceMatchAnimationStart",
					params = { 
						piece.extraData.score, 
						matchID, 
						0,
						piece.pieceView,
						true,
						specialCount},
					tags = { matchAnimationTag },
				}

				if(piece.extraData.matchProgress) then
					local hits = piece.extraData.hits
					RunScriptWithParams {
						target = rootGameView,
						script = "UpdateMatchProgress",
						params = {piece,hits},
						tags = { matchAnimationTag },
					}
				end

				if(piece.extraData.spawnedPieceView ~= nil) then
					RunScriptWithParams {
						target = rootGameView,
						script = "SpawnedPieceAnimation",
						params = { 
							piece.pieceView, 
							piece.extraData.spawnedPieceView
						},
						tags = { matchAnimationTag },
					}
				end
	
				AddScriptWithParams(rootGameView,"RemoveMatchedPieceAnimation", piece.pieceView)
			end
		end

		--Update properties instantly by deleting old view and showing new view
		for unused2,gemData in ipairs(patternData.removedPieces) do
			if(gemData ~= "Removed") then
				gemData.pieceView:Destroy()
			end
		end

		for unused2,gemData in ipairs(patternData.addedPieces) do
			if(gemData ~= "Removed") then
				gemData.pieceView.visible = true
			end
		end
	end
	
	if(#matchPatterns > 0) then
		ScriptDelay((patternApplicationTime - patternFlightDuration) * DemoAnimationSlowingFactor)
	end
	ScriptDelay(150)
end

function Scripts.QueuePowerupStartAnimation(rootGameView, piece, delay)
	ScriptDelay(delay)
	if(piece.pieceView == nil) then
		return
	end
	 
	--For splash powerups, hide main piece.
	if(piece.pieceView.AreaSplashDecoration ~= nil or
		piece.pieceView.VerticalSplashDecoration ~= nil or
		piece.pieceView.HorizontalSplashDecoration ~= nil or
		piece.pieceView.isPickAxe) then
		piece.pieceView.alpha = 0 --Hides the piece but not its aura or decoration.
	end

	if(piece.pieceView.AreaSplashDecoration)then
		piece.pieceView.AreaSplashDecoration.Overlay.visible = true
	end
	if(piece.pieceView.VerticalSplashDecoration)then
		piece.pieceView.VerticalSplashDecoration.Overlay.visible = true
	end
	if(piece.pieceView.HorizontalSplashDecoration)then
		piece.pieceView.HorizontalSplashDecoration.Overlay.visible = true
	end
	if(piece.pieceView.isPickAxe) then
		piece.pieceView.IconDecoration.IconDecoration.visible = true
	end
	if(piece.pieceView.isLink or piece.pieceView.isJailedLink) then
		--Set the piece glowing
		if(piece.pieceView.linkNumber == 1) then
			piece.pieceView.image = "LinkBlue_Glowing.png"
		end
		if(piece.pieceView.linkNumber == 2) then
			piece.pieceView.image = "LinkGreen_Glowing.png"
		end
		if(piece.pieceView.linkNumber == 3) then
			piece.pieceView.image = "LinkPurple_Glowing.png"
		end
		if(piece.pieceView.linkNumber == 4) then
			piece.pieceView.image = "LinkRed_Glowing.png"
		end
		if(piece.pieceView.linkNumber == 5) then
			piece.pieceView.image = "LinkYellow_Glowing.png"
		end
		if(piece.pieceView.linkNumber == 6) then
			piece.pieceView.image = "LinkOrange_Glowing.png"
		end
		--And spinning until stopped
		RotateAtSpeed(piece.pieceView,{revolutions=6},1000000,false)
	end
	if(piece.pieceView.isJailedLink) then
		--Explode the jail
		PlaySound("jail_cell_door_metal_impact_09.ogg")
		StartParticleEmitterFromFile {
			file = "JailBreak",
			position = piece.pieceView:GetGlobalPoint(Center),
		}
		piece.pieceView.LinkInJail:Destroy()
	end
	
	--Pulse icon with particles and hum.
	LoopSound("paddleChargeUpLoop.ogg")
	PlaySound("slot_machine_win_08.ogg")
	local pieceCharge = CreateInstance {
		name = "pieceCharge",
		class = "hgeParticleSystemActor",
		file = "ActivatedPowerupAura",
		alignment = "Center",
	}
	piece.pieceView:AddChild(pieceCharge)
	piece.pieceView:AddScript("PulseContinuously","")
end

function Scripts.QueueCandyMixAnimation(rootGameView, delay, allMatches, matchReasonPool)
	ScriptDelay(delay)
	PlaySound("Magic_Respawn18.ogg")
	--Make a table of all pieces involved in mixing
	local mixingPieces = {}
	for i,v in ipairs(allMatches) do
		--Get reason for piece
		for trash,potentialReason in ipairs(matchReasonPool) do
			--Skip any gem that is not mixed.
			if(potentialReason[2] ~= "plusSplash" and
				potentialReason[2] ~= "wideVerticalSplash" and
				potentialReason[2] ~= "wideHorizontalSplash" and
				potentialReason[2] ~= "widePlusSplash") then
				continue
			end
			if(potentialReason[1].x == v.boardCoordinate.x and
				potentialReason[1].y == v.boardCoordinate.y) then
				v.assignedMatchReason = potentialReason[2]
				mixingPieces[#mixingPieces + 1] = v
				break
			end
		end
	end

	--For each piece in the table
	for i,v in ipairs(mixingPieces) do
		for trash,target in ipairs(mixingPieces) do
			if(v == target) then
				continue
			end
			--Shoot sparks at each other
			local destination = GetGlobalCenterPoint(target.pieceView)
			local sparkle = CreateInstance {
				name = "",
				class = "hgeParticleSystemActor",
				file = "BigBrightTrail",
				position = GetGlobalCenterPoint(v.pieceView),
			}
			sparkle.destination  = destination
			sparkle.duration = 200 * DemoAnimationSlowingFactor
			rootGameView:AddChild(sparkle)
			sparkle:AddScript("FlyToDestination","")
		end
	end
	ScriptDelay(200 * DemoAnimationSlowingFactor)

	--Change icons to be the new icons
	for i,v in ipairs(mixingPieces) do
		local decorationActor
		if(v.pieceView.VerticalSplashDecoration ~= nil) then
			decorationActor = v.pieceView.VerticalSplashDecoration
		end
		if(v.pieceView.HorizontalSplashDecoration ~= nil) then
			decorationActor = v.pieceView.HorizontalSplashDecoration
		end
		if(v.pieceView.AreaSplashDecoration ~= nil) then
			decorationActor = v.pieceView.AreaSplashDecoration
		end
		if(v.pieceView.WideVerticalSplashDecoration ~= nil) then
			decorationActor = v.pieceView.WideVerticalSplashDecoration
		end
		if(v.pieceView.WideHorizontalSplashDecoration ~= nil) then
			decorationActor = v.pieceView.WideHorizontalSplashDecoration
		end
		if(v.pieceView.WidePlusSplashDecoration ~= nil) then
			decorationActor = v.pieceView.WidePlusSplashDecoration
		end
		if(v.pieceView.PlusSplashDecoration ~= nil) then
			decorationActor = v.pieceView.PlusSplashDecoration
		end
		if(decorationActor == nil) then
			v.pieceView.VerticalSplashDecoration = Resources.VerticalSplashDecoration
			decorationActor = v.pieceView.VerticalSplashDecoration
		end
		if(v.assignedMatchReason == "plusSplash") then
			decorationActor.image = "Plus.png"
		end
		if(v.assignedMatchReason == "widePlusSplash") then
			decorationActor.image = "WidePlus.png"
		end
		if(v.assignedMatchReason == "wideHorizontalSplash") then
			decorationActor.image = "WideHorizontal.png"
		end
		if(v.assignedMatchReason == "wideVerticalSplash") then
			decorationActor.image = "WideVertical.png"
		end
	end
	ScriptDelay(100 * DemoAnimationSlowingFactor)
end

function Scripts.PulseContinuously(actor)
	actor.oldExtraScale = actor.extraScale
	while(true) do
		ScaleTo(actor,{1.2,1.2},150)
		ScriptDelay(150)
		ScaleTo(actor,{1,1},150)
		ScriptDelay(150)
	end
end

function Scripts.FlyToDestination(actor)
	actor:SlideTo(actor.destination,actor.duration)
	ScriptDelay(actor.duration)
	if(actor.class == "hgeParticleSystemActor") then
		actor:Pause()
	end
	local hangtime = actor.hangtime or 500
	ScriptDelay(hangtime)
	actor:Destroy()
end

function Scripts.FlyToDestination_BURST(actor)
	local particleTrail = actor:FindChild("particleTrail",true)
	actor:SlideTo(actor.destination,actor.duration)
	ScriptDelay(actor.duration)
	if(actor.class == "hgeParticleSystemActor") then
		actor:Pause()
	end
	local hangtime = actor.hangtime or 500
	if(particleTrail)then
		particleTrail:Pause()
	end
	actor.alpha = 0
	FireParticle( actor,"StarToss1.smaller","Center")  
	FireParticle( actor,"SparkleBurst1","Center")
	FireParticle( actor,"ColorBomb1","Center")
	ScriptDelay(hangtime)
	actor:Destroy()
end

function Scripts.FlyToDestination_PATH(actor)
	if(actor.waypoint)then
		actor:SlideTo(actor.waypoint,actor.duration/2)
		ScriptDelay(actor.duration/2)
		actor:SlideTo(actor.destination,actor.duration/2)
		ScriptDelay(actor.duration/2)
	else
		actor:SlideTo(actor.destination,actor.duration)
		ScriptDelay(actor.duration)
	end
	if(actor.class == "hgeParticleSystemActor") then
		actor:Pause()
	end
	local hangtime = actor.hangtime or 500
	ScriptDelay(hangtime)
	actor:Destroy()
end

function Scripts.BoomAtDestination(actor)
	local duration = actor.duration or 0
	ScriptDelay(duration)
	FireParticle( actor,"StarToss1.smaller","Center")  
	FireParticle( actor,"SparkleBurst1","Center")
	FireParticle( actor,"ColorBomb1","Center")
end

function Scripts.BoomAtDestination_pattern(actor)
	local duration = actor.duration or 0
	ScriptDelay(duration)
	FireParticle( actor,"Boom1_small","Center")
end

function Scripts.BoomAtDestination_bedrock(actor,pieceView_IconImageName)
	local duration = actor.duration or 0

	PlaySound("ice_pick_impact_05.ogg")
	
	ScriptDelay(duration)

	local destination = GetGlobalCenterPoint(actor)

	local particleFileName = ""
	if(pieceView_IconImageName=="GemBlue.png")then
		particleFileName = "GemBlueToss"
	elseif(pieceView_IconImageName=="GemGreen.png")then
		particleFileName = "GemGreenToss"
	elseif(pieceView_IconImageName=="GemPurple.png")then
		particleFileName = "GemPurpleToss"
	elseif(pieceView_IconImageName=="GemRed.png")then
		particleFileName = "GemRedToss"
	elseif(pieceView_IconImageName=="GemYellow.png")then
		particleFileName = "GemYellowToss"
	elseif(pieceView_IconImageName=="GemOrange.png")then
		particleFileName = "GemOrangeToss"
	elseif(pieceView_IconImageName=="GemSpecialRed.png")then
		particleFileName = "GemSpecialRedToss"
	elseif(pieceView_IconImageName=="GemSpecialGreen.png")then
		particleFileName = "GemSpecialGreenToss"
	elseif(pieceView_IconImageName=="GemSpecialBlue.png")then
		particleFileName = "GemSpecialBlueToss"
	end

	if(particleFileName~="")then
		StartParticleEmitterFromFile {
			file = particleFileName,
			position = destination,
		}
	end

	FireParticle( actor,"ColorBomb2","Center")
	FireParticle( actor,"StoneBreak4_dark","Center")
	PlaySound("cave_rocks_crashing_07.ogg")
	--PlaySound("boulder_impact_on_stones_11.ogg")



end

function Scripts.RingToss(actor)

    local soundNumber = math.random(1,3)
    PlaySound("Rings"..soundNumber..".ogg")

	local destination = GetGlobalCenterPoint(actor)
	local particleFileName = "RingToss"

    StartParticleEmitterFromFile {
        file = "ColorBurst1_yellow",
        position = destination,
    }

	if(particleFileName~="")then
		StartParticleEmitterFromFile {
			file = particleFileName,
			position = destination,
		}
	end

    actor.IconDecoration.visible = false
end


local splashAnimationDuration = 510 * DemoAnimationSlowingFactor

function AddTrail(actor,particleName)
	local particleTrail = CreateInstance {
		name = "particleTrail",
		class = "hgeParticleSystemActor",
		file = particleName,
		alignment = Center
	}
	actor:AddChild(particleTrail)
end

function Scripts.QueuePowerupEndAnimation(rootGameView, piece, matchID, delay, reasonName, powerupPieces, hits)
	ScriptDelay(delay)

	if(piece.pieceView == nil) then
		return
	end
	--logprint("Running powerup end animation at " .. piece.boardCoordinate.x .. "," .. piece.boardCoordinate.y)

	local piecePosition = piece.boardCoordinate
	local upPosition = {x = piecePosition.x, y = piecePosition.y - 1}
	local downPosition = {x = piecePosition.x, y = piecePosition.y + 1}
	local leftPosition = {x = piecePosition.x - 1, y = piecePosition.y}
	local rightPosition = {x = piecePosition.x + 1, y = piecePosition.y}

	--BoardCell_%d_%d
	local upCellName = "BoardCell_" .. upPosition.x .. "_" .. upPosition.y
	local downCellName = "BoardCell_" .. downPosition.x .. "_" .. downPosition.y
	local leftCellName = "BoardCell_" .. leftPosition.x .. "_" .. leftPosition.y
	local rightCellName = "BoardCell_" .. rightPosition.x .. "_" .. rightPosition.y
	
	local upExists = rootGameView.AllBoardCells:FindChild(upCellName,true) ~= nil
	local downExists = rootGameView.AllBoardCells:FindChild(downCellName,true) ~= nil
	local leftExists = rootGameView.AllBoardCells:FindChild(leftCellName,true) ~= nil
	local rightExists = rootGameView.AllBoardCells:FindChild(rightCellName,true) ~= nil

	--Add custom animation depending on the type of powerup.
	if(reasonName == "superWild") then
		local gotOne = false
		local allSparkles = {}
		local origin = GetGlobalCenterPoint(piece.pieceView)
		piece.pieceView.shaderType = "SHADERTYPE_ADDITIVEALPHABLEND"
		for i,v in ipairs(powerupPieces) do
			gotOne = true
			local destination = GetGlobalCenterPoint(v.pieceView)
			allSparkles[i] = CreateInstance {
				name = "",
				class = "hgeParticleSystemActor",
				file = "SuperWildTrail2",
				position = origin,
			}
			allSparkles[i].destination  = destination
			allSparkles[i].duration = 250 * DemoAnimationSlowingFactor
			rootGameView:AddChild(allSparkles[i])
			allSparkles[i]:AddScript("FlyToDestination","")
			allSparkles[i]:AddScript("BoomAtDestination","")
		end
		if(gotOne)then
			PlaySound("laserShot1.ogg")
		end
		--Make sparkle actors travel to centers of each destination, and then be destroyed.
		ScriptDelay(200 * DemoAnimationSlowingFactor)
	end
	
	local splashEffectName = "SplashEffectLeft.png"
	if  (piece.pieceView.isLaser) then
		if (piece.pieceView.isRedLaser) then
			splashEffectName = "SplashEffectLeft_Red.png"
		elseif (piece.pieceView.isBlueLaser) then
			splashEffectName = "SplashEffectLeft_Blue.png"
		elseif (piece.pieceView.isGreenLaser) then
			splashEffectName = "SplashEffectLeft_Green.png"
		elseif (piece.pieceView.isPurpleLaser) then
			splashEffectName = "SplashEffectLeft_Purple.png"
		elseif (piece.pieceView.isYellowLaser) then
			splashEffectName = "SplashEffectLeft_Yellow.png"
		elseif (piece.pieceView.isOrangeLaser) then
			splashEffectName = "SplashEffectLeft_Orange.png"
		elseif (piece.pieceView.isWildLaser) then
			splashEffectName = "SplashEffectLeft_Wild.png"
		end
	end

	if(reasonName == "horizontalSplash") then
		ScriptDelay(50 * DemoAnimationSlowingFactor)
		if(piece.pieceView.isLaser) then
			PlaySound("Laser_Shot08.ogg")
		else
			PlaySound("Magic_CrystalBlast7.ogg")
		end
		--Make a wave-front in each direction
		--Fly it to the edge of the board
		local leftSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			layer = 7,
		}
		rootGameView:AddChild(leftSplashEffect)
        local leftDistance = piece.extraData.BurstLeft0
        if(leftDistance == nil) then
            leftDistance = 10
        end
		leftSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		leftSplashEffect.destination = {x = leftSplashEffect.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance, y = leftSplashEffect.position.y}
		leftSplashEffect.duration = splashAnimationDuration * leftDistance/10
		AddTrail(leftSplashEffect,"BigBrightTrail")
		leftSplashEffect:AddScript("FlyToDestination_BURST","")


		local rightSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 180},
			layer = 7,
		}
		rootGameView:AddChild(rightSplashEffect)
        local rightDistance = piece.extraData.BurstRight0
        if(rightDistance == nil) then
            rightDistance = 10
        end
		rightSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		rightSplashEffect.destination = {x = rightSplashEffect.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance, y = rightSplashEffect.position.y}
		rightSplashEffect.duration = splashAnimationDuration * rightDistance/10
		AddTrail(rightSplashEffect,"BigBrightTrail")
		rightSplashEffect:AddScript("FlyToDestination_BURST","")

		
		ScriptDelay(150 * DemoAnimationSlowingFactor)
	end
	
	if(reasonName == "verticalSplash") then
		ScriptDelay(50 * DemoAnimationSlowingFactor)
		if(piece.pieceView.isLaser) then
			PlaySound("Laser_Shot08.ogg")
		else
			PlaySound("Magic_CrystalBlast7.ogg")
		end
		--Make a wave-front in each direction
		--Fly it to the edge of the board
		local topSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 90},
			layer = 7,
		}
		rootGameView:AddChild(topSplashEffect)
        local upDistance = piece.extraData.BurstUp0
        if(upDistance == nil) then
            upDistance = 10
        end
		topSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		topSplashEffect.destination = {x = topSplashEffect.position.x, y = topSplashEffect.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance}
		topSplashEffect.duration = splashAnimationDuration * upDistance/10
		AddTrail(topSplashEffect,"BigBrightTrail")
		topSplashEffect:AddScript("FlyToDestination_BURST","")


		local bottomSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 270},
			layer = 7,
		}
		rootGameView:AddChild(bottomSplashEffect)
        local downDistance = piece.extraData.BurstDown0
        if(downDistance == nil) then
            downDistance = 10
        end
		bottomSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		bottomSplashEffect.destination = {x = bottomSplashEffect.position.x, y = bottomSplashEffect.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance}
		bottomSplashEffect.duration = splashAnimationDuration * downDistance/10
		AddTrail(bottomSplashEffect,"BigBrightTrail")
		bottomSplashEffect:AddScript("FlyToDestination_BURST","")


		ScriptDelay(150 * DemoAnimationSlowingFactor)

	end
	
	if(reasonName == "areaSplash") then
		--Make a circular graphic around the splash
		if(piece.pieceView.isLaser) then
			PlaySound("Laser_Shot08.ogg")
		else
			PlaySound("Magic_CrystalBlast7.ogg")
		end
		--Make a wave-front surrounding the source piece
		--Fly it to the edge of the board
		local areaSplashEffect = CreateInstance {
			resource = "Actor",
			image = "AreaSplashEffect.png",
			size = {piece.pieceView.size.x*3,piece.pieceView.size.y*3},
			layer = 7,
			visible = false,
		}
		rootGameView:AddChild(areaSplashEffect)
		areaSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)

		AddTrail(areaSplashEffect,"BigBrightTrail")

		ScaleIn(areaSplashEffect,{0.25,0.25},250 * DemoAnimationSlowingFactor,true)
		Pop(areaSplashEffect)
		--Make it go away again
		rootGameView:DestroyChild(areaSplashEffect)
	end
	
	if(reasonName == "wideHorizontalSplash") then
		--Make a wave-front in each direction
		ScriptDelay(50 * DemoAnimationSlowingFactor)
		if(piece.pieceView.isLaser) then
			PlaySound("Laser_Shot08.ogg")
		else
			PlaySound("Magic_CrystalBlast7.ogg")
		end

		--Make a wave-front in each direction
		--Fly it to the edge of the board
		local leftSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			layer = 7,
		}
		rootGameView:AddChild(leftSplashEffect0)
        local leftDistance0 = piece.extraData.BurstLeft0
        if(leftDistance0 == nil) then
            leftDistance0 = 10
        end
		leftSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		leftSplashEffect0.destination = {x = leftSplashEffect0.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance0, y = leftSplashEffect0.position.y}
		leftSplashEffect0.duration = splashAnimationDuration * leftDistance0 / 10
		AddTrail(leftSplashEffect0,"BigBrightTrail")
		leftSplashEffect0:AddScript("FlyToDestination_BURST","")
        
		if(upExists) then
			local leftSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				layer = 7,
			}
			rootGameView:AddChild(leftSplashEffect1)
			local leftDistance1 = piece.extraData.BurstLeft1
			if(leftDistance1 == nil) then
				leftDistance1 = 10
			end
			local offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y - piece.pieceView.size.y
			leftSplashEffect1.center = offsetCenter
			leftSplashEffect1.destination = {x = leftSplashEffect1.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance1, y = leftSplashEffect1.position.y}
			leftSplashEffect1.duration = splashAnimationDuration * leftDistance1 / 10
			AddTrail(leftSplashEffect1,"BigBrightTrail")
			leftSplashEffect1:AddScript("FlyToDestination_BURST","")
		end

		if(downExists) then
			local leftSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				layer = 7,
			}
			rootGameView:AddChild(leftSplashEffect2)
			local leftDistance2 = piece.extraData.BurstLeft2
			if(leftDistance2 == nil) then
				leftDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y + piece.pieceView.size.y
			leftSplashEffect2.center = offsetCenter
			leftSplashEffect2.destination = {x = leftSplashEffect2.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance2, y = leftSplashEffect2.position.y}
			leftSplashEffect2.duration = splashAnimationDuration * leftDistance2 / 10
			AddTrail(leftSplashEffect2,"BigBrightTrail")
			leftSplashEffect2:AddScript("FlyToDestination_BURST","")
		end


		local rightSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 180},
			layer = 7,
		}
		rootGameView:AddChild(rightSplashEffect0)
        local rightDistance0 = piece.extraData.BurstRight0
        if(rightDistance0 == nil) then
            rightDistance0 = 10
        end
		rightSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		rightSplashEffect0.destination = {x = rightSplashEffect0.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance0, y = rightSplashEffect0.position.y}
		rightSplashEffect0.duration = splashAnimationDuration * rightDistance0/10
		AddTrail(rightSplashEffect0,"BigBrightTrail")
		rightSplashEffect0:AddScript("FlyToDestination_BURST","")
		
		if(upExists) then
			local rightSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 180},
				layer = 7,
			}
			rootGameView:AddChild(rightSplashEffect1)
			local rightDistance1 = piece.extraData.BurstRight1
			if(rightDistance1 == nil) then
				rightDistance1 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y - piece.pieceView.size.y
			rightSplashEffect1.center = offsetCenter
			rightSplashEffect1.destination = {x = rightSplashEffect1.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance1, y = rightSplashEffect1.position.y}
			rightSplashEffect1.duration = splashAnimationDuration * rightDistance1/10
			AddTrail(rightSplashEffect1,"BigBrightTrail")
			rightSplashEffect1:AddScript("FlyToDestination_BURST","")
		end
        
		if(downExists) then
			local rightSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 180},
				layer = 7,
			}
			rootGameView:AddChild(rightSplashEffect2)
			local rightDistance2 = piece.extraData.BurstRight2
			if(rightDistance2 == nil) then
				rightDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y + piece.pieceView.size.y
			rightSplashEffect2.center = offsetCenter
			rightSplashEffect2.destination = {x = rightSplashEffect2.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance2, y = rightSplashEffect2.position.y}
			rightSplashEffect2.duration = splashAnimationDuration * rightDistance2/10
			AddTrail(rightSplashEffect2,"BigBrightTrail")
			rightSplashEffect2:AddScript("FlyToDestination_BURST","")
		end
		
		ScriptDelay(150 * DemoAnimationSlowingFactor)
	end
	
	if(reasonName == "wideVerticalSplash") then
        local globalCenter = GetGlobalCenterPoint(piece.pieceView)
		ScriptDelay(50 * DemoAnimationSlowingFactor)
		if(piece.pieceView.isLaser) then
			PlaySound("Laser_Shot08.ogg")
		else
			PlaySound("Magic_CrystalBlast7.ogg")
		end
		--Make a wave-front in each direction
		--Fly it to the edge of the board
		local topSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 90},
			layer = 7,
		}
		rootGameView:AddChild(topSplashEffect0)
        local upDistance0 = piece.extraData.BurstUp0
        if(upDistance0 == nil) then
            upDistance0 = 10
        end
		topSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		topSplashEffect0.destination = {x = topSplashEffect0.position.x, y = topSplashEffect0.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance0}
		topSplashEffect0.duration = splashAnimationDuration * upDistance0 / 10
		AddTrail(topSplashEffect0,"BigBrightTrail")
		topSplashEffect0:AddScript("FlyToDestination_BURST","")
        
		if(leftExists) then
			local topSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 90},
				layer = 7,
			}
			rootGameView:AddChild(topSplashEffect1)

			local upDistance1 = piece.extraData.BurstUp1
			if(upDistance1 == nil) then
				upDistance1 = 10
			end
			local offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x - piece.pieceView.size.x
			topSplashEffect1.center = offsetCenter
			topSplashEffect1.destination = {x = topSplashEffect1.position.x, y = topSplashEffect1.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance1}
			topSplashEffect1.duration = splashAnimationDuration * upDistance1 / 10
			AddTrail(topSplashEffect1,"BigBrightTrail")
			topSplashEffect1:AddScript("FlyToDestination_BURST","")
		end

		if(rightExists) then
			local topSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 90},
				layer = 7,
			}
			rootGameView:AddChild(topSplashEffect2)

			local upDistance2 = piece.extraData.BurstUp2
			if(upDistance2 == nil) then
				upDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x + piece.pieceView.size.x
			topSplashEffect2.center = offsetCenter
			topSplashEffect2.destination = {x = topSplashEffect2.position.x, y = topSplashEffect2.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance2}
			topSplashEffect2.duration = splashAnimationDuration * upDistance2 / 10
			AddTrail(topSplashEffect2,"BigBrightTrail")
			topSplashEffect2:AddScript("FlyToDestination_BURST","")
		end
		
		local bottomSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 270},
			layer = 7,
		}
		rootGameView:AddChild(bottomSplashEffect0)
        local downDistance0 = piece.extraData.BurstDown0
        if(downDistance0 == nil) then
            downDistance0 = 10
        end
		bottomSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		bottomSplashEffect0.destination = {x = bottomSplashEffect0.position.x, y = bottomSplashEffect0.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance0}
		bottomSplashEffect0.duration = splashAnimationDuration * downDistance0 / 10
		AddTrail(bottomSplashEffect0,"BigBrightTrail")
		bottomSplashEffect0:AddScript("FlyToDestination_BURST","")
		
		if(leftExists) then
			local bottomSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 270},
				layer = 7,
			}
			rootGameView:AddChild(bottomSplashEffect1)
			local downDistance1 = piece.extraData.BurstDown0
			if(downDistance1 == nil) then
				downDistance1 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x - piece.pieceView.size.x
			bottomSplashEffect1.center = offsetCenter
			bottomSplashEffect1.destination = {x = bottomSplashEffect1.position.x, y = bottomSplashEffect1.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance1}
			bottomSplashEffect1.duration = splashAnimationDuration * downDistance1 / 10
			AddTrail(bottomSplashEffect1,"BigBrightTrail")
			bottomSplashEffect1:AddScript("FlyToDestination_BURST","")
		end
		
		if(rightExists) then
			local bottomSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 270},
				layer = 7,
			}
			rootGameView:AddChild(bottomSplashEffect2)
			local downDistance2 = piece.extraData.BurstDown2
			if(downDistance2 == nil) then
				downDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x + piece.pieceView.size.x
			bottomSplashEffect2.center = offsetCenter
			bottomSplashEffect2.destination = {x = bottomSplashEffect2.position.x, y = bottomSplashEffect2.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance2}
			bottomSplashEffect2.duration = splashAnimationDuration * downDistance2 / 10
			AddTrail(bottomSplashEffect2,"BigBrightTrail")
			bottomSplashEffect2:AddScript("FlyToDestination_BURST","")
		end
		
		ScriptDelay(150 * DemoAnimationSlowingFactor)
	end
	
	if(reasonName == "plusSplash") then
		ScriptDelay(50 * DemoAnimationSlowingFactor)
		if(piece.pieceView.isLaser) then
			PlaySound("Laser_Shot08.ogg")
		else
			PlaySound("Magic_CrystalBlast7.ogg")
		end
		--Make a wave-front in each direction
		--Fly it to the edge of the board
		local leftSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			layer = 7,
		}
		rootGameView:AddChild(leftSplashEffect)
        local leftDistance = piece.extraData.BurstLeft0
        if(leftDistance == nil) then
            leftDistance = 10
        end
		leftSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		leftSplashEffect.destination = {x = leftSplashEffect.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance, y = leftSplashEffect.position.y}
		leftSplashEffect.duration = splashAnimationDuration * leftDistance / 10
		AddTrail(leftSplashEffect,"BigBrightTrail")
		leftSplashEffect:AddScript("FlyToDestination_BURST","")
		
		local rightSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 180},
			layer = 7,
		}
		rootGameView:AddChild(rightSplashEffect)
        local rightDistance = piece.extraData.BurstRight0
        if(rightDistance == nil) then
            rightDistance = 10
        end
		rightSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		rightSplashEffect.destination = {x = rightSplashEffect.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance, y = rightSplashEffect.position.y}
		rightSplashEffect.duration = splashAnimationDuration * rightDistance / 10
		AddTrail(rightSplashEffect,"BigBrightTrail")
		rightSplashEffect:AddScript("FlyToDestination_BURST","")
		
		local topSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 90},
			layer = 7,
		}
		rootGameView:AddChild(topSplashEffect)
        local upDistance = piece.extraData.BurstUp0
        if(upDistance == nil) then
            upDistance = 10
        end
		topSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		topSplashEffect.destination = {x = topSplashEffect.position.x, y = topSplashEffect.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance}
		topSplashEffect.duration = splashAnimationDuration * upDistance / 10
		AddTrail(topSplashEffect,"BigBrightTrail")
		topSplashEffect:AddScript("FlyToDestination_BURST","")
		
		local bottomSplashEffect = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 270},
			layer = 7,
		}
		rootGameView:AddChild(bottomSplashEffect)
        local downDistance = piece.extraData.BurstDown0
        if(downDistance == nil) then
            downDistance = 10
        end
		bottomSplashEffect.center = GetGlobalCenterPoint(piece.pieceView)
		bottomSplashEffect.destination = {x = bottomSplashEffect.position.x, y = bottomSplashEffect.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance}
		bottomSplashEffect.duration = splashAnimationDuration * downDistance/10
		AddTrail(bottomSplashEffect,"BigBrightTrail")
		bottomSplashEffect:AddScript("FlyToDestination_BURST","")
		
		ScriptDelay(150 * DemoAnimationSlowingFactor)
	end
	
	if(reasonName == "widePlusSplash") then
		ScriptDelay(50 * DemoAnimationSlowingFactor)
		if(piece.pieceView.isLaser) then
			PlaySound("Laser_Shot08.ogg")
		else
			PlaySound("Magic_CrystalBlast7.ogg")
		end
		--Make a wave-front in each direction
		--Fly it to the edge of the board
		local leftSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			layer = 7,
		}
		rootGameView:AddChild(leftSplashEffect0)
        local leftDistance0 = piece.extraData.BurstLeft0
        if(leftDistance0 == nil) then
            leftDistance0 = 10
        end
		leftSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		leftSplashEffect0.destination = {x = leftSplashEffect0.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance0, y = leftSplashEffect0.position.y}
		leftSplashEffect0.duration = splashAnimationDuration * leftDistance0 / 10
		AddTrail(leftSplashEffect0,"BigBrightTrail")
		leftSplashEffect0:AddScript("FlyToDestination_BURST","")
		
		if(upExists) then
			local leftSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				layer = 7,
			}
			rootGameView:AddChild(leftSplashEffect1)
			local leftDistance1 = piece.extraData.BurstLeft1
			if(leftDistance1 == nil) then
				leftDistance1 = 10
			end
			local offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y - piece.pieceView.size.y
			leftSplashEffect1.center = offsetCenter
			leftSplashEffect1.destination = {x = leftSplashEffect1.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance1, y = leftSplashEffect1.position.y}
			leftSplashEffect1.duration = splashAnimationDuration * leftDistance1 / 10
			AddTrail(leftSplashEffect1,"BigBrightTrail")
			leftSplashEffect1:AddScript("FlyToDestination_BURST","")
		end
		
		if(downExists) then
			local leftSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				layer = 7,
			}
			rootGameView:AddChild(leftSplashEffect2)
			local leftDistance2 = piece.extraData.BurstLeft2
			if(leftDistance2 == nil) then
				leftDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y + piece.pieceView.size.y
			leftSplashEffect2.center = offsetCenter
			leftSplashEffect2.destination = {x = leftSplashEffect2.position.x - piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * leftDistance2, y = leftSplashEffect2.position.y}
			leftSplashEffect2.duration = splashAnimationDuration * leftDistance2 / 10
			AddTrail(leftSplashEffect2,"BigBrightTrail")
			leftSplashEffect2:AddScript("FlyToDestination_BURST","")
		end
		
		local rightSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 180},
			layer = 7,
		}
		rootGameView:AddChild(rightSplashEffect0)
        local rightDistance0 = piece.extraData.BurstRight0
        if(rightDistance0 == nil) then
            rightDistance0 = 10
        end
		rightSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		rightSplashEffect0.destination = {x = rightSplashEffect0.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance0, y = rightSplashEffect0.position.y}
		rightSplashEffect0.duration = splashAnimationDuration * rightDistance0/10
		AddTrail(rightSplashEffect0,"BigBrightTrail")
		rightSplashEffect0:AddScript("FlyToDestination_BURST","")
		
		if(upExists) then
			local rightSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 180},
				layer = 7,
			}
			rootGameView:AddChild(rightSplashEffect1)
			local rightDistance1 = piece.extraData.BurstRight1
			if(rightDistance1 == nil) then
				rightDistance1 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y - piece.pieceView.size.y
			rightSplashEffect1.center = offsetCenter
			rightSplashEffect1.destination = {x = rightSplashEffect1.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance1, y = rightSplashEffect1.position.y}
			rightSplashEffect1.duration = splashAnimationDuration * rightDistance1/10
			AddTrail(rightSplashEffect1,"BigBrightTrail")
			rightSplashEffect1:AddScript("FlyToDestination_BURST","")
		end
		
		if(downExists) then 
			local rightSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 180},
				layer = 7,
			}
			rootGameView:AddChild(rightSplashEffect2)
			local rightDistance2 = piece.extraData.BurstRight2
			if(rightDistance2 == nil) then
				rightDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.y = offsetCenter.y - piece.pieceView.size.y
			rightSplashEffect2.center = offsetCenter
			rightSplashEffect2.destination = {x = rightSplashEffect2.position.x + piece.pieceView.oldExtraScale.x * piece.pieceView.size.x * rightDistance2, y = rightSplashEffect2.position.y}
			rightSplashEffect2.duration = splashAnimationDuration * rightDistance2/10
			AddTrail(rightSplashEffect2,"BigBrightTrail")
			rightSplashEffect2:AddScript("FlyToDestination_BURST","")
		end
		        
        local topSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 90},
			layer = 7,
		}
		rootGameView:AddChild(topSplashEffect0)
        local upDistance0 = piece.extraData.BurstUp0
        if(upDistance0 == nil) then
            upDistance0 = 10
        end
		topSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		topSplashEffect0.destination = {x = topSplashEffect0.position.x, y = topSplashEffect0.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance0}
		topSplashEffect0.duration = splashAnimationDuration * upDistance0 / 10
		AddTrail(topSplashEffect0,"BigBrightTrail")
		topSplashEffect0:AddScript("FlyToDestination_BURST","")
		
		if(leftExists) then
			local topSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 90},
				layer = 7,
			}
			rootGameView:AddChild(topSplashEffect1)

			local upDistance1 = piece.extraData.BurstUp1
			if(upDistance1 == nil) then
				upDistance1 = 10
			end
			local offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x - piece.pieceView.size.x
			topSplashEffect1.center = offsetCenter
			topSplashEffect1.destination = {x = topSplashEffect1.position.x, y = topSplashEffect1.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance1}
			topSplashEffect1.duration = splashAnimationDuration * upDistance1 / 10
			AddTrail(topSplashEffect1,"BigBrightTrail")
			topSplashEffect1:AddScript("FlyToDestination_BURST","")
		end

		if(rightExists) then
			local topSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 90},
				layer = 7,
			}
			rootGameView:AddChild(topSplashEffect2)

			local upDistance2 = piece.extraData.BurstUp2
			if(upDistance2 == nil) then
				upDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x + piece.pieceView.size.x
			topSplashEffect2.center = offsetCenter
			topSplashEffect2.destination = {x = topSplashEffect2.position.x, y = topSplashEffect2.position.y - piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * upDistance2}
			topSplashEffect2.duration = splashAnimationDuration * upDistance2 / 10
			AddTrail(topSplashEffect2,"BigBrightTrail")
			topSplashEffect2:AddScript("FlyToDestination_BURST","")
		end

		local bottomSplashEffect0 = CreateInstance {
			resource = "Actor",
			image = splashEffectName,
			size = {piece.pieceView.size.x,piece.pieceView.size.y},
			rotationAngle = {degrees = 270},
			layer = 7,
		}
		rootGameView:AddChild(bottomSplashEffect0)
        local downDistance0 = piece.extraData.BurstDown0
        if(downDistance0 == nil) then
            downDistance0 = 10
        end
		bottomSplashEffect0.center = GetGlobalCenterPoint(piece.pieceView)
		bottomSplashEffect0.destination = {x = bottomSplashEffect0.position.x, y = bottomSplashEffect0.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance0}
		bottomSplashEffect0.duration = splashAnimationDuration * downDistance0 / 10
		AddTrail(bottomSplashEffect0,"BigBrightTrail")
		bottomSplashEffect0:AddScript("FlyToDestination_BURST","")
		
		if(leftExists) then
			local bottomSplashEffect1 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 270},
				layer = 7,
			}
			rootGameView:AddChild(bottomSplashEffect1)
			local downDistance1 = piece.extraData.BurstDown0
			if(downDistance1 == nil) then
				downDistance1 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x - piece.pieceView.size.x
			bottomSplashEffect1.center = offsetCenter
			bottomSplashEffect1.destination = {x = bottomSplashEffect1.position.x, y = bottomSplashEffect1.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance1}
			bottomSplashEffect1.duration = splashAnimationDuration * downDistance1 / 10
			AddTrail(bottomSplashEffect1,"BigBrightTrail")
			bottomSplashEffect1:AddScript("FlyToDestination_BURST","")
		end
		
		if(rightExists) then
			local bottomSplashEffect2 = CreateInstance {
				resource = "Actor",
				image = splashEffectName,
				size = {piece.pieceView.size.x,piece.pieceView.size.y},
				rotationAngle = {degrees = 270},
				layer = 7,
			}
			rootGameView:AddChild(bottomSplashEffect2)
			local downDistance2 = piece.extraData.BurstDown2
			if(downDistance2 == nil) then
				downDistance2 = 10
			end
			offsetCenter = GetGlobalCenterPoint(piece.pieceView)
			offsetCenter.x = offsetCenter.x + piece.pieceView.size.x
			bottomSplashEffect2.center = offsetCenter
			bottomSplashEffect2.destination = {x = bottomSplashEffect2.position.x, y = bottomSplashEffect2.position.y + piece.pieceView.oldExtraScale.y * piece.pieceView.size.y * downDistance2}
			bottomSplashEffect2.duration = splashAnimationDuration * downDistance2 / 10
			AddTrail(bottomSplashEffect2,"BigBrightTrail")
			bottomSplashEffect2:AddScript("FlyToDestination_BURST","")
		end
		
		ScriptDelay(150 * DemoAnimationSlowingFactor)
	end

	--End the animations of how excited this piece is to do its thing.
	if(reasonName == piece.extraData.splashType) then --Don't run twice on dual-type squares
		StopSound("paddleChargeUpLoop.ogg")
		piece.pieceView:KillTaggedScripts("PulseContinuously")
		piece.pieceView.pieceCharge:Pause()
	end

	--Make the piece go away with normal matching animations.
	local specialCount = 0
	if(IsNumber(piece.extraData.specialGemAmount)) then
		specialCount = piece.extraData.specialGemAmount
	end
	RunScriptWithParams {
		target = rootGameView,
		script = "PieceMatchAnimationStart",
		params = { 
			piece.extraData.score, 
			matchID, 
			0,
			piece.pieceView,
			true,
			specialCount},
		tags = { matchAnimationTag },
	}

	if(piece.extraData.matchProgress) then
		if(reasonName == piece.extraData.splashType) then --Don't run twice on dual-type squares
			RunScriptWithParams {
				target = rootGameView,
				script = "UpdateMatchProgress",
				params = {piece,hits},
				tags = { matchAnimationTag },
			}
		end
	end

	if(piece.extraData.spawnedPieceView ~= nil) then
		RunScriptWithParams {
			target = rootGameView,
			script = "SpawnedPieceAnimation",
			params = { 
				piece.pieceView, 
				piece.extraData.spawnedPieceView
			},
			tags = { matchAnimationTag },
		}
	end
	
	if(reasonName == piece.extraData.splashType) then --Don't run twice on dual-type squares
		AddScriptWithParams(rootGameView,"RemoveMatchedPieceAnimation", piece.pieceView)
	end
end

function Scripts.ShowPieceCreation(rootGameView,pieceView,createdPieceView,sequence)
	local particleFile = "SuperWildTrail2"
	if(pieceView.namePrefix == "Blue_")then
		particleFile = "SuperWildTrail2_blue"
	end
	if(pieceView.namePrefix == "Green_")then
		particleFile = "SuperWildTrail2_green"
	end
	if(pieceView.namePrefix == "Purple_")then
		particleFile = "SuperWildTrail2_purple"
	end
	if(pieceView.namePrefix == "Red_")then
		particleFile = "SuperWildTrail2_red"
	end
	if(pieceView.namePrefix == "Yellow_")then
		particleFile = "SuperWildTrail2_yellow"
	end
	if(pieceView.namePrefix == "Orange_")then
		particleFile = "SuperWildTrail2_orange"
	end

	-- Create Sparkle
	local sparkle = CreateInstance {
		name = "",
		class = "hgeParticleSystemActor",
		file = particleFile,
		position = GetGlobalCenterPoint(pieceView),
	}

	-- Set position, destination, and waypoint (if horizontal or vertical burst)
	sparkle.destination = GetGlobalCenterPoint(createdPieceView)
	pieceView_position = GetGlobalCenterPoint(pieceView)
	
	if(createdPieceView.VerticalSplashAura)then
		sparkle.waypoint = GetGlobalCenterPoint(createdPieceView)
		sparkle.waypoint.y = pieceView_position.y
	elseif(createdPieceView.HorizontalSplashAura)then
		sparkle.waypoint = GetGlobalCenterPoint(createdPieceView)
		sparkle.waypoint.x = pieceView_position.x
	end

	sparkle.duration = 200 * DemoAnimationSlowingFactor

	createdPieceView:AddChild(sparkle)
	
	-- Since it is added to createdPieceView, need to convert the global positions of sparkle to local
	sparkle.position  = createdPieceView:GlobalToLocalCoord(sparkle.position)
	sparkle.destination  = createdPieceView:GlobalToLocalCoord(sparkle.destination)
	if(sparkle.waypoint)then
		sparkle.waypoint  = createdPieceView:GlobalToLocalCoord(sparkle.waypoint)
	end

	-- Now fly it using a PATH for waypoints
	sparkle:AddScript("FlyToDestination_PATH","")

	ScriptDelay(100 * DemoAnimationSlowingFactor)

	if(sequence==1)then
		PlaySound("slot_machine_win_09.ogg")
	end
	ScriptDelay(50 * DemoAnimationSlowingFactor)

	createdPieceView.visible = true --Created piece needs to only pop once, not many times
	if(createdPieceView.oldScale == nil) then
		createdPieceView.oldScale = createdPieceView.extraScale
		ScaleTo(createdPieceView,{1.15, 1.25},150)
		ScriptDelay(150)
		ScaleTo(createdPieceView,createdPieceView.oldScale,250)
		ScriptDelay(250)
	end
end

function Scripts.HideSpawnedGem(rootGameView,pieceView)
    pieceView.visible = false
end

function Scripts.QueuePieceAnimation(rootGameView, piece, matchID, delay,sequence,maxSequence,matchReason,hits, hitsRequired)
	local destroyAtEnd = hits == hitsRequired

	ScriptDelay(delay)

	if(sequence == 1) then
		--Play as much of the win tones as possible
		local soundIndex = maxSequence
		if(soundIndex>11)then
			soundIndex = 11
		end
		if(soundIndex < 3) then
			soundIndex = 3
		end
		local soundName = "Win"..soundIndex..".ogg"
		PlaySound(soundName)
	end
	
	if(sequence % 11 + 1 == 1) then
		local remainingSounds = maxSequence - sequence + 1
		local soundIndex = remainingSounds
		if(soundIndex>11)then
			soundIndex = 11
		end
		if(soundIndex < 3) then
			soundIndex = 3
		end
		local soundName = "Win"..soundIndex..".ogg"
		PlaySound(soundName)
	end

	--logprint("Queued piece animation for piece at " .. piece.boardCoordinate.x .. "," .. piece.boardCoordinate.y)
	--logprint("hits == " .. hits)
	--logprint("hits required == " .. hitsRequired)
	--logprint("delay == " .. delay)

	if(piece.extraData.createdPieceView ~= nil and (matchReason == "core" or matchReason == "gemswipe" or matchReason == "hammer" or matchReason == "stealPiece")) then
		RunScriptWithParams {
			target = rootGameView,
			script = "ShowPieceCreation",
			params = { 
				piece.pieceView, 
				piece.extraData.createdPieceView,
				sequence},
			tags = { matchAnimationTag },
		}
	end

	if(piece.extraData.spawnedPieceView ~= nil and destroyAtEnd) then
		RunScriptWithParams {
			target = rootGameView,
			script = "SpawnedPieceAnimation",
			params = { 
				piece.pieceView, 
				piece.extraData.spawnedPieceView
			},
			tags = { matchAnimationTag },
		}
	end

	if(hits <= hitsRequired and piece.pieceView ~= nil) then
		local specialCount = 0
		if(IsNumber(piece.extraData.specialGemAmount)) then
			specialCount = piece.extraData.specialGemAmount
		end
		RunScriptWithParams {
			target = rootGameView,
			script = "PieceMatchAnimationStart",
			params = { 
				piece.extraData.score, 
				matchID, 
				maxSequence,
				piece.pieceView,
				destroyAtEnd,
				specialCount,},
			tags = { matchAnimationTag },
		}
	end

	if(piece.extraData.matchProgress) then
		RunScriptWithParams {
			target = rootGameView,
			script = "UpdateMatchProgress",
			params = {piece, hits},
			tags = { matchAnimationTag },
		}
	end
	
	if(destroyAtEnd) then
		AddScriptWithParams(rootGameView,"RemoveMatchedPieceAnimation", piece.pieceView)
	end
end

function Scripts.UpdateMatchProgress(rootGameView, piece, hitNumber)
	local coord = piece.boardCoordinate
	local matchFraction = piece.extraData.matchFraction
	local remainingMatches = piece.extraData.remainingMatches
	local requiredMatches = piece.extraData.requiredMatches
	local oldMatchesMade = piece.extraData.oldMatchesMade
	local matchesMade = piece.extraData.matchesMade
	local totalHits = piece.extraData.hits
	local hitsAfterThis = totalHits - hitNumber
	local matchDeltaFromThisPlay = matchesMade - oldMatchesMade
	local matchDeltaAtThisPoint = matchDeltaFromThisPlay - hitsAfterThis
	if(matchDeltaAtThisPoint < 0) then --If the matches aren't happening until later (maybe because of ignore patterns on this gem type), do nothing.
		matchDeltaAtThisPoint = 0
	end
	local matchesMadeAtThisPoint = oldMatchesMade + matchDeltaAtThisPoint
	local remainingMatchesAtThisPoint = requiredMatches - matchesMadeAtThisPoint
	if(remainingMatchesAtThisPoint < remainingMatches) then
		remainingMatchesAtThisPoint = remainingMatches
	end

	-- Update extent of pattern completion
	local allUnmatchedSpaces = rootGameView:FindChild("AllUnmatchedSpaces",true)
	local nameOfMatchingActor = "BoardSpaceMatchedView_" .. coord.x .. "_" .. coord.y
	local matchingActor = allUnmatchedSpaces:FindChild(nameOfMatchingActor,true)
	--logprint("Update match progress: " .. matchFraction .. "," .. remainingMatches .. "," .. requiredMatches)
	local patternImage_table = PatternImageTable
	if(requiredMatches==1)then patternImage_table = PatternImageTable1 end
	if(requiredMatches==2)then patternImage_table = PatternImageTable2 end
	if(requiredMatches==3)then patternImage_table = PatternImageTable3 end
	if(requiredMatches==4)then patternImage_table = PatternImageTable4 end
	if(requiredMatches==5)then patternImage_table = PatternImageTable5 end
		
	if(matchingActor) then
		--Undo and interrupt previous animation
		matchingActor:KillTaggedScripts("Shake")
		StopScaling(matchingActor)
		matchingActor:StopRotation()
		matchingActor.rotationAngle = {degrees = 0}
		matchingActor.extraScale = 1

		if(remainingMatchesAtThisPoint == 0) then
			local oldScale = matchingActor.extraScale
			PlaySound("slot_machine_win_21_short3.ogg")
			ScaleOut(matchingActor,{2.0,2.0},300,"nowait")
			matchingActor.alpha = 255
			matchingActor.layer = 10
			Scripts.Interpolate(matchingActor, "alpha", 0, 300)
			matchingActor.extraScale = oldScale
			if(requiredMatches > table.getn(patternImage_table)) then
				requiredMatches = table.getn(patternImage_table)
			end
			local oldSize = matchingActor.size
			matchingActor.image = patternImage_table[requiredMatches]
			matchingActor.size = oldSize
		else
			PlaySound("slot_machine_win_21_short2.ogg")
			matchingActor:AddScript("Shake","Shake")
			ScaleIn(matchingActor,{1.4,1.4},270,"nowait")
			if(remainingMatches > table.getn(patternImage_table)) then
				remainingMatches = table.getn(patternImage_table)
			end
			local oldSize = matchingActor.size
			matchingActor.image = patternImage_table[remainingMatchesAtThisPoint]
			matchingActor.size = oldSize
		end
	end
end

Scripts.Shake = function(actor)
	local amount = 10
	local speed = 50
	RotateByAmount(actor,{degrees=-amount},speed,true)
	RotateByAmount(actor,{degrees=amount*2},speed*2,true)
	RotateByAmount(actor,{degrees=-amount},speed,true)
	RotateByAmount(actor,{degrees=-amount/2},speed/2,true)
	RotateByAmount(actor,{degrees=amount/2*2},speed/2*2,true)
	RotateByAmount(actor,{degrees=-amount/2},speed/2,true)	
	RotateByAmount(actor,{degrees=-amount/4},speed/4,true)
	RotateByAmount(actor,{degrees=amount/4*2},speed/4*2,true)
	RotateByAmount(actor,{degrees=-amount/4},speed/4,true)	
	actor.rotation = { degrees = 0 }		
end

Scripts.Jello = function(actor)
	local time = 100
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.5, 1.0}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.25, 1.0}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.15, 1.0}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.05, 1.0}})
end
Scripts.Jello2 = function(actor)
	local time = 100
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.5, 1.0}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0, 1.25}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.15, 1.0}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0, 1.05}})
end

Scripts.JelloSquare = function(actor)
	local time = 100
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.25, 1.0}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0, 1.15}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.10, 1.0}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0, 1.05}})
end

Scripts.JelloLoop = function(actor)
	local time = 400
	while(true)do
		PopKeepScale(actor,  {expandTime=time, shrinkTime=time, expandTo={1.05, 1.0}})
		PopKeepScale(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0, 1.12}})
	end
end

Scripts.JelloSquareLoop = function(actor)
	local time = 400
	while(true)do
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.15, 1.0}})
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0, 1.15}})
	end
end

Scripts.SqueezeSquareLoop = function(actor)
	local time = 400
	while(true)do
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={0.75, 0.75}})
		--CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={0.5, 0.5}})
	end
end

Scripts.SqueezeSquareLoopFlipped = function(actor)
	local time = 400
	while(true)do
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={-0.75, 0.75}, finalScale={-1.0, 1.0}})
		--CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={0.5, 0.5}})
	end
end

Scripts.JelloLanding = function(actor)
	local time = 100
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.2, 0.8}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={0.8, 1.2}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.1, 0.9}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={0.9, 1.1}})
end

Scripts.JelloLanding_quick = function(actor)
	local time = 100
	actor.scalingPoint = MidBottom
	local expandFactor = 0.05
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0+expandFactor,  1.0-expandFactor}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0-expandFactor,  1.0+expandFactor}})
end

Scripts.JelloLanding_quick2 = function(actor)
	local time = 100
	ScriptDelay(1000)
	actor.scalingPoint = MidBottom
	local expandFactor = 0.05
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0+expandFactor,  1.0-expandFactor}})
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0-expandFactor,  1.0+expandFactor}})
end

Scripts.JelloLanding_squash = function(actor)
	local time = 100
	actor.scalingPoint = MidBottom
	local expandFactor = 0.05
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0+expandFactor,  1.0-expandFactor}})
	--CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0-expandFactor,  1.0+expandFactor}})
end

Scripts.JelloLanding_flag = function(actor)
	local time = 100
	actor.scalingPoint = MidBottom
	local expandFactor = 0.10
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0+expandFactor,  1.0-expandFactor}})
	--CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.0-expandFactor,  1.0+expandFactor}})
end

Scripts.Squeeze  = function(actor)
	local time = 100
	CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={0.90, 0.90}})
end

Scripts.HeartBeatLoop = function(actor)
	local time = 100
	while(true)do
		CorrectedPop(actor,  {expandTime=time/2, shrinkTime=time/2, expandTo={1.20, 1.20}})
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.10, 1.10}})
		ScriptDelay(1500)
	end
end

Scripts.HeartBeatLoop_dialog = function(actor)
	local time = 100
	while(true)do
		CorrectedPop(actor,  {expandTime=time/2, shrinkTime=time/2, expandTo={1.20, 1.20}})
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.10, 1.10}})
		ScriptDelay(1500,"DialogTimers")
	end
end

Scripts.HeartBeatLoopFast = function(actor)
	local time = 100
	while(true)do
		CorrectedPop(actor,  {expandTime=time/2, shrinkTime=time/2, expandTo={1.25, 1.25}})
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.10, 1.10}})
		ScriptDelay(1000)
	end
end

Scripts.HeartBeatLoopFastest = function(actor)
	local time = 100
	while(true)do
		CorrectedPop(actor,  {expandTime=time/2, shrinkTime=time/2, expandTo={1.30, 1.30}})
		CorrectedPop(actor,  {expandTime=time, shrinkTime=time, expandTo={1.10, 1.10}})
		ScriptDelay(500)
	end
end

Scripts.SpinAndSlideOff_right = function(actor)
	RotateOut(actor,{revolutions=1.25},750,false)
	SlideOff(actor,750,"bottom","wait")
end

Scripts.SpinAndSlideOff_left = function(actor)
	RotateOut(actor,{revolutions=-1.25},750,false)
	SlideOff(actor,750,"bottom","wait")
end


function Scripts.RotateBackAndForth(actor)
	--logprint("RotateBackAndForth")
	local duration = 1000
		
	RotateTo(actor, {revolutions=-0.1}, duration/2,true)
	while(true)do
		RotateTo(actor, {revolutions=0.1}, duration,true)
		RotateTo(actor, {revolutions=-0.1}, duration,true)
	end
end

function Scripts.RotateBackAndForth_wide(actor)
	--logprint("RotateBackAndForth_wide")
	local duration = 1000
		
	RotateTo(actor, {revolutions=-0.2}, duration/2,true)
	while(true)do
		RotateTo(actor, {revolutions=0.2}, duration,true)
		RotateTo(actor, {revolutions=-0.2}, duration,true)
	end
end

Scripts.MakerOutput = function(actor)
	local amount = 6
	local speed = 100
	local slidePosition = actor.position
	local originalPosition = actor.position
	PlaySound("Chunka.ogg")
	PlaySound("cavedrip02.ogg")
    FireParticle(actor, "Launch1", "MidBottom")
    FireParticle(actor, "Launch1", "MidBottom")
    FireParticle(actor, "Steam2", "MidBottom")
    for index = 1, 1 do
		slidePosition.y=slidePosition.y-amount
		SlideAndWait(actor,slidePosition, speed/2)
		slidePosition.y=slidePosition.y+(amount*1.5)
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.y=slidePosition.y-(amount*0.5)
		SlideAndWait(actor,slidePosition, speed)
	end
	actor.position = originalPosition		
end

Scripts.ShakeSlide = function(actor)
	if(actor:GetTaggedScriptCount("ShakeSlide") > 1) then
		return
	end
	local amount = 2
	local speed = 32
	local slidePosition = actor.position
	local originalPosition = actor.position
	--logprint("shakeslide start " .. actor.name)
	
	for index = 1, 1 do
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x+(amount*2)
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
	end
	actor.position = originalPosition	
	--logprint("shakeslide end " .. actor.name)	
end

Scripts.FlexOut = function(actor)
	if(actor:GetTaggedScriptCount("FlexOut") > 1) then
		return
	end
	PopKeepScale(actor,{expandTo={1.05,1.05},expandTime=32*2, shrinkTime=32*4})
end

Scripts.SpawnedPieceAnimation = function(rootGameView, pieceView, spawnedPieceAnimation)
	local wasGiftBox = false
	if(pieceView.coveredPiece ~= nil) then
		wasGiftBox = (pieceView.coveredPiece.isGiftBox == true)
	end

	if(pieceView.isCoconut)then
		-- Make piece appear, but with alpha 0 to children can appear but not actor yet
		spawnedPieceAnimation.alpha = 0
		spawnedPieceAnimation.visible = true
		-- Set up pieces
		spawnedPieceAnimation.leftHalf = Resources.Actor:new {
			resource = "Actor",
			image = "Coconut_open_left.png",
			visible = true,
			zorder = -1,
			layer = 4,
		}
		spawnedPieceAnimation.rightHalf = Resources.Actor:new {
			resource = "Actor",
			image = "Coconut_open_right.png",
			visible = true,
			zorder = -1,
			layer = 4,
		}
		local leftHalf = spawnedPieceAnimation.leftHalf
		local rightHalf = spawnedPieceAnimation.rightHalf
		local coconutPos = spawnedPieceAnimation:GetGlobalPoint("Center")
		spawnedPieceAnimation:RemoveChild(leftHalf)
		spawnedPieceAnimation:RemoveChild(rightHalf)

		leftHalf.name = "coconutLeft" .. coconutNumber
		rightHalf.name = "coconutRight" .. coconutNumber
		coconutNumber = coconutNumber + 1
		rootGameView:AddChild(leftHalf)
		rootGameView:AddChild(rightHalf)
		leftHalf:SetGlobalPoint("Center",coconutPos)
		rightHalf:SetGlobalPoint("Center",coconutPos)

		-- Set up destination location of fly off
		local rightHalf_dest	= {550,coconutPos.y}
		local leftHalf_dest		= {-550,coconutPos.y}

		-- Play crack sound and start pieces flying out
		PlaySound("Baseball_Bat_Crack13.ogg")
		AddScriptWithParams {
			target = rightHalf,
			script = "Interpolate",
			params = { "position", rightHalf_dest, 1000 },
			tags = {},
		}
		AddScriptWithParams {
			target = leftHalf,
			script = "Interpolate",
			params = { "position", leftHalf_dest, 1000 },
			tags = {},
		}

		-- Start poof
		StartParticleEmitterFromFile {
			file = "MagicPoof1",
			position = spawnedPieceAnimation.center,
		}

		RotateAtSpeed(rightHalf,  {revolutions=2.0}, 1000, false)
		RotateAtSpeed(leftHalf,  {revolutions=-2.0}, 1000, false)

		-- Wait a bit to let pieces start to fly
		ScriptDelay(50)
		-- Show piece and pop it - so it seems like it burst out
		spawnedPieceAnimation.alpha = 255
		spawnedPieceAnimation.visible = true

		local oldScale = spawnedPieceAnimation.extraScale
		while(IsScaling(spawnedPieceAnimation)) do
			ScriptBreak()
		end
		ScaleTo(spawnedPieceAnimation,{1.2, 1.2},250)
		ScriptDelay(220)
		ScaleTo(spawnedPieceAnimation,oldScale,100)
		ScriptDelay(100)

		-- Wait a bit and then destroy flying pieces
		ScriptDelay(1000)
		rightHalf:Destroy()
		leftHalf:Destroy()
	elseif (pieceView.isGiftBox or pieceView.isGiftBox2 or pieceView.isGiftBox3 or wasGiftBox) then
		-- Make piece appear, but with alpha 0 so children can appear but not actor yet
		spawnedPieceAnimation.alpha = 0
		spawnedPieceAnimation.visible = true
		-- Set up pieces
		spawnedPieceAnimation.leftWall = Resources.Actor:new {
			resource = "Actor",
			image = "giftbox_leftside.png",
			visible = true,
			zorder = -1,
		}
		spawnedPieceAnimation.rightWall = Resources.Actor:new {
			resource = "Actor",
			image = "giftbox_rightside.png",
			visible = true,
			zorder = -1,
		}
		spawnedPieceAnimation.frontWall = Resources.Actor:new {
			resource = "Actor",
			image = "giftbox_front.png",
			visible = true,
			zorder = -1,
		}
		spawnedPieceAnimation.lid = Resources.Actor:new {
			resource = "Actor",
			image = "giftbox_lid.png",
			visible = true,
			zorder = -1,
		}
		
		-- Set up destination location of fly off
		local rightWall_dest	= {550,0}
		local leftWall_dest		= {-550,0}
		local frontWall_dest	= {0,550}
		local lid_dest		= {0,-550}

		-- Play crack sound and start pieces flying out
		PlaySound("bwv_sand_paper_set_down_01.ogg")
		PlaySound("magicwand_up5.ogg")
		AddScriptWithParams {
			target = spawnedPieceAnimation.rightWall,
			script = "Interpolate",
			params = { "position", rightWall_dest, 1000 },
			tags = {},
		}
		AddScriptWithParams {
			target = spawnedPieceAnimation.leftWall,
			script = "Interpolate",
			params = { "position", leftWall_dest, 1000 },
			tags = {},
		}
		AddScriptWithParams {
			target = spawnedPieceAnimation.frontWall,
			script = "Interpolate",
			params = { "position", frontWall_dest, 1000 },
			tags = {},
		}
		AddScriptWithParams {
			target = spawnedPieceAnimation.lid,
			script = "Interpolate",
			params = { "position", lid_dest, 1000 },
			tags = {},
		}

		-- Start poof
		StartParticleEmitterFromFile {
			file = "MagicPoof1",
			position = spawnedPieceAnimation.center,
		}

		RotateAtSpeed(spawnedPieceAnimation.rightWall,  {revolutions=2.0}, 1000, false)
		RotateAtSpeed(spawnedPieceAnimation.leftWall,  {revolutions=-2.0}, 1000, false)
		RotateAtSpeed(spawnedPieceAnimation.frontWall,  {revolutions=2.0}, 1000, false)
		RotateAtSpeed(spawnedPieceAnimation.lid,  {revolutions=-1.0}, 1000, false)

		-- Wait a bit to let pieces start to fly
		ScriptDelay(50)
		-- Show piece and pop it - so it seems like it burst out
		spawnedPieceAnimation.alpha = 255
		spawnedPieceAnimation.visible = true

		local oldScale = spawnedPieceAnimation.extraScale
		while(IsScaling(spawnedPieceAnimation)) do
			ScriptBreak()
		end
		ScaleTo(spawnedPieceAnimation,{1.25, 1.25},250)
		ScriptDelay(220)
		ScaleTo(spawnedPieceAnimation,oldScale,100)
		ScriptDelay(100)

		-- Wait a bit and then destroy flying pieces
		ScriptDelay(1000)
		spawnedPieceAnimation.rightWall:Destroy()
		spawnedPieceAnimation.leftWall:Destroy()
		spawnedPieceAnimation.frontWall:Destroy()
		spawnedPieceAnimation.lid:Destroy()
	elseif(pieceView.isSafe)then
		if(pieceView.isLastSafe) then
            --Scale new piece that emerges from safe
            --ScriptDelay(safeDialTime)

			while(IsScaling(spawnedPieceAnimation)) do
				ScriptBreak()
			end
            local gemScale = spawnedPieceAnimation.extraScale

            ScaleTo(spawnedPieceAnimation,{0.0,0.0},0)
            spawnedPieceAnimation.visible = true
            ScriptDelay(safeOpenTime/2) --wait for chips to clear
			PlaySound("Cascade_Chimes.ogg")
			PlaySound("Cascade_SFX_MatchCoins.ogg")
            ScaleTo(spawnedPieceAnimation,{gemScale.x,gemScale.y},safeOpenTime/2)
            ScriptDelay(safeOpenTime/2)
			PlaySound("Cascade_Ooo.ogg")
		else
            ScriptDelay(safeDialTime)
			spawnedPieceAnimation.visible = true

            --Start safe dial at new rotation
            if(pieceView.totalSpawns > 0) then
                local spawnsLeft = pieceView.extraSpawns
                if(spawnsLeft == 0) then
                    spawnsLeft = 1
                end
                local rotationAmount = 360 * spawnsLeft/pieceView.totalSpawns
                spawnedPieceAnimation.Dial.rotationAngle = {degrees = rotationAmount,}
            end
		end
	elseif (pieceView.isLaser) then
		ScaleIn(spawnedPieceAnimation,{0.0,0.0},0,"nowait")
	elseif (pieceView.isRealBarrel or pieceView.isRealBarrel2) then
		while(IsScaling(spawnedPieceAnimation)) do
			ScriptBreak()
		end
		ScaleIn(spawnedPieceAnimation,{0.0,0.0},250,"nowait")
	elseif (pieceView.isSwitch) then
		spawnedPieceAnimation.visible = true
		spawnedPieceAnimation:AddScript("FlexOut","FlexOut")
		if(pieceView.isToggle) then
			if (pieceView.isDungeonSwitch) then
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_DungeonSwitchOnGamePieceView)
				ScriptDelay(250)
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_DungeonSwitchToggleGamePieceView)
			elseif (pieceView.isEgyptSwitch) then
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_EgyptSwitchOnGamePieceView)
				ScriptDelay(250)
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_EgyptSwitchToggleGamePieceView)
			elseif (pieceView.isWhimsySwitch) then
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_WhimsySwitchOnGamePieceView)
				ScriptDelay(250)
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_WhimsySwitchToggleGamePieceView)
			else
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_SwitchOnGamePieceView)
				ScriptDelay(250)
				spawnedPieceAnimation:ApplyProperties(Resources.Gems_SwitchToggleGamePieceView)
			end
		end
	elseif (pieceView.isLink or pieceView.isJailedLink) then
		--The existence of a spawned piece indicates a failed link
		spawnedPieceAnimation.visible = true
		spawnedPieceAnimation:AddScript("SpinLink","")
	else
		spawnedPieceAnimation.visible = true
	end
end

Scripts.SpinLink = function(pieceView)
	if(pieceView.LinkInJail ~= nil) then
		PlaySound("jail_cell_door_metal_impact_09.ogg")
	end

	for speed = 5,1,-1 do
		if(pieceView.LinkInJail ~= nil) then
			RotateAtSpeed(pieceView.LinkInJail,  {revolutions=-speed}, 133, true)
		else
			RotateAtSpeed(pieceView,  {revolutions=speed}, 133, true)
		end
	end
	if(pieceView.LinkInJail ~= nil) then
		pieceView.LinkInJail.rotationAngle = {degrees = 0}
	else
		pieceView.rotationAngle = {degrees = 0}
	end
end

Scripts.ApplyPatternsAnimation = function(rootGameView, patternList)
	--logprint("ApplyPatternsAnimation queued")
	ScriptBreak()
	WaitForTaggedScriptCompletion(rootGameView, "Piece_Drop")
	--logprint("ApplyPatternsAnimation starting")

	for unused,patternData in ipairs(patternList) do
		--logprint (patternData.name)
		if(patternData.isSelfApplying) then
			--logprint("Self matching")
		else
			--logprint("not self matching")
			local allSparkles = {}
			local sparkleNumber = 0
			local hasBedRock = false
			--For each gem that applies this pattern
			for unused,sourceGemData in ipairs(patternData.applyingPieces) do
				local sourceSpot = {x = -1,y = -1}
				if(sourceGemData.pieceView == nil) then
					--It looks like the applying piece was removed. Get the piece view for the corresponding added piece.
					local idToSearchFor = sourceGemData.id
					local foundIndex = -1

					for index, removedPieceData in ipairs(patternData.removedPieces) do
						if(removedPieceData.id == idToSearchFor) then
							foundIndex = index
							local correspondingAddedPiece = patternData.addedPieces[index]
							if(correspondingAddedPiece ~= "removed" and sourceSpot.pieceView ~= nil) then
								sourceSpot = correspondingAddedPiece.pieceView:GetGlobalPoint(Center)
							end
							break
						end
					end
				else
					sourceSpot = sourceGemData.pieceView:GetGlobalPoint(Center)
				end
				--For each destination gem
				for index,gemData in ipairs(patternData.removedPieces) do
					if(gemData ~= "Removed") then
						if(sourceSpot.x < 0) then 
							continue
						end
						--Schedule a particle stream to fly from source to destination over a fixed time period, deleting itself on arrival
						sparkleNumber = sparkleNumber + 1
						local destination = GetGlobalCenterPoint(gemData.pieceView)
						allSparkles[sparkleNumber] = CreateInstance {
							name = "",
							class = "hgeParticleSystemActor",
							file = "SuperWildTrail2",
							position = sourceSpot,
						}
						allSparkles[sparkleNumber].destination  = destination
						allSparkles[sparkleNumber].duration = patternFlightDuration
						rootGameView:AddChild(allSparkles[sparkleNumber])
						allSparkles[sparkleNumber]:AddScript("FlyToDestination","")
						if(sourceGemData.pieceView ~= nil and sourceGemData.pieceView.isBedRock)then
							--logprint("BedRock 3")
							hasBedRock = true
							AddScriptWithParams(allSparkles[sparkleNumber],"BoomAtDestination_bedrock", sourceGemData.pieceView.IconDecoration.image)
						else 
							--logprint("Not BedRock 3")
							allSparkles[sparkleNumber]:AddScript("BoomAtDestination_pattern","")
						end
					end
				end
			end
		end
	end

	if(#patternList > 0) then
		if(hasBedRock)then
			PlaySound("fdn_convolution_pulse_07.ogg")
		else
			--PlaySound("laserShot1.ogg")
			PlaySound("Cascade_Open_SFX_10.ogg")
		end

		--After the fixed time period
		ScriptDelay(patternFlightDuration)
	end
	
	for unused,patternData in ipairs(patternList) do
		--Update properties instantly by deleting old view and showing new view
		for unused2,gemData in ipairs(patternData.removedPieces) do
			if(gemData ~= "Removed") then
				gemData.pieceView:Destroy()
			end
		end

		for unused2,gemData in ipairs(patternData.addedPieces) do
			if(gemData ~= "Removed") then
				gemData.pieceView.visible = true
			end
		end
		ScriptDelay(150)
	end
	--logprint("ApplyPatternsAnimation done")
end

function Scripts.PulseLinkLine(linkLine)
	while(true) do
		ScaleTo(linkLine,{1.0,2.0},150)
		ScriptDelay(150)
		ScaleTo(linkLine,{1.0,0.5},150)
		ScriptDelay(150)
	end
end

local particleColors = {
	["Blue"] = {1, 0, 0,1, },
	["Purple"] = {1, 1, 0, 1},
	["Green"] = {1,   0, 1, 0},
	["Red"] = {1,   1, 0, 0},
	["Yellow"] = {1,   1, 1, 0},
	["Orange"] = {1,   1, 0.5, 0},
}

function Scripts.ExplodeLinkLine(linkLine)
	local linkParticles = CreateInstance { 
		name = "CurrentHintAnimation",
		class = "hgeParticleSystemActor",
		file = "LinkConnector",
		alignment = "Center",
		rotationAngle = linkLine.rotationAngle,
	}
	linkLine:AddChild(linkParticles)

	--Change properties of particle system to match link line's properties
	linkParticles:Pause() --Forces particle system info to exist?
	linkParticles:Unpause()
	local info = linkParticles.info

	local scaleFactor = linkLine.size.x/100

	info.nEmission = info.nEmission * scaleFactor
	info.fStartPosXOffsetMin = info.fStartPosXOffsetMin * scaleFactor
	info.fStartPosXOffsetMax = info.fStartPosXOffsetMax * scaleFactor
	info.image = "Link" .. linkLine.particleColor .. ".png"
	info.colColorStart = particleColors[linkLine.particleColor]
	info.colCoclorEnd = particleColors[linkLine.particleColor]
	local steps = 10
	for step = 1,steps do --Can't fadeout because it does setvisible false, which hides the particles
		linkLine.alpha = 255 - (255 * steps/10)
		ScriptBreak()
	end
	ScriptDelay(1850)
	--linkLine:Destroy() --Uncomment to prevent old link lines from displaying
end

Scripts.CopySplashIcons = function(source,dest)
	--Copies splash icons from the source to the destination piece view
	if(source.VerticalSplashDecoration ~= nil) then
		dest.VerticalSplashDecoration = Resources.VerticalSplashDecoration
		dest.VerticalSplashAura = Resources.VerticalSplashAura
	end
	if(source.HorizontalSplashDecoration ~= nil) then
		dest.HorizontalSplashDecoration = Resources.HorizontalSplashDecoration
		dest.HorizontalSplashAura = Resources.HorizontalSplashAura
	end
	if(source.AreaSplashDecoration ~= nil) then
		dest.AreaSplashDecoration = Resources.AreaSplashDecoration
		dest.AreaSplashAura = Resources.AreaSplashAura
	end
	if(source.WideVerticalSplashDecoration ~= nil) then
		dest.WideVerticalSplashDecoration = Resources.WideVerticalSplashDecoration
		dest.AreaSplashAura = Resources.AreaSplashAura
	end
	if(source.WideHorizontalSplashDecoration ~= nil) then
		dest.WideHorizontalSplashDecoration = Resources.WideHorizontalSplashDecoration
		dest.AreaSplashAura = Resources.AreaSplashAura
	end
	if(source.WidePlusSplashDecoration ~= nil) then
		dest.WidePlusSplashDecoration = Resources.WidePlusSplashDecoration
		dest.AreaSplashAura = Resources.AreaSplashAura
	end
	if(source.PlusSplashDecoration ~= nil) then
		dest.PlusSplashDecoration = Resources.PlusSplashDecoration
		dest.AreaSplashAura = Resources.AreaSplashAura
	end
end