
--
-- SCORE FLASHES
--

Resources.AppScoreFlash = Resources.Actor:new {
	class = "BitmapFontLabel",
	fontName = "Bauhaus_21_border_2@2x.fnt",
	xAdvanceOffset = -1,
	color = "Yellow",
	layer = 20,
	scalingPoint = Center,
	extraScale = 0.5,   --Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
						--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
}

Resources.AppScoreFlash42 = Resources.Actor:new {
	class = "BitmapFontLabel",
	fontName = "Bauhaus_42_border_2@2x.fnt",
	xAdvanceOffset = -1,
	color = "Yellow",
	layer = 20,
	scalingPoint = Center,
	extraScale = 0.5,	--Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
						--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
}

function Scripts.InnerScoreFlashScript(scoreFlash) 
	ScriptDelay(400)
	
	if not SlideTargetPoint then
		--NOTE: Took this to see if it fixes gem drop pause bug
		--local slideTargetActor = Screens.currentScreen:FindChild("ScoreLabel", true)
		if(Screens.currentScreen.RootActor.AllGameViews)then
			local slideTargetActor = Screens.currentScreen.RootActor.AllGameViews.ScoreContainer.ScoreLabel
			if slideTargetActor then
				SlideTargetPoint = slideTargetActor:GetGlobalPoint("Center")
			end
		end
	end
	
	if SlideTargetPoint then
		Scripts.InterpolateGlobalPoint(scoreFlash, "Center", SlideTargetPoint, 150)
		FadeOut(scoreFlash,250)
		ScaleOut(scoreFlash,{0.0, 0.0},250)
		ScriptDelay(250)
	else
		Scripts.Interpolate(scoreFlash, "alpha", 0, 200)
	end
	
	scoreFlash.parent:DestroyChild(scoreFlash)
end

local possibleSpecialCounters = {
	"SpecialGem1_1",
	"SpecialGem2_1",
	"SpecialGem2_2",
	"SpecialGem3_1",
	"SpecialGem3_2",
	"SpecialGem3_3",
}

local InnerSpecialGemFlashScript_sound_is_playing =false
function Scripts.InnerSpecialGemFlashScript(scoreFlash) 
	ScriptDelay(550)

	local SlideTargetPoint
	local SlideTargetActor
	
	local specialGemType = scoreFlash.specialGemType
	local specialGemPanel = Screens.currentScreen.RootActor.AllGameViews.SpecialGemPanel
	for targetNumber = 1,6 do
		local targetName = possibleSpecialCounters[targetNumber]
		local targetCounter = specialGemPanel[targetName]
		if targetCounter.specialGemType == specialGemType then
			SlideTargetActor = targetCounter
			SlideTargetPoint = targetCounter:GetGlobalPoint("Center")
			break
		end
	end
	
	Scripts.InterpolateGlobalPoint(scoreFlash, "Center", SlideTargetPoint, 150)

	FadeOut(scoreFlash,250)

	--TODO: Find a better place for this so it only plays once
	if(not InnerSpecialGemFlashScript_sound_is_playing)then
		PlaySound("Pickup_Coins06.ogg")
		InnerSpecialGemFlashScript_sound_is_playing = true
	end


	SlideTargetActor.SpecialGem_label:SetScore(SlideTargetActor.SpecialGem_label.score + scoreFlash.extraSpecials,false)
	-- DEBUG
	-- local value = 10000
	-- SlideTargetActor.SpecialGem_label:SetScore(value,false)
	ScaleOut(scoreFlash,{0.0, 0.0},250)

    if(SlideTargetActor.isPopping == false) then
        SlideTargetActor.isPopping = true
        PopKeepScale(SlideTargetActor)
    end
    ScriptDelay(250)
	scoreFlash.parent:DestroyChild(scoreFlash)
	InnerSpecialGemFlashScript_sound_is_playing = false
end

--local ScoreFlashNumber = 1
function Scripts.CascadeScoreFlash(parentActor, alignmentTarget, rawScore, color, resource, wait)
    if(color==nil)then
        color = "White"
    end
	if(color=="Blue")then
        color = "ScoreFlash_Blue"
    end
	if(color=="Green")then
        color = "ScoreFlash_Green"
    end
	if(color=="Red")then
        color = "ScoreFlash_Red"
    end
	if(color=="Purple")then
        color = "ScoreFlash_Purple"
    end
	if(color=="Yellow")then
        color = "ScoreFlash_Yellow"
    end
	if(color=="Orange")then
        color = "ScoreFlash_Orange"
    end
    -- Format the score as appropriate:
	local formattedScore
	if IsNumber(rawScore) then
        formattedScore = Int2Comma(rawScore)
	else
        formattedScore = tostring(rawScore)
	end
    formattedScore = formattedScore:gsub("%W"," ")

	local flashResource = "AppScoreFlash"
	if(resource~=nil)then
		flashResource=resource
	end

	-- Create a score flash resource.
	local scoreFlashResource = Resources.AppScoreFlash:new {
		resource = flashResource,
        color = color,
	}
	
	-- Instantiate a new score flash.
	local scoreFlash = CreateInstance(scoreFlashResource)
	
	-- Set the score flash's text.  This is done after instantiation, to make
	-- sure all other properties (for font, etc.) get set, causing the text to
	-- only be rendered once.
	scoreFlash.text = formattedScore
	
	-- Center it on the actor it's being attached to
	local centerPoint
	if IsPoint(alignmentTarget) then
		centerPoint = Point:fromTable(alignmentTarget)
	else
		centerPoint = alignmentTarget:GetGlobalPoint("Center")
	end
	
	--logprintf("LogScoreFlash: %d,%d\n", centerPoint.x, centerPoint.y)
	
	-- Make sure the score flash is not positioned too far to the left.
	local left = centerPoint.x - (scoreFlash.width / 2)
	if left < 0 then
		centerPoint.x = centerPoint.x - left
	end
	
	-- Make sure the score flash is not positioned too far to the right.
	local right = centerPoint.x + (scoreFlash.width / 2)
	if right > parentActor.width then
		centerPoint.x = centerPoint.x - (right - parentActor.width)
	end

	--logprintf("... %d,%d\n", centerPoint.x, centerPoint.y)
	
	-- Add the score flash to the screen, then position it.
	parentActor:AddChild(scoreFlash)
	scoreFlash:SetGlobalPoint("Center", centerPoint)
	
	-- Start animating the score flash:
	RunScriptWithParams(scoreFlash, "InnerScoreFlashScript")
	
	-- Wait for the score flash to finish animating.
	WaitForTaggedScriptCompletion(scoreFlash, "InnerScoreFlashScript")
end

function Scripts.CascadeSpecialGemFlash(parentActor, alignmentTarget, rawScore, color, specialGemType)
    if(color==nil)then
        color = "White"
    end

	if(color=="Blue")then
        color = "ScoreFlash_Blue_Special"
    end
	if(color=="Green")then
        color = "ScoreFlash_Green_Special"
    end
	if(color=="Red")then
        color = "ScoreFlash_Red_Special"
    end
    -- Format the score as appropriate:
	local formattedScore
	if IsNumber(rawScore) then
        formattedScore = Int2Comma(rawScore)
	else
        formattedScore = tostring(rawScore)
	end
    formattedScore = formattedScore:gsub("%W"," ")

	local flashResource = "AppScoreFlash"
	if(resource~=nil)then
		flashResource=resource
	end

	-- Create a score flash resource.
	local scoreFlashResource = Resources.AppScoreFlash:new {
		resource = flashResource,
        color = color,
		specialGemType = specialGemType,
		extraSpecials = rawScore,
	}
	
	-- Instantiate a new score flash.
	local scoreFlash = CreateInstance(scoreFlashResource)
	
	-- Set the score flash's text.  This is done after instantiation, to make
	-- sure all other properties (for font, etc.) get set, causing the text to
	-- only be rendered once.
	scoreFlash.text = formattedScore
	
	-- Center it on the actor it's being attached to
	local centerPoint
	if IsPoint(alignmentTarget) then
		centerPoint = Point:fromTable(alignmentTarget)
	else
		centerPoint = alignmentTarget:GetGlobalPoint("Center")
	end
	
	--logprintf("LogScoreFlash: %d,%d\n", centerPoint.x, centerPoint.y)
	
	-- Make sure the score flash is not positioned too far to the left.
	local left = centerPoint.x - (scoreFlash.width / 2)
	if left < 0 then
		centerPoint.x = centerPoint.x - left
	end
	
	-- Make sure the score flash is not positioned too far to the right.
	local right = centerPoint.x + (scoreFlash.width / 2)
	if right > parentActor.width then
		centerPoint.x = centerPoint.x - (right - parentActor.width)
	end

	-- Lift up special score flash so it is on top of score, not over it
	centerPoint.y = centerPoint.y - (scoreFlash.height / 2)

	--logprintf("... %d,%d\n", centerPoint.x, centerPoint.y)
	
	-- Add the score flash to the screen, then position it.
	parentActor:AddChild(scoreFlash)
	scoreFlash:SetGlobalPoint("Center", centerPoint)
	
	-- Start animating the score flash:
	RunScriptWithParams(scoreFlash, "InnerSpecialGemFlashScript")
	
	-- Wait for the score flash to finish animating.
	WaitForTaggedScriptCompletion(scoreFlash, "InnerSpecialGemFlashScript")
end