
local function MakeBuilding(landNumber, discoveryIndex, buildingPosition, workerTable, allowedGemTypes, progressBarOffset, collectReminderOffset, discoverIconOffset)

	local buildingName = "Land"..landNumber.."_Discovery_"..discoveryIndex
    local workerOffset = progressBarOffset

    --Optional paramater #1
    if(collectReminderOffset == nil) then
        collectReminderOffset = {0,-25} --default setting
    end

    --Optional parameter #2
    if(discoverIconOffset == nil) then
        discoverIconOffset = {0,0}
    else
        workerOffset = discoverIconOffset
    end

	local building = {
		name = buildingName,
		buildingType = "Discovery",	
		basicProperties = {
			position = buildingPosition, --Relative to top of land
			progressBar = {
				class = "Actor",
				alignmentOffset = progressBarOffset,
			},
			workerCount = {
				class = "Label",
				alignmentOffset = workerOffset,
			},
            collectReminder = {
                class = "Button",
                alignmentOffset = collectReminderOffset,
            },
            discoverIcon = {
                class = "Actor",
                alignmentOffset = discoverIconOffset,
            },
		},
		unfinishedProperties = {
			image = buildingName.."_locked.png",
		},
		finishedProperties = {
			image = buildingName..".png",
		},
		dialogProperties = {
			image = buildingName..".png",
            extraScale = 1.2,
		},
		dialogProperties_unfinished = {
			image = buildingName.."_locked.png",
		},
		landNumber = landNumber,
		workers = workerTable,
		workerImage = "Land"..landNumber.."_Worker@2x.png",
		workerImageSmall = "Land"..landNumber.."_Worker_small@2x.png",
		allowedGemTypes = allowedGemTypes,
	}

	return building
end

Buildings = {
	-- Land2_Discovery_1
	MakeBuilding(2, 1, {30,1050},
		{
			[1] = {"SimpleCrateCoverage2",1},
			[2] = {"CoverageEmpty2",1},
			[3] = {"CoverageEmpty3_side",1},
		},
		{"SpecialGreen",},
		{x=0,y=0}
	),

	-- Land3_Discovery_1
	MakeBuilding(3, 1, {50,390},
		{
			[1] = {"Inaccessible",1},
			[2] = {"Drop_Empty_Large1_side",2},
			[3] = {"HardPattern",3},
		},
		{"SpecialGreen",},
		{x=0,y=0}
	),

	-- Land4_Discovery_1
	MakeBuilding(4, 1, {40,380},
		{
			[1] = {"JailCoverage1_side",1},
			[2] = {"JailDrop2",2},
			[3] = {"GemJailScore4",3},
		},
		{"SpecialGreen",},
		{x=0,y=0}
	),

	-- Land5_Discovery_1
	MakeBuilding(5, 1, {230,1295},
		{
			[1] = {"PyramidDrop",1},
			[2] = {"VinesDrop3",2},
			[3] = {"VinesPattern4_side",3},
		},
		{"SpecialGreen","SpecialBlue",},
		{x=0,y=0}
	),

	-- Land6_Discovery_1
	MakeBuilding(6, 1, {260,410},
		{
			[1] = {"WaffleIron",1},
			[2] = {"Barrels5_Updated_side",2},
			[3] = {"TrickLevel",3},
		},
		{"SpecialGreen","SpecialBlue",},
		{x=-18,y=0}
	),

	-- Land7_Discovery_1
	MakeBuilding(7, 1, {20,570},
		{
			[1] = {"BombFalling2",1},
			[2] = {"EverythingFall",2},
			[3] = {"BombFallingCombined_side",3},
		},
		{"SpecialGreen","SpecialBlue",},
		{x=15,y=0}
	),

	-- Land8_Discovery_1
	MakeBuilding(8, 1, {50,450},
		{
			[1] = {"Land8_Level3",1},
			[2] = {"Land8_Level13_side",2},
			[3] = {"Land8_Level11",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land9_Discovery_1
	MakeBuilding(9, 1, {25,795},
		{
			[1] = {"StoneWithBurst",1},
			[2] = {"Landslide",2},
			[3] = {"EdgeVines_side",2},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land10_Discovery_1
	MakeBuilding(10, 1, {185,170},
		{
			[1] = {"BottomVines",1},
			[2] = {"GemCollect4",2},
			[3] = {"GemTreasures_side",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land11_Discovery_1
	MakeBuilding(11, 1, {60,295},
		{
			[1] = {"LavaSplash_side",1},
			[2] = {"LavaBombs",2},
			[3] = {"StackOfBoulders",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land12_Discovery_1
	MakeBuilding(12, 1, {32,830},
		{
			[1] = {"LavaSeeds_Updated",1},
			[2] = {"CrateDropper",2},
			[3] = {"PottedPlants_side",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land13_Discovery_1
	MakeBuilding(13, 1, {207,280},
		{
			[1] = {"HalfReels",1},
			[2] = {"SplitLevel_side",2},
			[3] = {"Swimming",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land14_Discovery_1
	MakeBuilding(14, 1, {30,355},
		{
			[1] = {"VineGrowerTest_side",1},
			[2] = {"Multibomb",2},
			[3] = {"BombBarrels",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=10,y=0}
	),

	-- Land15_Discovery_1
	MakeBuilding(15, 1, {230,20},
		{
			[1] = {"CoconutTest_side",1},
			[2] = {"CoconutsVerseLava",2},
			[3] = {"CoconutLava",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=-12,y=0}
	),

	-- Land16_Discovery_1
	MakeBuilding(16, 1, {-42,32},
		{
			[1] = {"AmberTest_side",1},
			[2] = {"SuperwildBreakout",2},
			[3] = {"Impurities",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=35,y=0}
	),

	-- Land17_Discovery_1
	MakeBuilding(17, 1, {230,480},
		{
			[1] = {"FenceTest",1},
			[2] = {"ColoredBarrelTreasure_side",2},
			[3] = {"TreasureCave",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land18_Discovery_1
	MakeBuilding(18, 1, {20,1225},
		{
			[1] = {"OrangeChute",1},
			[2] = {"MakerIntro_side",2},
			[3] = {"ThinkBig",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=25,y=-3}
	),

	-- Land19_Discovery_1
	MakeBuilding(19, 1, {210,1330},
		{
			[1] = {"LockedGarden",1},
			[2] = {"LockTest_side",2},
			[3] = {"BombLocks",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=-80,y=-55},
        {x=-80,y=-10}, --collectReminder offset
        {x=-80,y=-10} --discoverIcon offset
	),

	-- Land20_Discovery_1
	MakeBuilding(20, 1, {30,1100},
		{
			[1] = {"JailhouseRocks5",1},
			[2] = {"ItTakesTwo_side",2},
			[3] = {"JailTrimmers",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land21_Discovery_1
	MakeBuilding(21, 1, {260,650},
		{
			[1] = {"MakerOfBoulders",1},
			[2] = {"Battlement1_side",2},
			[3] = {"Quarry",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=-22,y=-5}
	),

	-- Land22_Discovery_1
	MakeBuilding(22, 1, {40,620},
		{
			[1] = {"Stonewalling",1},
			[2] = {"PatternEater_side",2},
			[3] = {"Treasury",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land23_Discovery_1
	MakeBuilding(23, 1, {130,1060},
		{
			[1] = {"TowerToTheClouds",1},
			[2] = {"BasicBoxes_side",2},
			[3] = {"LockedLava",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land24_Discovery_1
	MakeBuilding(24, 1, {65,105},
		{
			[1] = {"YayPortal",1},
			[2] = {"YayPortal_side",2},
			[3] = {"PortalsGemEaters",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land25_Discovery_1
	MakeBuilding(25, 1, {0,1070},
		{
			[1] = {"LaserWall",1},
			[2] = {"LockTumblers_side",2},
			[3] = {"WildLasers",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land26_Discovery_1
	MakeBuilding(26, 1, {15,828},
		{
			[1] = {"AnInconvenientTrough",1},
			[2] = {"NoTwoAlike_side",2},
			[3] = {"TropicalIce",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land27_Discovery_1
	MakeBuilding(27, 1, {20,600},
		{
			[1] = {"BalloonTreasureTest",1},
			[2] = {"DuelingMakers",2},
			[3] = {"ClearTheAir_side",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=15,y=-40}
	),

	-- Land28_Discovery_1
	MakeBuilding(28, 1, {28,792},
		{
			[1] = {"Antigravity",1},
			[2] = {"SuperSafe",2},
			[3] = {"Cornerstone",3},
			[4] = {"StuckBalloons",2},
			[5] = {"BlockedBounce",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land29_Discovery_1
	MakeBuilding(29, 1, {52,928},
		{
			[1] = {"PrisonBreak",1},
			[2] = {"AxeWarp",2},
			[3] = {"DunkTank",3},
			[4] = {"PickaxePopup",2},
			[5] = {"LostKeys",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land30_Discovery_1
	MakeBuilding(30, 1, {230,680},
		{
			[1] = {"TwoWorlds",1},
			[2] = {"SkeetShooting",2},
			[3] = {"Binary",2},
			[4] = {"TowerDefense2",3},
			[5] = {"BreakerBox",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=0,y=0}
	),

	-- Land31_Discovery_1
	MakeBuilding(31, 1, {254,301},
		{
			[1] = {"PharaohsTreasure",1},
			[2] = {"BrassRing",2},
			[3] = {"SayWhen",2},
			[4] = {"Pinpoint",3},
			[5] = {"PresentPickup",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=-30,y=-70}
	),

	-- Land32_Discovery_1
	MakeBuilding(32, 1, {-1,785},
		{
			[1] = {"CurtainCutter",1},
			[2] = {"ThingsThatYouSee",3},
			[3] = {"ColorSwap",2},
			[4] = {"RingForService",2},
			[5] = {"ReverseReverse",3},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=25,y=0}
	),

	-- Land33_Discovery_1
	MakeBuilding(33, 1, {220,1030},
		{
            [1] = {"BigHeist",2},
            [2] = {"GatheringSupplies",3},
            [3] = {"RemoteSafe",2},
            [4] = {"SafePlacer",1},
            [5] = {"ShrinkingSafes",2},
		},
		{"SpecialGreen","SpecialBlue","SpecialRed",},
		{x=-20,y=-20}, --progressBar
        {x=-20,y=10}, --collectReminder offset
        {x=0,y=-15} --discoverIcon + worker count
	),

}

WorkerImagesByLevelAndStar = {
}

SmallWorkerImagesByLevelAndStar = {
}

local function CompileWorkerImages()
    for unused,building in ipairs(Buildings) do
        local workers = building.workers
        for unused,worker in ipairs(workers) do
            local levelName = worker[1]
            local starNumber = worker[2]
            if(WorkerImagesByLevelAndStar[levelName] == nil) then
                WorkerImagesByLevelAndStar[levelName] = {}
            end
            WorkerImagesByLevelAndStar[levelName][starNumber] = building.workerImage
			if(SmallWorkerImagesByLevelAndStar[levelName] == nil) then
                SmallWorkerImagesByLevelAndStar[levelName] = {}
            end
            SmallWorkerImagesByLevelAndStar[levelName][starNumber] = building.workerImageSmall
        end
    end
end

CompileWorkerImages()

Resources.Building = Resources.Button:new {
	resource = "Button",
	text = "",
	useFrames = false,
	isBuilding = true,
	image = "",
	zorder = 2,
	ignoreGamePause = true,
	clickSound = "Click6.ogg",

	workerCount = Resources.AppDialogLabel_small:new {
		resource = "AppDialogLabel_small",
		alignment = Center,
		alignTo = Center,
		alignmentOffset = {0,0},
		fgColor = "White",
		--shadowColor = "White",
		workerIcon = Resources.Actor:new {
			resource = "Actor",
			size = {70/2,76/2},
			resizeOnSetImage = false,
			alignment = MidRight,
			alignTo = MidLeft,
		},
	},

	discoverIcon = Resources.Actor:new {
		resource = "Actor",
        scripts = {"Groove"},
		image = "Discover_Icon.png",
		alignment = Center,
		visible = false,
	},

	progressBar = Resources.Actor:new {
		class = "Actor",
		image = "",
		alpha = 0,
		size = {90 + 4 + 4, 10 + 4},
		maximumWidth = 100,
		alignment = MidBottom,
		layer = 4,

		-- barBacking = Resources.Actor:new {
		-- 	class = "Actor",
		-- 	alignment = MidLeft,
		-- 	alignmentOffset = {2,0},
		-- 	size = {90 + 4,10},
		-- 	image = "",
		-- 	color = "Black",
		-- 	zorder = 1,
		-- },
        barBacking = Resources.Actor:new {
            class = "Actor",
            alignment = TopLeft,
            alignmentOffset = {-7,-4},
            image = "DiscoveryMeter_Base.png",
            zorder = 1,
        },

		barFilling = Resources.Actor:new {
			class = "Actor",
			alignment = MidLeft,
			alignmentOffset = {2,-1},
			size = {90,15},
			image = "",
			color = "White",
			zorder = 2,
		},

		divider1 = Resources.Actor:new {
			resource = "Actor",
			alignment = MidLeft,
			alignmentOffset = {32,-1},
			size = {3,15},
			image = "",
			color = "Cascade_BrownRed",
			zorder = 3,
		},

		divider2 = Resources.Actor:new {
			resource = "Actor",
			alignment = MidLeft,
			alignmentOffset = {65,-1},
			size = {3,15},
			image = "",
			color = "Cascade_BrownRed",
			zorder = 3,
		},

        barStroke = Resources.Actor:new {
            class = "Actor",
            alignment = TopLeft,
            alignmentOffset = {-1,-4},
            image = "DiscoveryMeter_OverlayStroke.png",
            zorder = 4,
        },

		upgradeIcon = Resources.Button:new {
			resource = "Actor",
			alignment = BottomRight,
			alignmentOffset = {-35,-10},
			image = "AddButton.png",
			visible = false,
			size = {30,30},
			useFrames = false,
			text = "",
			clickSound = "CLEEP_B.ogg",
			zorder = 5,
		
			SaleTag = Resources.SaleTag:new {
				resource = "SaleTag",
				alignment = MidBottom,
				alignTo = MidBottom,
				alignmentOffset = {-10,10},
				extraScale = 0.75,
				visible = (CONFIG:RetrieveConst("DebugSaleTags") == true),
			},
		},

	},
	collectReminder = Resources.Button:new {
		resource = "Button",
		alignment = MidTop,
		alignmentOffset = {0,-25},
		image = "Bubble_Blue.png",
        scripts = {"Groove"},

        --button stuff
        useFrames = false,
        isBuilding = true,
        ignoreGamePause = true,
        clickSound = "Click6.ogg",

		visible = false,
		layer = 6,
		specialGemImage = Resources.Actor:new {
			resource = "Actor",
			alignment = Center,
			alignmentOffset = {0,-12},
			size = {50,50},
			resizeOnSetImage = false,
		},
	},
}

Scripts.ShowCollectReminder = function(collectReminder) 
	if(collectReminder.visible == false) then
		FadeIn(collectReminder,250)
	end
end

Resources.BuildingScoreFlashResource = Resources.Actor:new {
	class = "BitmapFontLabel",
	fontName = "Bauhaus_42_border_2@2x.fnt",
	xAdvanceOffset = -1,
	layer = 20,
	scalingPoint = Center,
	extraScale = 0.5,
	scripts = {"ScoreFlashScript"},
}

local buildingScoreFlashNumber = 0

Scripts.BuildingScoreFlash = function(building, flashText, flashColor, newTotal, counter)
	local centerPoint = building:GetGlobalPoint(Center)

	PlaySound("coin_magic_06.ogg")
	ScriptDelay(200)
	
	if(flashColor=="Green")then
		FireParticle(building, "Explosion1_green")
	end
	if(flashColor=="Blue")then
		FireParticle(building, "Explosion1_blue")
	end
	if(flashColor=="Red")then
		FireParticle(building, "Explosion1_red")
	end

	StartBitmapScoreFlash(
		{
			alignmentTarget = building,
			color = flashColor,
			text = flashText,
			resource = "BuildingScoreFlashResource",
		}
	)

    local img = "GemSpecial" .. flashColor .. ".png"
    if (flashColor == "Yellow") then
        img = "Gold_1bar.png"
    end

	local flyingGemName = "flyingGem_" .. buildingScoreFlashNumber
	buildingScoreFlashNumber = buildingScoreFlashNumber + 1

	counter[flyingGemName] = Resources.Actor:new {
        class = "Actor",
		image = img,
		zorder = 10,
	}

	local flyingGem = counter[flyingGemName]

	AddTrail(flyingGem,"GoldSparkleTrail1")

	local dest = flyingGem:GetGlobalPoint(Center)
	flyingGem:SetGlobalPoint(Center,building:GetGlobalPoint(Center))
	ScaleOut(flyingGem,{0.0,0.0},250,false)
	Scripts.InterpolateGlobalPoint(flyingGem, "Center", dest, 250)
	flyingGem.particleTrail:Pause()
	counter:SetScore(newTotal,false)
	ScriptDelay(1000) --give time for particles to fade
	flyingGem:Destroy()

end

Scripts.FillProgressBar = function(building, startingFraction, endingFraction, runningTime)
	--Fractions are in terms of total bar size. Runs at most for 5 minutes.
	local progressBar = building.progressBar
	local initialWidth = progressBar.maximumWidth * startingFraction
	local finalWidth = progressBar.maximumWidth * endingFraction
	local barFilling = progressBar.barFilling
	barFilling.size = {x = initialWidth,y = barFilling.size.y}

	Scripts.Interpolate(barFilling, "size", {x = finalWidth, y = barFilling.size.y}, runningTime)
end