
function CreateBigImages(self)
	self.parent.bigImages = Resources.Actor:new {
		resource = "Actor",
		position = {0,0},
		alignment = "NoAlignment",
	}
	local bigImages = self.parent.bigImages
	for i=1,20 do
		local imageName = string.format("bigImage%d.png", i)
		local coord = (i-1)*5
		bigImages[imageName] = Resources.Actor:new {
			resource = "Actor",
			image = imageName,
			position = {coord,coord},
			alignment = "NoAlignment",
			size = {220,220},
		}
	end
end

Scripts.FailWithGenericError = function ()
	error("FailWithGenericError fake error")
end

Scripts.FailWithEngineAssertion = function ()
	GenerateFailedAssertion()
end

Resources.DebugScreen = {
	color = "Black",
	deviceOrientation = "Portrait",
	
	menuButton = Resources.Button:new {
		resource = "Button",
		text = "Back to Menu",
		alignment = "MidTop",
		alignmentOffset = {-80, 0},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			SwitchToScreen("MenuScreen")
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	crashTestA = Resources.Button:new {
		resource = "Button",
		text = "Crash Test\n(null deref)",
		alignment = "MidTop",
		alignmentOffset = {-80,40},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			CrashViaNullDeref()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	crashTestB = Resources.Button:new {
		resource = "Button",
		text = "Crash Test\n(inf. loop)",
		alignment = "MidTop",
		alignmentOffset = {-80,80},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			CrashViaInfiniteLoop()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	crashTestC = Resources.Button:new {
		resource = "Button",
		text = "...",
		alignment = "MidTop",
		alignmentOffset = {-80,120},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		UpdateText = function (self)
			local statusText
			if Players.current[1].Crash_By_Infinite_Loop_On_Startup then
				statusText = "ON"
			else
				statusText = "OFF"
			end
			self.text = string.format("Startup Crash Test\n(inf. loop: %s)", statusText)
		end,
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			Players.current[1].Crash_By_Infinite_Loop_On_Startup = not Players.current[1].Crash_By_Infinite_Loop_On_Startup
			SaveCurrentPlayerData()
			self:UpdateText()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
			self:UpdateText()
		end,
	},
	
	crashTestD = Resources.Button:new {
		resource = "Button",
		text = "Crash Test\n(mem leak)",
		alignment = "MidTop",
		alignmentOffset = {-80,160},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			self.text = "Crash Test\n(mem leak) ..."
			AddScriptWithParams(self, function ()
				ScriptDelay(2000)
				while true do
					AllocMemoryForCrashing()
					ScriptBreak()
				end
			end)
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	crashTestE = Resources.Button:new {
		resource = "Button",
		text = "Crash Test\n(LuaException)",
		alignment = "MidTop",
		alignmentOffset = {-80,200},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			CrashViaLuaException()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	showBigImages = Resources.Button:new {
		resource = "Button",
		text = "Show Big Images",
		alignment = "MidTop",
		alignmentOffset = {-80,240},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			if self.parent.bigImages then
				self.parent.bigImages:Destroy()
			else
				CreateBigImages(self)
			end
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	showBigImages2 = Resources.Button:new {
		resource = "Button",
		text = "Show Big Images\n(with delay)",
		alignment = "MidTop",
		alignmentOffset = {-80,280},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			if self.parent.bigImages then
				self.parent.bigImages:Destroy()
			else
				APP.delayNextDraw = true
				CreateBigImages(self)
			end
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	genericScriptError = Resources.Button:new {
		resource = "Button",
		text = "Script Error\nvia error()",
		alignment = "MidTop",
		alignmentOffset = {-80,320},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			AddScriptWithParams(self, "FailWithGenericError");
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},

	failedAssertionScriptError = Resources.Button:new {
		resource = "Button",
		text = "Script Error,\nvia ASSERT1()",
		alignment = "MidTop",
		alignmentOffset = {-80,360},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			AddScriptWithParams(self, "FailWithEngineAssertion");
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},

	flurryAnalyticsTestEvent = Resources.Button:new {
		resource = "Button",
		text = "Flurry Analytics\nTest Event",
		alignment = "MidTop",
		alignmentOffset = {-80,400},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			FlurryAnalytics_LogEvent("test event");
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	purchaseFiveChallengePoints = Resources.Button:new {
		resource = "Button",
		text = "Purchase Five\nChallenge Points",
		alignment = "MidTop",
		alignmentOffset = {80, 10},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			local productID = GetAppBundleID() .. ".FiveChallengePoints"
			TestProductPurchase(productID)
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	showChallengePointQuantity = Resources.Button:new {
		resource = "Button",
		text = "Show Purchased\nChallenge Points",
		alignment = "MidTop",
		alignmentOffset = {80, 60},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			TestConsumableProductQuantityRetrieval("ChallengePoints")
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	consumeOneChallengePoint = Resources.Button:new {
		resource = "Button",
		text = "Consume One\nChallenge Point",
		alignment = "MidTop",
		alignmentOffset = {80, 110},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			TestConsumableProductConsumption("ChallengePoints", 1)
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	consumeFiveChallengePoints = Resources.Button:new {
		resource = "Button",
		text = "Consume Five\nChallenge Points",
		alignment = "MidTop",
		alignmentOffset = {80, 160},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			TestConsumableProductConsumption("ChallengePoints", 5)
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	logProductInfo = Resources.Button:new {
		resource = "Button",
		text = "Log Product\nInfo",
		alignment = "MidTop",
		alignmentOffset = {80, 210},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			LogProductInfo()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
	
	logProductInfo2 = Resources.Button:new {
		resource = "Button",
		text = "Log Product\nInfo (alt ver.)",
		alignment = "MidTop",
		alignmentOffset = {80, 260},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			LogProductInfo_AltVersion()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
}
