
--
-- 'TotalLands' declares how many lands are in the game.
--
-- DavidL: Once set, this probably shouldn't be changed in mid-app-run.  It is,
-- okay to set this variable here and now, however the value shouldn't be
-- expected to completely take hold until a cold-[re]start of the app is performed.
-- This is because 'TotalLands' was a derivative of work done by a few people,
-- on top of a system that wasn't built to handle mid-app changes of land counts
-- (or changes in level count, for that matter).
--

TotalLands = 33

Resources.LandSign = Resources.Actor:new {
	image = "LandSign.png",
	LandNumber = Resources.AppDialogLabel:new {
		resource = "AppDialogLabel",
		text = "Land 13",
		alignment = MidTop,
		alignTo = MidTop,

		alignmentOffset = {0,22},
		shadowAlpha = 180,
		shadowColor = "White",
		--fgColor = "White",
		widthLimit = 250/2,
		alpha = 225,
		rotationAngle = { degrees = -4 },

        --[[
		font	= Resources.AppLandSignFont:new{
			resource = "AppLandSignFont",
		},
        
        --]]
        
        font	= Resources.MapButtonFont:new{
            resource = "MapButtonFont",
            fontSize = 20,
        },
        
	},
	LandName = Resources.AppDialogLabel:new {
		resource = "AppDialogLabel",
		text = "Ruins of Olympus",
		alignment = MidTop,
		alignTo = MidTop,
		alignmentOffset = {0,48},
		shadowAlpha = 180,
		shadowColor = "White",
		--fgColor = "White",
		widthLimit = 100,
		rotationAngle = { degrees = -4 },
		alpha = 225,
        
        --[[
		font	= Resources.AppLandSignFont:new{
			resource = "AppLandSignFont",
		},
        --]]
        font	= Resources.MapButtonFont:new{
            resource = "MapButtonFont",
        },
        
	},
}

Resources.Land = Resources.Actor:new {
	cellImage = "slot_6.png",
	slotImage = "",

	StageSign = Resources.Actor:new {
		resource = "Actor",
		image = "LandSign.png",
		alignment = MidBottom,
		alignTo = MidBottom,
		alignmentOffset = {100,-50},
		zorder = 1,
		titleName = Resources.AppDialogLabel:new {
			resource = "AppDialogLabel",
			text = STRINGS("Emerald Forest"),
			alignment = Center,		
			alignTo = Center,		
			alignmentOffset = {-8,-12},	
			--fgColor = landTextColorTable[landNumber],
			font	= Resources.DialogFont:new{
				resource = "DialogFont",
				fontSize = 18,
			},
			wrapLimit = 80,
			lineSpacing = 16,
			shadowColor = White,
			shadowAlpha = 50,
			highlightColor = Black,
			highlightAlpha = 50,
			rotationAngle = { degrees = -20 },
			visible = false,
		},
	},
	Shader = Resources.Actor:new {
		resource = "Actor",
		size = {768,512},
		image = "LandEnd_Disolve.png",
		alignment = MidBottom,
		alignTo = MidBottom,
		alignmentOffset = {0,128},
		zorder = 10,
		layer = 4,
	},
}

Scripts.MapCharacterGroove = function(actor)
	if(actor:GetTaggedScriptCount("MapCharacterGroove") > 1) then 
		return
	end
	
	local speed = 50
	local factor = math.random(10,20)
	local amount
	local pick = math.random(0,1)
	if(pick==0)then 
		amount = 0-factor 
	else
		amount = factor
	end
	
	local beatTime = 925

	local originalExtraScale = actor.extraScale

	local expandTo = {originalExtraScale.x, originalExtraScale.y+0.025}
	local expandTime = beatTime/4
	local shrinkTime = beatTime/4
	local tilt = 3
	local swing = 1

	local firstTime = 1


	while(true)do
		if(swing==1)then
			doTilt = -tilt
			if(firstTime==1) then 
				RotateByAmount(actor,{degrees=doTilt/2},100,true)
				firstTime = 0
			else
				RotateByAmount(actor,{degrees=doTilt},100,true)
			end
		end

		if(swing==3)then
			doTilt = tilt
			RotateByAmount(actor,{degrees=doTilt},100,true)
		end
		swing = swing + 1
		if(swing==5)then
			swing=1
		end

		actor.extraScale = originalExtraScale
		ScaleTo(actor, expandTo, expandTime)
		ScriptDelay(expandTime,"DialogTimers")
	
		ScaleTo(actor, originalExtraScale, shrinkTime)
		ScriptDelay(shrinkTime,"DialogTimers")

		ScriptDelay(10,"DialogTimers")

	end
	
	actor.rotation = { degrees = 0 }		
end

Scripts.QuickQuestMapIconGroove = function(actor)
	if(actor:GetTaggedScriptCount("MapCharacterGroove") > 1) then 
		return
	end
	
	local speed = 50
	local factor = math.random(10,20)
	local amount
	local pick = math.random(0,1)
	if(pick==0)then 
		amount = 0-factor 
	else
		amount = factor
	end
	
	local beatTime = 825

	local originalExtraScale = actor.extraScale

	local expandTo = {originalExtraScale.x, originalExtraScale.y+0.025}
	local expandTime = beatTime/4
	local shrinkTime = beatTime/4
	local tilt = 5
	local swing = 1

	local firstTime = 1


	while(true)do
		if(swing==1)then
			doTilt = -tilt
			if(firstTime==1) then 
				RotateByAmount(actor,{degrees=doTilt/2},100,true)
				firstTime = 0
			else
				RotateByAmount(actor,{degrees=doTilt},100,true)
			end
		end

		if(swing==3)then
			doTilt = tilt
			RotateByAmount(actor,{degrees=doTilt},100,true)
		end
		swing = swing + 1
		if(swing==5)then
			swing=1
		end

		actor.extraScale = originalExtraScale
		ScaleTo(actor, expandTo, expandTime)
		ScriptDelay(expandTime,"DialogTimers")
	
		ScaleTo(actor, originalExtraScale, shrinkTime)
		ScriptDelay(shrinkTime,"DialogTimers")

		--ScriptDelay(math.random(1000,10000),"DialogTimers")

	end
	
	--RotateByAmount(actor,{degrees=-amount},speed,true)
	--RotateByAmount(actor,{degrees=amount*2-(amount/2)},speed,true)
	--RotateByAmount(actor,{degrees=-((amount/2)+(amount/4))},speed,true)
	--RotateByAmount(actor,{degrees=(amount/4)*2},speed,true)
	--RotateByAmount(actor,{degrees=-amount/4},speed,true)
	actor.rotation = { degrees = 0 }		
end

Resources.CharacterMarker = Resources.Button:new {
	resource = "Button",
	image = "Character_Explorer.png",
	alignment = Center,
	alignTo = TopLeft,
	alignmentOffset = {0,-4},
	visible = false,
	scripts = {"MapCharacterGroove"},
	useFrames = false,
	rotationPoint = MidBottom,
	scalingPoint = MidBottom,
	--extraScale = 0.80,
	Radio = Resources.Actor:new {
		resource = "Actor",
		image = "Boombox.png",
		alignment = "MidLeft",
		alignmentOffset = {-15,8},
		zorder = -1,
		extraScale = 0.5,
	},
}

Resources.QuickQuestMapIcon = Resources.Actor:new {
	resource = "Actor",
    layer = 5,

    icon = Resources.Button:new {
        resource = "Button",
        alignment = "Center",
		alignmentOffset = {25+2,-12-20},
        scripts = {"QuickQuestMapIconGroove"},
        useFrames = false,
        rotationPoint = MidBottom,
        scalingPoint = MidBottom,
		resizeOnSetImage = true,
		alpha = 200,
        clickSound = "Dink.ogg",
        OnPostSpawn = function (self)
            self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
            self.image = QQ_MGR:GetImagePath(false,0.45)
        end,
        zorder = 1,

        OnButtonClick = function(self,event)
            if self ~= event.button then return false end
            self.screen:ShowQuickQuestDialog()
            return true
        end,
    },

    Glow = Resources.Actor:new {
		resource = "Actor",
		image = "Glow.png",
		alignment = "Center",
        alignmentOffset = {25+2,-12-20},
		zorder = 0,
		scripts = {
			function (self)
				local revolutionsPerSecond = 0.025
				RotateAtSpeed(self, {revolutions=revolutionsPerSecond})
			end,
		},
		shaderType = "SHADERTYPE_ADDITIVEALPHABLEND",
		alpha = 100,
		extraScale = 0.75,
	},

    timeRemainingLabel = Resources.AppDialogLabel:new {
        resource = "AppDialogLabel",
        alignment = "Center",
		alignmentOffset = {25,-12},
        font = Resources.DialogBoldFont:new{
            resource = "DialogBoldFont",
            fontSize = 12,
        },
        textColor = "White",
        shadowColor = "Black",

        Tick = function(self)
            self.text = QQ_MGR:GetTimeRemainingAsString()
        end,
        zorder = 2,
		visible = false,
    },
}



local function MakeLevel(levelName, levelNumber, landNumber, alignmentOffset)
	return Resources.Actor:new {
		resource = "Actor",
		image = "Flag.png", --just to get matching size of MainButton
		alpha = 0,
		alignment = "TopLeft",
		alignTo = "TopLeft",
		alignmentOffset = alignmentOffset,
		definitionName = levelName,
		landNumber = landNumber,
		zorder = 3,
        tag = "Level_" .. levelNumber,
		--extraScale = 0.80,
		
		MainButton = Resources.AppMenuMediumSignButton:new {
			resource = "AppMenuMediumSignButton",
			image = "Flag.png", --just to get matching size of MainButton
			useFrames = false,
			text = STRINGS(""),
			--text = tostring(levelNumber),
			levelNumber = levelNumber,
			textAlignmentOffset = {3,-10},
			alpha = 255,
			textWidthLimit = 46,
			textDisabledColor = "Sign_Brown",
			font	= Resources.MapButtonFont:new{
				resource = "MapButtonFont",
				fontSize = 28,
			},
			highlightChild = "levelNumberLabel_Rollover",
			levelNumberLabel = {
				class = "BitmapFontLabel",
				fontName = "Bangers.fnt",
				xAdvanceOffset = -1,
				color = "Cascade_DarkBrown",
				scalingPoint = Center,
				extraScale = 0.28,   --Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
									--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
				alignment = Center,
				alignmentOffset = {5,-10},
				text = tostring(levelNumber),
			},
			levelNumberLabel_Rollover = {
				class = "BitmapFontLabel",
				fontName = "Bangers.fnt",
				xAdvanceOffset = -1,
				color = "Blue",
				scalingPoint = Center,
				extraScale = 0.28,   --Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
									--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
				alignment = Center,
				alignmentOffset = {5,-10},
				text = tostring(levelNumber),
				zorder = 2,
				visible = false,
			},
			levelNumberLabel_Disabled = {
				class = "BitmapFontLabel",
				fontName = "Bangers.fnt",
				xAdvanceOffset = -1,
				color = "Sign_Brown",
				scalingPoint = Center,
				extraScale = 0.28,   --Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
									--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
				alignment = Center,
				alignmentOffset = {5,-8},
				text = tostring(levelNumber),
				zorder = 2,
				visible = false,
			},

			SelectionHighlight = Resources.Actor:new {
				image = "Check.png",
				zorder = -1,
				alignment = TopLeft,		
				alignTo = TopLeft,		
				alignmentOffset = {-12,-12},	
				zorder = 1,
				visible = false,
			},
			Lock = Resources.Actor:new {
				resource = "Actor",
				image = "FeatureLock.png",
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {-16,6},
				zorder = 3,
			},
			Pickaxe = Resources.Actor:new {
				resource = "Actor",
				image = "PickAxe.png",
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {-4,4},
				visible = false,
			},
			Stars = Resources.Actor:new {
				alignment = MidTop,
				alignTo = MidBottom,
				size = {24*3,30},
				alpha = 0,
				color = "White",
				alignmentOffset = {5,-26},
				zorder = 4,
				GoldBars0Star = Resources.Actor:new {
					resource = "Actor",
					image = "Gold_1bar_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-26,-10},
					zorder = 4,
					visible = false,
				},
				GoldBars1Star = Resources.Actor:new {
					resource = "Actor",
					image = "Gold_3bars_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-16,-5},
					zorder = 4,
					visible = false,
				},
				GoldBars2Star = Resources.Actor:new {
					resource = "Actor",
					image = "Gold_5bars_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-4,-14},
					zorder = 4,
					visible = false,
				},
				
				Worker1 = Resources.Actor:new {
					resource = "Actor",
					alignment = Center,
					alignTo = MidTop,
					alignmentOffset = {-10-4,-26+6},
					visible = false,
					size = {70/4,76/4},
					resizeOnSetImage = false,
				},
				Worker2 = Resources.Actor:new {
					resource = "Actor",
					alignment = Center,
					alignTo = MidTop,
					alignmentOffset = {0-4,-26+5},
					visible = false,
					size = {70/4,76/4},
					resizeOnSetImage = false,
				},
				Worker3 = Resources.Actor:new {
					resource = "Actor",
					alignment = Center,
					alignTo = MidTop,
					alignmentOffset = {10-4,-26+4},
					visible = false,
					size = {70/4,76/4},
					resizeOnSetImage = false,
				},

				GemSpecialRed = Resources.Actor:new {
					resource = "Actor",
					image = "GemSpecialRed_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-26+3,-10+6},
					zorder = 5,
					visible = false,
				},
				GemSpecialGreen = Resources.Actor:new {
					resource = "Actor",
					image = "GemSpecialGreen_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-16+3,-5+8},
					zorder = 5,
					visible = false,
				},
				GemSpecialBlue = Resources.Actor:new {
					resource = "Actor",
					image = "GemSpecialBlue_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-4+3-2,-10+6},
					zorder = 5,
					visible = false,
				},
			},

		},
	}
end

local function MakeSidePathLevel(levelName, landNumber, indexInLand, alignmentOffset)
	return Resources.Actor:new {
		resource = "Actor",
		image = "Flag.png", --just to get matching size of MainButton
		alpha = 0,
		--zorder = levelNumber,
		alignment = "TopLeft",
		alignTo = "TopLeft",
		alignmentOffset = alignmentOffset,
		definitionName = levelName,
		landNumber = landNumber,
		sideLevelIndex = indexInLand,
		zorder = 2,
        tag = "SideLevel_" .. landNumber .. "_" .. indexInLand,
		--extraScale = 0.80,
		
		MainButton = Resources.AppMenuMediumSignButton:new {
			resource = "AppMenuMediumSignButton",
			image = "SecretTunnel_Locked.png", 
			useFrames = false,
			--text = STRINGS("?"),
			textAlignmentOffset = {3,-10},
			alpha = 255,
			textWidthLimit = 46,
			textDisabledColor = "Sign_Brown",
			landNumber = landNumber,
			sideLevelIndex = indexInLand,
			font	= Resources.MapButtonFont:new{
				resource = "MapButtonFont",
				fontSize = 28,
			},
			highlightChild = "levelNumberLabel_Rollover",
			levelNumberLabel = {
				class = "BitmapFontLabel",
				fontName = "Bangers.fnt",
				xAdvanceOffset = -1,
				color = "Cascade_DarkBrown",
				scalingPoint = Center,
				extraScale = 0.22,   --Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
									--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
				alignment = Center,
				alignmentOffset = {0,-16},
				text = landNumber .. "-" .. indexInLand,
			},
			levelNumberLabel_Rollover = {
				class = "BitmapFontLabel",
				fontName = "Bangers.fnt",
				xAdvanceOffset = -1,
				color = "Purple",
				scalingPoint = Center,
				extraScale = 0.22,   --Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
									--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
				alignment = Center,
				alignmentOffset = {0,-16},
				text = landNumber .. "-" .. indexInLand,
				zorder = 2,
				visible = false,
			},
			levelNumberLabel_Disabled = {
				class = "BitmapFontLabel",
				fontName = "Bangers.fnt",
				xAdvanceOffset = -1,
				color = "Sign_Brown",
				scalingPoint = Center,
				extraScale = 0.22,   --Note: Right now this is scaled up and then scaled down to see if we can 2x resolution
									--TODO: Rework this so it works naturally with @2x pngs without the need to create 2x fonts and scale down
				alignment = Center,
				alignmentOffset = {0,-12},
				text = landNumber .. "-" .. indexInLand,
				zorder = 2,
				visible = false,
			},

			SelectionHighlight = Resources.Actor:new {
				image = "Check.png",
				zorder = -1,
				alignment = TopLeft,		
				alignTo = TopLeft,		
				alignmentOffset = {-12,-12},	
				zorder = 1,
				visible = false,
			},
			Lock = Resources.Actor:new {
				resource = "Actor",
				image = "FeatureLock.png",
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {-16,6},
				zorder = 3,
			},
			Pickaxe = Resources.Actor:new {
				resource = "Actor",
				image = "PickAxe.png",
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {-4,4},
				visible = false,
			},
			Stars = Resources.Actor:new {
				alignment = MidTop,
				alignTo = MidBottom,
				size = {24*3,30},
				alpha = 0,
				color = "White",
				alignmentOffset = {2,-26},
				zorder = 4,
				GoldBars0Star = Resources.Actor:new {
					resource = "Actor",
					image = "Gold_1bar_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-26+14,-10+2},
					zorder = 4,
					visible = false,
				},
				GoldBars1Star = Resources.Actor:new {
					resource = "Actor",
					image = "Gold_3bars_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-16+14,-5+2},
					zorder = 4,
					visible = false,
				},
				GoldBars2Star = Resources.Actor:new {
					resource = "Actor",
					image = "Gold_5bars_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-4+14,-14+2},
					zorder = 4,
					visible = false,
				},
				
				Worker1 = Resources.Actor:new {
					resource = "Actor",
					alignment = Center,
					alignTo = MidTop,
					alignmentOffset = {-12,-32},
					visible = false,
					size = {70/4,76/4},
					resizeOnSetImage = false,
				},
				Worker2 = Resources.Actor:new {
					resource = "Actor",
					alignment = Center,
					alignTo = MidTop,
					alignmentOffset = {-2,-32},
					visible = false,
					size = {70/4,76/4},
					resizeOnSetImage = false,
				},
				Worker3 = Resources.Actor:new {
					resource = "Actor",
					alignment = Center,
					alignTo = MidTop,
					alignmentOffset = {12-4,-32},
					visible = false,
					size = {70/4,76/4},
					resizeOnSetImage = false,
				},

				GemSpecialRed = Resources.Actor:new {
					resource = "Actor",
					image = "GemSpecialRed_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-26+3+10,-10+6+2},
					zorder = 5,
					visible = false,
				},
				GemSpecialGreen = Resources.Actor:new {
					resource = "Actor",
					image = "GemSpecialGreen_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-16+3+10,-5+8+2},
					zorder = 5,
					visible = false,
				},
				GemSpecialBlue = Resources.Actor:new {
					resource = "Actor",
					image = "GemSpecialBlue_small.png",
					alignment = "Center",		
					alignTo = "MidBottom",	
					alignmentOffset = {-4+3-2+10,-10+6+2},
					zorder = 5,
					visible = false,
				},
			},

		},
	}
end


landTextColorTable = {
	[1] = "Cascade_DarkerGreenBlue",
	[2] = "Cascade_DarkerCyan2",
	[3] = "Cascade_DarkerPurple",
	[4] = "Cascade_DarkOrange",
	[5] = "Cascade_DarkTan",
	[6] = "Cascade_DarkerPurple",
	[7] = "Cascade_DarkBrown",
	[8] = "Blue_Shine",
	[9] = "Blue_Shine",
}

landSignTable = {
	[1] = {"LandSign1", "Land 1", "Emerald Cave",		1,		{60,0}},		--{0,-60+-200}},
	[2] = {"LandSign2", "Land 2", "Sapphire Lake",		2,		{100,16}},		--{40,-60+-400}},
	[3] = {"LandSign3", "Land 3", "Purple Mountains",	3,		{90,-155}},		--{0,-60+-600}},
	[4] = {"LandSign4", "Land 4", "Crystal Caves",		4,		{70,-50}},		--{40,-60+-800}},
	[5] = {"LandSign5", "Land 5", "Gem Jungle",			5,		{-80,-390}},		--{0,-60+-1000}},
	[6] = {"LandSign6", "Land 6", "Lost Gold City",		6,		{-110,-210}},		--{40,-60+-1200}},
	[7] = {"LandSign7", "Land 7", "Grumpy Volcano",		7,		{-100,-170}},		--{0,-60+-1400}},
	[8] = {"LandSign8", "Land 8", "Diamond Glacier",	8,		{-80,-240}},		--{40,-60+-1600}},
	[9] = {"LandSign9", "Land 9", "Emerald Cave",		1,		{100,-145}},		--{0,-60+-200}},
	[10] = {"LandSign10", "Land 10", "Emerald Cave",	1,		{95,-100}},
}

if TotalLands >= 11 then
	table.insert(landSignTable, {"LandSign11", "Land 11", "Emerald Cave",	1,		{97,-90}})
end

if TotalLands >= 12 then
	table.insert(landSignTable, {"LandSign12", "Land 12", "Emerald Cave",	1,		{100,-112}})
end

if TotalLands >= 13 then
	table.insert(landSignTable, {"LandSign13", "Land 13", "Emerald Cave",	1,		{95,-125}})
end

if TotalLands >= 14 then
	table.insert(landSignTable, {"LandSign14", "Land 14", "Emerald Cave",	1,		{100,-50}})
end

if TotalLands >= 15 then
	table.insert(landSignTable, {"LandSign15", "Land 15", "Emerald Cave",	1,		{100,-50}})
end

if TotalLands >= 16 then
	table.insert(landSignTable, {"LandSign16", "Land 16", "Emerald Cave",	1,		{90,-37}})
end

if TotalLands >= 17 then
	table.insert(landSignTable, {"LandSign17", "Land 17", "Emerald Cave",	1,		{97,-70}})
end

if TotalLands >= 18 then
	table.insert(landSignTable, {"LandSign18", "Land 18", "Emerald Cave",	1,		{-90,-310}})
end

if TotalLands >= 19 then
	table.insert(landSignTable, {"LandSign19", "Land 19", "Emerald Cave",	1,		{-100,-200}})
end

if TotalLands >= 20 then
	table.insert(landSignTable, {"LandSign20", "Land 20", "Emerald Cave",	1,		{-98,-40}})
end

if TotalLands >= 21 then
	table.insert(landSignTable, {"LandSign21", "Land 21", "Emerald Cave",	1,		{90,-130}})
end

if TotalLands >= 22 then
	table.insert(landSignTable, {"LandSign22", "Land 22", "Emerald Cave",	1,		{-100,-40}})
end

if TotalLands >= 23 then
	table.insert(landSignTable, {"LandSign23", "Land 23", "Emerald Cave",	1,		{-100,-40}})
end

if TotalLands >= 24 then
	table.insert(landSignTable, {"LandSign24", "Land 24", "Emerald Cave",	1,		{-95,-50}})
end

if TotalLands >= 25 then
	table.insert(landSignTable, {"LandSign25", "Land 25", "Emerald Cave",	1,		{-95,-40}})
end

if TotalLands >= 26 then
	table.insert(landSignTable, {"LandSign26", "Land 26", "Emerald Cave",	1,		{-95,-40}})
end

if TotalLands >= 27 then
	table.insert(landSignTable, {"LandSign27", "Land 27", "Emerald Cave",	1,		{-95,-30}})
end

if TotalLands >= 28 then
	table.insert(landSignTable, {"LandSign28", "Land 28", "Emerald Cave",	1,		{-100,-30}})
end

if TotalLands >= 29 then
	table.insert(landSignTable, {"LandSign29", "Land 29", "Emerald Cave",	1,		{-100,-30}})
end

if TotalLands >= 30 then
	table.insert(landSignTable, {"LandSign30", "Land 30", "Emerald Cave",	1,		{-95,-30}})
end

if TotalLands >= 31 then
	table.insert(landSignTable, {"LandSign31", "Land 31", "Emerald Cave",	1,		{-95,-30}})
end

if TotalLands >= 32 then
	table.insert(landSignTable, {"LandSign32", "Land 32", "Emerald Cave",	1,		{-95,-30}})
end

if TotalLands >= 33 then
	table.insert(landSignTable, {"LandSign33", "Land 33", "Emerald Cave",	1,		{-95,-30}})
end

totalHeight = 1536*27 + 2560*(TotalLands - 27)		-- 1536: height of 6-slice land; 2560: height of 10-slice land
local landWidth = 384
numSlices = 6

landOffsets = {
	[1] = 1536,
	[2] = 1536*2,
	[3] = 1536*3,
	[4] = 1536*4,
	[5] = 1536*5,
	[6] = 1536*6,
	[7] = 1536*7,
	[8] = 1536*8,
	[9] = 1536*9,
	[10] = 1536*10,
	[11] = 1536*11,
	[12] = 1536*12,
	[13] = 1536*13,
	[14] = 1536*14,
	[15] = 1536*15,
	[16] = 1536*16,
	[17] = 1536*17,
	[18] = 1536*18,
	[19] = 1536*19,
	[20] = 1536*20,
	[21] = 1536*21,
	[22] = 1536*22,
	[23] = 1536*23,
	[24] = 1536*24,
	[25] = 1536*25,
	[26] = 1536*26,
	[27] = 1536*27,
	[28] = 1536*27 + 2560,
	[29] = 1536*27 + 2560*2,
	[30] = 1536*27 + 2560*3,
	[31] = 1536*27 + 2560*4,
	[32] = 1536*27 + 2560*5,
	[33] = 1536*27 + 2560*6,
	[34] = 1536*27 + 2560*7,
}

Resources.LandSlice = Resources.Actor:new {
	resource = "Actor",
	desiredImage = "Land1_1.jpg",
    moreDesirableImage = "",
	alignment = "MidTop",
	alignmentOffset = {0,0},
	zorder = 0,
}

Resources.LandSliceOverlay = Resources.Actor:new {
	resource = "Actor",
	desiredImage = "Land2_1_Overlay.png",
    moreDesirableImage = "",
	alignment = "MidTop",
	alignmentOffset = {0,0},
	zorder = 0,
}

local function Land(params)
	--
	-- Argument checks and extractions:
	--
	assert(IsNumber(params.landNumber))
	assert(params.landNumber >= 1)

	assert(IsNumber(params.firstLevelNumber))
	assert(params.firstLevelNumber >= 1)

	--assert(IsNumber(params.numLevels))
	--assert(params.numLevels >= 1)

	assert(IsTable(params.landLevels))
	local numLevels = table.getn(params.landLevels)
	local numSlices = params.numSlices
	--assert(table.getn(params.landLevels) == numLevels)

	assert(IsNil(params.foregroundImageName) or IsString(params.foregroundImageName))

	assert(IsNil(params.signTextNotVisible) or IsBoolean(params.signTextNotVisible))

	local hgeLayerTag = "LandParticlesLayer" .. params.landNumber

	--
	-- Resource Building:
	--
	local theResource = Resources.Land:new {
		resource = "Land",
		isLand = true,
		alignment = "MidTop",
		alignTo = "MidTop",
		color = "Cascade_GreenBlue",
		landNumber = params.landNumber,
		landName = params.landName,
		numSlices = params.numSlices,
		size = {landWidth, 256*numSlices},
		foregroundImageName = params.foregroundImageName,
		zorder = params.landNumber,

		hgeParticleLayer = {
			class = "hgeParticleLayer",
			alignment = "MidTop",
			alignTo = "MidTop",
			zorder = 10000,
			tag = hgeLayerTag,
			zorder = 1,
		},

		alignmentOffset = {
			0,
			totalHeight - landOffsets[params.landNumber],
		},
		StageSign = {
			class = "Actor",
			resource = "LandSign",
			alignmentOffset = landSignTable[params.landNumber][5],
		},
		OnPostSpawn = function(self)
			local landText = STRINGS("Land")
			self.StageSign.LandNumber.text = string.format("%s %s", landText, self.landNumber)
			self.StageSign.LandName.text = self.landName or "No Name"
		end,
		Path = params.Path or {class = "Actor", visible = true},
	}

	--
	-- Apply extra properties to the land:
	--
	for k,v in pairs(params) do
		theResource[k] = v
	end

	--
	--Setup land slices and overlays
	--


	for sliceNumber = 1, params.numSlices do
		local slicePosition = {0, 256 * (params.numSlices - sliceNumber)}
		local sliceName = string.format("Slice%d", sliceNumber)
		local sliceImageName = string.format("Land%d_%d.jpg", params.landNumber, sliceNumber)
        local sliceImageName2 = ""

        --Slices 6+ can have optional "B" versions
        if(sliceNumber > 5) then
            sliceImageName2 = string.format("Land%d_%d_b.jpg", params.landNumber, sliceNumber)
        end

		local slice = Resources.LandSlice:new{ resource = "LandSlice", desiredImage = sliceImageName, moreDesirableImage = sliceImageName2, alignmentOffset = slicePosition, }
		theResource[sliceName] = slice
		
		local overlayName = string.format("OverlaySlice%d", sliceNumber)
		if (params[overlayName] == true) then
			local overlayImageName = string.format("Land%d_%d_Overlay.png", params.landNumber, sliceNumber)
            local overlayImageName2 = ""

            --Overlays 6+ can have optional "B" versions
            if(sliceNumber > 5) then
                overlayImageName2 = string.format("Land%d_%d_Overlay_b.png", params.landNumber, sliceNumber)
            end

			local overlay = Resources.LandSliceOverlay:new{ resource = "LandSliceOverlay", desiredImage = overlayImageName, moreDesirableImage = overlayImageName2, alignmentOffset = slicePosition, layer = 4, }
			theResource[overlayName] = overlay
		end
	end

	--
	-- Setup levels:
	--
	local lastLevelNumber = params.firstLevelNumber + numLevels - 1
	for levelNumber = params.firstLevelNumber, lastLevelNumber do
		local levelNumberInLand = (levelNumber - params.firstLevelNumber + 1)

		assert(
			IsPoint(params.landLevels[levelNumberInLand].alignmentOffset),
			string.format("No 'alignmentOffset' for level %s (in land %s; level offset in land: %s)",
				tostring(levelNumber),
				tostring(params.landNumber),
				tostring(levelNumberInLand)
			)
		)

		local level = MakeLevel(
			string.format("Level %d", levelNumber),
			levelNumber,
			params.landNumber,
			params.landLevels[levelNumberInLand].alignmentOffset
		)
		theResource[string.format("Level_%d", levelNumber)] = level

		--[[
		logprintf("*** LAND %2s, LEVEL %3s (land offset %2s): ao={%4d, %4d}\n",
			tostring(params.landNumber),
			tostring(levelNumber),
			tostring(levelNumberInLand),
			params.landLevels[levelNumberInLand].alignmentOffset[1],
			params.landLevels[levelNumberInLand].alignmentOffset[2])
		--]]
	end
	
	-- Side paths/ secret tunnels
	if(IsTable(params.sideLevels)) then
		local numSideLevels = table.getn(params.sideLevels)
		for sideLevelIndex = 1, numSideLevels do
		
			assert(
				IsPoint(params.landLevels[sideLevelIndex].alignmentOffset),
				string.format("No 'alignmentOffset' for side level %s in land %s",
					tostring(sideLevelIndex),
					tostring(params.landNumber)
				)
			)

			local level = MakeSidePathLevel(
				string.format("SideLevel %d", sideLevelIndex),
				params.landNumber,
				sideLevelIndex,
				params.sideLevels[sideLevelIndex].alignmentOffset
			)
			theResource[string.format("SideLevel_%d_%d", params.landNumber, sideLevelIndex)] = level
		end
	end

	-- Add buildings

	for unused,building in pairs(Buildings) do
		if(building.landNumber == params.landNumber) then
			theResource[building.name] = Resources.Building:new {
				resource = "Building",
			}
		end
	end

	-- Add dig spots
	local digSpotsForLand = DigSpots["Land" .. params.landNumber] 
	if(IsTable(digSpotsForLand)) then
		for index,digspot in pairs(digSpotsForLand) do
			local digSpotName = "Land" .. params.landNumber .. "_DigSpot" .. index
			--logprint("Adding dig spot named: " .. digSpotName)
			--logprint("with properties:")
			--logprint(SerializeToString(digspot))
			if(digspot.NumberLabel ~= nil) then
				digspot.NumberLabel.text = Int2Comma(index)
			end
			theResource[digSpotName] = digspot
		end
	end

	--
	-- Done!
	--
	return theResource
end


Resources.Gate = Resources.Button:new {
	resource = "Button",
	isGate = true,
	image = "Land1_GATE1.png",
	lockedImage = "Land1_GATE1.png",
	unlockedImage = "Land1_GATE3.png",
	useFrames = false,
	alignment = "MidBottom",
	alignmentOffset = {0, -40},
	layer = 1,
	clickSound = "Chunka.ogg",
	Sockets = Resources.Actor:new {
		resources = "Actor",
		size = {75,30},
		alignment = "MidBottom",
		Socket_Green = Resources.Actor:new {
			resources = "Actor",
			image = "Gate_Socket_Leaf.png",
			alignment = "MidLeft",
			filled = Resources.Actor:new {
				resources = "Actor",
				image = "Gate_Gem_Leaf.png",
				alignment = "Center",
				visible = false,
				zorder = 1,
			},
			filled_glow = Resources.Actor:new {
				resources = "Actor",
				image = "Gate_Gem_Leaf_Glo.png",
				alignment = "Center",
				visible = false,
				zorder = 2,
			},
		},
		Socket_Red = Resources.Actor:new {
			resources = "Actor",
			image = "Gate_Socket_Fire.png",
			alignment = "MidRight",
			filled = Resources.Actor:new {
				resources = "Actor",
				image = "Gate_Gem_Fire.png",
				alignment = "Center",
				visible = false,
				zorder = 1,
			},
			filled_glow = Resources.Actor:new {
				resources = "Actor",
				image = "Gate_Gem_Fire_Glo.png",
				alignment = "Center",
				visible = false,
				zorder = 2,
			},
		},
		Socket_Blue = Resources.Actor:new {
			resources = "Actor",
			image = "Gate_Socket_Water.png",
			alignment = "Center",
			filled = Resources.Actor:new {
				resources = "Actor",
				image = "Gate_Gem_Water.png",
				alignment = "Center",
				visible = false,
				zorder = 1,
			},
			filled_glow = Resources.Actor:new {
				resources = "Actor",
				image = "Gate_Gem_Water_Glo.png",
				alignment = "Center",
				visible = false,
				zorder = 2,
			},
		},

		Gate_TapHerePointer = Resources.Actor:new {
			resource = "Actor",
			image = "TapHere.png",
			rotationPoint = "MidBottom",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {40,-20},
			visible = false,
			scripts = {"TapHereAction"},
			zorder = 3,
			layer = 100,
			OnPostSpawn = function (self)
			end
		},
	},


}

local function Gate(params)

	local pathEndOffset = {x=0, y=0}
	if params.pathEndOffset ~= nil then
		pathEndOffset = params.pathEndOffset
	end
	--
	-- Resource Building:
	--
	local theResource = Resources.Gate:new {
		resource = "Gate",
		pathEndOffset = pathEndOffset,
	}

	--
	-- Apply extra properties:
	--
	for k,v in pairs(params) do
		theResource[k] = v
	end

	--
	-- Done!
	--
	return theResource
end

local NO_PATH = "NO_PATH"
--alllevels
Lands = {
	Land {
		landNumber = 1,
		landName = STRINGS("Cascade Hills"),
		firstLevelNumber = 1,
		numSlices = 6,
		landLevels = {
			{ alignmentOffset = {200, 1277}, },			--1
			{ alignmentOffset = {240, 1167}, },			--2
			{ alignmentOffset = {137, 1044},},				--3
			{ alignmentOffset = {245, 922}, },			--4
			{ alignmentOffset = {130, 802}, },			--5
			{ alignmentOffset = {240, 682}, },			--6
			{ alignmentOffset = {147, 584},},				--7
			{ alignmentOffset = {160, 462}, },			--8
			{ alignmentOffset = {240, 352}, },			--9
			{ alignmentOffset = {120, 205}, },			--10
		},
		Path = NO_PATH,
		stageSignImage = "Land1_Sign.png",
		signTextNotVisible = false,
	},
	
	Gate {
		image = "Gate_Land1-2.png",
		lockedImage = "Gate_Land1-2.png",
		alignmentOffset = {0-(152/2)+20,60-7/2},
		layer = 4,
		Sockets = {
			class = "Actor",
			alignmentOffset = {-55, -20},
		},
	},

	Land {
		landNumber = 2,
		landName = STRINGS("Cascade Lakes"),
		firstLevelNumber = 11,
		numSlices = 6,

		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {160, 1362}, },			--11
			{ alignmentOffset = {220, 1212}, },			--12
			{ alignmentOffset = {255, 1022}, },			--13
			{ alignmentOffset = {180, 862}, },			--14
			{ alignmentOffset = {80, 722}, },			--15
			{ alignmentOffset = {170, 622}, },			--16
			{ alignmentOffset = {225, 517}, },			--17
			{ alignmentOffset = {150, 397}, },			--18
			{ alignmentOffset = {240, 302}, },			--19
			{ alignmentOffset = {180, 202}, },			--20
		},

		sideLevels = {
			{ alignmentOffset = {160, 920},},--13-1
			{ alignmentOffset = {96, 904},},--13-2
			{ alignmentOffset = {32, 883},},--13-3
            { alignmentOffset = {70, 280},},--19-4
		},

		Path = NO_PATH,
		stageSignImage = "Land2_Sign.png",
		signTextNotVisible = false,
	},

	Gate {
		image = "Gate_Land2-3.png",
		lockedImage = "Gate_Land2-3.png",
		alignmentOffset = {0-78/2-10,-25-19/2},
		pathEndOffset = {x=45, y=0},
		Sockets = {
			class = "Actor",
			alignmentOffset = {25, -60},
		},
	},

	Land {
		landNumber = 3,
		landName = STRINGS("Purple Mountain"),
		firstLevelNumber = 21,
		numSlices = 6,

		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {150, 1425}, },			--21
			{ alignmentOffset = {130, 1230}, },			--22
			{ alignmentOffset = {180, 1157}, },			--23
			{ alignmentOffset = {120, 1057}, },			--24
			{ alignmentOffset = {120, 977}, },			--25
			{ alignmentOffset = {160, 877}, },			--26
			{ alignmentOffset = {120, 772}, },			--27
			{ alignmentOffset = {210, 692}, },			--28
			{ alignmentOffset = {220, 592}, },			--29
			{ alignmentOffset = {180, 542}, },			--30
			{ alignmentOffset = {170, 400},},			--31
			{ alignmentOffset = {195, 327}, },			--32
			{ alignmentOffset = {200, 227}, },			--33
			{ alignmentOffset = {260, 127}, },			--34
			{ alignmentOffset = {180, 87}, },			--35
		},

		sideLevels = {
			{ alignmentOffset = {70, 1370},},
			{ alignmentOffset = {180, 790},},
			{ alignmentOffset = {71, 550},},
            { alignmentOffset = {100, 125},},--3-4
		},

		stageSignImage = "Land3_Sign.png",
		signTextNotVisible = false,
	},

	Gate {
		image = "Gate_Land3-4.png",
		lockedImage = "Gate_Land3-4.png",
		alignmentOffset = {0-164/2+18,28-15/2},
		Sockets = {
			class = "Actor",
			alignmentOffset = {15, -20},
		},
	},

	Land {
		landNumber = 4,
		landName = STRINGS("Sapphire Caves"),
		firstLevelNumber = 36,
		numSlices = 6,
		
		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {140, 1342}, },			--36
			{ alignmentOffset = {200, 1197}, },			--37
			{ alignmentOffset = {239, 1034},},				--38
			{ alignmentOffset = {111, 997},},				--39
			{ alignmentOffset = {140, 907}, },			--40
			{ alignmentOffset = {280, 837}, },			--41
			{ alignmentOffset = {260, 697}, },			--42
			{ alignmentOffset = {120, 697}, },			--43
			{ alignmentOffset = {140, 537}, },			--44
			{ alignmentOffset = {290, 497}, },			--45
			{ alignmentOffset = {185, 327}, },			--46
			{ alignmentOffset = {95, 295}, },			--47
			{ alignmentOffset = {130, 227}, },			--48
			{ alignmentOffset = {260, 217}, },			--49
			{ alignmentOffset = {240, 117}, },			--50
		},

		sideLevels = {
            { alignmentOffset = {280, 1150},},
            { alignmentOffset = {145, 1060},},
            { alignmentOffset = {230, 890},},
            { alignmentOffset = {281, 400},},
		},

		Path = NO_PATH,
		stageSignImage = "Land4_Sign.png",
		signTextNotVisible = false,
	},

	Gate {
		image = "Gate_Land4-5.png",
		lockedImage = "Gate_Land4-5.png",
		alignmentOffset = {0-36,62-12},
		Sockets = {
			class = "Actor",
			alignmentOffset = {20, -25},
		},
	},

	Land {
		landNumber = 5,
		landName = STRINGS("Jungle Gems"),
		firstLevelNumber = 51,
		numSlices = 6,

		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {193, 1373}, },			--51
			{ alignmentOffset = {197, 1270}, },			--52
			{ alignmentOffset = {160, 1210}, },			--53
			{ alignmentOffset = {230, 1162}, },			--54
			{ alignmentOffset = {278, 1073}, },			--55
			{ alignmentOffset = {281, 969}, },			--56
			{ alignmentOffset = {186, 944}, },			--57
			{ alignmentOffset = {242, 780}, },			--58
			{ alignmentOffset = {249, 693}, },			--59
			{ alignmentOffset = {131, 611}, },			--60
			{ alignmentOffset = {150, 527}, },			--61
			{ alignmentOffset = {270, 427}, },			--62
			{ alignmentOffset = {171, 268}, },			--63
			{ alignmentOffset = {252, 132}, },			--64
			{ alignmentOffset = {167, 22}, },			--65
		},
		sideLevels = {
			{ alignmentOffset = {95, 870},},
			{ alignmentOffset = {110, 810},},
			{ alignmentOffset = {220, 300},},
            { alignmentOffset = {150, 170},},
		},
		stageSignImage = "Land5_Sign.png",
		signTextNotVisible = false,
	},

	Gate {
		image = "Gate_Land5-6.png",
		lockedImage = "Gate_Land5-6.png",
		alignmentOffset = {-55,2},
		Sockets = {
			class = "Actor",
			alignmentOffset = {10, -30},
		},
	},

	Land {
		landNumber = 6,
		landName = STRINGS("Lost City"),
		firstLevelNumber = 66,
		numSlices = 6,
		
		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {140, 1337}, },			--66
			{ alignmentOffset = {240, 1297}, },			--67
			{ alignmentOffset = {260, 1177}, },			--68
			{ alignmentOffset = {295, 1077},},				--69
			{ alignmentOffset = {280, 977}, },			--70
			{ alignmentOffset = {270, 837}, },			--71
			{ alignmentOffset = {170, 777}, },			--72
			{ alignmentOffset = {120, 710}, },			--73
			{ alignmentOffset = {110, 647},},				--74
			{ alignmentOffset = {79, 505},},				--75
			{ alignmentOffset = {100, 427}, },			--76
			{ alignmentOffset = {290, 357}, },			--77
			{ alignmentOffset = {250, 277}, },			--78
			{ alignmentOffset = {160, 207}, },			--79
			{ alignmentOffset = {140, 127}, },			--80
		},
		sideLevels = {
			{ alignmentOffset = {215, 885},},
            { alignmentOffset = {30, 725},},
			{ alignmentOffset = {180, 335},},
			{ alignmentOffset = {95, 335},},
		},
		stageSignImage = "Land6_Sign.png",
		signTextNotVisible = false,
	},

	Gate {
		image = "Gate_Land6-7.png",
		lockedImage = "Gate_Land6-7.png",
		alignmentOffset = {0-30,-6-10},
		Sockets = {
			class = "Actor",
			alignmentOffset = {0, -70},
		},
	},

	Land {
		landNumber = 7,
		landName = STRINGS("Volcano Trail"),
		firstLevelNumber = 81,
		numSlices = 6,
		
		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {150, 1425}, },			--81
			{ alignmentOffset = {260, 1262}, },			--82
			{ alignmentOffset = {240, 1160}, },			--83
			{ alignmentOffset = {170, 1037}, },			--84
			{ alignmentOffset = {120, 917}, },			--85
			{ alignmentOffset = {160, 837}, },			--86
			{ alignmentOffset = {210, 777}, },			--87
			{ alignmentOffset = {300, 700}, },			--88
			{ alignmentOffset = {250, 597}, },			--89
			{ alignmentOffset = {150, 527}, },			--90
			{ alignmentOffset = {120, 427}, },			--91
			{ alignmentOffset = {190, 372}, },			--92
			{ alignmentOffset = {245, 297}, },			--93
			{ alignmentOffset = {280, 197}, },			--94
			{ alignmentOffset = {260, 117}, },			--95
		},
		sideLevels = {
			{ alignmentOffset = {75, 980},},
			{ alignmentOffset = {150, 705},},
			{ alignmentOffset = {60, 485},},
            { alignmentOffset = {90, 350},}
		},
		stageSignImage = "Land7_Sign.png",
		signTextNotVisible = false,
	},

	Gate {
		image = "Gate_Land7-8.png",
		lockedImage = "Gate_Land7-8.png",
		alignmentOffset = {0-35,52-12},
		Sockets = {
			class = "Actor",
			alignmentOffset = {10, -60},
		},
    },

	Land {
		landNumber = 8,
		landName = STRINGS("Diamond Glacier"),
		firstLevelNumber = 96,
		numSlices = 6,

		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {x=166, y=1410},},				--96
			{ alignmentOffset = {256, 1232},},				--97
			{ alignmentOffset = {301, 1155},},				--98
			{ alignmentOffset = {247, 1076},},				--99
			{ alignmentOffset = {157, 1019},},				--100
			{ alignmentOffset = {x=124, y=921},},				--101
			{ alignmentOffset = {x=143, y=830},},				--102
			{ alignmentOffset = {x=217, y=750},},				--103
			{ alignmentOffset = {267, 593},},				--104
			{ alignmentOffset = {285, 515},},				--105
			{ alignmentOffset = {261, 439},},				--106
			{ alignmentOffset = {239, 382},},				--107
			{ alignmentOffset = {x=274, y=320},},				--108
			{ alignmentOffset = {x=106, y=127},},				--109
			{ alignmentOffset = {x=140, y=30},},				--110
		},
		sideLevels = {
			{ alignmentOffset = {200, 965},},
			{ alignmentOffset = {205, 900},},
			{ alignmentOffset = {210, 835},},
			{ alignmentOffset = {266, 154},},
		},
		stageSignImage = "Land8_Sign.png",
		signTextNotVisible = false,
	},

	-- LAND 9
	Gate {
		image = "Gate_Land8-9.png",
		lockedImage = "Gate_Land8-9.png",
		alignmentOffset = {-50,-10-10},
		Sockets = {
			class = "Actor",
			alignmentOffset = {0, -25},
		},
    },

	Land {
		landNumber = 9,
		landName = STRINGS("Mystery Beach"),
		firstLevelNumber = 111,
		numSlices = 6,

		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {200, 1330}, },			--111
			{ alignmentOffset = {200, 1220}, },			--112
			{ alignmentOffset = {177, 1114},},			--113
			{ alignmentOffset = {147, 1000},},			--114
			{ alignmentOffset = {235, 922}, },			--115
			{ alignmentOffset = {200, 860}, },			--116
			{ alignmentOffset = {250, 802}, },			--117
			{ alignmentOffset = {220, 742}, },			--118
			{ alignmentOffset = {180, 652}, },			--119
			{ alignmentOffset = {157, 548},},			--120
			{ alignmentOffset = {200, 442}, },			--121
			{ alignmentOffset = {244, 352}, },			--122
			{ alignmentOffset = {170, 270}, },			--123
			{ alignmentOffset = {160, 170}, },			--124
			{ alignmentOffset = {200, 70}, },			--125
		},
		sideLevels = {
			{ alignmentOffset = {120, 705},},
			{ alignmentOffset = {60, 645},},
			{ alignmentOffset = {160, 385},},
			{ alignmentOffset = {100, 30},},
		},
		stageSignImage = "Land9_Sign.png",
		signTextNotVisible = false,
	},

	-- LAND 10
	Gate {
		image = "Gate_Land9-10.png",
		lockedImage = "Gate_Land9-10.png",
		alignmentOffset = {0-50,-40-10},
		Sockets = {
			class = "Actor",
			alignmentOffset = {0, -15},
		},
    },

	Land {
		landNumber = 10,
		landName = STRINGS("Hidden Temple"),
		firstLevelNumber = 126,
		numSlices = 6,

		OverlaySlice1 = true,

		landLevels = {
			{ alignmentOffset = {163, 1342},},				--126
			{ alignmentOffset = {190, 1255},},				--127
			{ alignmentOffset = {223, 1204},},				--128
			{ alignmentOffset = {195, 1085},},				--129
			{ alignmentOffset = {249, 978},},				--130
			{ alignmentOffset = {215, 893},},				--131
			{ alignmentOffset = {190, 812},},				--132
			{ alignmentOffset = {236, 714},},				--133
			{ alignmentOffset = {212, 655},},				--134
			{ alignmentOffset = {160, 580},},				--135
			{ alignmentOffset = {209, 520},},				--136
			{ alignmentOffset = {185, 430},},				--137
			{ alignmentOffset = {155, 242},},				--138
			{ alignmentOffset = {160, 176},},				--139
			{ alignmentOffset = {204, 59},},				--140

		},
		sideLevels = {
			{ alignmentOffset = {95, 1240},},
			{ alignmentOffset = {105, 1045},},
			{ alignmentOffset = {120, 640},},
			{ alignmentOffset = {85, 460},},
		},
		stageSignImage = "Land10_Sign.png",
		signTextNotVisible = false,
	},
}

if TotalLands >= 11 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land10-11.png",
			lockedImage = "Gate_Land10-11.png",
			alignmentOffset = {-20,32},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -17+5},
			},
		},

		Land {
			landNumber = 11,
			landName = STRINGS("Bamboo Forest"),
			firstLevelNumber = 141,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {168, 1350},},				--141
				{ alignmentOffset = {225, 1268},},				--142
				{ alignmentOffset = {273, 1200},},				--143
				{ alignmentOffset = {173, 1110},},				--144
				{ alignmentOffset = {118, 1024},},				--145
				{ alignmentOffset = {270, 918},},				--146
				{ alignmentOffset = {230, 838},},				--147
				{ alignmentOffset = {155, 750},},				--148
				{ alignmentOffset = {251, 700},},				--149
				{ alignmentOffset = {190, 500},},				--150
				{ alignmentOffset = {133, 430},},				--151
				{ alignmentOffset = {220, 375},},				--152
				{ alignmentOffset = {265, 315},},				--153
				{ alignmentOffset = {160, 170},},				--154
				{ alignmentOffset = {166, 85},},				--155

			},
			sideLevels = {
				{ alignmentOffset = {175, 1180},},
				{ alignmentOffset = {125, 810},},
				{ alignmentOffset = {265, 635},},
				{ alignmentOffset = {235, 435},},
			},
			stageSignImage = "Land11_Sign.png",
			signTextNotVisible = false,
		},
	})
end

if TotalLands >= 12 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land11-12.png",
			lockedImage = "Gate_Land11-12.png",
			alignmentOffset = {0-40,12},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -13},
			},
		},

		Land {
			landNumber = 12,
			landName = STRINGS("Himalayan Quest"),
			firstLevelNumber = 156,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {188, 1340},},				--156
				{ alignmentOffset = {145, 1260},},				--157
				{ alignmentOffset = {193, 1170},},				--158
				{ alignmentOffset = {233, 1110},},				--159
				{ alignmentOffset = {218, 1024},},				--160
				{ alignmentOffset = {180, 928},},				--161
				{ alignmentOffset = {200, 843},},				--162
				{ alignmentOffset = {170, 734},},				--163
				{ alignmentOffset = {231, 615},},				--164
				{ alignmentOffset = {260, 520},},				--165
				{ alignmentOffset = {240, 440},},				--166
				{ alignmentOffset = {230, 375},},				--167
				{ alignmentOffset = {265, 315},},				--168
				{ alignmentOffset = {260, 200},},				--169
				{ alignmentOffset = {246, 100},},				--170

			},
			sideLevels = {
				{ alignmentOffset = {75, 1180},},
				{ alignmentOffset = {170, 545},},
				{ alignmentOffset = {140, 305},},
				{ alignmentOffset = {132, 62},},
			},
			stageSignImage = "Land12_Sign.png",
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 13 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land12-13.png",
			lockedImage = "Gate_Land12-13.png",
			alignmentOffset = {0-20,72},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -47},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {0,-2},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {0,-4},
				},
			},
		},

		Land {
			landNumber = 13,
			landName = STRINGS("Ruins of Olympus"),
			firstLevelNumber = 171,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {210, 1390},},				--171
				{ alignmentOffset = {225, 1230},},				--172
				{ alignmentOffset = {173, 1130},},				--173
				{ alignmentOffset = {190, 1064},},				--174
				{ alignmentOffset = {230, 985},},				--175
				{ alignmentOffset = {260, 912},},				--176
				{ alignmentOffset = {240, 835},},				--177
				{ alignmentOffset = {190, 754},},				--178
				{ alignmentOffset = {231, 615},},				--179
				{ alignmentOffset = {220, 500},},				--180
				{ alignmentOffset = {190, 410},},				--181
				{ alignmentOffset = {170, 325},},				--182
				{ alignmentOffset = {265, 230},},				--183
				{ alignmentOffset = {250, 160},},				--184
				{ alignmentOffset = {236, 70},},				--185

			},
			sideLevels = {
				{ alignmentOffset = {224, 1150},},
				{ alignmentOffset = {262, 1000},},
				{ alignmentOffset = {60, 165},},
				{ alignmentOffset = {78, 110},},
			},
			stageSignImage = "Land13_Sign.png",
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 14 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land13-14.png",
			lockedImage = "Gate_Land13-14.png",
			alignmentOffset = {-30,42},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -16},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {0,-1},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {0,-2},
				},
			},
		},

		Land {
			landNumber = 14,
			landName = STRINGS("The Labyrinth"),
			firstLevelNumber = 186,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {178, 1400},},				--186
				{ alignmentOffset = {195, 1310},},				--187
				{ alignmentOffset = {138, 1210},},				--188
				{ alignmentOffset = {90, 1098},},				--189
				{ alignmentOffset = {139, 1020},},				--190
				{ alignmentOffset = {265, 1038},},				--191
				{ alignmentOffset = {280, 847},},				--192
				{ alignmentOffset = {130, 790},},				--193
				{ alignmentOffset = {133, 650},},				--194
				{ alignmentOffset = {248, 540},},				--195
				{ alignmentOffset = {280, 440},},				--196
				{ alignmentOffset = {258, 345},},				--197
				{ alignmentOffset = {205, 265},},				--198
				{ alignmentOffset = {200, 160},},				--199
				{ alignmentOffset = {120, 80},},				--200

			},
			sideLevels = {
				{ alignmentOffset = {280, 670},},
				{ alignmentOffset = {35, 250},},
				{ alignmentOffset = {50, 190},},
				{ alignmentOffset = {280, 190},},
			},
			stageSignImage = "Land14_Sign.png",
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 15 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land14-15.png",
			lockedImage = "Gate_Land14-15.png",
			alignmentOffset = {-20,76-10},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0-2, -28+4},
			},
		},

		Land {
			landNumber = 15,
			landName = STRINGS("The Oasis"),
			firstLevelNumber = 201,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {188, 1420},},				--201
				{ alignmentOffset = {195, 1325},},				--202
				{ alignmentOffset = {238, 1210},},				--203
				{ alignmentOffset = {220, 1107},},				--204
				{ alignmentOffset = {199, 1020},},				--205
				{ alignmentOffset = {190, 938},},				--206
				{ alignmentOffset = {160, 847},},				--207
				{ alignmentOffset = {180, 730},},				--208
				{ alignmentOffset = {193, 650},},				--209
				{ alignmentOffset = {228, 580},},				--210
				{ alignmentOffset = {210, 440},},				--211
				{ alignmentOffset = {208, 355},},				--212
				{ alignmentOffset = {205, 265},},				--213
				{ alignmentOffset = {185, 160},},				--214
				{ alignmentOffset = {176, 85},},				--215

			},
			sideLevels = {
				{ alignmentOffset = {120, 1150},},
				{ alignmentOffset = {290, 910},},
				{ alignmentOffset = {80, 570},},
				{ alignmentOffset = {250, 210},},
			},
			stageSignImage = "Land15_Sign.png",
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 16 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land15-16.png",
			lockedImage = "Gate_Land15-16.png",
			alignmentOffset = {0-25,44-5},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -28+5},
			},
		},

		Land {
			landNumber = 16,
			landName = STRINGS("Autumn Forest"),
			firstLevelNumber = 216,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {188, 1430},},				--216
				{ alignmentOffset = {200, 1315},},				--217
				{ alignmentOffset = {170, 1230},},				--218
				{ alignmentOffset = {180, 1140},},				--219
				{ alignmentOffset = {200, 1065},},				--220
				{ alignmentOffset = {255, 930},},				--221
				{ alignmentOffset = {225, 847},},				--222
				{ alignmentOffset = {180, 770},},				--223
				{ alignmentOffset = {263, 650},},				--224
				{ alignmentOffset = {250, 560},},				--225
				{ alignmentOffset = {210, 475},},				--226
				{ alignmentOffset = {188, 395},},				--227
				{ alignmentOffset = {185, 285},},				--228
				{ alignmentOffset = {220, 195},},				--229
				{ alignmentOffset = {206, 105},},				--230

			},
			sideLevels = {
				{ alignmentOffset = {100, 995},},
				{ alignmentOffset = {235, 755},},
				{ alignmentOffset = {90, 247},},
				{ alignmentOffset = {270, 130},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 17 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land16-17.png",
			lockedImage = "Gate_Land16-17.png",
			alignmentOffset = {0-25,44},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-5, -28+5},
			},
		},

		Land {
			landNumber = 17,
			landName = STRINGS("Savannah"),
			firstLevelNumber = 231,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {218, 1410},},				--231
				{ alignmentOffset = {195, 1310},},				--232
				{ alignmentOffset = {240, 1205},},				--233
				{ alignmentOffset = {220, 1110},},				--234
				{ alignmentOffset = {205, 1020},},				--235
				{ alignmentOffset = {175, 930},},				--236
				{ alignmentOffset = {175, 840},},				--237
				{ alignmentOffset = {150, 750},},				--238
				{ alignmentOffset = {203, 670},},				--239
				{ alignmentOffset = {220, 575},},				--240
				{ alignmentOffset = {210, 475},},				--241
				{ alignmentOffset = {238, 385},},				--242
				{ alignmentOffset = {235, 255},},				--243
				{ alignmentOffset = {240, 185},},				--244
				{ alignmentOffset = {276, 115},},				--245

			},
			sideLevels = {
				{ alignmentOffset = {80, 875},},
				{ alignmentOffset = {70, 540},},
                { alignmentOffset = {140, 175},},
                { alignmentOffset = {75, 130},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 18 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land17-18.png",
			lockedImage = "Gate_Land17-18.png",
			alignmentOffset = {0,-4-10},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -18+8},
			},
		},

		Land {
			landNumber = 18,
			landName = STRINGS("Lost Castle"),
			firstLevelNumber = 246,
			numSlices = 6,

			OverlaySlice1 = true,
			OverlaySlice5 = true,
			OverlaySlice6 = true,

			landLevels = {
				{ alignmentOffset = {228, 1330},},				--246
				{ alignmentOffset = {235, 1260},},				--247
				{ alignmentOffset = {260, 1185},},				--248
				{ alignmentOffset = {260, 1110},},				--249
				{ alignmentOffset = {235, 1020},},				--250
				{ alignmentOffset = {245, 930},},				--251
				{ alignmentOffset = {235, 860},},				--252
				{ alignmentOffset = {250, 780},},				--253
				{ alignmentOffset = {210, 700},},				--254
				{ alignmentOffset = {220, 610},},				--255
				{ alignmentOffset = {250, 535},},				--256
				{ alignmentOffset = {238, 455},},				--257
				{ alignmentOffset = {235, 275},},				--258
				{ alignmentOffset = {240, 210},},				--259
				{ alignmentOffset = {246, 125},},				--260

			},
			sideLevels = {
				{ alignmentOffset = {115, 811},},
				{ alignmentOffset = {80, 725},},
                { alignmentOffset = {130, 205},},
                { alignmentOffset = {80, 170},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 19 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land18-19.png",
			lockedImage = "Gate_Land18-19.png",
			alignmentOffset = {0,146},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -30},
			},
		},

		Land {
			landNumber = 19,
			landName = STRINGS("Floating City"),
			firstLevelNumber = 261,
			numSlices = 6,

			OverlaySlice6 = true,

			landLevels = {
				{ alignmentOffset = {168, 1470},},				--261
				{ alignmentOffset = {185, 1300},},				--262
				{ alignmentOffset = {280, 1185},},				--263
				{ alignmentOffset = {275, 1090},},				--264
				{ alignmentOffset = {235, 1020},},				--265
				{ alignmentOffset = {245, 930},},				--266
				{ alignmentOffset = {215, 850},},				--267
				{ alignmentOffset = {190, 750},},				--268
				{ alignmentOffset = {150, 660},},				--269
				{ alignmentOffset = {230, 575},},				--270
				{ alignmentOffset = {240, 480},},				--271
				{ alignmentOffset = {238, 390},},				--272
				{ alignmentOffset = {245, 295},},				--273
				{ alignmentOffset = {240, 210},},				--274
				{ alignmentOffset = {216, 125},},				--275

			},
			sideLevels = {
				{ alignmentOffset = {150, 925},},
                { alignmentOffset = {160, 515},},
                { alignmentOffset = {160, 350},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 20 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land19-20.png",
			lockedImage = "Gate_Land19-20.png",
			alignmentOffset = {0-80,135-6},
			Sockets = {
				class = "Actor",
				alignmentOffset = {5, -48-14},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {0,-8},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {0,-16},
				},
			},
		},

		Land {
			landNumber = 20,
			landName = STRINGS("Red Rock Arches"),
			firstLevelNumber = 276,
			numSlices = 6,

			OverlaySlice1 = true,
			OverlaySlice5 = true,

			landLevels = {
				{ alignmentOffset = {208, 1430},},				--276
				{ alignmentOffset = {245, 1320},},				--277
				{ alignmentOffset = {230, 1225},},				--278
				{ alignmentOffset = {200, 1080},},				--279
				{ alignmentOffset = {235, 1020},},				--280
				{ alignmentOffset = {245, 930},},				--281
				{ alignmentOffset = {235, 860},},				--282
				{ alignmentOffset = {250, 780},},				--283
				{ alignmentOffset = {230, 700},},				--284
				{ alignmentOffset = {220, 600},},				--285
				{ alignmentOffset = {240, 505},},				--286
				{ alignmentOffset = {238, 405},},				--287
				{ alignmentOffset = {215, 245},},				--288
				{ alignmentOffset = {200, 170},},				--289
				{ alignmentOffset = {176, 95},},				--290

			},
			sideLevels = {
				{ alignmentOffset = {115, 765},},
                { alignmentOffset = {125, 460},},
                { alignmentOffset = {105, 150},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 21 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land20-21.png",
			lockedImage = "Gate_Land20-21.png",
			alignmentOffset = {-50,96-4},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -48},
			},
		},

		Land {
			landNumber = 21,
			landName = STRINGS("Great Wall"),
			firstLevelNumber = 291,
			numSlices = 6,

			OverlaySlice1 = true,
			OverlaySlice2 = true,
			OverlaySlice3 = true,
			OverlaySlice4 = true,
			OverlaySlice5 = true,
			OverlaySlice6 = true,

			landLevels = {
				{ alignmentOffset = {185, 1420},},				--291
				{ alignmentOffset = {210, 1350},},				--292
				{ alignmentOffset = {240, 1150},},				--293
				{ alignmentOffset = {285, 1070},},				--294
				{ alignmentOffset = {300, 1010},},				--295
				{ alignmentOffset = {305, 940},},				--296
				{ alignmentOffset = {275, 855},},				--297
				{ alignmentOffset = {235, 790},},				--298
				{ alignmentOffset = {210, 740},},				--299
				{ alignmentOffset = {145, 655},},				--300
				{ alignmentOffset = {148, 578},},				--301
				{ alignmentOffset = {165, 520},},				--302
				{ alignmentOffset = {195, 275},},				--303
				{ alignmentOffset = {180, 210},},				--304
				{ alignmentOffset = {170, 140},},				--305

			},
			sideLevels = {
				{ alignmentOffset = {115, 975},},
                { alignmentOffset = {120, 745},},
                { alignmentOffset = {77, 615},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 22 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land21-22.png",
			lockedImage = "Gate_Land21-22.png",
			alignmentOffset = {-40,125-6},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -6},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {0,-3},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {0,-6},
				},
			},
		},

		Land {
			landNumber = 22,
			landName = STRINGS("Grimm Woods"),
			firstLevelNumber = 306,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {195, 1450},},				--306
				{ alignmentOffset = {210, 1370},},				--307
				{ alignmentOffset = {210, 1180},},				--308
				{ alignmentOffset = {202, 1099},},				--309
				{ alignmentOffset = {267, 1035},},				--310
				{ alignmentOffset = {288, 954},},				--311
				{ alignmentOffset = {249, 878},},				--312
				{ alignmentOffset = {241, 678},},				--313
				{ alignmentOffset = {244, 590},},				--314
				{ alignmentOffset = {230, 523},},				--315
				{ alignmentOffset = {222, 443},},				--316
				{ alignmentOffset = {242, 362},},				--317
				{ alignmentOffset = {220, 272},},				--318
				{ alignmentOffset = {237, 171},},				--319
				{ alignmentOffset = {206, 93},},				--320

			},
			sideLevels = {
				{ alignmentOffset = {100, 855},},
                { alignmentOffset = {150, 559},},
                { alignmentOffset = {95, 520},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 23 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land22-23.png",
			lockedImage = "Gate_Land22-23.png",
			alignmentOffset = {-20,80-10},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-8, -45+10},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {0,-3},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {0,-6},
				},
			},
		},

		Land {
			landNumber = 23,
			landName = STRINGS("English Garden"),
			firstLevelNumber = 321,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {200, 1438},},				--321
				{ alignmentOffset = {216, 1352},},				--322
				{ alignmentOffset = {237, 1249},},				--323
				{ alignmentOffset = {253, 1160},},				--324
				{ alignmentOffset = {196, 983},},				--325
				{ alignmentOffset = {220, 910},},				--326
				{ alignmentOffset = {275, 852},},				--327
				{ alignmentOffset = {260, 766},},				--328
				{ alignmentOffset = {177, 647},},				--329
				{ alignmentOffset = {169, 554},},				--330
				{ alignmentOffset = {186, 475},},				--331
				{ alignmentOffset = {210, 358},},				--332
				{ alignmentOffset = {182, 281},},				--333
				{ alignmentOffset = {163, 197},},				--334
				{ alignmentOffset = {227, 90},},				--335

			},
			sideLevels = {
				{ alignmentOffset = {65, 935},},
                { alignmentOffset = {85, 524},},
                { alignmentOffset = {115, 415},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 24 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land23-24.png",
			lockedImage = "Gate_Land23-24.png",
			alignmentOffset = {1,28},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -25+10},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {0,-1},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {0,-2},
				},
			},
		},

		Land {
			landNumber = 24,
			landName = STRINGS("Northern Lights"),
			firstLevelNumber = 336,
			numSlices = 6,

			OverlaySlice1 = true,
			OverlaySlice6 = true,

			landLevels = {
				{ alignmentOffset = {265, 1425},},				--336
				{ alignmentOffset = {282, 1349},},				--337
				{ alignmentOffset = {243, 1222},},				--338
				{ alignmentOffset = {150, 1159},},				--339
				{ alignmentOffset = {212, 1058},},				--340
				{ alignmentOffset = {266, 949},},				--341
				{ alignmentOffset = {181, 860},},				--342
				{ alignmentOffset = {252, 755},},				--343
				{ alignmentOffset = {184, 652},},				--344
				{ alignmentOffset = {253, 567},},				--345
				{ alignmentOffset = {206, 468},},				--346
				{ alignmentOffset = {237, 351},},				--347
				{ alignmentOffset = {225, 279},},				--348
				{ alignmentOffset = {196, 201},},				--349
				{ alignmentOffset = {240, 121},},				--350
			},
			sideLevels = {
				{ alignmentOffset = {115, 765},},
                { alignmentOffset = {85, 705},},
                { alignmentOffset = {125, 248},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 25 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land24-25.png",
			lockedImage = "Gate_Land24-25.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {51,80},
			Sockets = {
				class = "Actor",
				alignmentOffset = {5, -14},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {-8,-18},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {-4,-9},
				},
			},
		},

		Land {
			landNumber = 25,
			landName = STRINGS("Martian Plains"),
			firstLevelNumber = 351,
			numSlices = 6,

			OverlaySlice1 = true,
			OverlaySlice6 = true,

			landLevels = {
				{ alignmentOffset = {245, 1435},},				--351
				{ alignmentOffset = {252, 1349},},				--352
				{ alignmentOffset = {173, 1282},},				--353
				{ alignmentOffset = {230, 1159},},				--354
				{ alignmentOffset = {212, 1058},},				--355
				{ alignmentOffset = {166, 949},},				--356
				{ alignmentOffset = {281, 860},},				--357
				{ alignmentOffset = {282, 755},},				--358
				{ alignmentOffset = {284, 652},},				--359
				{ alignmentOffset = {253, 567},},				--360
				{ alignmentOffset = {206, 468},},				--361
				{ alignmentOffset = {237, 351},},				--362
				{ alignmentOffset = {265, 279},},				--363
				{ alignmentOffset = {246, 201},},				--364
				{ alignmentOffset = {180, 165},},				--365
			},
			sideLevels = {
				{ alignmentOffset = {80, 1220},},
				{ alignmentOffset = {150, 410},},
				{ alignmentOffset = {120, 340},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 26 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land25-26.png",
			lockedImage = "Gate_Land25-26.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {-12,130},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-12, -60},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {10,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {5,0},
				},
			},
		},

		Land {
			landNumber = 26,
			landName = STRINGS("Ice Age"),
			firstLevelNumber = 366,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {245, 1435},},				--366
				{ alignmentOffset = {252, 1349},},				--367
				{ alignmentOffset = {183, 1282},},				--368
				{ alignmentOffset = {230, 1159},},				--369
				{ alignmentOffset = {182, 1058},},				--370
				{ alignmentOffset = {226, 949},},				--371
				{ alignmentOffset = {281, 860},},				--372
				{ alignmentOffset = {142, 755},},				--373
				{ alignmentOffset = {184, 652},},				--374
				{ alignmentOffset = {213, 567},},				--375
				{ alignmentOffset = {236, 468},},				--376
				{ alignmentOffset = {260, 321},},				--377
				{ alignmentOffset = {255, 249},},				--378
				{ alignmentOffset = {225, 190},},				--379
				{ alignmentOffset = {160, 171},},				--380
			},
			sideLevels = {
				{ alignmentOffset = {120, 1140},},
				{ alignmentOffset = {220, 720},},
				{ alignmentOffset = {175, 352},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 27 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land26-27.png",
			lockedImage = "Gate_Land26-27.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {5,135},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-8, -60},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {14,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {7,0},
				},
			},
		},

		Land {
			landNumber = 27,
			landName = STRINGS("Sky Castle"),
			firstLevelNumber = 381,
			numSlices = 6,

			OverlaySlice1 = true,

			landLevels = {
				{ alignmentOffset = {245, 1430},},				--381
				{ alignmentOffset = {252, 1359},},				--382
				{ alignmentOffset = {193, 1292},},				--383
				{ alignmentOffset = {160, 1219},},				--384
				{ alignmentOffset = {212, 1058},},				--385
				{ alignmentOffset = {276, 979},},				--386
				{ alignmentOffset = {251, 890},},				--387
				{ alignmentOffset = {150, 712},},				--388
				{ alignmentOffset = {204, 652},},				--389
				{ alignmentOffset = {253, 587},},				--390
				{ alignmentOffset = {246, 518},},				--391
				{ alignmentOffset = {229, 457},},				--392
				{ alignmentOffset = {192, 387},},				--393
				{ alignmentOffset = {205, 295},},				--394
				{ alignmentOffset = {220, 170},},				--395
			},
			sideLevels = {
				{ alignmentOffset = {82, 1300},},
				{ alignmentOffset = {110, 1000},},
				{ alignmentOffset = {109, 137},},
			},
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 28 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land27-28.png",
			lockedImage = "Gate_Land27-28.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {12,93},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-8, -20},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {14,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {7,0},
				},
			},
		},

		Land {
			landNumber = 28,
			landName = STRINGS("Cascade Resort"),
			firstLevelNumber = 396,
			numSlices = 10,
			OverlaySlice1 = true,
			OverlaySlice10 = true,

			landLevels = {
				{ alignmentOffset = {220, 2425},},				--396
				{ alignmentOffset = {252, 2359},},				--397
				{ alignmentOffset = {293, 2292},},				--398
				{ alignmentOffset = {260, 2200},},				--399
				{ alignmentOffset = {150, 2100},},				--400
				{ alignmentOffset = {226, 1969},},				--401
				{ alignmentOffset = {311, 1865},},				--402
				{ alignmentOffset = {232, 1800},},				--403
				{ alignmentOffset = {137, 1742},},				--404
				{ alignmentOffset = {253, 1650},},				--405
				{ alignmentOffset = {253, 1550},},				--406
				{ alignmentOffset = {185, 1459},},				--407
				{ alignmentOffset = {282, 1322},},				--408
				{ alignmentOffset = {176, 1219},},				--409
				{ alignmentOffset = {100, 1170},},				--410
				{ alignmentOffset = {266, 1079},},				--411
				{ alignmentOffset = {101, 960},},				--412
				{ alignmentOffset = {210, 902},},				--413
				{ alignmentOffset = {194, 732},},				--414
				{ alignmentOffset = {243, 667},},				--415
				{ alignmentOffset = {236, 578},},				--416
				{ alignmentOffset = {219, 457},},				--417
				{ alignmentOffset = {182, 335},},				--418
				{ alignmentOffset = {226, 250},},				--419
				{ alignmentOffset = {190, 171},},				--420
			},
			sideLevels = {
				{ alignmentOffset = {152, 2310},},
				{ alignmentOffset = {275, 2060},},
				{ alignmentOffset = {130, 1600},},
				{ alignmentOffset = {93, 668},},
				{ alignmentOffset = {109, 500},},
			},
			signTextNotVisible = false,
		},

	})
end

if TotalLands >= 29 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land28-29.png",
			lockedImage = "Gate_Land28-29.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {-3,123},
			Sockets = {
				class = "Actor",
				alignmentOffset = {0, -22},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {0,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {0,0},
				},
			},
		},

		Land {
			landNumber = 29,
			landName = STRINGS("Klondike Mine"),
			firstLevelNumber = 421,
			numSlices = 10,
			OverlaySlice1 = true,
			OverlaySlice8 = true,

			landLevels = {
				{ alignmentOffset = {230,2469},},               --421
				{ alignmentOffset = {214,2394},},               --422
				{ alignmentOffset = {166,2331},},               --423
				{ alignmentOffset = {117,2228},},               --424
				{ alignmentOffset = {210,2148},},               --425
				{ alignmentOffset = {177,2030},},               --426
				{ alignmentOffset = {121,1923},},               --427
				{ alignmentOffset = {84,1839},},                --428
				{ alignmentOffset = {127,1761},},               --429
				{ alignmentOffset = {194,1670},},               --430
				{ alignmentOffset = {235,1583},},               --431
				{ alignmentOffset = {196,1490},},               --432
				{ alignmentOffset = {135,1421},},               --433
				{ alignmentOffset = {207,1332},},               --434
				{ alignmentOffset = {143,1216},},				--435
				{ alignmentOffset = {130, 1124},},				--436
				{ alignmentOffset = {208,1029},}, 	            --437
				{ alignmentOffset = {234,970},}, 	            --438
				{ alignmentOffset = {208,865},}, 	            --439
				{ alignmentOffset = {227, 749},}, 	            --440
				{ alignmentOffset = {156,554},},				--441
				{ alignmentOffset = {210,468},},				--442
				{ alignmentOffset = {211,338},},				--443
				{ alignmentOffset = {146,235},},				--444
				{ alignmentOffset = {173,127},},				--445
			},
            sideLevels = {
                { alignmentOffset = {222,2259},},
                { alignmentOffset = {212,1920},},
                { alignmentOffset = {292,1256},},
                { alignmentOffset = {86,687},},  
                { alignmentOffset = {258,201},}, 
            },
			
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 30 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land29-30.png",
			lockedImage = "Gate_Land29-30.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {0,70},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-16, -24},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {4,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {2,0},
				},
			},
		},

		Land {
			landNumber = 30,
			landName = STRINGS("Adventure Dungeon"),
			firstLevelNumber = 446,
			numSlices = 10,

			OverlaySlice2 = true,
			OverlaySlice10 = true,

			landLevels = {
				{ alignmentOffset = {x=248, y=2460},},				--446
				{ alignmentOffset = {x=220, y=2384},},				--447
				{ alignmentOffset = {x=195, y=2310},},				--448
				{ alignmentOffset = {x=275, y=2032},},				--449
				{ alignmentOffset = {x=190, y=1982},},				--450
				{ alignmentOffset = {x=109, y=1936},},				--451
				{ alignmentOffset = {x=187, y=1889},},				--452
				{ alignmentOffset = {x=237, y=1837},},				--453
				{ alignmentOffset = {x=226, y=1767},},				--454
				{ alignmentOffset = {x=194, y=1705},},				--455
				{ alignmentOffset = {x=299, y=1475},},				--456
				{ alignmentOffset = {x=225, y=1429},},				--457
				{ alignmentOffset = {x=269, y=1345},},				--458
				{ alignmentOffset = {x=243, y=1236},},				--459
				{ alignmentOffset = {x=162, y=1176},},				--460
				{ alignmentOffset = {x=253, y=1088},},				--461
				{ alignmentOffset = {x=239, y=1015},},				--462
				{ alignmentOffset = {x=229, y=945},},				--463
				{ alignmentOffset = {x=222, y=857},},				--464
				{ alignmentOffset = {x=200, y=725},},				--465
				{ alignmentOffset = {x=257, y=550},},				--466
				{ alignmentOffset = {x=291, y=468},},				--467
				{ alignmentOffset = {x=300, y=364},},				--468
				{ alignmentOffset = {x=281, y=290},},				--469
				{ alignmentOffset = {x=229, y=241},},				--470
			},
			sideLevels = {
                { alignmentOffset = {61,2029},},
                { alignmentOffset = {85,1816},},
                { alignmentOffset = {106,1011},},
                { alignmentOffset = {86,787},},  
                { alignmentOffset = {88,494},}, 
            },
			
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 31 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land30-31.png",
			lockedImage = "Gate_Land30-31.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {0,210},
			Sockets = {
				class = "Actor",
				alignmentOffset = {4, -24},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {-4,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {-2,0},
				},
			},
		},

		Land {
			landNumber = 31,
			landName = STRINGS("Valley of the Kings"),
			firstLevelNumber = 471,
			numSlices = 10,

			OverlaySlice4 = true,
			OverlaySlice5 = true,
			OverlaySlice7 = true,
			OverlaySlice10 = true,

			landLevels = {
				{ alignmentOffset = {x=224, y=2471},},				--471
				{ alignmentOffset = {x=201, y=2352},},				--472
				{ alignmentOffset = {x=147, y=2249},},				--473
				{ alignmentOffset = {x=184, y=2136},},				--474
				{ alignmentOffset = {x=127, y=2029},},				--475
				{ alignmentOffset = {x=154, y=1909},},				--476
				{ alignmentOffset = {x=205, y=1823},},				--477
				{ alignmentOffset = {x=206, y=1716},},				--478
				{ alignmentOffset = {x=175, y=1643},},				--479
				{ alignmentOffset = {x=161, y=1440},},				--480
				{ alignmentOffset = {x=107, y=1376},},				--481
				{ alignmentOffset = {x=181, y=1278},},				--482
				{ alignmentOffset = {x=268, y=1200},},				--483
				{ alignmentOffset = {x=246, y=1121},},				--484
				{ alignmentOffset = {x=197, y=1039},},				--485
				{ alignmentOffset = {x=136, y=831},},				--486
				{ alignmentOffset = {x=176, y=747},},				--487
				{ alignmentOffset = {x=215, y=648},},				--488
				{ alignmentOffset = {x=171, y=558},},				--489
				{ alignmentOffset = {x=130, y=465},},				--490
				{ alignmentOffset = {x=147, y=375},},				--491
				{ alignmentOffset = {x=193, y=286},},				--492
				{ alignmentOffset = {x=225, y=192},},				--493
				{ alignmentOffset = {x=137, y=152},},				--494
				{ alignmentOffset = {x=156, y=71},},				--495
			},
            sideLevels = {
                { alignmentOffset = {x=237, y=2009},},
                { alignmentOffset = {x=78, y=1744},},
                { alignmentOffset = {x=212, y=1348},},
                { alignmentOffset = {x=96, y=1161},},
                { alignmentOffset = {x=250, y=499},},
            },
			
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 32 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land31-32.png",
			lockedImage = "Gate_Land31-32.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {0,68},
			Sockets = {
				class = "Actor",
				alignmentOffset = {2, -21},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {8,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {4,0},
				},
			},
		},

		Land {
			landNumber = 32,
			landName = STRINGS("Whimsy World"),
			firstLevelNumber = 496,
			numSlices = 10,

			OverlaySlice1 = true,
			OverlaySlice10 = true,

			landLevels = {
				{ alignmentOffset = {x=224, y=2471},},				--496
				{ alignmentOffset = {x=238, y=2381},},				--497
				{ alignmentOffset = {x=272, y=2285},},				--498
				{ alignmentOffset = {x=239, y=2205},},				--499
				{ alignmentOffset = {x=175, y=2096},},				--500
				{ alignmentOffset = {x=276, y=2022},},				--501
				{ alignmentOffset = {x=201, y=1928},},				--502
				{ alignmentOffset = {x=225, y=1829},},				--503
				{ alignmentOffset = {x=100, y=1767},},				--504
				{ alignmentOffset = {x=263, y=1699},},				--505
				{ alignmentOffset = {x=291, y=1585},},				--506
				{ alignmentOffset = {x=299, y=1434},},				--507
				{ alignmentOffset = {x=294, y=1287},},				--508
				{ alignmentOffset = {x=204, y=1251},},				--509
				{ alignmentOffset = {x=281, y=1162},},				--510
				{ alignmentOffset = {x=252, y=1014},},				--511
				{ alignmentOffset = {x=151, y=999},},				--512
				{ alignmentOffset = {x=216, y=848},},				--513
				{ alignmentOffset = {x=244, y=744},},				--514
				{ alignmentOffset = {x=215, y=648},},				--515
				{ alignmentOffset = {x=242, y=539},},				--516
				{ alignmentOffset = {x=251, y=382},},				--517
				{ alignmentOffset = {x=297, y=324},},				--518
				{ alignmentOffset = {x=285, y=256},},				--519
				{ alignmentOffset = {x=225, y=207},},				--520
			},

			sideLevels = {
                { alignmentOffset = {x=81, y=2230},},
				{ alignmentOffset = {x=145, y=1520},},	
				{ alignmentOffset = {x=99, y=690},},
				{ alignmentOffset = {x=98, y=326},},
				{ alignmentOffset = {x=99, y=253},},
            },
			
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 33 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land32-33.png",
			lockedImage = "Gate_Land32-33.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {12,185},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-10, -48},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {8,16},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {4,8},
				},
			},
		},

		Land {
			landNumber = 33,
			landName = STRINGS("Casino Heist"),
			firstLevelNumber = 521,
			numSlices = 10,

			OverlaySlice3 = true,

			landLevels = {
				{ alignmentOffset = {x=274, y=2479},},				--521
				{ alignmentOffset = {x=231, y=2399},},				--522
				{ alignmentOffset = {x=255, y=2316},},				--523
				{ alignmentOffset = {x=155, y=2285},},				--524
				{ alignmentOffset = {x=108, y=2219},},				--525
				{ alignmentOffset = {x=97, y=2095},},				--526
				{ alignmentOffset = {x=214, y=2013},},				--527
				{ alignmentOffset = {x=266, y=1837},},				--528
				{ alignmentOffset = {x=229, y=1745},},				--529
				{ alignmentOffset = {x=288, y=1591},},				--530
				{ alignmentOffset = {x=213, y=1505},},				--531
				{ alignmentOffset = {x=180, y=1387},},				--532
				{ alignmentOffset = {x=215, y=1230},},				--533
				{ alignmentOffset = {x=180, y=1070},},				--534
				{ alignmentOffset = {x=252, y=990},},				--535
				{ alignmentOffset = {x=161, y=954},},				--536
				{ alignmentOffset = {x=226, y=846},},				--537
				{ alignmentOffset = {x=233, y=751},},				--538
				{ alignmentOffset = {x=223, y=653},},				--539
				{ alignmentOffset = {x=249, y=548},},				--540
				{ alignmentOffset = {x=204, y=470},},				--541
				{ alignmentOffset = {x=195, y=364},},				--542
				{ alignmentOffset = {x=230, y=271},},				--543
				{ alignmentOffset = {x=275, y=201},},				--544
				{ alignmentOffset = {x=225, y=143},},				--545
			},

			sideLevels = {
                { alignmentOffset = {x=295, y=2238},},
				{ alignmentOffset = {x=280, y=2152},},	
				{ alignmentOffset = {x=118, y=1656},},
				{ alignmentOffset = {x=122, y=584},},
				{ alignmentOffset = {x=132, y=230},},
            },
			
			signTextNotVisible = false,
		}
	})
end

if TotalLands >= 34 then
	table.insertall(Lands, {
		Gate {
			image = "Gate_Land33-34.png",
			lockedImage = "Gate_Land33-34.png",
			layer = 5, -- Need to set this in order to make door appear over inside of spaceship - not sure why it was showing through
			alignmentOffset = {0,112},
			Sockets = {
				class = "Actor",
				alignmentOffset = {-10, -45},
				Socket_Green = {
					class = "Actor",
				},
				Socket_Red = {
					class = "Actor",
					alignmentOffset = {18,0},
				},
				Socket_Blue = {
					class = "Actor",
					alignmentOffset = {9,0},
				},
			},
		},

		Land {
			landNumber = 34,
			landName = STRINGS(""),
			firstLevelNumber = 546,
			numSlices = 10,

			landLevels = {
				{ alignmentOffset = {x=217, y=2479},},				--546
				{ alignmentOffset = {x=244, y=2356},},				--547
				{ alignmentOffset = {x=247, y=2289},},				--548
				{ alignmentOffset = {x=191, y=2233},},				--549
				{ alignmentOffset = {x=147, y=2164},},				--550
				{ alignmentOffset = {x=189, y=2086},},				--551
				{ alignmentOffset = {x=214, y=2013},},				--552
				{ alignmentOffset = {x=291, y=1943},},				--553
				{ alignmentOffset = {x=308, y=1837},},				--554
				{ alignmentOffset = {x=300, y=1737},},				--555
				{ alignmentOffset = {x=233, y=1608},},				--556
				{ alignmentOffset = {x=258, y=1483},},				--557
				{ alignmentOffset = {x=252, y=1332},},				--558
				{ alignmentOffset = {x=215, y=1260},},				--559
				{ alignmentOffset = {x=225, y=1104},},				--560
				{ alignmentOffset = {x=252, y=997},},				--561
				{ alignmentOffset = {x=161, y=954},},				--562
				{ alignmentOffset = {x=233, y=751},},				--563
				{ alignmentOffset = {x=223, y=653},},				--564
				{ alignmentOffset = {x=249, y=548},},				--565
				{ alignmentOffset = {x=204, y=470},},				--566
				{ alignmentOffset = {x=195, y=364},},				--567
				{ alignmentOffset = {x=230, y=271},},				--568
				{ alignmentOffset = {x=275, y=201},},				--569
				{ alignmentOffset = {x=225, y=133},},				--570
			},

			sideLevels = {
                { alignmentOffset = {x=97, y=2031},},
				{ alignmentOffset = {x=142, y=1923},},	
				{ alignmentOffset = {x=130, y=1526},},
				{ alignmentOffset = {x=131, y=1378},},
				{ alignmentOffset = {x=116, y=590},},
            },

			signTextNotVisible = false,
		}
	})
end

function Lands:Compile()
	local lastLandNumber
	local maxElements = #self

	-- Apply 'startGate' actor resources to each land.  This should cause each
	-- gate to appear on-screen, inside the land that the gate opens:
	for i = 1, maxElements do
		if self[i].isLand then
			local land = self[i]
			local prevGateElementIndex = Lands:GetPreviousGateIndexForLandNumber(land.landNumber)
			local prevGate = self[prevGateElementIndex]
			if prevGate then
				land.startGate = prevGate
			end
		end
	end

	-- Tag gates with land numbers.
	-- These are used later on to help find land actors, via TAG_MGR.
	for i = 1, maxElements do
		if self[i].isGate then
			local landNumber = self:GetLandNumberAfterGate(i)
			if IsNumber(landNumber) then
				self[i].tag = "GateToLand" .. tostring(landNumber)
				self[i].gateToLandNumber = landNumber
			end
		end
	end
end

function Lands:CountLands()
	local count = 0;
	local maxElements = #self
	for i = 1, maxElements do
		local element = self[i]
		if element.isLand then
			count = count + 1
		end
	end
	return count
end

function Lands:GetFirstLevelNumberForLandNumber(landNumber)
	local land, elementIndex = self:GetLand(landNumber)
	if land then
		return land.firstLevelNumber or 0
	else
		return 0
	end
end

function Lands:GetLand(landNumber)
	assert(IsNumber(landNumber), string.format("invalid input: type(landNumber) == <%s>", type(landNumber)))

	local maxElements = #self
	for i = 1, maxElements do
		if self[i].isLand then
			landNumber = landNumber - 1
			if landNumber == 0 then
				return self[i], i
			end
		end
	end
	logprint("ERROR: There is no land for land number: " .. landNumber)
end

function Lands:GetLandNumberAfterGate(elementIndex)
	assert(IsNumber(elementIndex), string.format("invalid input: type(elementIndex) == <%s>", type(elementIndex)))
	assert(elementIndex >= 1, string.format("invalid input: 'elementIndex' was too small; elementIndex == %s", tostring(elementIndex)))
	assert(elementIndex <= #self, string.format("invalid input: 'elementIndex' was too big; elementIndex == %s; #self == %d", tostring(elementIndex), tostring(#self)))
	assert(self[elementIndex].isGate, string.format("invalid input: elementIndex == %s", tostring(elementIndex)))

	local maxElements = #self
	for i = elementIndex, maxElements do
		local element = self[i]
		if element.isLand then
			return element.landNumber
		end
	end
end

function Lands:GetPreviousGateIndexForLandNumber(landNumber)
	assert(IsNumber(landNumber), string.format("invalid input: type(landNumber) == <%s>", type(landNumber)))

	local land, elementIndex = self:GetLand(landNumber)
	assert(land, string.format("invalid input: no land exists at landIndex, %d", landNumber))

	for i = elementIndex, 1, -1 do
		if self[i].isGate then
			return i
		end
	end

	return 0
end



-- HACK: declare global functions for easy C++ access to 'Lands'
function Lands_CountLands() return Lands:CountLands() end
function Lands_GetFirstLevelNumberForLandNumber(landNumber) return Lands:GetFirstLevelNumberForLandNumber(landNumber) end
function Lands_GetLandNumberAfterGate(elementIndex)	return Lands:GetLandNumberAfterGate(elementIndex) end
function Lands_GetPreviousGateIndexForLandNumber(landNumber) return Lands:GetPreviousGateIndexForLandNumber(landNumber) end



Lands:Compile()		-- This call should only be made once all of 'Lands', and its methods, get declared!



Resources.PathLocation = Resources.Actor:new {
	resource = "Actor",
	alignment = "TopLeft",
	alignTo = "MidBottom",
	image = "StarSmall.png",
	zorder = 100000,
}

local function PathLocations(pathOffsets)
	local returnResource = Resources.Actor:new {
		resource = "Actor",
		alignment = "MidBottom",
		alignTo = "MidBottom",
		zorder = 10000,
	}
	for index = 1, #pathOffsets do
		local pathLocation = Resources.PathLocation:new {
			resource = "PathLocation",
			alignmentOffset = pathOffsets[index],
		}
		returnResource["location"] = pathLocation
	end
	return returnResource
end

local global_fractionForegroundScrolls = 1.25;
local landForegroundHeight = 1536*global_fractionForegroundScrolls
local LandMapForeground_Adjust_left = Layout{-50,ipad=-50,iphone5=-25,iphone4=-25}
local LandMapForeground_Adjust_right = Layout{50,ipad=50,iphone5=25,iphone4=25}

Resources.SkyLeft = Resources.Actor:new{
	resource = "Actor",
	alignment = "BottomLeft",
	alignTo = "BottomLeft",
	layer = 4,
}

Resources.SkyRight = Resources.Actor:new{
	resource = "Actor",
	alignment = "BottomRight",
	alignTo = "BottomRight",
	layer = 4,
}

function StandardForeground(landNumber)
	return Resources.Actor:new {
		resource = "Actor",
		size = {768/2,landForegroundHeight},
		zorder = 50,
		alignmentOffset = {0, totalHeight*global_fractionForegroundScrolls - landForegroundHeight * (landNumber - 1)},
		alignment = "MidBottom",
		alignTo = "MidBottom",
		rolloverable = false,
		SkyLeft_1 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land1_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-250}, },
		SkyLeft_2 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land1_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-600}, },
		SkyLeft_3 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land1_SkyLeft3.png",alignmentOffset = {LandMapForeground_Adjust_left,-950}, },
		SkyLeft_4 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land1_SkyLeft4.png",alignmentOffset = {LandMapForeground_Adjust_left,-1200}, },
		SkyLeft_5 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land1_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-1550}, },
		SkyLeft_6 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land1_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-1700}, },
		SkyRight_1 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land1_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-250}, },
		SkyRight_2 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land1_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-600}, },
		SkyRight_3 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land1_SkyRight3.png",alignmentOffset = {LandMapForeground_Adjust_right,-950}, },
		SkyRight_4 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land1_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-1200}, },
		SkyRight_5 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land1_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-1550}, },
		SkyRight_6 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land1_SkyRight3.png",alignmentOffset = {LandMapForeground_Adjust_right,-1700}, },
	}
end

Resources.LandMapForegrounds = Resources.Actor:new {
	resource = "Actor",
	zorder = 50,
	alignment = "MidTop",
	alignTo = "MidTop",
	alignmentOffset = {0, 65},
	Foreground1 = StandardForeground(1),
	Foreground2 = StandardForeground(2),
	Foreground3 = StandardForeground(3),
	Foreground4 = Resources.Actor:new {
		resource = "Actor",
		--image = "Land4_MapBackground_foreground.png",
		size = {768/2,landForegroundHeight},
		zorder = 50,
		alignmentOffset = {0, totalHeight*global_fractionForegroundScrolls - landForegroundHeight*3},
		alignment = "MidBottom",
		alignTo = "MidBottom",
		rolloverable = false,

		SkyLeft_1 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land4_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-250}, },
		SkyLeft_2 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land4_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-500}, },
		SkyLeft_3 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land4_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-750}, },
		SkyLeft_4 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land4_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-1000}, },
		SkyLeft_5 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land4_SkyLeft3.png",alignmentOffset = {LandMapForeground_Adjust_left,-1250}, },
		SkyLeft_6 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land4_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-1500}, },
		SkyRight_1 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land4_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-250}, },
		SkyRight_2 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land4_SkyRight3.png",alignmentOffset = {LandMapForeground_Adjust_right,-500}, },
		SkyRight_3 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land4_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-750}, },
		SkyRight_4 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land4_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-1000}, },
		SkyRight_5 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land4_SkyRight3.png",alignmentOffset = {LandMapForeground_Adjust_right,-1250}, },
		SkyRight_6 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land4_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-1500}, },
	},
	Foreground5 = Resources.Actor:new {
		resource = "Actor",
		--image = "Land5_MapBackground_foreground.png",
		size = {768/2,landForegroundHeight},
		zorder = 50,
		alignmentOffset = {0, totalHeight*global_fractionForegroundScrolls - landForegroundHeight*4},
		alignment = "MidBottom",
		alignTo = "MidBottom",
		rolloverable = false,

		SkyLeft_1 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land5_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-250}, },
		SkyLeft_2 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land5_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-500}, },
		SkyLeft_3 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land5_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-750}, },
		SkyLeft_4 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land5_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-1000}, },
		SkyLeft_5 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land5_SkyLeft3.png",alignmentOffset = {LandMapForeground_Adjust_left,-1250}, },
		SkyLeft_6 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage = "Land5_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-1500}, },
		SkyRight_1 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land5_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-350}, },
		SkyRight_2 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land5_SkyRight3.png",alignmentOffset = {LandMapForeground_Adjust_right,-500}, },
		SkyRight_3 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land5_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-750}, },
		SkyRight_4 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land5_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-1000}, },
		SkyRight_5 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land5_SkyRight3.png",alignmentOffset = {LandMapForeground_Adjust_right,-1250}, },
		SkyRight_6 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land5_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-1500}, },
	},
	Foreground6 = StandardForeground(6),
	Foreground7 = StandardForeground(7),
	Foreground8 = StandardForeground(8),
	Foreground9 = StandardForeground(9),
	Foreground10 = Resources.Actor:new {
		resource = "Actor",
		size = {768/2,landForegroundHeight},
		zorder = 50,
		rolloverable = false,
		alignment = "MidBottom",
		alignTo = "MidBottom",
		alignmentOffset = {0, totalHeight*global_fractionForegroundScrolls - landForegroundHeight*9},

		SkyLeft_1 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land10_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-250-350}, },
		SkyLeft_2 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land10_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-600-200}, },
		SkyLeft_3 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land10_SkyLeft3.png",alignmentOffset = {LandMapForeground_Adjust_left,-950-200}, },
		SkyLeft_4 = Resources.SkyLeft:new{resource = "SkyLeft", desiredImage	= "Land10_SkyLeft4.png",alignmentOffset = {LandMapForeground_Adjust_left,-1200-300}, },
		SkyLeft_5 = Resources.SkyLeft:new{resource = "SkyLeft", visible = false, desiredImage	= "Land10_SkyLeft1.png",alignmentOffset = {LandMapForeground_Adjust_left,-250-300}, },
		SkyLeft_6 = Resources.SkyLeft:new{resource = "SkyLeft",visible = false, desiredImage	= "Land10_SkyLeft2.png",alignmentOffset = {LandMapForeground_Adjust_left,-600-200}, },
		SkyRight_1 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land10_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-250-350}, },
		SkyRight_2 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land10_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-600-300}, },
		SkyRight_3 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land10_SkyRight3.png",alignmentOffset = {LandMapForeground_Adjust_right,-950-100}, },
		SkyRight_4 = Resources.SkyRight:new{resource = "SkyRight", desiredImage = "Land10_SkyRight4.png",alignmentOffset = {LandMapForeground_Adjust_right,-1200-400}, },
		SkyRight_5 = Resources.SkyRight:new{resource = "SkyRight",visible = false, desiredImage = "Land10_SkyRight1.png",alignmentOffset = {LandMapForeground_Adjust_right,-250-300}, },
		SkyRight_6 = Resources.SkyRight:new{resource = "SkyRight",visible = false, desiredImage = "Land10_SkyRight2.png",alignmentOffset = {LandMapForeground_Adjust_right,-600-200}, },
	},
	Foreground11 = StandardForeground(11),
}

if TotalLands >= 12 then
	Resources.LandMapForegrounds.Foreground12 = StandardForeground(12)
end

if TotalLands >= 13 then
	Resources.LandMapForegrounds.Foreground13 = StandardForeground(13)
end

if TotalLands >= 14 then
	Resources.LandMapForegrounds.Foreground14 = StandardForeground(14)
end

if TotalLands >= 15 then
	Resources.LandMapForegrounds.Foreground15 = StandardForeground(15)
end

if TotalLands >= 16 then
	Resources.LandMapForegrounds.Foreground16 = StandardForeground(16)
end

if TotalLands >= 17 then
	Resources.LandMapForegrounds.Foreground17 = StandardForeground(17)
end

if TotalLands >= 18 then
	Resources.LandMapForegrounds.Foreground18 = Resources.Actor:new {
		-- no foreground in land 18
	}
end

if TotalLands >= 19 then
	Resources.LandMapForegrounds.Foreground19 = StandardForeground(19)
end

if TotalLands >= 20 then
	Resources.LandMapForegrounds.Foreground20 = StandardForeground(20)
end

if TotalLands >= 21 then
	Resources.LandMapForegrounds.Foreground21 = StandardForeground(21)
end

if TotalLands >= 22 then
	Resources.LandMapForegrounds.Foreground22 = StandardForeground(22)
end

if TotalLands >= 23 then
	Resources.LandMapForegrounds.Foreground23 = StandardForeground(23)
end

if TotalLands >= 24 then
	Resources.LandMapForegrounds.Foreground24 = StandardForeground(24)
end

if TotalLands >= 25 then
	Resources.LandMapForegrounds.Foreground25 = Resources.Actor:new {
		-- no foreground in land 25
	}
end

if TotalLands >= 26 then
	Resources.LandMapForegrounds.Foreground26 = StandardForeground(26)
end

if TotalLands >= 27 then
	Resources.LandMapForegrounds.Foreground27 = StandardForeground(27)
end

if TotalLands >= 28 then
	Resources.LandMapForegrounds.Foreground28 = StandardForeground(28)
end

if TotalLands >= 29 then
	Resources.LandMapForegrounds.Foreground29 = StandardForeground(29)
end

if TotalLands >= 30 then
	Resources.LandMapForegrounds.Foreground30 = StandardForeground(30)
end

if TotalLands >= 31 then
	Resources.LandMapForegrounds.Foreground31 = StandardForeground(31)
end

if TotalLands >= 32 then
	Resources.LandMapForegrounds.Foreground32 = StandardForeground(32)
end

if TotalLands >= 33 then
	Resources.LandMapForegrounds.Foreground33 = StandardForeground(33)
end

if TotalLands >= 34 then
	Resources.LandMapForegrounds.Foreground34 = StandardForeground(34)
end

Resources.LandMapScrollableBackground = Resources.Actor:new{
	resource = "Actor",
	alignment = "MidTop",
	alignTo = "MidTop",

	ComingSoon1 = Resources.Actor:new {
		resource = "Actor",
		alignment = "MidTop",
		alignTo = "MidTop",
		alignmentOffset = {0, -256},
		image = "Land_ComingSoon.jpg",
	},
	
	DownloadMarker = Resources.Actor:new {
		resource = "Actor",
		image = "Land_DownloadNow.png",
		layer = 6,
		RolloverBlocker = Resources.Actor:new {
			resource = "Actor",
			autoFillParent = true,
			alignment = Center,
			alignmentOffset = {0,-77},
			alpha = 0,
			color = "Red",
			rolloverable = true,
		},
		visible = false,
		alpha = 255,
		DownloadButton = Resources.EndGameDialogBigButton:new {
			resource = "EndGameDialogBigButton",
            scripts = {"DialogButtonGroove"},
			image = "ButtonPlastic_Long_Green.png",
			shortcutKey = "d",
			clickSound = "coin_magic_05.ogg",
			alignment = Center,
			alignmentOffset = {0, 0},
			text = STRINGS("Download Now"),
            font = Resources.MapButtonFont:new{
                resource = "MapButtonFont",
                fontSize = 22,
            },	
			zorder = 2,
			MoleGuyTarget = Resources.Actor:new {
				resource = "Actor",
				visible = false,
				alignment = MidRight,
				alignTo = MidLeft,
				alignmentOffset = {-35, 0},			
                zorder = 3,
			},
		},
	},

	hgeParticleLayer = {
		class = "hgeParticleLayer",
		alignment = "MidTop",
		alignTo = "MidTop",
		zorder = 10000,
		tag = "MapParticleLayer"
	},

	dirtPath = Resources.Actor:new {
		resource = "Actor",
		zorder = 1,
	},
	
	Land1Levels = Lands:GetLand(1),
	Land2Levels = Lands:GetLand(2),
	Land3Levels = Lands:GetLand(3),
	Land4Levels = Lands:GetLand(4),
	Land5Levels = Lands:GetLand(5),
	Land6Levels = Lands:GetLand(6),
	Land7Levels = Lands:GetLand(7),
	Land8Levels = Lands:GetLand(8),
	Land9Levels = Lands:GetLand(9),
	Land10Levels = Lands:GetLand(10),
}

if TotalLands >= 11 then
	Resources.LandMapScrollableBackground.Land11Levels = Lands:GetLand(11)
end

if TotalLands >= 12 then
	Resources.LandMapScrollableBackground.Land12Levels = Lands:GetLand(12)
end

if TotalLands >= 13 then
	Resources.LandMapScrollableBackground.Land13Levels = Lands:GetLand(13)
end

if TotalLands >= 14 then
	Resources.LandMapScrollableBackground.Land14Levels = Lands:GetLand(14)
end

if TotalLands >= 15 then
	Resources.LandMapScrollableBackground.Land15Levels = Lands:GetLand(15)
end

if TotalLands >= 16 then
	Resources.LandMapScrollableBackground.Land16Levels = Lands:GetLand(16)
end

if TotalLands >= 17 then
	Resources.LandMapScrollableBackground.Land17Levels = Lands:GetLand(17)
end

if TotalLands >= 18 then
	Resources.LandMapScrollableBackground.Land18Levels = Lands:GetLand(18)
end

if TotalLands >= 19 then
	Resources.LandMapScrollableBackground.Land19Levels = Lands:GetLand(19)
end

if TotalLands >= 20 then
	Resources.LandMapScrollableBackground.Land20Levels = Lands:GetLand(20)
end

if TotalLands >= 21 then
	Resources.LandMapScrollableBackground.Land21Levels = Lands:GetLand(21)
end

if TotalLands >= 22 then
	Resources.LandMapScrollableBackground.Land22Levels = Lands:GetLand(22)
end

if TotalLands >= 23 then
	Resources.LandMapScrollableBackground.Land23Levels = Lands:GetLand(23)
end

if TotalLands >= 24 then
	Resources.LandMapScrollableBackground.Land24Levels = Lands:GetLand(24)
end

if TotalLands >= 25 then
	Resources.LandMapScrollableBackground.Land25Levels = Lands:GetLand(25)
end

if TotalLands >= 26 then
	Resources.LandMapScrollableBackground.Land26Levels = Lands:GetLand(26)
end

if TotalLands >= 27 then
	Resources.LandMapScrollableBackground.Land27Levels = Lands:GetLand(27)
end

if TotalLands >= 28 then
	Resources.LandMapScrollableBackground.Land28Levels = Lands:GetLand(28)
end

if TotalLands >= 29 then
	Resources.LandMapScrollableBackground.Land29Levels = Lands:GetLand(29)
end

if TotalLands >= 30 then
	Resources.LandMapScrollableBackground.Land30Levels = Lands:GetLand(30)
end

if TotalLands >= 31 then
	Resources.LandMapScrollableBackground.Land31Levels = Lands:GetLand(31)
end

if TotalLands >= 32 then
	Resources.LandMapScrollableBackground.Land32Levels = Lands:GetLand(32)
end

if TotalLands >= 33 then
	Resources.LandMapScrollableBackground.Land33Levels = Lands:GetLand(33)
end

if TotalLands >= 34 then
	Resources.LandMapScrollableBackground.Land34Levels = Lands:GetLand(34)
end

Resources.LandMap = Resources.Actor:new{
	class = "LandMap",
	alignment = "MidTop",
	alignTo = "MidTop",

	upperLoadingLayer = Resources.LoadingLayer:new {
		resource = "LoadingLayer",
		alignment = MidTop,
		alignmentOffset = {0,20+50},
		size = {200,100},
		alpha = 0,
		color = "Red",
	},
	lowerLoadingLayer = Resources.LoadingLayer:new {
		resource = "LoadingLayer",
		alignment = MidBottom,
		alignmentOffset = {0,470},
		size = {200,100},
		alpha = 0,
		color = "Blue",
	},

	fractionForegroundScrolls = global_fractionForegroundScrolls,
	
	Foregrounds = Resources.LandMapForegrounds:new {
		resource = "LandMapForegrounds",
	},
	
	GatePathLocations = Resources.Actor:new{
		resource = "Actor",
		PathLocations = {
			{{x=0, y=0},},												--Gate 1 is before Land 1 and isn't used
			{{x=113, y=1714},},											--2
			{{x=157, y=1729}, {x=209, y=1656},},						--3
			{{x=139, y=1644},},											--4
			{{x=170, y=1657},},											--5
			{{x=192, y=1685},},											--6
			{{x=137, y=1633},},											--7
			{{x=208, y=1660},},											--8
			{{x=146, y=1549},},											--9
			{{x=121, y=1559},},											--10
			{{x=121, y=1559},},											--11
			{{x=121, y=1559},},											--12
			{{x=121, y=1559},},											--13
			{{x=121, y=1559},},											--14
			{{x=121, y=1559},},											--15
			{{x=121, y=1559},},											--16
			{{x=121, y=1559},},											--17
			{{x=121, y=1559},},											--18
			{{x=205, y=1720},},											--19
			{{x=121, y=1559},},											--20
			{{x=121, y=1559},},											--21
			{{x=121, y=1559},},											--22
			{{x=121, y=1559},},											--23
			{{x=121, y=1559},},											--24
			{{x=121, y=1559},},											--25
			{{x=121, y=1559},},											--26
			{{x=121, y=1559},},											--27
			{{x=121, y=1559},},											--28
			{{x=121, y=1559},},											--29
			{{x=121, y=1559},},											--30
			{{x=121, y=1559},},											--31
			{{x=121, y=1559},},											--32
			{{x=200, y=2780},},											--33
			{{x=121, y=1559},},											--34
		},
	},

	--allpaths
	LevelPathLocations = Resources.Actor:new{
		resource = "Actor",
		PathLocations = {
			{{x=128, y=1461}, {x=118, y=1373},},		--1
			{{x=259, y=1268},},				--2
			{{x=157, y=1177},},				--3
			{{x=185, y=1047},{x=263, y=1014},},			--4
			{{x=182, y=909},},				--5
			{{x=208, y=780},},				--6
			{{x=210, y=671},},				--7
			{{x=115, y=561},},				--8
			{{x=254, y=462},},				--9
			{{x=193, y=323},{x=123, y=296},},				--10
			{{x=131, y=1596}, {x=101, y=1456},},				--11
			{{x=167, y=1285},},				--12
			{{x=263, y=1128},},				--13
			{{x=247, y=945},},				--14
			{{x=118, y=824},},				--15
			{{x=110, y=712},},				--16
			{{x=209, y=619},},				--17
			{{x=189, y=484},},				--18
			{{x=202, y=388},},				--19
			{{x=217, y=283},},				--20
			{{x=116, y=1520},},				--21
			{{x=76, y=1389},},				--22
			{{x=152, y=1258},},				--23
			{{x=144, y=1140},},				--24
			{{x=93, y=1049},},				--25
			{{x=145, y=949},},				--26
			{{x=142, y=863},},				--27
			{{x=165, y=763},},				--28
			{{x=228, y=667},},				--29
			{{x=179, y=588},},				--30
			{{x=127, y=500},},				--31
			{{x=161, y=418},},				--32
			{{x=207, y=315},},				--33
			{{x=207, y=201},},				--34
			{{x=203, y=181},},				--35
			{{x=127, y=1557},{x=118, y=1470},},				--36
			{{x=131, y=1281},},				--37
			{{x=291, y=1192},{x=293, y=1132},},				--38
			{{x=150, y=1067},},				--39
			{{x=88, y=992},},				--40
			{{x=228, y=935},},				--41
			{{x=295, y=791},},				--42
			{{x=184, y=758},},				--43
			{{x=99, y=653},},				--44
			{{x=235, y=594},},				--45
			{{x=283, y=414},},				--46
			{{x=135, y=366},},				--47
			{{x=76, y=314},},				--48
			{{x=193, y=226},},				--49
			{{x=279, y=194},},				--50
			{{x=143, y=1633},{x=122, y=1550},{x=145, y=1473},},				--51
			{{x=198, y=1364},},				--52
			{{x=159, y=1311},},				--53
			{{x=148, y=1213},},				--54
			{{x=257, y=1172},},				--55
			{{x=251, y=1073},},				--56
			{{x=199, y=1032},},				--57
			{{x=116, y=881},},				--58
			{{x=276, y=798},},				--59
			{{x=200, y=676},},				--60
			{{x=127, y=610},},				--61
			{{x=243, y=530},},				--62
			{{x=105, y=368},},				--63
			{{x=277, y=232},},				--64
			{{x=177, y=119},},				--65
			{{x=129, y=1612},{x=131, y=1497},},				--66
			{{x=218, y=1393},},				--67
			{{x=285, y=1271},},				--68
			{{x=317, y=1162},},				--69
			{{x=282, y=1054},},				--70
			{{x=287, y=937},},				--71
			{{x=210, y=844},},				--72
			{{x=109, y=824},},				--73
			{{x=72, y=712},},				--74
			{{x=66, y=613},},				--75
			{{x=72, y=512},},				--76
			{{x=160, y=437},},				--77
			{{x=297, y=371},},				--78
			{{x=177, y=317},},				--79
			{{x=126, y=213},},				--80
			{{x=146, y=1593},{x=149, y=1528},},				--81
			{{x=253, y=1382},},				--82
			{{x=225, y=1254},},				--83
			{{x=227, y=1129},},				--84
			{{x=106, y=1017},},				--85
			{{x=110, y=900},},				--86
			{{x=170, y=836},},				--87
			{{x=246, y=773},},				--88
			{{x=298, y=687},},				--89
			{{x=203, y=586},},				--90
			{{x=96, y=526},},				--91
			{{x=128, y=417},},				--92
			{{x=181, y=372},},				--93
			{{x=285, y=289},},				--94
			{{x=249, y=196},},				--95
			{{x=148, y=1609},{x=148, y=1514},},				--96
			{{x=192, y=1350},},				--97
			{{x=288, y=1240},},				--98
			{{x=258, y=1179},},				--99
			{{x=214, y=1089},},				--100
			{{x=94, y=1021},},				--101
			{{x=113, y=917},},				--102
			{{x=185, y=827},},				--103
			{{x=145, y=767}, {x=218, y=672},},				--104
			{{x=247, y=584},},				--105
			{{x=250, y=524},},				--106
			{{x=225, y=466},},				--107
			{{x=245, y=390},},				--108
			{{x=179, y=278},},				--109
			{{x=126, y=111},},				--110

			{{x=122, y=1507},{x=167, y=1437},},				--111
			{{x=206, y=1306},},				--112
			{{x=172, y=1207},},				--113
			{{x=159, y=1093},},				--114
			{{x=183, y=991},},				--115
			{{x=195, y=936},},				--116
			{{x=205, y=857},},				--117
			{{x=211, y=813},},				--118
			{{x=171, y=732},},				--119
			{{x=151, y=631},},				--120
			{{x=167, y=526},},				--121
			{{x=221, y=435},},				--122
			{{x=197, y=349},},				--123
			{{x=145, y=254},},				--124
			{{x=162, y=148},},				--125
			{{x=120, y=1586},{x=145, y=1492},{x=146, y=1402},},				--126
			{{x=135, y=1322},},		 		--127
			{{x=212, y=1272},}, 			--128
			{{x=207, y=1177},},				--129
			{{x=214, y=1057},},				--130
			{{x=232, y=970},},				--131
			{{x=185, y=887},},				--132
			{{x=225, y=781},},				--133
			{{x=180, y=750},},				--134
			{{x=154, y=657},},				--135
			{{x=185, y=602},},				--136
			{{x=197, y=532},},				--137
			{{x=166, y=417},},				--138
			{{x=158, y=261},},				--139
			{{x=147, y=157},},				--140
			{{x=183, y=1550},},				--141
			{{x=161, y=1370},},				--142
			{{x=271, y=1284},},				--143
			{{x=196, y=1195},},				--144
			{{x=123, y=1111},},				--145
			{{x=143, y=1000},},				--146
			{{x=239, y=918},},				--147
			{{x=158, y=806},},				--148
			{{x=204, y=736},},				--149
			{{x=270, y=615},},				--150
			{{x=165, y=544},},				--151
			{{x=184, y=419},},				--152
			{{x=253, y=399},},				--153
			{{x=202, y=258},},				--154
			{{x=141, y=185},},				--155
			{{x=183, y=1550},},				--156
			{{x=141, y=1300},},				--157
			{{x=151, y=1234},},				--158
			{{x=196, y=1195},},				--159
			{{x=223, y=1111},},				--160
			{{x=193, y=1040},},				--161
			{{x=189, y=948},},				--162
			{{x=158, y=806},},				--163
			{{x=180, y=706},},				--164
			{{x=270, y=615},},				--165
			{{x=235, y=534},},				--166
			{{x=214, y=449},},				--167
			{{x=253, y=399},},				--168
			{{x=252, y=258},},				--169
			{{x=241, y=185},},				--170
			{{x=183, y=1550},},				--171
			{{x=211, y=1330},},				--172
			{{x=181, y=1214},},				--173
			{{x=176, y=1135},},				--174
			{{x=213, y=1050},},				--175
			{{x=233, y=980},},				--176
			{{x=239, y=918},},				--177
			{{x=208, y=830},},				--178
			{{x=204, y=736},},				--179
			{{x=270, y=615},},				--180
			{{x=165, y=504},},				--181
			{{x=184, y=419},},				--182
			{{x=253, y=329},},				--183
			{{x=242, y=258},},				--184
			{{x=241, y=185},},				--185
			{{x=163, y=1600},},				--186
			{{x=171, y=1400},},				--187
			{{x=151, y=1274},},				--188
			{{x=36, y=1195},},				--189
			{{x=143, y=1071},},				--190
			{{x=243, y=1040},},				--191
			{{x=239, y=938},},				--192
			{{x=158, y=886},},				--193
			{{x=130, y=756},},				--194
			{{x=170, y=605},},				--195
			{{x=235, y=534},},				--196
			{{x=264, y=429},},				--197
			{{x=233, y=369},},				--198
			{{x=202, y=258},},				--199
			{{x=151, y=180},},				--200
			{{x=163, y=1600},},				--201
			{{x=171, y=1400},},				--202
			{{x=211, y=1294},},				--203
			{{x=196, y=1195},},				--204
			{{x=193, y=1121},},				--205
			{{x=213, y=1040},},				--206
			{{x=179, y=958},},				--207
			{{x=158, y=886},},				--208
			{{x=160, y=736},},				--209
			{{x=200, y=635},},				--210
			{{x=235, y=534},},				--211
			{{x=214, y=429},},				--212
			{{x=213, y=339},},				--213
			{{x=202, y=258},},				--214
			{{x=171, y=180},},				--215
			{{x=195, y=1600},},				--216
			{{x=205, y=1400},},				--217
			{{x=151, y=1294},},				--218
			{{x=171, y=1195},},				--219
			{{x=181, y=1121},},				--220
			{{x=200, y=1035},},				--221
			{{x=221, y=908},},				--222
			{{x=197, y=856},},				--223
			{{x=165, y=736},},				--224
			{{x=252, y=645},},				--225
			{{x=215, y=554},},				--226
			{{x=185, y=449},},		 		--227
			{{x=182, y=369},}, 				--228
			{{x=207, y=258},},				--229
			{{x=214, y=180},},				--230
			{{x=163, y=1600},},				--231
			{{x=211, y=1400},},				--232
			{{x=211, y=1294},},				--233
			{{x=226, y=1195},},				--234
			{{x=213, y=1121},},				--235
			{{x=213, y=1040},},				--236
			{{x=179, y=958},},				--237
			{{x=158, y=856},},				--238
			{{x=160, y=736},},				--239
			{{x=200, y=655},},				--240
			{{x=205, y=554},},				--241
			{{x=214, y=469},},				--242
			{{x=213, y=339},},				--243
			{{x=230, y=258},},				--244
			{{x=251, y=175},},				--245
			{{x=195, y=1600},},				--246
			{{x=225, y=1340},},				--247
			{{x=230, y=1254},},				--248
			{{x=221, y=1195},},				--249
			{{x=221, y=1121},},				--250
			{{x=200, y=1035},},				--251
			{{x=231, y=928},},				--252
			{{x=197, y=856},},				--253
			{{x=205, y=756},},				--254
			{{x=222, y=665},},				--255
			{{x=215, y=594},},				--256
			{{x=215, y=519},},		 		--257
			{{x=182, y=369},}, 				--258
			{{x=207, y=258},},				--259
			{{x=214, y=180},},				--260
			{{x=163, y=1600},},				--261
			{{x=145, y=1400},},				--262
			{{x=205, y=1294},},				--263
			{{x=276, y=1195},},				--264
			{{x=243, y=1111},},				--265
			{{x=213, y=1040},},				--266
			{{x=229, y=958},},				--267
			{{x=210, y=856},},				--268
			{{x=160, y=736},},				--269
			{{x=200, y=655},},				--270
			{{x=205, y=554},},				--271
			{{x=214, y=469},},				--272
			{{x=213, y=369},},				--273
			{{x=230, y=258},},				--274
			{{x=221, y=185},},				--275
			{{x=115, y=1600},},				--276
			{{x=225, y=1370},},				--277
			{{x=230, y=1284},},				--278
			{{x=181, y=1195},},				--279
			{{x=201, y=1121},},				--280
			{{x=200, y=1035},},				--281
			{{x=231, y=928},},				--282
			{{x=197, y=856},},				--283
			{{x=205, y=756},},				--284
			{{x=222, y=665},},				--285
			{{x=215, y=594},},				--286
			{{x=215, y=519},},		 		--287
			{{x=182, y=369},}, 				--288
			{{x=207, y=258},},				--289
			{{x=174, y=160},},				--290
			{{x=125, y=1554},},				--291
			{{x=180, y=1409},},				--292
			{{x=144, y=1246},},				--293
			{{x=241, y=1166},},				--294
			{{x=275, y=1095},},				--295
			{{x=294, y=1030},},				--296
			{{x=294, y=965},},				--297
			{{x=270, y=886},},				--298
			{{x=216, y=805},},				--299
			{{x=145, y=733},},				--300
			{{x=129, y=632},},				--301
			{{x=160, y=574},},				--302
			{{x=238, y=426},},				--303
			{{x=170, y=297},},				--304
			{{x=150, y=215},},				--305
			{{x=145, y=1559},},				--306
			{{x=191, y=1431},},				--307
			{{x=209, y=1294},},				--308
			{{x=198, y=1186},},				--309
			{{x=239, y=1078},},				--310
			{{x=286, y=1014},},				--311
			{{x=268, y=953},},				--312
			{{x=217, y=838},},				--313
			{{x=286, y=666},},				--314
			{{x=238, y=617},},				--315
			{{x=216, y=547},},				--316
			{{x=242, y=422},},				--317
			{{x=228, y=355},},				--318
			{{x=235, y=265},},				--319
			{{x=238, y=205},},				--320
			{{x=151, y=1511},},				--321
			{{x=188, y=1415},},				--322
			{{x=227, y=1312},},				--323
			{{x=241, y=1210},},				--324
			{{x=243, y=1120},},				--325
			{{x=206, y=981},},				--326
			{{x=232, y=904},},				--327
			{{x=257, y=843},},				--328
			{{x=212, y=745},},				--329
			{{x=134, y=626},},				--330
			{{x=158, y=523},},				--331
			{{x=192, y=423},},				--332
			{{x=182, y=360},},				--333
			{{x=157, y=274},},				--334
			{{x=182, y=157},},				--335
			{{x=181, y=1536},},				--336
			{{x=237, y=1411},},				--337
			{{x=226, y=1312},},				--338
			{{x=138, y=1191},},				--339
			{{x=155, y=1122},},				--340
			{{x=209, y=1022},},				--341
			{{x=229, y=939},},				--342
			{{x=219, y=841},},				--343
			{{x=199, y=741},},				--344
			{{x=219, y=622},},				--345
			{{x=239, y=538},},				--346
			{{x=218, y=459},},				--347
			{{x=201, y=362},},				--348
			{{x=203, y=265},},				--349
			{{x=217, y=174},},				--350
			{{x=231, y=1511},},				--351
			{{x=208, y=1415},},				--352
			{{x=157, y=1342},},				--353
			{{x=171, y=1230},},				--354
			{{x=203, y=1120},},				--355
			{{x=166, y=981},},				--356
			{{x=232, y=904},},				--357
			{{x=257, y=843},},				--358
			{{x=232, y=745},},				--359
			{{x=204, y=626},},				--360
			{{x=208, y=523},},				--361
			{{x=192, y=423},},				--362
			{{x=192, y=340},},				--363
			{{x=217, y=274},},				--364
			{{x=162, y=217},},				--365
			{{x=181, y=1536},},				--366
			{{x=197, y=1411},},				--367
			{{x=166, y=1362},},				--368
			{{x=168, y=1241},},				--369
			{{x=155, y=1122},},				--370
			{{x=209, y=1022},},				--371
			{{x=229, y=939},},				--372
			{{x=219, y=841},},				--373
			{{x=149, y=741},},				--374
			{{x=179, y=622},},				--375
			{{x=209, y=538},},				--376
			{{x=218, y=459},},				--377
			{{x=221, y=332},},				--378
			{{x=203, y=265},},				--379
			{{x=217, y=174},},				--380
			{{x=231, y=1511},},				--381
			{{x=208, y=1415},},				--382
			{{x=157, y=1342},},				--383
			{{x=151, y=1250},},				--384
			{{x=193, y=1120},},				--385
			{{x=236, y=999},},				--386
			{{x=232, y=904},},				--387
			{{x=167, y=833},},				--388
			{{x=152, y=695},},				--389
			{{x=214, y=606},},				--390
			{{x=218, y=593},},				--391
			{{x=192, y=523},},				--392
			{{x=193, y=438},},				--393
			{{x=185, y=352},},				--394
			{{x=213, y=257},},				--395
			{{x=194, y=2594},},				--396
			{{x=200, y=2397},},				--397
			{{x=257, y=2332},},				--398
			{{x=262, y=2274},},				--399
			{{x=193, y=2192},},				--400
			{{x=159, y=2032},},				--401
			{{x=315, y=1947},},				--402
			{{x=264, y=1860},},				--403
			{{x=124, y=1817},},				--404
			{{x=203, y=1708},},				--405
			{{x=237, y=1622},},				--406
			{{x=193, y=1552},},				--407
			{{x=192, y=1393},},				--408
			{{x=220, y=1311},},				--409
			{{x=134, y=1229},},				--410
			{{x=193, y=1120},},				--411
			{{x=218, y=1053},},				--412
			{{x=179, y=957},},				--413
			{{x=274, y=858},},				--414
			{{x=207, y=705},},				--415
			{{x=222, y=636},},				--416
			{{x=186, y=543},},				--417
			{{x=183, y=423},},				--418
			{{x=190, y=297},},				--419
			{{x=238, y=222},},				--420
			{{x=194, y=2594},},				--421
			{{x=180, y=2440},},				--422
			{{x=150, y=2360},},				--423
			{{x=104, y=2307},},				--424
			{{x=176, y=2211},},				--425
			{{x=206, y=2109},},				--426
			{{x=155, y=1999},},				--427
			{{x=98, y=1910},},				--428
			{{x=97, y=1807},},				--429
			{{x=164, y=1724},},				--430
			{{x=219, y=1641},},				--431
			{{x=220, y=1565},},				--432
			{{x=156, y=1496},},				--433
			{{x=171, y=1375},},				--434
			{{x=199, y=1281},},				--435
			{{x=129, y=1194},},				--436
			{{x=159, y=1092},},				--437
			{{x=202, y=1011},},				--438
			{{x=234, y=934},},				--439
			{{x=216, y=830},},				--440
			{{x=284, y=632},},				--441
			{{x=200, y=529},},				--442
			{{x=222, y=418},},				--443
			{{x=179, y=320},},				--444
			{{x=159, y=204},},				--445
			{{x=191, y=2576},},				--446
			{{x=219, y=2460},},				--447
			{{x=199, y=2379},},				--448
			{{x=224, y=2160},},				--449
			{{x=228, y=2052},},				--450
			{{x=144, y=2005},},				--451
			{{x=163, y=1913},},				--452
			{{x=222, y=1870},},				--453
			{{x=227, y=1825},},				--454
			{{x=203, y=1766},},				--455
			{{x=251, y=1573},},				--456
			{{x=261, y=1492},},				--457
			{{x=249, y=1395},},				--458
			{{x=233, y=1321},},				--459
			{{x=224, y=1254},},				--460
			{{x=216, y=1142},},				--461
			{{x=220, y=1085},},				--462
			{{x=214, y=1001},},				--463
			{{x=199, y=920},},				--464
			{{x=196, y=801},},				--465
			{{x=222, y=623},},				--466
			{{x=273, y=525},},				--467
			{{x=290, y=432},},				--468
			{{x=287, y=369},},				--469
			{{x=256, y=320},},				--470
			{{x=194, y=2594},},				--471
			{{x=176, y=2440},},				--472
			{{x=144, y=2334},},				--473
			{{x=145, y=2211},},				--474
			{{x=130, y=2121},},				--475
			{{x=120, y=1982},},				--476
			{{x=158, y=1875},},				--477
			{{x=194, y=1795},},			    --478
			{{x=169, y=1721},},			    --479
			{{x=176, y=1543},},				--480
			{{x=117, y=1453},},				--481
			{{x=95, y=1347},}, 				--482
			{{x=174, y=1279},},				--483
			{{x=216, y=1199},},				--484
			{{x=184, y=1116},},				--485
			{{x=155, y=970},}, 				--486
			{{x=137, y=797},}, 				--487
			{{x=176, y=710},}, 				--488
			{{x=174, y=637},}, 			    --489
			{{x=135, y=546},}, 			    --490
			{{x=118, y=432},}, 		        --491
			{{x=150, y=332},}, 			    --492
			{{x=158, y=274},}, 			    --493
			{{x=122, y=215},}, 			    --494
			{{x=113, y=126},}, 		    	--495
			{{x=199, y=2580},},				--496
			{{x=218, y=2455},},				--497
			{{x=238, y=2347},},				--498
			{{x=238, y=2282},},				--499
			{{x=171, y=2199},},				--500
			{{x=218, y=2045},},				--501
			{{x=225, y=2017},},				--502
			{{x=193, y=1884},},				--503
			{{x=153, y=1847},},				--504
			{{x=214, y=1765},},				--505
			{{x=258, y=1660},},				--506
			{{x=277, y=1513},},				--507
			{{x=297, y=1380},},				--508
			{{x=248, y=1321},},				--509
			{{x=241, y=1207},},				--510
			{{x=264, y=1115},},				--511
			{{x=160, y=1083},},				--512
			{{x=176, y=907},},				--513
			{{x=218, y=802},},				--514
			{{x=218, y=729},},				--515
			{{x=212, y=601},},				--516
			{{x=212, y=472},},				--517
			{{x=244, y=342},},				--518
			{{x=235, y=290},},				--519
			{{x=192, y=221},},				--520
			{{x=222, y=2607},},				--521
			{{x=233, y=2488},},				--522
			{{x=231, y=2384},},				--523
			{{x=201, y=2351},},				--524
			{{x=113, y=2304},},				--525
			{{x=80, y=2198},},				--526
			{{x=161, y=2059},},				--527
			{{x=193, y=1924},},				--528
			{{x=217, y=1828},},				--529
			{{x=198, y=1649},},				--530
			{{x=232, y=1600},},				--531
			{{x=172, y=1477},},				--532
			{{x=178, y=1340},},				--533
			{{x=264, y=1220},},				--534
			{{x=230, y=1055},},				--535
			{{x=226, y=987},},				--536
			{{x=198, y=915},},				--537
			{{x=214, y=823},},				--538
			{{x=214, y=732},},				--539
			{{x=215, y=615},},				--540
			{{x=215, y=550},},				--541
			{{x=178, y=443},},				--542
			{{x=195, y=312},},				--543
			{{x=243, y=241},},				--544
			{{x=240, y=206},},				--545
			{{x=185, y=2633},},				--546
			{{x=207, y=2427},},				--547
			{{x=231, y=2353},},				--548
			{{x=213, y=2306},},				--549
			{{x=149, y=2251},},				--550
			{{x=148, y=2138},},				--551
			{{x=184, y=2063},},				--552
			{{x=249, y=1987},},				--553
			{{x=284, y=1909},},				--554
			{{x=282, y=1825},},				--555
			{{x=238, y=1726},},				--556
			{{x=184, y=1537},},				--557
			{{x=211, y=1429},},				--558
			{{x=206, y=1344},},				--559
			{{x=210, y=1207},},				--560
			{{x=233, y=1068},},				--561
			{{x=202, y=1027},},				--562
			{{x=210, y=883},},				--563
			{{x=203, y=740},},				--564
			{{x=217, y=615},},				--565
			{{x=223, y=563},},				--566
			{{x=187, y=452},},				--567
			{{x=202, y=344},},				--568
			{{x=247, y=245},},				--569
			{{x=241, y=211},},				--570
		},
	},

	ScrollableArea = Resources.Actor:new {
		resource = "Actor",
		class = "ScrollableArea",
		zorder = 25,
		horizontalScrollLocked = true,
		alignment = "MidTop",
		alignTo = "MidTop",
		SlowFraction = .95,
		stoppingVelocity = 2,
		
		scrollableBackground = Resources.LandMapScrollableBackground:new {
			resource = "LandMapScrollableBackground",
		},
	},
}

Resources.LevelZeroActor = Resources.Actor:new {
	resource = "Actor",
	offsetX = 135,
	offsetY = 1530,
}

Scripts.Present_LandMap = function(currentLevelsArea)
	PlaySound("Move1_up.ogg")
	currentLevelsArea.visible = true
end

Scripts.Present_CurrentLevelsArea = function(currentLevelsArea)
	PlaySound("Move1_up.ogg")
	currentLevelsArea.visible = true
end

Scripts.Leave_CurrentLevelsArea = function(currentLevelsArea)
	PlaySound("Move1_down.ogg")
	currentLevelsArea.visible = false
end

Scripts.LevelComesFromLeft = function(levelNumber)
	for i,v in ipairs(Resources.PathsComingFromLeft) do
		if v == levelNumber then
			return true
		end
	end
	return false
end

local LEFT = {-10, 0}
local CENTERLEFT = {-5, 0}
local CENTER = {0, 0}
local RIGHT = {5, 0}

Resources.LevelStartPathOffsets = {
	[34] = {0, 20}
}

Resources.LevelEndPathOffsets = {
	{-30, 0},
	CENTERLEFT,
	CENTER,
	CENTER,
	CENTER,				--5
	CENTERLEFT,
	CENTER,
	LEFT,
	CENTER,
	CENTER,				--10
	LEFT,
	LEFT,
	CENTER,
	CENTER,
	LEFT,				--15
	LEFT,
	CENTER,
	CENTER,
	CENTERLEFT,
	CENTER,				--20
	LEFT,
	LEFT,
	LEFT,
	CENTER,
	LEFT,				--25
	LEFT,
	CENTERLEFT,
	{-25, -5},
	CENTER,
	RIGHT,				--30
	CENTER,
	LEFT,
	CENTERLEFT,
	LEFT,
	RIGHT,				--35
	LEFT,
	LEFT,
	CENTER,
	CENTER,
	LEFT,				--40
	LEFT,
	CENTER,
	CENTER,
	LEFT,
	LEFT,				--45
	CENTER,
	CENTER,
	LEFT,
	{-30, -30},
	RIGHT,				--50
	LEFT,
	LEFT,
	CENTER,
	LEFT,
	LEFT,				--55
	LEFT,
	CENTER,
	{-30, -15},
	CENTER,
	{20, 0},			--60
	LEFT,
	{-10, 0},
	{20, 0},
	LEFT,
	CENTER,				--65
	{-10, 0},
	CENTER,
	CENTER,
	CENTER,
	CENTER,				--70
	CENTER,
	{20, 0},
	{-10, 0},
	CENTER,
	CENTER,				--75
	LEFT,
	{-30, -15},
	CENTER,
	{-10, 0},
	LEFT,				--80
	CENTER,
	CENTER,
	LEFT,
	CENTER,
	CENTER,				--85
	{-30, -15},
	{-30, -15},
	LEFT,
	{20, 5},
	{20, 0},			--90
	LEFT,
	{-30, -15},
	CENTER,
	CENTER,
	LEFT,				--95
	LEFT,
	CENTER,
	CENTERLEFT,
	{10, 0},
	{20, 0},			--100
	{-20, 0},
	CENTER,
	LEFT,
	CENTER,
	{-15, 0},			--105
	LEFT,
	{27, 15},
	{-15, 0},
	CENTER,
	LEFT,				--110
	{-20, 0},
	CENTER,
	{-15, 0},
	CENTER,
	{-30, -15},			--115
	{-15, 0},
	{-30, -15},
	CENTER,
	{-5, 0},
	{-15, 0},			--120
	{-20, 0},
	{-15, 0},
	CENTER,
	{-15, 0},
	{-30, 0},			--125
	CENTER,
	{-15, 0},
	CENTER,
	CENTER,
	CENTER,				--130
	CENTER,
	{-15, 0},
	CENTER,
	CENTER,
	CENTER,				--135
	CENTER,
	CENTER,
	CENTER,
	CENTER,
	CENTER,				--140
}

Resources.SoilLand1 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand1",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand2 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand2",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}		

Resources.SoilLand3 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand3",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand4 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand4",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand5 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand5",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand6 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand6",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand7 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand7",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand8 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand8",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand9 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand9",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand10 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand10",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand11 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand11",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand12 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand12",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand13 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand13",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand14 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand14",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand15 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand15",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand16 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand16",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand17 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand17",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand18 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand18",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand19 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand19",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}	

Resources.SoilLand20 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand20",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand21 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand21",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand22 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand22",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand23 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand23",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand24 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand24",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand25 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand25",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand26 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand26",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand27 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand27",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand28 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand28",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand29 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand29",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand30 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand30",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand31 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand31",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand32 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand32",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand33 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand33",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.SoilLand34 = {
	name = "ThrowSoilContinuous",
	class = "hgeParticleSystemActor",
	file = "SoilLand34",
	targetTag = "MapParticleLayer",
	zorder = 1000,
	alignment = "Center",
    alignTo = "Center",
    alignmentOffset = {0,3},
}

Resources.DirtLand1 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land1.png",
	},
}

Resources.DirtLand2 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land2.png",
	},
}
Resources.DirtLand3 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land3.png",
	},
}
Resources.DirtLand4 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land4.png",
	},
}

Resources.DirtLand5 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land5.png",
	},
}

Resources.DirtLand6 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land6.png",
	},
}

Resources.DirtLand7 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land7.png",
	},
}

Resources.DirtLand8 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land8.png",
	},
}

Resources.DirtLand9 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land9.png",
	},
}

Resources.DirtLand10 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land10.png",
	},
}

Resources.DirtLand11 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land11.png",
	},
}

Resources.DirtLand12 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land12.png",
	},
}
Resources.DirtLand13 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land13.png",
	},
}
Resources.DirtLand14 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land14.png",
	},
}

Resources.DirtLand15 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land15.png",
	},
}

Resources.DirtLand16 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land16.png",
	},
}

Resources.DirtLand17 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land17.png",
	},
}

Resources.DirtLand18 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land18.png",
	},
}

Resources.DirtLand19 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land19.png",
	},
}

Resources.DirtLand20 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land20.png",
	},
}

Resources.DirtLand21 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land21.png",
	},
}

Resources.DirtLand22 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land22.png",
	},
}
Resources.DirtLand23 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land23.png",
	},
}
Resources.DirtLand24 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land24.png",
	},
}
Resources.DirtLand25 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land25.png",
	},
}

Resources.DirtLand26 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land26.png",
	},
}

Resources.DirtLand27 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land27.png",
	},
}

Resources.DirtLand28 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land28.png",
	},
}

Resources.DirtLand29 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land29.png",
	},
}

Resources.DirtLand30 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land30.png",
	},
}

Resources.DirtLand31 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land31.png",
	},
}

Resources.DirtLand32 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land32.png",
	},
}

Resources.DirtLand33 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land33.png",
	},
}

Resources.DirtLand34 = Resources.Actor:new {
	resource = "Actor",
	dirtImage = Resources.Actor:new {
		alignment = Center,
		alignTo = Center,
		image = "Mound_Land34.png",
	},
}

Scripts.LandMap_DisplayLoading = function(landMap, topOrBottom)
	if(topOrBottom == "top") then
		landMap.upperLoadingLayer.visible = true
	else
		landMap.lowerLoadingLayer.visible = true
	end
	ScriptDelay(100)
	landMap:UpdateCaching()
	if(topOrBottom == "top") then
		landMap.upperLoadingLayer.visible = false
	else
		landMap.lowerLoadingLayer.visible = false
	end
end

JasperGateOffsets = {
	-- Remember, [index] is the number of the NEXT land, not the preceding land
	[2] = {x=75,y=130},
	[3] = {x=120,y=160},
	[4] = {x=65,y=95},
	[5] = {x=90,y=120},
	[6] = {x=20,y=80},
	[7] = {x=70,y=130},
	[8] = {x=85,y=120},
	[9] = {x=100,y=100},
	[10] = {x=60,y=125},
	[11] = {x=50,y=80},
	[12] = {x=110,y=70},
	[13] = {x=80,y=110},
	[14] = {x=45,y=85},
	[15] = {x=120,y=70},
	[16] = {x=10,y=85},
	[17] = {x=90,y=90},
	[18] = {x=60,y=110},
	[19] = {x=30,y=80},
	[20] = {x=75,y=90},
	[21] = {x=40,y=110},
	[22] = {x=-15,y=75},
	[23] = {x=50,y=95},
	[24] = {x=115,y=65},
	[25] = {x=-35,y=155},
	[26] = {x=53,y=198},
	[27] = {x=3,y=188},
	[28] = {x=53,y=115},
	[29] = {x=120,y=78},
	[30] = {x=120,y=108},
	[31] = {x=120,y=108},
	[32] = {x=120,y=108},
	[33] = {x=50,y=133},
	[34] = {x=120,y=145},
}