local function MakeLevel(levelName,mapLevelNumber,mapLandNumber)
	local land = Lands:GetLand(mapLandNumber)

    --List out #/id/name for each level
    -- logprint("Level " .. mapLevelNumber .. ": " .. levelName .. " (" .. LevelDefs[levelName].levelName .. ")")

	local level = {
		definitionName = levelName,
		--spot = LevelSpots[mapLevelNumber],
        landNumber = mapLandNumber,
		slotImage = land.slotImage,
		cellImage = land.cellImage,
		gameScreenResource = string.format("GameScreen_Land%d", mapLandNumber),
		levelNumber = mapLevelNumber,
	}

	-- use large board resource for large board
	if(LevelDefs[levelName].Features.largeBoard)then
		level.gameScreenResource = level.gameScreenResource .. "_large"
	end

	return level
end

local function MakeSidePathLevel(levelName,landIndex,mapLandNumber,branchLevel)
	local land = Lands:GetLand(mapLandNumber)

    --List out #/id/name for each level
    -- logprint("SideLevel " .. mapLandNumber .. "-" .. landIndex .. ": " .. levelName .. " (" .. LevelDefs[levelName].levelName .. ")")

	local level = {
		definitionName = levelName,
        landNumber = mapLandNumber,
		slotImage = land.slotImage,
		cellImage = land.cellImage,
		gameScreenResource = string.format("GameScreen_Land%d", mapLandNumber),
		indexInLand = landIndex,
		branchOrigin = branchLevel,
	}

	-- For event levels, use the level definition to set the background name
	if(LevelDefs[levelName].hidden == true) then
		level.gameScreenResource = LevelDefs[levelName].resource
	end

	-- use large board resource for large board
	if(LevelDefs[levelName].Features.largeBoard)then
		level.gameScreenResource = level.gameScreenResource .. "_large"
	end

	return level
end


Levels = {
	-- LAND 1
	-- Tutorial
	--MakeLevel("EasyTestLevel",		1,1),
	MakeLevel("Welcome",		1,1),
	MakeLevel("Level2",			2,1),
	MakeLevel("Level3",			3,1),
	MakeLevel("Level4",			4,1),
	MakeLevel("Level5",			5,1),
	
	-- COVERAGE
	MakeLevel("SimpleCoverage1",6,1),
	MakeLevel("SimpleCoverage2",7,1),
	MakeLevel("SimpleCoverage3",8,1),  -- TEST: Level switch using Config Config.Land1_Level8_Name - "SimpleCoverage3"
	MakeLevel("SimpleCoverage4",9,1),
	MakeLevel("BigBoard2",		10,1),

	-- Missing Cells and Orange
	--[[
	MakeLevel("MissingCells",	6,1),
	MakeLevel("HolePunch",		7,1),
	MakeLevel("OrangeGems",		8,1),
	MakeLevel("OrangeMissing",	9,1),
	MakeLevel("BigBoard",		10,1),
	--]]
}
	
if Config.OldLand2 then
	table.insertall(Levels, {
		-- LAND 2
		MakeLevel("SimpleCrateCoverage1",11,2),
		MakeLevel("SimpleCrateCoverage2",12,2),
		MakeLevel("SimpleCrateCoverage3",13,2),
		MakeLevel("CoverageEmpty1",14,2),
		MakeLevel("CoverageEmpty2",15,2),
		MakeLevel("CoverageEmpty3",16,2),
		MakeLevel("BigCrateBoard1",17,2),
		MakeLevel("BigCrateBoard2",18,2),
		MakeLevel("BigCrateBoard3",19,2),
		MakeLevel("BigCrateBoard4",20,2),
	})
else
	table.insertall(Levels, {
		-- LAND 2
		MakeLevel("SimpleCrateCoverage1",11,2),
		MakeLevel("SimpleCrateCoverage2",12,2),
		MakeLevel("SimpleCrateCoverage3",13,2),
		MakeLevel("CoverageEmpty1",14,2),
		MakeLevel("CoverageEmpty2",15,2),
		MakeLevel("CoverageEmpty3",16,2),
		MakeLevel("BigCrateBoard1",17,2),
		MakeLevel("HorizontalBursts",18,2),
		MakeLevel("VerticalBursts",19,2),
		MakeLevel("BigCrateBoard4",20,2),
	})
end

table.insertall(Levels, {
	-- LAND 3
	MakeLevel("Dropping1",21,3),
	MakeLevel("Dropping3",22,3),
	MakeLevel("CrossBoardCoverage1",23,3),
	MakeLevel("CrateBlockDrop1",24,3),
	MakeLevel("StairClear",25,3),
	MakeLevel("CoverageCrate_RightSide",26,3),
	MakeLevel("Inaccessible",27,3), --Was boring level: CoverageCrate_Stairs
	MakeLevel("CoverageCrate_Frame",28,3), 
	MakeLevel("CrateBlockDrop2",29,3),  -- Drop
	MakeLevel("Chest_Floor",30,3), --Was broken level: CoverageCrate_UpperLine1
	MakeLevel("HardPattern",31,3), --DROP -- Was broken level CrateBlockDrop3
	MakeLevel("CoverageCrate_Blocks1",32,3), -- Was broken level: 
	MakeLevel("Drop_Empty1",33,3), -- COVERAGE Double BOARD
	MakeLevel("DrillDown",34,3),
	MakeLevel("Drop_Empty_Large1",35,3), -- BIG BOARD DROP CRATE 

	-- LAND 4
	MakeLevel("GemJail1",		36,4), --jail
	MakeLevel("GemJailScore1",	37,4),
	MakeLevel("JailGate1",		38,4),
	MakeLevel("JailDrop1",		39,4),
	MakeLevel("JailCoverage1",	40,4),
	MakeLevel("Continuation", 41,4), --Was boring level: JailCoverage1_large
	MakeLevel("GemJailScore2",	42,4),
	MakeLevel("JailDrop2",		43,4),
	MakeLevel("CustomColor1",	44,4),
	MakeLevel("GemJailScore3",	45,4),
	MakeLevel("JailDrop3",		46,4),
	MakeLevel("ImpossibleColor_Updated", 47,4), --Was flawed level: JailCoverage2
	MakeLevel("GemJailScore4",	48,4),
	MakeLevel("JailDrop4",		49,4),
	MakeLevel("ASplashOfGreatness",50,4), --Was flawed/boring level JailCoverage2_large

	-- LAND 5
	MakeLevel("VinesScore1",51,5),
	MakeLevel("VinesRemove1",52,5),
	MakeLevel("VinesDrop1",53,5),
	MakeLevel("VinesPattern1",54,5),
	MakeLevel("PyramidDrop",55,5),  -- Was poorly-performing level VinesRemove2
	MakeLevel("VinesDrop2",56,5),
	MakeLevel("VinesPattern2_Updated",57,5),
	MakeLevel("VinesRemove3",58,5),
	MakeLevel("VinesDrop3",59,5),
	MakeLevel("EntrenchedVine_Updated",60,5), --Was diamond level VinesDiamond1
	MakeLevel("JailPairs",61,5), -- Was poorly-performing level VinesRemove4
	MakeLevel("VinesPattern3",62,5),
	MakeLevel("VinesDrop4",63,5),
	MakeLevel("VinesPattern4",64,5),
	MakeLevel("VinesRemove5_Updated",65,5),

	-- LAND 6
	MakeLevel("Barrels1",66,6),
	MakeLevel("BarrelsPattern1",67,6),
	MakeLevel("BarrelsDrop1",68,6),
	MakeLevel("BarrelQueue",69,6), --Was diamond level BarrelsDiamonds1
	MakeLevel("WaffleIron",70,6), --Was under-performing level Barrels2
	MakeLevel("BarrelsPattern2",71,6),
	MakeLevel("BarrelsDrop2",72,6),
	MakeLevel("GoWild",73,6), --Was broken level Barrels3 (could get stuck in unwinnable state)
	MakeLevel("BarrelsPattern3",74,6),
	MakeLevel("BarrelsDrop3",75,6),
	MakeLevel("Barrels4_Updated",76,6),
	MakeLevel("ShiftingSands",77,6), --Was BarrelsPattern4, which was boring.
	MakeLevel("BarrelsDrop4",78,6),
	MakeLevel("TrickLevel",79,6), --Was BarrelsDiamonds2, a diamond level
	MakeLevel("Barrels5_Updated",80,6),

	-- LAND 7
	MakeLevel("Bomb1",			81,7),
	MakeLevel("BombFalling1",	82,7),
	MakeLevel("BombDungeon1",	83,7),
	MakeLevel("BombFalling2",	84,7),
	MakeLevel("BombDungeon2",	85,7),
	MakeLevel("BombTower1",		86,7),
	MakeLevel("BombFalling3",   87,7),
	MakeLevel("EverythingFall",	88,7),
	MakeLevel("BombFalling4",	89,7),
	MakeLevel("BombDungeon3",	90,7),
	MakeLevel("BombPattern1",	91,7),
	MakeLevel("BombDrop1",		92,7),
	MakeLevel("BombVines1",		93,7),
	MakeLevel("BombFallingCombined",	94,7),
	MakeLevel("BombDungeon4_Updated",	95,7),
	
	-- LAND 8  
	MakeLevel("Land8_Level1",	96,8), -- Stone rid
	MakeLevel("Land8_Level2",	97,8), -- Pattern
	MakeLevel("Land8_Level3",	98,8), -- Dropping
	MakeLevel("Land8_Level4",	99,8), -- Jail
	MakeLevel("Land8_Level5_Updated",	100,8),-- Vines
	MakeLevel("Land8_Level6",	101,8),-- Barrels
	MakeLevel("Land8_Level7",	102,8),-- Bomb
	MakeLevel("Land8_Level8",	103,8),-- Stone rid + Vines Large
	MakeLevel("Land8_Level9",	104,8),-- Pattern + Jail Large
	MakeLevel("Land8_Level10_Updated",	105,8),-- Dropping + Barrels Large
	MakeLevel("Land8_Level11",	106,8),-- Jail + Bomb Large
	MakeLevel("EmptyColumns",	107,8),-- Was Vines + Stone Large "Land8_Level12", replaced with a themed level
	MakeLevel("Land8_Level13",	108,8),-- Barrels + Vines Large
	MakeLevel("Land8_Level14_Updated",	109,8),-- Bomb + Vines Large
	MakeLevel("Land8_Level15",	110,8),-- Bomb + Barrels + Vines Large
	
	--MakeLevel("TripleStone",	110,8),-- Triple Stone Test


	-- LAND 9 
	MakeLevel("Land9_Sample",		111,9), -- 3 hit stone + pattern
	MakeLevel("FourQuarts",			112,9), -- 3 hit stone + vines
	MakeLevel("StoneWithBurst",		113,9), -- 3 hit stone + pattern + burst gems
	MakeLevel("TripleDecker",		114,9), -- 3 hit stone + bombs (large)
	MakeLevel("FieldOfStones",		115,9), -- 3 hit stone + boulders

	MakeLevel("Landslide",			116,9), -- 3 hit stone + treasure chests	
	MakeLevel("TreasureRow",		117,9), -- 3 hit stone + treasure bombs 
	MakeLevel("JailsV",				118,9), -- 3 hit stone + jails + pattern (empty squares)
	MakeLevel("BigDiamond",			119,9), -- 3 hit stone + jails + crates + patterns (large)
	MakeLevel("StonePoints",		120,9), -- 3 hit stone + points (large)

	MakeLevel("SuperStone",			121,9), -- 3 hit stone + super wilds + points (large)	
	MakeLevel("LotsOBombs",			122,9), -- Bombs
	MakeLevel("JailStack",			123,9), -- Jails (large)
	MakeLevel("GottaCatchEmAll",	124,9), -- Pattern + boulders
	MakeLevel("EdgeVines",			125,9), -- 3 hit stone + vines (large)


	-- LAND 10
	MakeLevel("GemCollect1",		126,10), -- Crates + match blues
	MakeLevel("GemClear1",			127,10), -- Remove reds
	MakeLevel("GemCollect2",		128,10), -- Match yellows
	MakeLevel("BombCollect_Updated",129,10), -- Bombs + remove oranges
	MakeLevel("BottomVines",		130,10), -- Vines + remove greens
	
	MakeLevel("SideColumns",		131,10), -- Jails + remove yellows
	MakeLevel("GemCollect4",		132,10), -- Missing squares + match blues
	MakeLevel("6ColorClear_Updated",133,10), -- Remove orange
	MakeLevel("BottomRow",			134,10), -- Missing squares + remove yellows
	MakeLevel("TopVines",			135,10), -- Vines + match greens
	
	MakeLevel("GemTreasures",		136,10), -- Treasure chests + non-reel colors
	MakeLevel("BombCollect2",		137,10), -- Bombs + match purples
	MakeLevel("YellowMiniTowers",	138,10), -- Jails + remove oranges
	MakeLevel("EverythingIsTicking",139,10), -- Was unpopular level "GemClearStripes"
	MakeLevel("BombVines",			140,10), -- Bombs + vines + match oranges
})

if TotalLands >= 11 then
	table.insertall(Levels, {
	-- LAND 11
	MakeLevel("LavaPool",				141,11), -- Lava
	MakeLevel("2HitBoulderTest_Updated",142,11), -- 2-hit boulders
	MakeLevel("JailedLava",				143,11), -- Lava + jails
	MakeLevel("BoulderTreasure",		144,11), -- 2-hit boulders and chests
	MakeLevel("LavaSplash",				145,11), -- Was lava & splashes; changed to just lots of lava

	MakeLevel("BouldersAndLava",		146,11), -- 2-hit boulders + lava
	MakeLevel("LavaBombs",				147,11), -- Lava and bombs
	MakeLevel("TreasureJails",			148,11), -- Treasure chests and jails
	MakeLevel("LavaVines",				149,11), -- Vines and lava
	MakeLevel("DoubleBoulderBombs",		150,11), -- 2-hit boulder bombs

	MakeLevel("LavaChests",				151,11), -- Lava + chests
	MakeLevel("SplashVines",			152,11), -- Vines and recurring splashes
	MakeLevel("LavaPattern_Updated",	153,11), -- Lava + match pattern
	MakeLevel("StackOfBoulders",		154,11), -- 2-hit boulders + jails
	MakeLevel("AvoidLava",				155,11), -- Lava, but need to avoid clearing until points are built up
	--MakeLevel("BarrelTest",			155,11),
	})
end

if TotalLands >= 12 then
	table.insertall(Levels, {
	-- LAND 12

	--[[
	-- barrel test levels

	MakeLevel("BarrelGemTest",		156,12), 
	MakeLevel("BarrelSuperwildTest",		157,12), 
	MakeLevel("BarrelBurstTest",		158,12), 
	MakeLevel("BarrelLavaTest",		159,12), 
	MakeLevel("BarrelBoulderTest",		160,12), 

	MakeLevel("BarrelVineTest",		161,12), 
	MakeLevel("BarrelTreasureTest",		162,12), 
	MakeLevel("BarrelJailTest",		163,12),
	MakeLevel("BarrelBombTest",		164,12),
	MakeLevel("BarrelStoneTest",		165,12),

	MakeLevel("BarrelBombsAndVines",		166,12), 
	MakeLevel("BarrelBurstsAndCrates",		167,12), 
	MakeLevel("BarrelsOfTreasure_boulders",		168,12), 
	MakeLevel("BarrelsOfSuperWilds_jails",		169,12),
	MakeLevel("BarrelsOfCrazy_pattern",		170,12), 
	--]]

	-- actual Land 12 levels (to be filled in)

	MakeLevel("BarrelGemTest",			156,12), --1 Naomi: Welcome to barrels
	MakeLevel("TargetPractice",				157,12), --2 Jon: Target Practice: There's unreachable crates on the sides. Barrels drop in at a 10% rate and have horizontal bursts. Break all the crates. 
	MakeLevel("WildJungle_Updated",					158,12), --3 DavidB: Wild Jungle: Wilds fall in barrels, they are used to clear vines.
	MakeLevel("BarrelsOfTreasure_boulders",	159,12), --5 Dave: Treasure or boulders
	MakeLevel("LavaSeeds_Updated",			160,12), --4 Jon: Lava Seeds: Barrels drop in at a 20% rate. Score level. Barrels have lava in them, so player is trying to manage where lava gets set up
	MakeLevel("DoubleBreakout",				161,12), --6 Jon: Double Break-out: A thick pattern of barrels. The barrels contained jailed gems. Get the pattern behind the barrels/jails. 

	MakeLevel("BarrelsOfSuperWilds_jails",	162,12), --7 Dave: Made Super wilds or jails - pretty fun, use super wilds to try to break jails that are forming - needs tuning.
	MakeLevel("ExplodingLava",				163,12), --8 Jon: Exploding Lava: Lava with a bomb in it. As the lava spreads, so does the bomb countdown.
	MakeLevel("SecureAmmunition",			164,12), --10 DavidB: Secure Munitions - Area Splash pieces on the sides of the board, surrounded above and below by jails. To the inside of the splash pieces are missing cells. Horizontal splash pieces fall in the barrels and can be used to detonate the splash gems and clear the jails.
	MakeLevel("CrateDropper",				165,12), --11 Naomi: Warehouse: Clear pattern. Barrels drop crates.

	MakeLevel("Terraforming",				166,12), --12 DavidB: Terraforming: Area, horizontal, and vertical splash gems fall in barrels. They are used to clear lava.
	MakeLevel("BarrelsOfBombs",			167,12), --13 Naomi: Dynamite: Barrels containing bombs. 
	MakeLevel("TonsOfTreasure",		    	168,12), --14 Jon: Tons of Treasure: 25% piece mix of treasure chests. This should be enough that they really get in the way; treasure as a barrier instead of a help.
	MakeLevel("BarrelsOfCrazy_pattern",		169,12), --9 Dave: WIP: Barrels of Crazy -  Large board with everything in it (may be better boss level) TODO
	MakeLevel("PottedPlants",				170,12), --15 Naomi: Potted Plants: Eliminate vines, with vines coming out of barrels

	})
end

if TotalLands >= 13 then
	table.insertall(Levels, {
	-- LAND 13
	MakeLevel("SplitLevel",	            171,13), --Big board with center missing. Intro to feature for score
	MakeLevel("SideShot",	            172,13), --Reels missing on one side, pattern on that side.
	MakeLevel("NarrowContinuation",	    173,13), --Using missing reels to make the biggest spin even bigger
	MakeLevel("HalfReels",	            174,13), --Missing reels in the columns with treasure chests
	MakeLevel("BombColumn",	            175,13), --Bombs protected by stone
   
	MakeLevel("BurstReels",	            176,13), --Clear triple stones with reel bursts
	MakeLevel("LavaTide_Updated",		177,13), --Clear lava that is spreading from opposite side of reels
	MakeLevel("MiniCascade",			178,13), --4x5 board
	MakeLevel("RemoteJungle",			179,13), --Reels missing from center jungle column
	MakeLevel("Snake",					180,13), --Long path only one gem wide, 3 gem types
    
	MakeLevel("Swimming",				181,13), --Long narrow level with boulders
	MakeLevel("JungleRace",				182,13), --Bomb in the middle vines
	MakeLevel("StepByStep",				183,13), --A series of actions needed to drop a chest
	MakeLevel("Columns",				184,13), --Score level split into 2-1-2
	MakeLevel("Breakout",				185,13), --Center reels, corner obstacles
	})
end

if TotalLands >= 14 then
	table.insertall(Levels, {
	-- LAND 14
	MakeLevel("VineGrowerTest",			186,14), -- Germination: Intro for vine growers: two generators, one on each side
	MakeLevel("VineGrowerStone",			187,14), -- Plant Box: Center vine generators with stone covering pattern
	MakeLevel("Squeeze",					188,14), -- Squeeze: Vine generators on each side, play for high score
	MakeLevel("BurstCollector",				189,14), -- Burst Collector: make burst gems
	MakeLevel("VineGrowerTreasure",		190,14), -- Undergrowth: Vine generators at bottom, getting in the way of treasure chests

	MakeLevel("EmptySpacesVineGenerator",191,14), -- Greenhouse: Vine generators on the other side of a gulf of empty rows
	MakeLevel("Multibomb",					192,14), -- Minefield: every kind of bomb at once
	MakeLevel("StemTheTide",				193,14), -- David B - Stem the Tide: Vines grow from a vine generator along a track, player can use strategically placed horizontal burst gems to hold them back
	MakeLevel("VineGrowerBombs",			194,14), -- Bomb Box: Vine generators and crates blocking access to bombs
	MakeLevel("Deforestation",				195,14), -- David B - Deforestation: Vines from several vine generators, player has burst gems in barrels of fun and must clear a specific number of vines
  
	MakeLevel("BombBarrels",				196,14), -- Ticking Barrels: Bomb barrels full of stones
	MakeLevel("CornerVineGenerator",		197,14), -- Funnel: Get rid of jails. Breaking a crate frees the vine generator.
	MakeLevel("Grove",						198,14), -- Grove: Corner box of vines needs to be zapped by bursts
	MakeLevel("TripleTimeThreat",			199,14), -- No Pressure: Vines, lava, and bombs all at once
	MakeLevel("Vinewheel",					200,14), -- Vinewheel: Vine generators in corners, pinwheel of vines
	})
end

if TotalLands >= 15 then
	table.insertall(Levels, {
	-- LAND 15
	MakeLevel("CoconutTest",			201,15), -- Coconuts
	MakeLevel("DiagonalDig",			202,15), -- 4-hit stone, empty squares, pattern
	MakeLevel("CoconutsAnd4Stone",		203,15), -- Stone Islands: Coconuts, missing reels, 4-hit stone
	MakeLevel("UpAndDown",				204,15), -- Coconuts, stone, empty squares
	MakeLevel("BadCoconut",				205,15), 
	MakeLevel("BuriedTreasure",			206,15), -- Treasure on side to drop down via super wilds
	MakeLevel("CoconutsVerseLava",		207,15), 
	MakeLevel("DoubleBombPressure",		208,15), -- Bomb coconuts full of bombs
	MakeLevel("SkyTop",					209,15), -- Pattern on top to match via super wilds
	MakeLevel("BarrelsOfCoconuts",		210,15), 
	MakeLevel("CoconutLava",			211,15), -- Coconuts of Fire
	MakeLevel("RockinTree",				212,15), 
	MakeLevel("CoconutOrange",			213,15), -- Hidden Oranges
	MakeLevel("4StoneTest",				214,15), -- Fortress: Treasure bombs, barrels of bombs, 4-hit stone
	MakeLevel("VineChestCoconut",		215,15), -- Entanglement
	})
end

if TotalLands >= 16 then
	table.insertall(Levels, {
	-- LAND 16
	MakeLevel("AmberTest",			216,16), --Blocks of Amber: large block of amber. Naomi
	MakeLevel("TickingAmber",		217,16), --Trapped Bombs: Bombs in amber
	MakeLevel("AmberTreasure",		218,16), --Treasures in Amber: treasure chests covered by amber. Naomi
	MakeLevel("MatrioshkaBursts",	219,16), --Matrioshka Bursts: Bursts in amber in barrels. Fun level for score. jsupnik
	MakeLevel("Layers",				220,16), --Geology: vine generators, inside lava, inside amber, inside pattern

	MakeLevel("Checkers",			221,16), --Checkers: Amber, crates, jails. Pattern level.
	MakeLevel("AmberRing",			222,16), --Pipeline: Long skinny tunnel of ambers that have to matched one at a time. Naomi
	MakeLevel("SuperwildBreakout",	223,16), --Superwild Breakout: Use amber superwilds to get off-color jails
	MakeLevel("AmberPool",			224,16), --Amber Pool: amber blocked by stone
	MakeLevel("AmberBoulders",		225,16), --Rocks in Amber: boulders trapped in amber

	MakeLevel("CoconutKnockdown",	226,16), --Coconuts in amber with stones in
	MakeLevel("TimedExcavation",	227,16), --Layers of obstacles with bombs on.
	MakeLevel("Impurities",			228,16), --Like biggest spin, but with a foreign color occasionally
	MakeLevel("AmberRainbow",		229,16), --Amber Rainbow, Preplaced colors with ambers and jails
	MakeLevel("CornersAndMid",		230,16), --Bursts in amber to shoot into protected barrels
	})
end

if TotalLands >= 17 then
	table.insertall(Levels, {
	-- LAND 17
	MakeLevel("ColoredBarrelTreasure",	231,17), -- Treasure Barrels: get chests, which are only found in colored barrels
	MakeLevel("PatternVille",			232,17), 
	MakeLevel("BarrelReel2",			233,17), -- Barrel Jails: barrels of jails and boulders in reels, break jails
	MakeLevel("OneColorReels",			234,17), 
	MakeLevel("FenceTest",				235,17), 
	MakeLevel("FencePatterns",			236,17), 
	MakeLevel("RubeMachine",			237,17), 
	MakeLevel("BarrelsInBarrels",		238,17), 
	MakeLevel("LavaTube",				239,17), 
	MakeLevel("SuperWildHunt2",			240,17), 
	MakeLevel("TreasureCave",			241,17),
	MakeLevel("BurstChest",				242,17), 
	MakeLevel("Firecracker",			243,17), -- Long Fuse: chain of barrels full of area bursts, crate bomb
	MakeLevel("ImpossibleBarrels",		244,17), -- Impossible Barrels: orange barrels of oranges in a 4-color level
	MakeLevel("BarrelReel",				245,17), -- Reel Crazy: like Barrels of Crazy, but barrels are all in reels
	})
end

if TotalLands >= 18 then
	table.insertall(Levels, {
	-- LAND 18
	MakeLevel("MakerIntro",	246,18),
	MakeLevel("CoconutFall",	247,18),
	MakeLevel("BlockedTubes",	248,18),
	MakeLevel("OrangeChute",	249,18),
	MakeLevel("AmberMakers",	250,18),
	MakeLevel("BombChutes",	251,18),
	MakeLevel("WhackARed",	252,18),
	MakeLevel("PowersCombined",	253,18),
	MakeLevel("Scarcity",		254,18),
	MakeLevel("Rube2",			255,18),
	MakeLevel("ThinkBig",		256,18),
	MakeLevel("Stripes",		257,18),
	MakeLevel("MakerFloor",		258,18),
	MakeLevel("SpaceCrunch",	259,18),
	MakeLevel("CrazyMakers",	260,18),
	})
end

if TotalLands >= 19 then
	table.insertall(Levels, {
	-- LAND 19
	MakeLevel("LockTest",			261,19),
	MakeLevel("ColoredBarrelLocks",	262,19),
	MakeLevel("LockAmber",			263,19),
	MakeLevel("SkinnyLocks",		264,19),
	MakeLevel("LockedGarden",		265,19),
	MakeLevel("LockChests",			266,19),
	MakeLevel("LockPattern",		267,19),
	MakeLevel("BarrierBusting",		268,19),
	MakeLevel("Dispensers",			269,19),
	MakeLevel("LockCols",			270,19),
	MakeLevel("BombLocks",			271,19),
	MakeLevel("LockCoconuts",		272,19),
	MakeLevel("FaultLines",			273,19),
	MakeLevel("GrowingSpace",		274,19),
	MakeLevel("PandorasLocks",		275,19),
	})
end

if TotalLands >= 20 then
	table.insertall(Levels, {
	-- LAND 20
	MakeLevel("ItTakesTwo",             276,20),
	MakeLevel("JailhouseRocks5",		277,20), --good alternatives = JailhouseRocks3, JailhouseRocks5, JailhouseRocks6, JailhouseRocks7
	MakeLevel("BehindBars2",            278,20),
	MakeLevel("DoubleQuarantine2",		279,20),
	MakeLevel("JailBroken4",             280,20), --good alternatives = JailBroken, JailBroken2, JailBroken3, Jailbroken4
	MakeLevel("Barrel2Test",            281,20),
	MakeLevel("BarreledTreasure",       282,20),
	MakeLevel("HedgeTrimmers",          283,20),
	MakeLevel("DoABarrelRoll",          284,20),
	MakeLevel("DoNotOpen2",             285,20),
	MakeLevel("DoubleBarrelSlotGun",	286,20),
	MakeLevel("JailTrimmers",           287,20),
	MakeLevel("BailJarrel",             288,20),
	MakeLevel("PrisonLattice",          289,20),
    MakeLevel("PrisonersDualemma4",		290,20),
	})
end

if TotalLands >= 21 then
	table.insertall(Levels, {
	-- LAND 21
	MakeLevel("Battlement1",            291,21),
	MakeLevel("PictureFrame",           292,21),
	MakeLevel("Persistence",			293,21),
	MakeLevel("VerticalHits",           294,21),
	MakeLevel("MakerOfBoulders",        295,21),
	MakeLevel("DropIt1",				296,21),
	MakeLevel("Sand",					297,21),
	MakeLevel("ColorWheel",             298,21),
	MakeLevel("BurningBush",            299,21),
	MakeLevel("SerialThreats",          300,21),
	MakeLevel("Quarry",					301,21),
	MakeLevel("BurstCrazy",             302,21),
	MakeLevel("Spider",					303,21),
	MakeLevel("HashTag1",				304,21),
	MakeLevel("Re:Fuse",				305,21),
	})
end

if TotalLands >= 22 then
	table.insertall(Levels, {
	-- LAND 22
	MakeLevel("PatternEater",			306,22),
	MakeLevel("BurstHoarding",			307,22),
	MakeLevel("OrangeEaters",			308,22),
	MakeLevel("ColumnEater",			309,22),
	MakeLevel("Stonewalling",           310,22),
	MakeLevel("UseItOrLoseIt_Jon",	    311,22),
	MakeLevel("ChestEaters",            312,22),
	MakeLevel("HalfAndHalf",			313,22),
	MakeLevel("RockCollection",         314,22),
	MakeLevel("BurstTraps",				315,22),
	MakeLevel("Treasury",				316,22),
	MakeLevel("BadBarrels",             317,22),
	MakeLevel("AutoTrimmer",			318,22),
	MakeLevel("ImpossibleChests",		319,22),
	MakeLevel("SpinEater",				320,22),
	})
end

if TotalLands >= 23 then
	table.insertall(Levels, {
	-- LAND 23
	MakeLevel("BasicBoxes",				321,23),
	MakeLevel("Pinata",					322,23),
	MakeLevel("LockedBoxes",			323,23),
	MakeLevel("ColorSpout",				324,23),
	MakeLevel("TowerToTheClouds",		325,23),
	MakeLevel("UpAndDownAgain",			326,23),
	MakeLevel("SuperWall",				327,23),
	MakeLevel("UnwelcomeGifts",			328,23),
	MakeLevel("BoxSpiral",				329,23),
	MakeLevel("Plumbing",				330,23),
	MakeLevel("LockedLava",				331,23),
	MakeLevel("PresentPlacer",			332,23),
	MakeLevel("AirDrop",              	333,23),
	MakeLevel("TowerRaid",				334,23),
	MakeLevel("Regifting",				335,23),
	})
end

if TotalLands >= 24 then
	table.insertall(Levels, {
	-- LAND 24
	MakeLevel("YayPortal",				336,24), -- easy intro portal level (chest drop)
	MakeLevel("TwoTowers",				337,24), -- portal pattern match
	MakeLevel("PortalsGemEaters",		338,24), -- portal chest drop
	MakeLevel("TriplePortalRow",		339,24), -- portal bomb match
	MakeLevel("BurstChain",				340,24), -- no portals
	MakeLevel("RockFall",				341,24), -- portal boulder removal
	MakeLevel("OverAndOver",			342,24), -- portal chest drop	
	MakeLevel("GemShower",				343,24), -- no portals
	MakeLevel("SplitFall",				344,24), -- portal chest drop
	MakeLevel("FillTheTank",			345,24), -- portal pattern match
	MakeLevel("LongWayDown2",			346,24), -- portal chest drop
	MakeLevel("LavaChecks",				347,24), -- no portals
	MakeLevel("Rube3",					348,24), -- portal chest drop (but more focused on bursts)
	MakeLevel("BombChase",				349,24), -- portal bomb (boulder) match
	MakeLevel("CornerBlocks",			350,24), -- portal vine removal
	})
end

if TotalLands >= 25 then
	table.insertall(Levels, {
	-- LAND 25
	MakeLevel("LaserWelcome",			351,25),
	MakeLevel("LaserWall",				352,25),
	MakeLevel("LaserDispenser",			353,25),
	MakeLevel("LaseTheRainbow",			354,25),
	MakeLevel("PewPew",					355,25),
	MakeLevel("OlympusMons",			356,25),
	MakeLevel("LockTumblers",			357,25),
	MakeLevel("LaserSusan",             358,25),
	MakeLevel("SeekAndDestroy",			359,25),
	MakeLevel("SlowFill",				360,25),
	MakeLevel("LengthyLaser",			361,25),
	MakeLevel("MoltenCore",             362,25),
	MakeLevel("RandomRocks",			363,25),
	MakeLevel("WildLasers",				364,25),
	MakeLevel("ImpossibleBlast",		365,25),
	})
end

if TotalLands >= 26 then
	table.insertall(Levels, {
	-- LAND 26
	MakeLevel("FirstFrost",		366,26),
	MakeLevel("NoTwoAlike",		367,26),
	MakeLevel("ShootingGallery",368,26),
	MakeLevel("AnInconvenientTrough",369,26),
	MakeLevel("Permafrost",		370,26),
	MakeLevel("StainedGlass",	371,26),
	MakeLevel("BackAndForth",	372,26),
	MakeLevel("GlacialPace",	373,26),
	MakeLevel("IceChest",		374,26),
	MakeLevel("Icebox",			375,26),
	MakeLevel("Cryolock",		376,26),
	MakeLevel("FreezerBurn",	377,26),
	MakeLevel("TropicalIce",	378,26),
	MakeLevel("TripleSpreader",	379,26),
	MakeLevel("IceBarrels",		380,26),
	})
end

if TotalLands >= 27 then
	table.insertall(Levels, {
	-- LAND 27
	MakeLevel("BalloonsTest",		381,27),
	MakeLevel("BalloonMakerTest",	382,27),
	MakeLevel("BalloonTreasureTest",383,27),
	MakeLevel("Amulet",				384,27),
	MakeLevel("BombChest2",			385,27),
	MakeLevel("ParadiseRises",		386,27),
	MakeLevel("FloatOn",			387,27),
	MakeLevel("HeliumTank",			388,27),
	MakeLevel("DuelingMakers",		389,27),
	MakeLevel("Meltdown",			390,27),
	MakeLevel("LaserDarts",			391,27),
	MakeLevel("BalloonReels",		392,27),
	MakeLevel("BlockAndBust",		393,27),
	MakeLevel("CoconutForever",		394,27),
	MakeLevel("BalloonBelt",		395,27),
	})
end

if TotalLands >= 28 then
	table.insertall(Levels, {
	-- LAND 28
	MakeLevel("WindowRows",			396,28),
	MakeLevel("NoTouchTreasure",	397,28),
	MakeLevel("Antigravity",		398,28),
	MakeLevel("Oven",				399,28),
	MakeLevel("GlassSafe",			400,28),
	MakeLevel("HiddenCloud",		401,28),
	MakeLevel("GlassHouses",		402,28),
	MakeLevel("BalloonStand",		403,28),
	MakeLevel("ColorBalance",		404,28),
	MakeLevel("Cornerstone",		405,28),
	MakeLevel("MixAndMatch",		406,28),
	MakeLevel("GlassCheckerboard",	407,28),
	MakeLevel("BlockedBombs",		408,28),
	MakeLevel("GlassHalfEmpty",		409,28),
	MakeLevel("GetWild",			410,28),
	MakeLevel("StuckBalloons",		411,28),
	MakeLevel("DelayedGratification",412,28),
	MakeLevel("UnstableBalloons",	413,28),
	MakeLevel("BalloonTube",		414,28),
	MakeLevel("SteelPlacer",		415,28),
	MakeLevel("Passersby",		416,28),
	MakeLevel("InCaseOfEmergency",	417,28),
	MakeLevel("BlockedBounce",		418,28),
	MakeLevel("RoomForRed",		419,28),
	MakeLevel("LockedCabinet",		420,28),

	})
end

if TotalLands >= 29 then
	table.insertall(Levels, {
	-- LAND 29
	MakeLevel("KeyGemTest",			421,29),
	MakeLevel("PrisonBreak",		422,29),
	MakeLevel("Parfait",			423,29),
	MakeLevel("RotatingLasers",		424,29),
	MakeLevel("TimeToBlast",		425,29),
	MakeLevel("HitPay",				426,29),
	MakeLevel("ScrollingBursts",	427,29),
	MakeLevel("SteelDig",			428,29),
	MakeLevel("Hypobaric",		    429,29),
	MakeLevel("AxeWarp",			430,29),
	MakeLevel("CoalVein",			431,29),
	MakeLevel("LavaDam",		    432,29),
	MakeLevel("RockCity",		    433,29),
	MakeLevel("Stratum",            434,29),
	MakeLevel("BalloonParty",		435,29),
	MakeLevel("DunkTank",			436,29),
	MakeLevel("LongestChest",		437,29),
	MakeLevel("PrizeInside",		438,29),
	MakeLevel("LaserJail",			439,29),
	MakeLevel("BlockedPlumbing",	440,29),
	MakeLevel("GoldDrop",			441,29),
	MakeLevel("SideCut",		    442,29),
	MakeLevel("DugTooDeep",			443,29),
	MakeLevel("LostKeys",			444,29),
	MakeLevel("JungleTrap",			445,29),

	})
end

if TotalLands >= 30 then
	table.insertall(Levels, {
	-- LAND 30
	MakeLevel("SwitchClicker",			446,30),
	MakeLevel("MoveTheBlocker",			447,30),
	MakeLevel("TwoWorlds",				448,30),
	MakeLevel("ReplenishingBedrock",	449,30),
	MakeLevel("Clearance",				450,30),
	MakeLevel("Metamorphosis",			451,30),
	MakeLevel("RaceTheClock",			452,30),
	MakeLevel("FlickeringWall_Updated",	453,30),
	MakeLevel("SkeetShooting",			454,30),
	MakeLevel("EmergencyEscape",	    455,30),
	MakeLevel("AxeChase",				456,30),
	MakeLevel("SwitchSequence",			457,30),
	MakeLevel("Teleporter",				458,30),
	MakeLevel("LongRise",				459,30),
	MakeLevel("GravitySwapper",        	460,30),
	MakeLevel("SwitchSequence2",		461,30),
	MakeLevel("ControlRoom",			462,30),
	MakeLevel("BombColumns",	        463,30),
	MakeLevel("DefusingSwitch",			464,30),
	MakeLevel("TowerDefense2",			465,30),
	MakeLevel("OrangeFactory",       	466,30),
	MakeLevel("TasteTheRainbow",        467,30),
	MakeLevel("FlameRetardant",			468,30),
	MakeLevel("BreakerBox",				469,30),
	MakeLevel("LaserToggle",			470,30),
	})
end

if TotalLands >= 31 then
	table.insertall(Levels, {
	-- LAND 31
	MakeLevel("RingChase",				471,31),
	MakeLevel("RingGrid",      			472,31),
	MakeLevel("PharaohsTreasure",		473,31),
	MakeLevel("RingSearch",				474,31),
	MakeLevel("SuperColumns",			475,31),
	MakeLevel("WhatsInTheBox",			476,31),
	MakeLevel("BrassRing",				477,31),
	MakeLevel("FreeRefills",			478,31),
	MakeLevel("RainbowStripe",			479,31),
	MakeLevel("GiftWrapping",			480,31),
	MakeLevel("SquishyPattern",			481,31),
	MakeLevel("MountDoom",				482,31),
	MakeLevel("PunchClockVillain",		483,31),
	MakeLevel("RainDance",	    		484,31),
	MakeLevel("GarbagePile",			485,31),
	MakeLevel("DoABarrelRoll2",			486,31),
	MakeLevel("Pinpoint",				487,31),
	MakeLevel("RingMaker",     			488,31),
	MakeLevel("JewelryShop",			489,31),
	MakeLevel("RingStamper",			490,31),
	MakeLevel("Bobbing",	          	491,31),
	MakeLevel("QuickThaw",				492,31),
	MakeLevel("PresentPickup",			493,31),
	MakeLevel("DropsOfGold",	      	494,31),
	MakeLevel("PandorasLocks2",			495,31),
	})
end

if TotalLands >= 32 then
	table.insertall(Levels, {
	-- LAND 32
	MakeLevel("TreasureDispenser",			496,32),
	MakeLevel("CurtainCutter",				497,32),
	MakeLevel("CallforBackup",				498,32),
	MakeLevel("BadSwitches",				499,32),
	MakeLevel("ThingsThatYouSee",			500,32),
	MakeLevel("DoubleAct",					501,32),
	MakeLevel("FillTheVoid",				502,32),
	MakeLevel("OffRamp",					503,32),
	MakeLevel("Grinch",						504,32),
	MakeLevel("RainbowSwitch",				505,32),
	MakeLevel("SnoozeButton",				506,32),
	MakeLevel("PowerUp",					507,32),
	MakeLevel("TwoDays",					508,32),
	MakeLevel("WabbitSeason",				509,32),
	MakeLevel("12RedBlue",					510,32),
	MakeLevel("LightsOut",					511,32),
	MakeLevel("RingForService",				512,32),
	MakeLevel("PickaxeFactory",				513,32),
	MakeLevel("SkyScraper",					514,32),
	MakeLevel("TargetLocked",				515,32),
	MakeLevel("TheCreature",				516,32),
	MakeLevel("FilterMachine",				517,32),
	MakeLevel("ReverseReverse",				518,32),
	MakeLevel("MatchForMore",				519,32),
	MakeLevel("DangerousChoice",			520,32),
	})
end

if TotalLands >= 33 then
	table.insertall(Levels, {
	-- LAND 33
	MakeLevel("CodenameRainbow",            521,33),
    MakeLevel("SafeInside",                 522,33),
    MakeLevel("SafeTreasure",               523,33),
    MakeLevel("Trespasser",                 524,33),
    MakeLevel("BigHeist",                   525,33),
    MakeLevel("SurpriseParole",             526,33),
    MakeLevel("LaserProof",                 527,33),
    MakeLevel("HatchingAPlan",              528,33),
    MakeLevel("GreenOrBlue",                529,33),
    MakeLevel("CombiLock",                  530,33),
    MakeLevel("GatheringSupplies",          531,33),
    MakeLevel("StepByStepByStep",           532,33),
    MakeLevel("AtBay",                      533,33),
    MakeLevel("SideShooter",                534,33),
    MakeLevel("SafeBelt",                   535,33),
    MakeLevel("StrongArm",                  536,33),
    MakeLevel("TripWire",                   537,33),
    MakeLevel("LaserField",                 538,33),
    MakeLevel("HiddenSafe",                 539,33),
    MakeLevel("RemoteSafe",                 540,33),
    MakeLevel("CutToTheChase",              541,33),
    MakeLevel("DangerSafes",                542,33),
    MakeLevel("GreatEscape",                543,33),
    MakeLevel("SafeBombs",                  544,33),
    MakeLevel("BigStruggle",                545,33),
	})
end

if TotalLands >= 34 then
	table.insertall(Levels, {
	-- LAND 34
	MakeLevel("Turnover",					546,34),
	MakeLevel("SharkBait",					547,34),
	MakeLevel("DropTheAnchor",			    548,34),
	MakeLevel("RisingTide",				    549,34),
	MakeLevel("TidalWave",				    550,34),
	MakeLevel("SteelFlood",				    551,34),
	MakeLevel("ToughBombs",                 552,34),
	MakeLevel("Goldfishing",				553,34),
	MakeLevel("SaveTheBooty",				554,34),
	MakeLevel("SideSlide",            		555,34),
	MakeLevel("GrapesOfWrath",              556,34),
	MakeLevel("PlunderingRings",            557,34),
	MakeLevel("BalloonEater",               558,34),
	MakeLevel("FavoriteLetter",			    559,34),
	MakeLevel("Timing",						560,34),
	MakeLevel("DavyJones",					561,34),
	MakeLevel("CuckoooForCoconuts",		    562,34),
	MakeLevel("XSpot",				        563,34),
	MakeLevel("HungryChest",				564,34),
	MakeLevel("JammingMachine",			    565,34),
	MakeLevel("CheckeredEaters",			566,34),
	MakeLevel("EmptyTheTraps",              567,34),
	MakeLevel("BombOrBooty",                568,34),
	MakeLevel("LavaLevel",                  569,34),
	MakeLevel("PirateBattle",               570,34),
	})
end

--
-- TEST LEVELS:
--
table.insertall(Levels, {
	--MakeLevel("GoldTest",	108,8),	   
	--MakeLevel("SteelTest",	109,8),	   
	--MakeLevel("BarrierTest",	110,8),

	--[[
	MakeLevel("VinesTest",		111,8),
	MakeLevel("SplashTest",	112,8),
	MakeLevel("BombTest",		113,8),
	MakeLevel("RecurringBombTest",114,8),
	MakeLevel("StoneTest",		115,8),

	MakeLevel("BarrelTest",	116,8),
	MakeLevel("PatternMatcher",117,8),
	MakeLevel("TwoLines",		118,8),
	MakeLevel("Crates!",		 119,8),
	MakeLevel("CratePattern",	 120,8),
	MakeLevel("CrateJump",		 121,8),
	MakeLevel("Escape!",		 122,8),
	MakeLevel("LetMeOut",		 123,8),
	MakeLevel("DiamondFall",	 124,8),
	MakeLevel("Some1Some4",	 125,8),
	--]]
	
	--[[
	MakeLevel("CrateFall",		 105,4),
	MakeLevel("Temple",			 106,4),
	MakeLevel("CustomColors",	 107,4),
	MakeLevel("TreasureChest",	 108,4),
	--]]

	--[[
	MakeLevel("GemEaterTest",102,1),
	MakeLevel("PortalTest",103,1),
	MakeLevel("FirstScore",104,1),
	MakeLevel("FirstScore",105,1),
	MakeLevel("FirstScore",106,1),
	MakeLevel("FirstScore",107,1),
	MakeLevel("FirstScore",108,1),
	MakeLevel("FirstScore",109,1),
	MakeLevel("FirstScore",110,1),
	--]]

	--[[
	MakeLevel("SafesTest2",111,1),
	--]]
})

SideLevels = {
	[1] = {
		-- Invisible side levels used for launching from TODAY menu
		MakeSidePathLevel("PompeiiFive", 1, 1, 1),
		MakeSidePathLevel("LaserLab", 2, 1, 1),
		MakeSidePathLevel("LaserLab2", 3, 1, 1),
		MakeSidePathLevel("LaserLab3", 4, 1, 1),
		MakeSidePathLevel("LaserLab4", 5, 1, 1),
		MakeSidePathLevel("WindowLab1", 6, 1, 1),
		MakeSidePathLevel("WindowLab2", 7, 1, 1),
		MakeSidePathLevel("WindowLab3", 8, 1, 1),
		MakeSidePathLevel("CrowdedLab1", 9, 1, 1),
		MakeSidePathLevel("CrowdedLab2", 10, 1, 1),
		MakeSidePathLevel("CrowdedLab3", 11, 1, 1),
		MakeSidePathLevel("BalloonLab1", 12, 1, 1),
		MakeSidePathLevel("BalloonLab2", 13, 1, 1),
		MakeSidePathLevel("BalloonLab3", 14, 1, 1),
		MakeSidePathLevel("LockLab1", 15, 1, 1),
		MakeSidePathLevel("LockLab2", 16, 1, 1),
		MakeSidePathLevel("LockLab3", 17, 1, 1),
        MakeSidePathLevel("TreasureLab1", 18, 1, 1),
        MakeSidePathLevel("TreasureLab2", 19, 1, 1),
        MakeSidePathLevel("TreasureLab3", 20, 1, 1),
		MakeSidePathLevel("IceLab1", 21, 1, 1),
		MakeSidePathLevel("IceLab2", 22, 1, 1),
		MakeSidePathLevel("IceLab3", 23, 1, 1), 
		MakeSidePathLevel("AmberLab1", 24, 1, 1),
		MakeSidePathLevel("AmberLab2", 25, 1, 1),
		MakeSidePathLevel("AmberLab3", 26, 1, 1),
		MakeSidePathLevel("BurstReelLab1",27,1,1), 
		MakeSidePathLevel("BurstReelLab2",28,1,1),
		MakeSidePathLevel("BurstReelLab3",29,1,1),
		MakeSidePathLevel("BombLab1", 30, 1, 1),
		MakeSidePathLevel("BombLab2", 31, 1, 1),	
		MakeSidePathLevel("BombLab3", 32, 1, 1),
		MakeSidePathLevel("LavaLab1", 33, 1, 1),
		MakeSidePathLevel("LavaLab2", 34, 1, 1),
		MakeSidePathLevel("LavaLab3", 35, 1, 1),
		MakeSidePathLevel("GemEaterLab1",36,1,1), 
		MakeSidePathLevel("GemEaterLab2",37,1,1), 
		MakeSidePathLevel("GemEaterLab3",38,1,1),

		MakeSidePathLevel("CoconutLab1", 39, 1, 1),
		MakeSidePathLevel("CoconutLab2", 40, 1, 1),
		MakeSidePathLevel("CoconutLab3", 41, 1, 1),
		MakeSidePathLevel("FourElementsLab1", 42, 1, 1),
		MakeSidePathLevel("FourElementsLab2", 43, 1, 1), 
		MakeSidePathLevel("FourElementsLab3", 44, 1, 1),
		MakeSidePathLevel("HoleLab1", 45, 1, 1),
		MakeSidePathLevel("HoleLab2", 46, 1, 1),
		MakeSidePathLevel("HoleLab3", 47, 1, 1), 
		
		MakeSidePathLevel("Continuation_Copy", 48, 1, 1), 
		MakeSidePathLevel("ShootingGallery_Copy", 49, 1, 1), 
		MakeSidePathLevel("Rube2_Copy", 50, 1, 1), 
		MakeSidePathLevel("Land15Side4_side_Copy", 51, 1, 1), 
		MakeSidePathLevel("WaffleIron_Copy", 52, 1, 1), 
		MakeSidePathLevel("TrickLevel_Copy", 53, 1, 1), 
		MakeSidePathLevel("FillTheTank_Copy", 54, 1, 1),

		MakeSidePathLevel("Minilab1", 55, 1, 1),
		MakeSidePathLevel("Minilab2", 56, 1, 1),
		MakeSidePathLevel("Minilab3", 57, 1, 1),

		MakeSidePathLevel("VineLab1", 58, 1, 1),
		MakeSidePathLevel("VineLab2", 59, 1, 1),
		MakeSidePathLevel("VineLab3", 60, 1, 1),

		MakeSidePathLevel("BarrelLab1", 61, 1, 1),
		MakeSidePathLevel("BarrelLab2", 62, 1, 1),
		MakeSidePathLevel("BarrelLab3", 63, 1, 1),

		MakeSidePathLevel("YellowLab1", 64, 1, 1),
		MakeSidePathLevel("YellowLab2", 65, 1, 1), -- placeholder until Yellow Lab 2 is completed
		MakeSidePathLevel("YellowLab1", 66, 1, 1), -- placeholder until Yellow Lab 3 is completed

		MakeSidePathLevel("SafeLab1", 67, 1, 1),
		MakeSidePathLevel("SafeLab2", 68, 1, 1),
		MakeSidePathLevel("SafeLab3", 69, 1, 1),

		MakeSidePathLevel("BonusLab1", 70, 1, 1),
		MakeSidePathLevel("BonusLab2", 71, 1, 1),
		MakeSidePathLevel("BonusLab3", 72, 1, 1),
	},
	[2] = {
		-- LAND 2
		MakeSidePathLevel("CoverageEmpty3_side",	1,2,13),
		MakeSidePathLevel("BigCrateBoard1_side",	2,2,13),
		MakeSidePathLevel("BigCrateBoard4_side",	3,2,13),
        MakeSidePathLevel("VerticalBursts_side",    4,2,19), --Needs tuning
	},
	[3] = {
	-- LAND 3
		MakeSidePathLevel("StairClear_side",		1,3,21),
		MakeSidePathLevel("DrillDown_side",			2,3,26),
		MakeSidePathLevel("Drop_Empty_Large1_side",	3,3,29),
        MakeSidePathLevel("Drop_Empty1_side",       4,3,33),
	},
	[4] = {
	-- LAND 4
		MakeSidePathLevel("JailCoverage1_side",	1,4,37),	
		MakeSidePathLevel("CustomColor1_side",	2,4,38),	
		MakeSidePathLevel("GemJailScore4_side",	3,4,40),
        MakeSidePathLevel("GemJail1_side",      4,4,45),
	},
	[5] = {
	-- LAND 5
		MakeSidePathLevel("EntrenchedVine_Updated_side",	1,5,57),	
		MakeSidePathLevel("VinesPattern4_side",				2,5,57),	
		MakeSidePathLevel("VinesRemove5_Updated_side",		3,5,62),	
        MakeSidePathLevel("VinesPattern1_side",             4,5,63),
	},
	[6] = {
	-- LAND 6
		MakeSidePathLevel("BarrelsDrop3_side",		1,6,70),	
		MakeSidePathLevel("BarrelsDrop4_side",		2,6,73),	
		MakeSidePathLevel("Barrels5_Updated_side",	3,6,77),
		MakeSidePathLevel("GoWild_side",			4,6,77),	

	},
	[7] = {
	-- LAND 7
		MakeSidePathLevel("BombDungeon3_side",		1,7,84),	
		MakeSidePathLevel("BombDrop1_side",			2,7,87),	
		MakeSidePathLevel("BombFallingCombined_side",	3,7,90),
        MakeSidePathLevel("BombDungeon1_side",	4,7,92),
	},
	[8] = {
	-- LAND 8
		MakeSidePathLevel("Land8_Level8_side",		1,8,100),	
		MakeSidePathLevel("Land8_Level9_side",		2,8,100),	
		MakeSidePathLevel("Land8_Level13_side",		3,8,100),
		MakeSidePathLevel("Land8Side4_side",	    4,8,108),
	},
	[9] = {
	-- LAND 9
		MakeSidePathLevel("BigDiamond_side",		1,9,118),	
		MakeSidePathLevel("GottaCatchEmAll_side",	2,9,118),	
		MakeSidePathLevel("EdgeVines_side",			3,9,121),
		MakeSidePathLevel("Land9Side4_side",		4,9,125),
	},
	[10] = {
	-- LAND 10
		MakeSidePathLevel("BottomVines_side",		1,10,127),	
		MakeSidePathLevel("GemCollect4_side",		2,10,129),	
		MakeSidePathLevel("GemTreasures_side",		3,10,134),
		MakeSidePathLevel("Land10Side4_side",		4,10,136),
	},
	[11] = {
	-- LAND 11
		MakeSidePathLevel("LavaSplash_side",		1,11,143),	
		MakeSidePathLevel("DoubleBoulderBombs_side",		2,11,147),	
		MakeSidePathLevel("LavaChests_side",	3,11,149),
		MakeSidePathLevel("Land11Side4_side",		4,11,150),	
	},
	[12] = {
	-- LAND 12
		MakeSidePathLevel("LavaSeeds_side",		1,12,157),	
		MakeSidePathLevel("BarrelsOfBombs_side",	2,12,164),	
		MakeSidePathLevel("PottedPlants_side",	3,12,167),
		MakeSidePathLevel("Land12Side4_side",	4,12,170),
	},
	[13] = {
	-- LAND 13
		MakeSidePathLevel("SplitLevel_side",		1,13,172),	
		MakeSidePathLevel("HalfReels_side",		2,13,174),
		MakeSidePathLevel("JungleRace_side",	3,13,183),
		MakeSidePathLevel("Land13Side4_side",		4,13,184),
	},
	[14] = {
	-- LAND 14
		MakeSidePathLevel("VineGrowerTest_side",		1,14,194),
		MakeSidePathLevel("EmptySpacesVineGenerator_side", 2, 14, 198),	
		MakeSidePathLevel("BombBarrels_side", 3, 14, 198),
		MakeSidePathLevel("Land14Side4_side", 4, 14, 198),
	},
	[15] = {
	-- LAND 15
		MakeSidePathLevel("CoconutTest_side",		1,15,203),	
		MakeSidePathLevel("BadCoconut_side", 2,15,206),
		MakeSidePathLevel("SkyTop_side", 3, 15, 209),
		MakeSidePathLevel("Land15Side4_side", 4, 15, 213),
	},
	[16] = {
	-- LAND 16
		MakeSidePathLevel("AmberTest_side",		1,16,220),	
		MakeSidePathLevel("Checkers_side", 2,16,223),
		MakeSidePathLevel("AmberPool_side", 3,16,228),
		MakeSidePathLevel("Land16Side4_side", 4,16,229),
	},
	[17] = {
	-- LAND 17
		MakeSidePathLevel("ColoredBarrelTreasure_side",		1,17,236),
        MakeSidePathLevel("FenceTest_side",                 2,17,240),
        MakeSidePathLevel("BurstChest_side",                3,17,243),
		MakeSidePathLevel("Land17Side4_side",                4,17,243),
	},
	[18] = {
	-- LAND 18
		MakeSidePathLevel("MakerIntro_side",		1,18,252),
        MakeSidePathLevel("OrangeChute_side",		2,18,253),
        MakeSidePathLevel("MakerFloor_side",		3,18,258),
		MakeSidePathLevel("Land18Side4_side",		4,18,258),
	},
	[19] = {
	-- LAND 19
		MakeSidePathLevel("LockTest_side",		1,19,266),
        MakeSidePathLevel("LockedGarden_side",      2,19,270),
        MakeSidePathLevel("LockChests_side",		3,19,272),
	},
	[20] = {
	-- LAND 20
		MakeSidePathLevel("ItTakesTwo_side",		1,20,283),
        MakeSidePathLevel("DoNotOpen2_side",		2,20,286),
        MakeSidePathLevel("JailhouseRocks5_side",	3,20,289),
	},
	[21] = {
	-- LAND 21
		MakeSidePathLevel("Battlement1_side",		1,21,296),
        MakeSidePathLevel("PictureFrame_side",		2,21,299),
        MakeSidePathLevel("VerticalHits_side",		3,21,300),
	},
	[22] = {
	-- LAND 22
		MakeSidePathLevel("PatternEater_side",		1,22,312),
        MakeSidePathLevel("OrangeEaters_side",		2,22,314),
        MakeSidePathLevel("HalfAndHalf_side",		3,22,314),
	},
	[23] = {
	-- LAND 23
		MakeSidePathLevel("BasicBoxes_side",		1,23,325),
        MakeSidePathLevel("TowerToTheClouds_side",	2,23,330),
        MakeSidePathLevel("UpAndDownAgain_side",	3,23,331),
	},
	[24] = {
	-- LAND 24
		MakeSidePathLevel("YayPortal_side",			1,24,342),
        MakeSidePathLevel("RockFall_side",          2,24,342),
        MakeSidePathLevel("OverAndOver_side",		3,24,348),
	},
	[25] = {
	-- LAND 25
		MakeSidePathLevel("LaserWelcome_side",		1,25,353),
		MakeSidePathLevel("LockTumblers_side",		2,25,361),
		MakeSidePathLevel("LengthyLaser_side",		3,25,361),	
	},
	[26] = {
	-- LAND 26
		MakeSidePathLevel("GlacialPace_side",	1,26,369),
		MakeSidePathLevel("NoTwoAlike_side",	2,26,373),
		MakeSidePathLevel("IceChest_side",		3,26,376),	
	},
	[27] = {
	-- LAND 27
		MakeSidePathLevel("Fountain_side",	1,27,382),
		MakeSidePathLevel("BalloonWall",	2,27,385),	
		MakeSidePathLevel("ClearTheAir_side",	3,27,394),
	},
	[28] = {
	-- LAND 28
		MakeSidePathLevel("Conservatory",	1,28,397),
		MakeSidePathLevel("SuperSafe",	2,28,400),
		MakeSidePathLevel("PictureFrame2",	3,28,405),
		MakeSidePathLevel("DelayedGratification2",	4,28,414),
		MakeSidePathLevel("ColorImbalance",	5,28,416),
	},
    [29] = {
    -- LAND 29
        MakeSidePathLevel("BarrelEscape",		1,29,423),
        MakeSidePathLevel("ReleaseTheBurst",	2,29,426),
        MakeSidePathLevel("PayDirt",			3,29,434),
        MakeSidePathLevel("PickaxePopup",		4,29,440),
        MakeSidePathLevel("Tripmining",			5,29,443),
    }
}

if TotalLands >= 30 then
	SideLevels[30] = {
    -- LAND 30
        MakeSidePathLevel("Crabgrass",				1,30,448),
        MakeSidePathLevel("AirLock",				2,30,451),
        MakeSidePathLevel("Binary",					3,30,461),
        MakeSidePathLevel("SafetyCage",				4,30,464),
		MakeSidePathLevel("DefusingSwitch_side",	5,30,466),
    }
end

if TotalLands >= 31 then
	SideLevels[31] = {
    -- LAND 31
        MakeSidePathLevel("RecoveringPattern",		1,31,474),
		MakeSidePathLevel("RingChase_side",       	2,31,477),
        MakeSidePathLevel("SayWhen",				3,31,480),
        MakeSidePathLevel("RainbowStripe_side", 	4,31,482),
        MakeSidePathLevel("GarbagePile_side",   	5,31,489),
    }
end

if TotalLands >= 32 then
	SideLevels[32] = {
    -- LAND 32
        MakeSidePathLevel("DucksinaRow",				1,32,498),
		MakeSidePathLevel("ColorSwap",					2,32,506),
        MakeSidePathLevel("RainbowCycle",				3,32,514),
        MakeSidePathLevel("RailroadTracks", 			4,32,517),
        MakeSidePathLevel("PandorasSwitches",			5,32,517),
    }
end

if TotalLands >= 33 then
	SideLevels[33] = {
    -- LAND 33
        MakeSidePathLevel("JungleSafes",		1,33,523),
		MakeSidePathLevel("Hustle",				2,33,523),
        MakeSidePathLevel("SafePlacer",				3,33,529),
        MakeSidePathLevel("ShrinkingSafes", 	4,33,539),
        MakeSidePathLevel("RemoteSafe2",		5,33,543),
    }
end

if TotalLands >= 34 then
	SideLevels[34] = {
    -- LAND 34
        MakeSidePathLevel("ClearPath",		    1,34,551),--needs minimum 551
		MakeSidePathLevel("FaultySieve",		2,34,552),--needs minimum 551
        MakeSidePathLevel("FeedingTime",		3,34,556),--needs minimum 551
        MakeSidePathLevel("CannonBall",	 	    4,34,557),--needs minimum 561
        MakeSidePathLevel("LavaPiper",		    5,34,564),--needs minimum 561
    }
end
