
local ScreenScaleFactor = 2
local TOP_SCORE_SPACING = 28
local Debug_Alpha = 0 --128

local iPhone5_menu_nudge = 30
--alignmentOffset = {0, APP.screenSize.y * 0.2},

Scripts.AppScoreRowHighlighter = function (self)
	-- Script Settings:
	local fadeTimeInMS = 1500
	local initialDelay = 250
	local brightestAlpha = 192
	local darkestAlpha = 96

	-- Script Body:
	self.alpha = brightestAlpha
	ScriptDelay(initialDelay)
	while true do
		Scripts.Interpolate(self, "alpha", darkestAlpha, fadeTimeInMS)
		Scripts.Interpolate(self, "alpha", brightestAlpha, fadeTimeInMS)
	end
end

Resources.AppScoreRow = Resources.Actor:new {
	resource = "Actor",
	image = "namePanel.png",
	
	ClearValues = function (self)
		self.scoreRowNumber.text = STRINGS("")
		self.playerName.text = STRINGS("")
		self.gameScore.text = STRINGS("")
		self.highlightPanel:KillTaggedScripts("AppScoreRowHighlighter")
		self.highlightPanel.visible = false
	end,
	
	--[[
	SetValues = function (self, values)
		self.gameCenterIcon.visible = (values.isGameCenter == true)
		if values.isGameCenter then
			--self.scoreRowNumber.alignmentOffset = {33, 2}
			self.playerName.alignmentOffset = {45, 2}
		else
			--self.scoreRowNumber.alignmentOffset = {10, 2}
			self.playerName.alignmentOffset = {25, 2}
		end
	end,
	--]]
	
	SetValues = function (self, values)
		self.highlightPanel.visible = (values.isCurrentPlayer == true)
		if values.isCurrentPlayer then
			if not self.highlightPanel:HasTaggedScript("AppScoreRowHighlighter") then
				self.highlightPanel:AddScript("AppScoreRowHighlighter", "")
			end
		else
			self.highlightPanel:KillTaggedScripts("AppScoreRowHighlighter")
		end
	end,

	highlightPanel = Resources.Actor:new {
		resource = "Actor",
		image = "namePanelHighlight.png",
		visible = false,
		zorder = 0,
	},

	gameCenterIcon = Resources.Actor:new {
		resource = "Actor",
		image = "GameCenterButton.png",
		size = {16, 16},
		alignment = "MidLeft",
		alignmentOffset = {25, 1},
		visible = false,
		zorder = 1,
	},

	scoreRowNumber = Resources.Label:new {
		resource = "Label",
		font = {
			fontName = "dabd____.ttf",
			fontSize = 16,
		},
		fgColor = "White",
		shadowColor = "Black",
		shadowOffset = {1,1},
		shadowAlpha = 100,
		widthLimit = 30,
		SetValue = function (self, value)
			self.text = string.format("%s.", tostring(value))
		end,
		alignment = "MidLeft",
		alignmentOffset = {10,0},
		zorder = 1,
	},
	
	playerName = Resources.Label:new {
		resource = "Label",
		font = {
			fontName = "dabd____.ttf",
			fontSize = 16,
		},
		fgColor = "White",
		shadowColor = "Black",
		shadowOffset = {1,1},
		shadowAlpha = 100,
		widthLimit = 200,
		alignment = "MidLeft",
		alignmentOffset = {25,0},
		zorder = 1,
	},
	
	gameScore = Resources.Label:new {
		resource = "Label",
		font = {
			fontName = "dabd____.ttf",
			fontSize = 16,
		},
		fgColor = "White",
		shadowColor = "Black",
		shadowOffset = {1,1},
		shadowAlpha = 100,
		widthLimit = 200,
		SetValue = function (self, value)
			local score = tonumber(value)
			if score then
				self.text = Int2Comma(score)
			else
				self.text = tostring(score)
			end
		end,
		alignment = "MidRight",
		alignmentOffset = {-10,0},
		zorder = 1,
	},
}

function MarkAsNew(item)
	logprint("MarkAsNew")
	if(item ~= NIL) then
		item.NewMarker = Resources.NewMarker
		if(item.newOffset) then
			item.NewMarker.alignmentOffset = item.newOffset
		end
	end
end

function MarkAsOld(item)
	if(item ~= NIL) then
		if(item.NewMarker ~= NIL) then
			item.NewMarker:Destroy()
		end
	end
end

Resources.NewMarker = Resources.NotificationCircle:new {
	resource = "NotificationCircle",
	zorder = 25,
	layer = 2,
	visible = false,
}

local playPanel_y_offset = -20
logoTopScoresPanel_y_offset = Layout{16,pc=16+20}
local nudge_bottom_button_y = 3

Resources.OriginalMenuScreen = {
	deviceOrientation = "Portrait",
	
	backgroundImage = Resources.Actor:new {
		resource = "Actor",
		image = "menu_background.png",
		zorder = -3,
		scripts = {"SparkleRandomly"},--"SparkRandomly"},
	},

	discoBallFX = Resources.Actor:new {
		resource = "Actor",
		image = "fx_discoBall_1.2.png",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,0},
		alpha = 28,
		zorder = -2,
		scripts = {
			function (self)
				local revolutionsPerSecond = 0.025
				RotateAtSpeed(self, {revolutions=revolutionsPerSecond})
			end,
		},
		shaderType = "SHADERTYPE_ADDITIVEALPHABLEND",
		visible = true,
	},
	
	appLogo = Resources.Actor:new {
		resource = "Actor",
		image = Layout{"menu_logo_medium.png", pc="logoMed.png"},
		alignment = "MidTop",
		--alignmentOffset = {0,10+logoTopScoresPanel_y_offset},
		--alignmentOffset = {0,(UseIPhone5AppAssets() and 28+logoTopScoresPanel_y_offset) or 10+logoTopScoresPanel_y_offset},
		alignmentOffset = Layout {{0,10+logoTopScoresPanel_y_offset}, iphone5={0,28+logoTopScoresPanel_y_offset}, pc={0,16}},

		--alignmentOffset = {0, APP.screenSize.y * 0.045},
		zorder = -1,
		scripts = {"SparkleRandomly"},
	},
	
	versionInfo = Resources.Label:new {
		resource = "Label",
		text = GetVersionAsString(),
		alignment = "MidBottom",
		alignmentOffset = {0, -9},
		fgColor = "White",
		shadowColor = "Black",
		shadowOffset = {1,1},
		font = {
			fontName = "arial.ttf",
			fontSize = 9,
		},
		visible = false,
	},
	
	-- BEGIN BFG BUTTONS
	
	BFGMoreGamesButton = Resources.AppMenuButton:new {
		resource = "AppMenuButton",
		text = STRINGS("More Games"),
		alignment = "MidBottom",
		alignmentOffset = {0, 4},
		scripts = {"SparkleRandomly"},
		textLineSpacing = 3,
		OnButtonClick = function (self, event)
			--logprint("OnButtonClick MoreGames")
			if self ~= event.button then return false end
			if BIGFISH then
				BIGFISH:ShowMoreGames()
			end
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 2,
		visible = false,
	},
	
	BFGTellAFriendButton = Resources.AppMenuButton:new {
		resource = "AppMenuButton",
		text = STRINGS("Tell A Friend"),
		alignment = "BottomRight",
		alignmentOffset = {20,-140},
		scripts = {"SparkleRandomly"},
		textLineSpacing = 3,
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			if BIGFISH then
				BIGFISH:ShowTellAFriend();
			end
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 1,
		visible = false,
	},
	
	BFGRatingButton = Resources.AppMenuButton:new {
		resource = "AppMenuButton",
		text = STRINGS("Rate Now"),
		alignment = "BottomRight",
		alignmentOffset = {20,-90},
		scripts = {"SparkleRandomly"},
		textLineSpacing = 3,
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			if BIGFISH then
				BIGFISH:Rating_MainMenuRateApp();
			end
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 1,
		visible = false,
	},
	
	-- Per an update from BFG, this isn't needed any more
	BFGSplashNewsletterButton = Resources.Button:new {
		resource = "Button",
		image = "iSplash_blue1-iphone.png",
		alignment = BottomLeft,
		alignmentOffset = {-40,-90},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			if BIGFISH then
				BIGFISH:ShowSplashNewsletter();
			end
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		visible = false,
	},
	
	-- END BFG BUTTONS

	GameCenterButton = Resources.GameCenterButton:new {
		resource = "GameCenterButton",
		image = "GameCenterButton_cascade.png",
		alignment = TopRight,
		alignTo = TopRight,
		alignmentOffset = Layout{{-7, 9}, ipad={-6,8}},
		HandleGameCenterReadyClick = function (self)
			GuruGameCenter_ShowLeaderboard()
			return true
		end,
	},

	FacebookButton = Resources.Button:new {
		resource = "Button",
		alignment = TopLeft,
		alignTo = TopLeft,
		alignmentOffset = Layout{{7, 9}, ipad={8,8}, pc={7,9+30}},

		--size = {30, 30},
		image = "FacebookLogo_cascade.png",

		OnButtonClick = function (self, event)
			if event.button ~= self then
				return
			end
            -- I think config gets stored and doesn't update from the default
            -- And I'm changing this from what it was so am forcing it here:
            Config.AppFacebookURL = "https://www.facebook.com/255393097898012",
			logprint(tostring(Config.AppFacebookURL))
			if IsString(Config.AppFacebookURL) then
				OpenWebPage(Config.AppFacebookURL)
			end
			return true
		end,	
	},
	
	leaderboard = Resources.Actor:new {
		resource = "Actor",
		alignment = "MidTop",
		--alignmentOffset = {0, 83+logoTopScoresPanel_y_offset},
		--alignmentOffset = {0, APP.screenSize.y * 0.21},
		--alignmentOffset = {0, (UseIPhone5AppAssets() and 119+logoTopScoresPanel_y_offset) or 86+logoTopScoresPanel_y_offset},
		alignmentOffset = Layout{{0,86+logoTopScoresPanel_y_offset}, iphone5={0,112+logoTopScoresPanel_y_offset}, pc={0,40+logoTopScoresPanel_y_offset}}, 
		scripts = {"SparkleRandomly"},

		topScoreViewManager = Resources.TopScoreViewManager:new {
			--resource = "TopScoreViewManager",
			class = "TopScoreViewManager",
			titleActorName = "topScoreTitle",
			prevButtonName = "prevButton",
			nextButtonName = "nextButton",
			rowNameFormat = "topScoreRow%d",
		},

		topScoreTitle = Resources.Actor:new {
			resource = "Actor",
			image = "namePanel.png",
			alignment = "MidTop",
			alignmentOffset = {0, 10},
			zorder = 1,

			title = Resources.Label:new {
				resource = "Label",
				font = {
					fontName = "dabd____.ttf",
					fontSize = 18,
				},
				fgColor = "White",
				shadowColor = "Black",
				shadowOffset = {1,1},
				shadowAlpha = 100,
				heightLimit = 40,
				widthLimit = 186,
				alignment = "Center",
				alignmentOffset = {0,-2},
				--wrapLimit = 210,
				--aligner = {class="Actor", size={186,40}, color="Green", alpha=128, alignment="Center",},

				SetValue = function (self, value)
					self.text = tostring(value)
				end,
			},
			
			---[[
			gameCenterIconLeft = Resources.Actor:new {
				resource = "Actor",
				image = "GameCenterButton.png",
				size = {16, 16},
				alignment = "MidLeft",
				alignmentOffset = {31, -1},
			},

			gameCenterIconRight = Resources.Actor:new {
				resource = "Actor",
				image = "GameCenterButton.png",
				size = {16, 16},
				alignment = "MidRight",
				alignmentOffset = {-31, -1},
			},
			
			SetValues = function (self, values)
				self.gameCenterIconLeft.visible = (values.isGameCenter == true)
				self.gameCenterIconRight.visible = (values.isGameCenter == true)
				if values.isGameCenter == true then
					self.title.widthLimit = 180
				else
					self.title.widthLimit = 210
				end
			end,
			--]]
		},
		
		prevButton = Resources.Button:new {
			resource = "Button",
			text = STRINGS(""),
			alignment = "MidTop",
			alignmentOffset = {-130, 10},
			image = "LeftArrow.png",
			--clickUpSound = "Soft Impact 74a.ogg",
			
			zorder = 2,
		},
		
		nextButton = Resources.Button:new {
			resource = "Button",
			text = STRINGS(""),
			alignment = "MidTop",
			alignmentOffset = {130, 10},
			image = "RightArrow.png",
			--clickUpSound = "Soft Impact 74a.ogg",
			zorder = 2,
		},

		topScoreRows = Resources.Actor:new {
			resource = "Actor",
			alignment = "MidTop",
			alignmentOffset = {0, 36},
			topScoreRow1 = Resources.AppScoreRow:new {
				resource = "AppScoreRow",
				alignment = "MidTop",
				alignmentOffset = {0,0},
				zorder = 1,
			},
			topScoreRow2 = Resources.AppScoreRow:new {
				resource = "AppScoreRow",
				alignment = "MidTop",
				alignmentOffset = {0,1*TOP_SCORE_SPACING},
				zorder = 2,
			},
			topScoreRow3 = Resources.AppScoreRow:new {
				resource = "AppScoreRow",
				alignment = "MidTop",
				alignmentOffset = {0,2*TOP_SCORE_SPACING},
				zorder = 3,
			},
			topScoreRow4 = Resources.AppScoreRow:new {
				resource = "AppScoreRow",
				alignment = "MidTop",
				alignmentOffset = {0,3*TOP_SCORE_SPACING},
				zorder = 4,
			},
			topScoreRow5 = Resources.AppScoreRow:new {
				resource = "AppScoreRow",
				alignment = "MidTop",
				alignmentOffset = {0,4*TOP_SCORE_SPACING},
				zorder = 5,
			},
		},
	},
	
	optionsPanel = Resources.Actor:new {
		resource = "Actor",
		alignment = "MidTop",
		--alignmentOffset = {-32, 301+playPanel_y_offset},
		alignmentOffset = Layout{{-32, APP.screenSize.y * 0.59},pc={80,243}},
		layoutManager = Resources.GridLayout:new {
			resource = "GridLayout",
			--defaultColumnWidth = 130,
			--defaultRowHeight = 27,
			columns = 2,
			rows = 3,
		},
		color = "Red",
		alpha = Debug_Alpha,
		size = {280,105},
		visible = false,
		
		children = {
			Resources.Actor:new {
				name = "titleA",
				resource = "Actor",
				--text = STRINGS("Board"),
				image = "label_board.png",
			},
			Resources.MenuOptionGroup:new {
				name = "buttonSet1",
				resource = "MenuOptionGroup",
				color = "Blue",
				alpha = Debug_Alpha,
				size = {200,21},
				layoutManager = Resources.GridLayout:new {
					resource = "GridLayout",
					defaultColumnWidth = 130,
					defaultRowHeight = 27,
					columns = 2,
					rows = 1,
				},
				groups = {
					OptionARadioButtons = Resources.RadioButtonGroup,
				},
				buttonA = Resources.MenuOptionButton:new {
					resource = "MenuOptionButton",
					--text = STRINGS("Regular"),
					baseImageName = "rawbtnRegular",
					radioButtonGroup = "OptionARadioButtons",
					UpdateFromConfig = function (self)
						self.checkBoxState = (Config.MainMenu_Board == "regular")
					end,
					ApplyToConfig = function (self)
						Config.MainMenu_Board = "regular"
					end,
				},
				buttonB = Resources.MenuOptionButton:new {
					resource = "MenuOptionButton",
					--text = STRINGS("Large"),
					baseImageName = "rawbtnLarge",
					radioButtonGroup = "OptionARadioButtons",
					UpdateFromConfig = function (self)
						self.checkBoxState = (Config.MainMenu_Board == "large")
					end,
					ApplyToConfig = function (self)
						Config.MainMenu_Board = "large"
					end,
				},
			},
			--[[
			Resources.Actor:new {
				name = "titleB",
				resource = "Actor",
				--text = STRINGS("Match"),
				image = "label_match.png",
			},
			Resources.MenuOptionGroup:new {
				name = "buttonSet2",
				resource = "MenuOptionGroup",
				color = "Blue",
				alpha = Debug_Alpha,
				size = {200,21},
				
				layoutManager = Resources.GridLayout:new {
					resource = "GridLayout",
					defaultColumnWidth = 130,
					defaultRowHeight = 27,
					columns = 2,
					rows = 1,
				},
				groups = {
					OptionBRadioButtons = Resources.RadioButtonGroup,
				},
				buttonA = Resources.MenuOptionButton:new {
					resource = "MenuOptionButton",
					--text = STRINGS("Blast"),
					baseImageName = "rawbtnBlast",
					radioButtonGroup = "OptionBRadioButtons",
					UpdateFromConfig = function (self)
						self.checkBoxState = (Config.MainMenu_Match == "blast")
					end,
					ApplyToConfig = function (self)
						Config.MainMenu_Match = "blast"
					end,
				},
				buttonB = Resources.MenuOptionButton:new {
					resource = "MenuOptionButton",
					--text = STRINGS("Classic"),
					baseImageName = "rawbtnClassic",
					radioButtonGroup = "OptionBRadioButtons",
					UpdateFromConfig = function (self)
						self.checkBoxState = (Config.MainMenu_Match == "classic")
					end,
					ApplyToConfig = function (self)
						Config.MainMenu_Match = "classic"
					end,
				},
			},
			--]]
			--[[
			Resources.Actor:new {
				name = "titleB",
				resource = "Actor",
				--text = STRINGS("Mode"),
				image = "label_mode.png",
			},
			Resources.MenuOptionGroup:new {
				name = "buttonSet2",
				resource = "MenuOptionGroup",
				color = "Blue",
				alpha = Debug_Alpha,
				size = {200,21},
				layoutManager = Resources.GridLayout:new {
					resource = "GridLayout",
					defaultColumnWidth = 130,
					defaultRowHeight = 27,
					columns = 2,
					rows = 1,
				},
				groups = {
					OptionCRadioButtons = Resources.RadioButtonGroup,
				},
				buttonA = Resources.MenuOptionButton:new {
					resource = "MenuOptionButton",
					--text = STRINGS("Spins"),
					baseImageName = "rawbtnSpins",
					radioButtonGroup = "OptionCRadioButtons",
					UpdateFromConfig = function (self)
						self.checkBoxState = (Config.MainMenu_Mode == "spins")
					end,
					ApplyToConfig = function (self)
						Config.MainMenu_Mode = "spins"
					end,
				},
				buttonB = Resources.MenuOptionButton:new {
					resource = "MenuOptionButton",
					--text = STRINGS("Blitz"),
					baseImageName = "rawbtnBlitz",
					radioButtonGroup = "OptionCRadioButtons",
					UpdateFromConfig = function (self)
						self.checkBoxState = (Config.MainMenu_Mode == "blitz")
					end,
					ApplyToConfig = function (self)
						Config.MainMenu_Mode = "blitz"
					end,
				},
			},
			--]]
		},
	},
	
	playGameButton = Resources.AppMenuButton:new {
		resource = "AppMenuButton",--resource = "AppMenuButton",
		text = STRINGS("Arcade"),
		alignment = "MidTop",
		alignmentOffset = Layout{{-60, APP.screenSize.y * 0.59},iphone5={-60, APP.screenSize.y * 0.57},ipad={-60, APP.screenSize.y * 0.57},pc={140,280}},
		--alignmentOffset = Layout{{0, APP.screenSize.y * 0.59},iphone5={0, APP.screenSize.y * 0.57},ipad={0, APP.screenSize.y * 0.57},pc={140,280}},
		scripts = {"SparkleRandomly"},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			-- REWRITE, IF NEEDED: start a game here
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 1,
	},
	playQuestButton = Resources.AppMenuButton:new {
		resource = "AppMenuButton",
		text = STRINGS("Quest"),
		alignment = "MidTop",
		alignmentOffset = Layout{{60, APP.screenSize.y * 0.59},iphone5={60, APP.screenSize.y * 0.57},ipad={60, APP.screenSize.y * 0.57},pc={140,280}},
		scripts = {"SparkleRandomly"},
		textLineSpacing = 3,
		shortcutKey = "q",
		OnButtonClick = function (self, event)
			--logprint("OnButtonClick GoMap")
			if self ~= event.button then return false end
				self.screen:GoMap()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 1,
		visible = true,
	},
	StartChallengeButton = Resources.AppMenuBigButton:new {
		resource = "AppMenuBigButton",
		text = STRINGS("Daily Challenge"),
		alignment = "MidTop",
		alignmentOffset =  Layout{{0, 375-44}, iphone5={0, 440-56}, ipad={0, 375+30-56},pc={-140,280}},
		scripts = {"SparkleRandomly"},
		textLineSpacing = 3,
		--[[
		textImage = Resources.Actor:new {
			resource = "Actor",
			alignment = "Center",
			alignmentOffset = {0,0},
			image = "label_play.png",
		},
		--]]
		CheckMark = Resources.Actor:new {
			resource = "Actor",
			image = "Check.png",
			alignment = TopRight,
			alignmentOffset = {15,-15},
			visible = false,
			zorder = 10,
		},	
		lockMark = Resources.Actor:new {
			resource = "Actor",
			zorder = 5,
			image = "FeatureLock.png",
			alignment = Center,
			alignTo = Center,
			alignmentOffset = {0,-3},
			visible = false,
			darken = Resources.Actor:new {
				resource = "Actor",
				color = "Black",
				size = powerupSize,
				alpha = 150,
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {0,0},
				zorder = -1,
			},
		},
		NewMarker = Resources.NewMarker:new {
			resource = "NewMarker",
			visible = false,
		},
		Progress = Resources.Actor:new {
			resource = "Actor",
			alignment = Center,
			alignTo = Center,
			alignmentOffset = {0,-4},
			size = {132,20},
			color = "White",
			visible = false,
			zorder = 4,
			
			ProgressInside = Resources.Actor:new {
				resource = "Actor",
				alignment = Center,
				size = {130,18},
				color = "Black",
				zorder = 1,
			},
			ProgressBar = Resources.Actor:new {
				resource = "Actor",
				alignment = MidLeft,
				alignmentOffset = {1,0},
				size = {0,18},
				color = "Green",
				zorder = 2,
			},
			
			Building = Resources.Label:new {
				resource = "Label",
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {0,1},
				text = STRINGS("BuildingChallenge"),
				font	= Resources.DialogLabelFont:new{
					resource = "DialogLabelFont",
					fontSize = 18
				},	
				fgColor = "White",
				wrapLimit = 190,
				shadowColor = "Black",
				shadowAlpha = 100,
				highlightColor = "White",
				highlightAlpha = 0,
				zorder = 3,
			},			
		},	

		--[[
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			-- REWRITE, IF NEEDED: start a game here
			return true
		end,
		--]]
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 1,

	},

	FriendChallengeButton = Resources.GameCenterButton:new {
		resource = "GameCenterButton",
		image = "btnMenuBig.png",
		clickSound			= "SPLEEP_B.ogg",
		textAlignmentOffset = {0,-5},
		textColor = "Black",
		textRolloverColor = "Blue",
        textAlignmentOffset = {0,-5},
        font	= Resources.AppButtonFont:new{
            resource = "AppButtonFont",
            fontSize = 28,
        },
		text = STRINGS("Friend Challenge"),
        alwaysUseClickOffset = true,
        clickOffset = {0,1},

		disableUnlessGameCenterFeatures = {"GameCenterFeature_ScoreChallenges"},
		alignment = "MidTop",
		alignmentOffset =  Layout{{0, 425-46}, iphone5={0, 460-18}, ipad={0, 425+30-50},pc={-140,300-50}},

		scripts = {"SparkleRandomly"},
		textLineSpacing = 3,

		CheckMark = Resources.Actor:new {
			resource = "Actor",
			image = "Check.png",
			alignment = TopRight,
			alignmentOffset = {15,-15},
			visible = false,
			zorder = 10,
		},	
		lockMark = Resources.Actor:new {
			resource = "Actor",
			zorder = 5,
			image = "FeatureLock.png",
			alignment = Center,
			alignTo = Center,
			alignmentOffset = {0,-3},
			visible = false,
			darken = Resources.Actor:new {
				resource = "Actor",
				color = "Black",
				size = powerupSize,
				alpha = 150,
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {0,0},
				zorder = -1,
			},
		},
		NewMarker = Resources.NewMarker:new {
			resource = "NewMarker",
			visible = false,
		},
		--[[
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			-- REWRITE, IF NEEDED: start a game here
			return true
		end,
		--]]
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 1,

	},
	
	PlayerButton = Resources.AppButton:new {
		resource = "AppButton",
		image = "btn_blank_small.png",
		--Text is player name
		text = STRINGS("Player"),
		textWidthLimit = 180/ScreenScaleFactor,
		alignment = "MidTop",
		alignTo = "MidTop",
		--alignmentOffset = {0, 5},
		--alignmentOffset = {0,(UseIPhone5AppAssets() and 8) or 5},
		alignmentOffset = Layout{{0,10}, iphone5={0,10}, ipad={0,10}, pc={-200,10}},

		--alignment = "MidBottom",
		--alignmentOffset = {55, -10},
		shortcutKey = "N",
		keyTipPosition = {-10,30},
		--scalingPoint = TopRight,
		
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			self.screen:ShowPlayerDialog()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
			self.visible = true --(Config.enable_all_game_types == true)
		end,
	},	
	
    moreFunButton = Resources.AppMenuButton:new {
		resource = "AppMenuButton",
		text = STRINGS("more fun"),
		alignment = "MidBottom",
		alignmentOffset = {0, 4},
		textLineSpacing = 4,
        visible = true,
		NewMarker = Resources.NewMarker:new {
			resource = "NewMarker",
			alignmentOffset = {-10, 4},
			extraScale = 0.2,
			visible = false,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			self.screen:GoMoreFun()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
			--self.visible = (Config.enable_all_game_types == true)
            self.visible = true
		end,
		zorder = 1,
	},
	
	shareButton = Resources.AppButton:new {
		resource = "AppButton",
		image = "btnRoundShare.png",
		--image = "btn_blank_small.png",
		--text = STRINGS("awards"),
		alignment = "MidBottom",
		alignmentOffset = Layout{{82, -3+nudge_bottom_button_y}, iphone5={82, -3+nudge_bottom_button_y}, ipad={82+20, -3+nudge_bottom_button_y}, pc={82+40, -3+nudge_bottom_button_y}},
                
        OnButtonClick = function (self, event)
  			if self ~= event.button then
				return false
			end
            logprint("SHARING BUTTON")
			self.textToShare = STRINGS("AppInviteMessage", "I'm playing Cascade! Wanna join me?")
			if IsString(self.textToShare) then
				ShareText(self.textToShare, self)
			end
			return true
		end,

		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
			EVENT_HUB:AddObserver(EventMap.GURU_SCREENTRANSITION, self)
		end,
		
		UpdateVisibility = function (self)
			self.visible = SOCIAL_MGR_IsAvailable() or CanComposeEMail()
		end,
		OnScreenTransition = function (self, event)
			if self.screen == event.newScreen then
				self:UpdateVisibility()
			end
		end,
		OnWindowEvent = function (self, event)
			if event.eventName == "SDL_WINDOWEVENT_RESTORED" then
				self:UpdateVisibility()
			end
		end,
	},
	
    awardsButton = Resources.AppButton:new {
		resource = "AppButton",
		image = "btnRoundAwards.png",
		--image = "btn_blank_small.png",
		--text = STRINGS("awards"),
		alignment = "MidBottom",
		alignmentOffset = {-82, -3+nudge_bottom_button_y},
		alignmentOffset = Layout{{-82, -3+nudge_bottom_button_y}, iphone5={-82, -3+nudge_bottom_button_y}, ipad={-82-20, -3+nudge_bottom_button_y}, pc={-82-40, -3+nudge_bottom_button_y}},

		textLineSpacing = 4,
		OnButtonClick = function (self, event)
			--logprint("OnButtonClick GoAwards")
			if self ~= event.button then return false end
				self.screen:GoAwards()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
			self.visible = true --(Config.enable_all_game_types == true)
		end,
		zorder = 1,
	},	

	HelpButton = Resources.AppButton:new{
		resource = "AppButton",
		image = "btnHelp.png",
		shortcutKey = "?",
		functionKey = 282,--SDLK_F1
		alignment = "BottomRight",
		alignmentOffset = Layout{{-6, -3+nudge_bottom_button_y}, iphone5={-6, -3+nudge_bottom_button_y}, ipad={0, -3+nudge_bottom_button_y}},
		--size = {40,40},
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			self.screen:ShowHelp()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
		
    OptionsButton = Resources.Button:new {
		resource = "Button",
		image = "btnOptions.png",
		--text = STRINGS("options"),
		alignment = "BottomLeft",
		alignmentOffset = Layout{{6, -3+nudge_bottom_button_y}, iphone5={6, -3+nudge_bottom_button_y}, ipad={0, -3+nudge_bottom_button_y}},

		textColor = "White",
        textAlignmentOffset = {0,-2},
		font = {
			fontName = "fubd____.ttf",
			fontSize = 22,
		},
		textLineSpacing = 4,
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			-- TEST BEAT THIS
			-- self.screen:ShowBeatThisDialog()
			self.screen:ShowOptionsDialog()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
		zorder = 1,
	},
	
	playOtherGamesButton = Resources.Button:new {
		resource = "Button",
		image = "btnBlankSmall.png",
		text = STRINGS("Custom"),
		alignment = "BottomLeft",
		alignmentOffset = {5, -50},
		textAlignmentOffset = {0,-2},
        textRolloverOffset = {0,-1},
		
		font = {
			fontName = "fubd____.ttf",
			fontSize = 14,
		},
		textLineSpacing = 4,
		OnButtonClick = function (self, event)
			if self ~= event.button then return false end
			self.screen:StartCustomGame()
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
			self.visible = (Config.enable_all_game_types == true)
		end,
		zorder = 1,
	},

--[[
	leaderboardButton = Resources.Button:new {
		resource = "Button",
		text = STRINGS("Leaderboards"),
		alignment = "MidTop",
		alignmentOffset = {0, 200},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			GuruGameCenter_ShowLeaderboard("CascadeSlots.MainLeaderboard")
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
--]]
	
--[[
	debugButton = Resources.Button:new {
		resource = "Button",
		text = STRINGS("Debug Menu"),
		alignment = "MidTop",
		alignmentOffset = {0, 250},
		fgColor = "White",
		font = {
			fontName = "IT441___.TTF",
			fontSize = 14,
		},
		OnButtonClick = function (self, event)
			if self ~= event.button then
				return false
			end
			SwitchToScreen("DebugScreen")
			return true
		end,
		OnPostSpawn = function (self)
			self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		end,
	},
--]]
}

Scripts.UpdateChallenge = function(screen)
	--Set up challenge progress bar
	local challengeButton = screen.RootActor:FindChild("StartChallengeButton",true)
	challengeButton.active = false
	challengeButton.CheckMark.visible = false
	challengeButton.Progress.visible = true
	
	ScriptDelay(550)	 --Allows menu to present first
	
	--While challenge isn't done, update challenge progress bar
	--local progress = screen:GetSimulatorProgress()
	local progress = 0
	while(progress < 1.0) do
		ScriptBreak()
		--screen:TickSimulator()
		--progress = screen:GetSimulatorProgress()
		progress = progress + 0.05
		local goalWidth = challengeButton.Progress.width - 2
		goalWidth = goalWidth * progress
		challengeButton.Progress.ProgressBar.width = goalWidth
	end
	
	challengeButton.Progress.visible = false
	
	PlaySound("sparkle_little.ogg")
	screen:OnSimulatorFinish()
end

Scripts.ChallengeReady = function(screen)
end

Scripts.Preshow_MenuScreen = function(screen)
	if(not screen.openingShown)then

	else

	end
end

Scripts.Present_MenuScreen = function(screen)
	if(not screen.openingShown) then

	end
end

Scripts.BeatenChallenge = function(screen)
	local challengeButton = screen.RootActor.StartChallengeButton
	challengeButton.CheckMark.visible = false
	ScriptDelay(500) --Let the star transition finish
	challengeButton.CheckMark.visible = true
	Pop(challengeButton.CheckMark)
	FireParticle(challengeButton.CheckMark,"StarBurst_Blue_Small","Center")
	PlaySound("slot_machine_win_19.ogg")
end

Scripts.NewMarkerScript = function(newMarker)
	newMarker.visible = false
	--Wait for star transition to end
	-- ScriptDelay(400)
	--Wait a random amount more time so that not all NEWs show up at once
	ScriptDelay(math.random(0,1000))
	--Pop in
	if(newMarker.parent:IsGloballyVisible()) then
		ScaleIn(newMarker, {0.0, 0.0}, 200)
		PlaySound("slot_machine_win_23.ogg")
		FireParticle(newMarker,"SparkleBurst1","Center")
		--PlaySound("pop_sparkle_01.ogg")
		Pop(newMarker, newMarker.popAdjustments)
	end
	while(true)do
		ScriptDelay(10000)
		ScriptBreak()
	end

end

