BorderResizeHandler = function(eventData, componentResizeHandler)
	if eventData == nil then return end
	--logprint(eventData.receiver.name..":OnSizeChangedHandler() entered\n")
	local self = eventData.receiver
	local source = eventData.sender
	
	self.size = componentResizeHandler(source)
	
	--logprint(eventData.receiver.name..":OnSizeChangedHandler() exited\n")
	return true
end

BorderResizeHandlers = 
{
	left = function(eventData)
		local customResizeHandler = function(source) 
			--logprint(eventData.receiver.name..":BorderResizeHandler.left() entered\n")
			return {source.left.size.x, source.btmLeft.position.y - (source.topLeft.position.y + source.topLeft.size.y)} 
		end
		
		return BorderResizeHandler(eventData, customResizeHandler)
	end,

	right = function(eventData)
		local customResizeHandler = function(source) 
			--logprint(eventData.receiver.name..":BorderResizeHandler.right() entered\n")
			return {source.right.size.x, source.btmLeft.position.y - (source.topLeft.position.y + source.topLeft.size.y)}
		end
		
		return BorderResizeHandler(eventData, customResizeHandler)
	end,

	top = function(eventData)
		local customResizeHandler = function(source) 
			--logprint(eventData.receiver.name..":BorderResizeHandler.top() entered\n")
			return {source.topRight.position.x - (source.topLeft.position.x + source.topLeft.size.x) , source.top.size.y}
		end
		
		return BorderResizeHandler(eventData, customResizeHandler)
	end,

	bottom = function(eventData)
		local customResizeHandler = function(source) 
			--logprint(eventData.receiver.name..":BorderResizeHandler.bottom() entered\n")
			return {source.topRight.position.x - (source.topLeft.position.x + source.topLeft.size.x), source.bottom.size.y}
		end
		
		return BorderResizeHandler(eventData, customResizeHandler)
	end,
}

-- Aligns a border component
function AlignBorderComponent(component, location, borderAlignment)
	local location = location or component.alignTo or MidLeft
	local borderAlignment = borderAlignment or Center
	component.alignmentOffset = {1, 1}
	
	--logprint("AlignBorderComponent() aligning component named \'"..component.name.."\' at location \'"..location.."\' to "..borderAlignment)
	
	if borderAlignment == Inside then 
		component.alignment = location
	elseif borderAlignment == Center then
		component.alignment = Center
	elseif location == MidLeft	then
		component.alignment = MidRight
	elseif location == MidRight then
		component.alignment = MidLeft
	elseif location == MidTop then
		component.alignment = MidBottom
	elseif location == MidBottom then
		component.alignment = MidTop
	elseif location == TopLeft then
		component.alignment = BottomRight
	elseif location == TopRight then
		component.alignment = BottomLeft
	elseif location == BottomLeft then
		component.alignment = TopRight
	elseif location == BottomRight then
		component.alignment = TopLeft
	else
		logprint("Invalid border component location \'"..location.."\'")
	end
	return component
end

LineBorderComponent = function(location, thickness, hiColor, loColor)
	local location	= location	or MidLeft
	local thickness	= thickness or DefaultThickness
	local hiColor	= hiColor	or White
	local loColor	= loColor	or Black

	if thickness == nil then
		thickness = DefaultThickness
	elseif type(thickness) == "number" or type(thickness) == "integer" then 
		local t = thickness
		thickness = 
		{
			left	= t, 
			right	= t, 
			top		= t, 
			bottom	= t,
		}
	else
		assert(type(thickness) == "table", "Invalid Type specified for thickness values") 
	end
	
	local comp = Resources.Actor:new
	{
		color		= color,
		-- Borders are normally higher than other actors within 
		-- the same container but this is not a requirement. 
		zorder		= 100,		
		alignTo		= location,		
		shaderType	= "SHADERTYPE_BLENDCOMBINEDALPHA",
	}
	
	if location == MidLeft then
		comp.name	= "left"
		comp.size	= {thickness.left, 1}
		comp.color	= hiColor
	elseif location == MidRight then
		comp.name	= "right"
		comp.size = {thickness.right, 1}
		comp.color	= loColor
	elseif location == MidTop then
		comp.name	= "top"
		comp.size = {1, thickness.top}
		comp.color	= hiColor
	elseif location == MidBottom then
		comp.name	= "bottom"
		comp.size = {1, thickness.bottom}
		comp.color	= loColor
	elseif location == TopLeft then
		comp.name	= "topLeft"
		comp.size = {thickness.left, thickness.top}
		comp.color	= hiColor
	elseif location == TopRight then
		comp.name	= "topRight"
		comp.size = {thickness.right, thickness.top}
		comp.color	= hiColor
	elseif location == BottomLeft then
		comp.name	= "btmLeft"
		comp.size = {thickness.left, thickness.bottom}
		comp.color	= hiColor
	elseif location == BottomRight then
		comp.name	= "btmRight"
		comp.size = {thickness.right, thickness.bottom}
		comp.color	= loColor
	else
		logprint ("Invalid location ".. tostring(location) .. " for BorderComponent\n")
		return nil
	end 
	--logprint ("Created Border Component named "..comp.name.." for location ".. tostring(location).."\n")

	return comp
end 

DefaultThickness = 
{
	left	= 2,
	right	= 2, 
	top		= 2,
	bottom	= 2,
}

--[[
	Class: Border
	Description: Adds a lined border around an Actor
	Properties:
		All Actor Properties
		offset		- Point containing the x and y offsets of the border
		thickness	- Rectangle containing the thickness values for each side
--]]
Resources.Border = Resources.Actor:new
{
	borderAlignment	= Outside,
	thickness		= DefaultThickness,
	hiColor			= LightGray,
	loColor			= DarkGray,
	autoFillParent	= true,
	shaderType		= "SHADERTYPE_BLENDCOMBINEDALPHA",

	OnPostSpawn = function(self)
		--logprint(self.name..":OnPostSpawn() entered")

		--logprint("Creating Border Components")
		self:CreateComponents()

		--logprint("Adding Border Component Resize Handlers")
		self.left.GURU_ACTOR_RESIZED	= BorderResizeHandlers.left
		self.right.GURU_ACTOR_RESIZED	= BorderResizeHandlers.right
		self.top.GURU_ACTOR_RESIZED		= BorderResizeHandlers.top
		self.bottom.GURU_ACTOR_RESIZED	= BorderResizeHandlers.bottom
		
		--logprint("Adding Border Component Resize Observers")
		self:AddObserver(EventMap.GURU_ACTOR_RESIZED, self.left)
		self:AddObserver(EventMap.GURU_ACTOR_RESIZED, self.right)
		self:AddObserver(EventMap.GURU_ACTOR_RESIZED, self.bottom)
		self:AddObserver(EventMap.GURU_ACTOR_RESIZED, self.top)
		
		--logprint("Aligning Border Components")
		self:AlignBorder()

		--logprint(self.name..":OnPostSpawn() exited")
	end,
	
	CreateComponents = function(self)
		self.left			= LineBorderComponent(MidLeft,		self.thickness, self.hiColor, self.loColor)
		self.right			= LineBorderComponent(MidRight,		self.thickness, self.hiColor, self.loColor)
		self.top			= LineBorderComponent(MidTop,		self.thickness, self.hiColor, self.loColor)
		self.bottom			= LineBorderComponent(MidBottom,	self.thickness, self.hiColor, self.loColor)
		self.topLeft		= LineBorderComponent(TopLeft,		self.thickness, self.hiColor, self.loColor)
		self.topRight		= LineBorderComponent(TopRight,		self.thickness, self.hiColor, self.loColor)
		self.btmLeft		= LineBorderComponent(BottomLeft,	self.thickness, self.hiColor, self.loColor)
		self.btmRight		= LineBorderComponent(BottomRight,	self.thickness, self.hiColor, self.loColor)
	end,

	AlignBorder		= function(self, borderAlignment)
		self.borderAlignment = borderAlignment or self.borderAlignment or Center
		--logprint("Setting BorderAlignment to "..self.borderAlignment)
		
		if (self.borderAlignment ~= Center and 
			self.borderAlignment ~= Outside and 
			self.borderAlignment ~= Inside) then 
			logprint(tostring(borderAlignment).." is an invalid BorderAlignment. Setting to Center")
			self.borderAlignment = Center
		end
		
		if(self.left)		then AlignBorderComponent(self.left,		MidLeft,		self.borderAlignment) end
		if(self.right)		then AlignBorderComponent(self.right,		MidRight,		self.borderAlignment) end
		if(self.top)		then AlignBorderComponent(self.top,			MidTop,			self.borderAlignment) end
		if(self.bottom)		then AlignBorderComponent(self.bottom,		MidBottom,		self.borderAlignment) end
		if(self.topLeft)	then AlignBorderComponent(self.topLeft,		TopLeft,		self.borderAlignment) end
		if(self.topRight)	then AlignBorderComponent(self.topRight,	TopRight,		self.borderAlignment) end
		if(self.btmLeft)	then AlignBorderComponent(self.btmLeft,		BottomLeft,		self.borderAlignment) end
		if(self.btmRight)	then AlignBorderComponent(self.btmRight,	BottomRight,	self.borderAlignment) end
				
		if (self.parent) then 
			-- force a resize 
			self.size = self.parent.size 
		end
	end,
	
	GURU_ACTOR_RESIZED = function(eventData)
		if eventData == nil then return false end
		--logprint(eventData.receiver.name..":OnSizeChangedHandler() entered\n")
		local self = eventData.receiver
		local source = eventData.sender
		
		if(self == source) then return false end
		
		--logprint("Resizing Border from "..tostring(Size:new(self.size)).." to "..
		--		  tostring(Size:new(source.size.x - 1, source.size.y - 1)))
		self.size = {source.size.x - 1, source.size.y - 1}
		--logprint(eventData.receiver.name..":OnSizeChangedHandler() exited\n")
		return true
	end,
	
	SetAlpha			= function(self, alpha)
		alpha = alpha or 255
		--[[ NOTE: This didn't seem to work because for example, alpha comes in at 20 but is type number and then it makes it 255
		if(type(alpha) == "integer") then
			if(alpha > 255) then
				while (alpha > 255) do alpha = alpha - 256 end
			elseif(alpha < 0) then
				while (alpha < 0) do alpha = alpha + 256 end
			end
		elseif(type(alpha) == "number") then
			if(alpha > 1.0) then 
				alpha = 255 
			elseif(alpha < 0.0) then
				alpha = 0
			else
				alpha = tointeger(alpha * 255)
			end
		end
		--]]
		-- NOTE: This seems to not work because parent is NIL during a OnPostSpawn
		--self.parent.shaderType = "SHADERTYPE_BLENDCOMBINEDALPHA"
		if(self.left)		then self.left.alpha		= alpha end
		if(self.right)		then self.right.alpha		= alpha end
		if(self.top)		then self.top.alpha			= alpha end
		if(self.bottom)		then self.bottom.alpha		= alpha end
		if(self.topLeft)	then self.topLeft.alpha		= alpha end
		if(self.topRight)	then self.topRight.alpha	= alpha end
		if(self.btmLeft)	then self.btmLeft.alpha		= alpha end
		if(self.btmRight)	then self.btmRight.alpha	= alpha end
	end,

	SetThickness		= function(self, thickness)
		if type(thickness) == "number" or type(thickness) == "integer" then
			self.thickness.left		= thickness
			self.thickness.right	= thickness
			self.thickness.top		= thickness
			self.thickness.bottom	= thickness
		elseif type(thickness) == "table" then
			self.thickness = thickness			
		elseif type(thickness) == "function" then
			self.thickness = thickness()		
		end			
	end,
}

LabeledLineBorder = Resources.Border:new
{
	resource	= "Border",
	label		= Resources.Actor:new
	{
		text = Resources.Label:new
		{
			text			= "Label",
			textColor		= White,
			size			= Size:new(100,25),
			alignment		= Center,
			alignTo			= Center,
			font			= Resources.Font:new{resource = "Font", fontSize = 20},
		},
		size			= Size:new(100,25),
		alignment		= Center,
		alignTo			= Center,
		color			= Black,
		border			= Resources.Border,
	},	

	OnPostSpawn = function(self)
		Resources.Border.OnPostSpawn(self)
	end
}
