
-- Button Image Frames:
DefaultButtonFrame_Active = 0
DefaultButtonFrame_Disabled = 1
DefaultButtonFrame_Highlighted = 2
DefaultButtonFrame_Pressed = 3

--[[
	Class: Button
	Parent Class: Actor
	Properties:
		All Actor Properties
		textColor
		text
		textBackground1
		textBackground2
		useFrames
		font
		textAlignment
		pressUpOnMouseLeave
		shortcutKey
		enterKey
		escapeKey
		functionKey
		altFunctionKey
		useAltShortcutKey
		active
		clickSound
		highlightSound
--]]
Resources.Button = Resources.Actor:new
{
	--
	-- HACK: This resource is duplicated in EngineSpecificCompiledTemplateResources.cpp.
	-- This duplication a quick way to get a performance boost when instantiating a lot of
	-- objects with this resource.
	--
	
	class				= "Button",
	clickSound			= "",
	font				= Resources.ButtonFont,
}

-- A button's keytip.
-- Please note, this must have a child Label called "KeyTipLabel".
Resources.KeyTip = Resources.Actor:new 
{
	image		= "objKeyTip.png",
	children	=
	{
		KeyTipLabel = Resources.Label:new
		{
			font = Resources.Font:new
			{
				resource = "ButtonFont",
			},
		},
	},
}

-- The default Label for all Buttons.   no
Resources.ButtonLabel = Resources.Label:new
{
	font = Resources.Font,
}

local Button = Resources.Button

--[[
	Class: RadioButton
	Description: Special class of Button that is used to select one option from a group.
	Parent Class: Button
--]]
Resources.RadioButton = Resources.Button:new
{
	class 				= "RadioButton",
	radioButtonGroup	= ""
}

--[[
	Class: RadioButtonGroup
	Description: Container class to hold and manage a group of RadioButtons
	Parent Class: ActorGroup
	Properties: 
		All ActorGroup Properties
--]]
Resources.RadioButtonGroup = Resources.ActorGroup:new
{
	class 	= "RadioButtonGroup",
}

--[[
	Class: Grid
	Parent Class: Actor
	Description:
	Properties:
		All Actor Properties
		cellSize
		margins
--]]
Resources.Grid = Resources.Actor:new
{
	class			= "Grid",
	cellsize 		= {40,20},
	margins 		= {0, 0},
	--layoutManager	= Resources.GridLayout:new(),
}

--[[
	Class: RadioButtonGroup
	Description: Container class to hold and manage a group of RadioButtons
	Parent Class: ActorGroup
	Properties: 
		All ActorGroup Properties
--]]
Resources.RadioButtonGrid = Resources.Grid:new
{
	class 			= "RadioButtonGrid",
	radioButtons	= {},
}

--[[
	Button Templates 
	NOTE: These are for use in scripts and have no connection to the C++ 
	ButtonTemplate class which is not exposed to lua.
--]]
-- Arrow Buttons
Resources.NextArrowButton = Resources.Button:new
{
	image		= "btnBlankSmall.png", 
	text		= ">", 
	name		= "NextArrowButton", 
	shortcutKey = GURUKEY_RIGHT, 
	font		= Resources.ButtonFont,
}

Resources.PrevArrowButton = Resources.Button:new
{
	image		= "btnBlankSmall.png", 
	text		= "<", 
	name		= "PrevArrowButton", 
	shortcutKey = GURUKEY_LEFT,
	font		= Resources.ButtonFont,
}

-- Text Buttons
Resources.CancelButton = Resources.Button:new
{
	image		= "Button.png",
	text		= "cancel",       
	name		= "CancelButton",
	shortcutKey = GURUKEY_c,
	font		= Resources.ButtonFont,
}

Resources.YesButton = Resources.Button:new
{
	image		= "Button.png",          
	text		= "yes",			
	name		= "YesButton",
	shortcutKey = GURUKEY_y,
	enterKey	= true,
	font		= Resources.ButtonFont,
}

Resources.NoButton = Resources.Button:new
{
	image		= "Button.png",          
	text		= "no",			
	name		= "NoButton",
	shortcutKey = GURUKEY_n,
	font		= Resources.ButtonFont,
}

Resources.ChangeNameButton = Resources.Button:new	
{
	image		= "Button.small.png",        
	text		= "rename", 
	name		= "ChangeNameButton", 
	shortcutKey = GURUKEY_r,
	font		= Resources.ButtonFont
}

Resources.ClearPlayerButton = Resources.Button:new 
{
	image		= "Button.small.png",        
	text		= "clear",
	name		= "ClearPlayerButton",
	shortcutKey = GURUKEY_c,
	font		= Resources.ButtonFont
}

Resources.ChangeButton = Resources.Button:new		
{
	image		= "Button.small.png",        
	text		= "change",
	name		= "ChangeButton",
	shortcutKey = GURUKEY_c,
	font		= Resources.ButtonFont
}

Resources.UpdateButton = Resources.Button:new		
{
	image		= "Button.small.png",        
	text		= "update",
	name		= "UpdateButton",
	shortcutKey = GURUKEY_u,
	font		= Resources.ButtonFont
}

Resources.OkayButton = Resources.Button:new		
{
	image		= "Button.png",        
	text		= "okay",         
	name		= "OkayButton",
	shortcutKey = GURUKEY_o,
	enterKey	= true,
	font		= Resources.ButtonFont,
}

Resources.OptionsButton	= Resources.Button:new	
{
	image		= "Button.png",              
	text		= "options",      
	name		= "OptionsButton",
	shortcutKey = GURUKEY_o,
	font		= Resources.ButtonFont,
}

Resources.QuitButton = Resources.Button:new		
{
	image		= "Button.png",          
	text		= "quit",         
	name		= "QuitButton",
	shortcutKey = GURUKEY_q,
	font		= Resources.ButtonFont,
}

Resources.SmallOptionsButton = Resources.Button:new
{
	image		= "Button.small.png",        
	text		= "options",      
	name		= "OptionsButton",
	shortcutKey = GURUKEY_o,
	font		= Resources.SmallButtonFont,
}

Resources.SmallQuitButton = Resources.Button:new  
{
	image		= "Button.small.red.png",    
	text		= "quit",         
	name		= "QuitButton",
	shortcutKey = GURUKEY_q,
	font		= Resources.SmallButtonFont,
}

Resources.NextButton = Resources.Button:new
{
	image			= "Button.small.red.png",    
	text			= "next",
	shortcutKey		= GURUKEY_n,
	enterKey		= true,
	font			= Resources.ButtonFont,
}

Resources.PrevButton = Resources.Button:new
{
	image			= "Button.small.red.png",    
	text			= "prev",
	shortcutKey		= GURUKEY_BACKSPACE,
	font			= Resources.ButtonFont,
}

Resources.CheckBox = Resources.Button:new
{
	resource		= "Button",
	text			= "",
	image			= "boxCheck.check.png",
	useFrames		= false,
	font = {resource = "ButtonFont",},
	
    clickSound = buttonClickSound,
	
	OnPostSpawn = function (self)
		self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
	end,
}


Resources.ConfigKeyAwareView = Resources.Base:new {
	configKey = nil,
	configKeyAwareProperties = {
		_default = nil,
	},
	updateOnScreenTransitions = true,

	InitConfigKeyAwareView = function (self)
		EVENT_HUB:AddObserver(EventMap.GURU_SCREENTRANSITION, self)
		if IsString(self.configKey) then
			CONFIG:AttachObserver(self, self.configKey)
		end
		self:UpdateConfigKeyAwareView()
	end,

	OnConfigUpdate = function (self, event)
		if (event.configData.key == self.configKey) then
			self:UpdateConfigKeyAwareView()
		end
	end,
	
	GetPropertiesToApply = function (self)
		local configValue = nil
		if IsString(self.configKey) then
			configValue = CONFIG:RetrieveConst(self.configKey)
		end

		local properties = nil
		if IsTable(self.configKeyAwareProperties) then
			if configValue ~= nil then
				properties = self.configKeyAwareProperties[configValue]
			end
			if properties == nil then
				properties = self.configKeyAwareProperties._default
			end
		end
		return properties
	end,
	
	UpdateConfigKeyAwareView = function (self)
		--logprintf("TODO: Update \"%s\", a CONFIG-aware view (against the CONFIG key, %s)\n",
		--	tostring(self.name),
		--	((type(self.configKey) == "string" and string.format("\"%s\"", tostring(self.configKey))) or tostring(self.configKey)))

		local properties = self:GetPropertiesToApply()
		if IsTable(properties) then
			self:ApplyProperties(properties)
		end
	end,
	
	OnScreenTransition = function (self, event)
		if not self.screen or self.screen ~= event.newScreen then
			return
		end
		if (self.updateOnScreenTransitions == true) then
			self:UpdateConfigKeyAwareView()
		end
	end,
}


--
-- AuthenticationAwareDecorator
--
-- Use one of these to adjust an Actor's appearance or state based on
-- authentication state.
--
Resources.AuthenticationAwareDecorator = Resources.Base:new
{
	---------------------------------------------------------------------------
	-- CONFIGURABLE, OVERRIDE-ABLE, OR OTHERWISE IMPORTANT STUFF
	---------------------------------------------------------------------------

	-- If 'true', AuthenticationAwareDecorator may attempt to change the
	-- appearance of its host-Actor if and when authentication status changes.
	-- Set to 'false' to prevent it from making appearance-related changes.
	enableAppearanceUpdates = true,

	-- Settings to control the host-Actor's visibility if and when a person is
	-- connected, or not.
	--
	-- "enableAppearanceUpdates" should also be set to 'true', in order for these
	-- to work.
	visibleWhenConnected = true,
	visibleWhenDisconnected = true,

	-- If 'true', AuthenticationAwareDecorator may attempt to change the
	-- "text" property of its host-Actor.  When applied to a Label, this can
	-- cause appearance changes.
	--
	-- "enableAppearanceUpdates" should also be set to 'true', in order for this
	-- to work.
	decoratorCanChangeText = false,

	-- Possible values for the 'text' property.  Set these to strings to use them.
	--
	-- Both "decoratorCanChangeText" and "enableAppearanceUpdates" should also
	-- be set to 'true', in order for these to work.
	connectText = nil,		-- the player is logged-out, and the UI control's use will log them in
	connectingText = nil,	-- the player is actively authenticating
	logoutText = nil,		-- the player is logged-in, and the UI control's use will log them out

	-- If true, and the global Lua function, ShouldShowConnectUI(), returns false,
	-- the control will be hidden.
	--
	-- ShouldShowConnectUI() typically always returns 'true', but may return
	-- 'false' in builds that shouldn't show connect-related UIs, such as
	-- gaming-expo show-floor builds.
	--
	-- TL;DR - To prevent the host-Actor from showing up in gaming expo show floor
	-- builds, set 'isConnectUI' to 'true', otherwise set this value to 'false'.
	isConnectUI = false,

	-- Set this to "Facebook" to require use of Facebook-based authentication.
	-- This can be used, for example, to display Facebook-specific Connect
	-- buttons (that log a player into Facebook, even if they're logged in via
	-- some other, non-Facebook service).
	raveLoginType = nil,

	-- IMPORTANT: Call this from the host-actor's OnPostSpawn Lua method!
	InitAuthenticationAwareDecorator = function (self)
		EVENT_HUB:AddObserver(EventMap.GURU_BASE_AUTHENTICATION_STATUS_CHANGE, self)
		EVENT_HUB:AddObserver(EventMap.GURU_MAIN_AUTHENTICATION_STATUS_CHANGE, self)
		EVENT_HUB:AddObserver(EventMap.GURU_SCREENTRANSITION, self)
		EVENT_HUB:AddObserver(EventMap.GURU_RAVE_FACEBOOK_ACCOUNT_LINKAGE_STATUS_CHANGE, self)
		self:TryUpdatingAppearance(self)
	end,

	--
	-- UpdateAppearance: Override this to make changes to the host-actor if
	-- and when the authentication status changes!
	--
	-- Be sure to set 'enableAppearanceUpdates' to true, otherwise this method
	-- will not get called!
	--
	UpdateAppearance = function (self)
		self.lastAuthStatus = self:GetAuthenticationStatus()

		--logprintf("%s(\"%s\"), UpdateAppearance: status=<%s>, ShouldShowConnectUI=%s\n", tostring(self.class), tostring(self.name), tostring(self.lastAuthStatus), tostring(ShouldShowConnectUI()))
		
		if (self.isConnectUI == true) and (not ShouldShowConnectUI()) then
			self.visible = false
		elseif self.lastAuthStatus == "Authenticating" then
			if self.decoratorCanChangeText ~= false then
				if IsString(self.connectingText) then
					self.text = self.connectingText
				end
			end
			self.active = false
		elseif self.lastAuthStatus == "Ready" then
			if self.decoratorCanChangeText ~= false then
				if IsString(self.logoutText) then
					self.text = self.logoutText
				end
			end
			self.active = (self.visibleWhenConnected == true)
			self.visible = (self.visibleWhenConnected == true)
		else
			if self.decoratorCanChangeText ~= false then
				if IsString(self.connectText) then
					self.text = self.connectText
				end
			end
			--logprint("Setting visibility")
			--logprintf("Visibility is <%s>\n", tostring(self.visibleWhenDisconnected))
			self.active = (self.visibleWhenDisconnected == true)
			self.visible = (self.visibleWhenDisconnected == true)
		end
	end,


	---------------------------------------------------------------------------
	-- INTERNAL STUFF
	---------------------------------------------------------------------------

	GetAuthenticationStatus = function (self)
		local mixedAuthStatus = Authentication_GetAuthenticationStatus()
		if self.raveLoginType == "Facebook" then
			if not RAVE then
				return "Error"
			elseif mixedAuthStatus == "Ready" then
				return RAVE:GetFacebookAccountLinkageStatus()
			end
		end
		return mixedAuthStatus
	end,
	
	TryUpdatingAppearance = function (self)
		if (self.isConnectUI == true) and (not ShouldShowConnectUI()) then
			-- HACK: update if 'isConnectUI' is true here too, to disable UIs
			-- that have 'enableAppearanceUpdates' set to false.
			self.visible = false
		elseif self.enableAppearanceUpdates ~= false then
			self:UpdateAppearance()
		end
	end,
	
	OnScreenTransition = function (self, event)
		if not self.screen or self.screen ~= event.newScreen then
			return
		end

		--logprintf("DEBUG SCREENTRANSITION\n")
		
		self:TryUpdatingAppearance()
	end,

	OnBaseAuthenticationStatusChange = function (self, event)
		--[[
		logprintf("Button, OnBaseAuthenticationStatusChange called: engine status=<%s>, control status=<%s>, event.receiver=<%s>, self.name=<%s>, self.screen=<%s>, Screens.currentScreen=<%s>\n",
			tostring(Authentication_GetAuthenticationStatus()),
			tostring(self:GetAuthenticationStatus()),
			tostring(event.target),
			tostring(self.name),
			tostring(self.screen),
			tostring(Screens.currentScreen))
		--]]

		if event.receiver ~= self then
			return
		elseif self.screen ~= Screens.currentScreen then
			return
		end

		self:TryUpdatingAppearance(self)
	end,


	OnMainAuthenticationStatusChange = function (self, event)
		--[[
		logprintf("Button, OnMainAuthenticationStatusChange called: engine status=<%s>, control status=<%s>, event.receiver=<%s>, self.name=<%s>, self.screen=<%s>, Screens.currentScreen=<%s>\n",
			tostring(Authentication_GetAuthenticationStatus()),
			tostring(self:GetAuthenticationStatus()),
			tostring(event.target),
			tostring(self.name),
			tostring(self.screen),
			tostring(Screens.currentScreen))
		--]]

		if event.receiver ~= self then
			return
		elseif self.screen ~= Screens.currentScreen then
			return
		end

		self:TryUpdatingAppearance(self)
	end,
	
	OnRaveFacebookAccountLinkageStatusChange = function (self, event)
		--[[
		logprintf("Button, OnRaveFacebookAccountLinkageStatusChange called: engine status=<%s>, control status=<%s>, event.receiver=<%s>, self.name=<%s>, self.screen=<%s>, Screens.currentScreen=<%s>\n",
			tostring(Authentication_GetAuthenticationStatus()),
			tostring(self:GetAuthenticationStatus()),
			tostring(event.target),
			tostring(self.name),
			tostring(self.screen),
			tostring(Screens.currentScreen))
		--]]
	
		if event.receiver ~= self then
			return
		elseif self.screen ~= Screens.currentScreen then
			return
		elseif self.raveLoginType ~= "Facebook" then
			-- Only update from GURU_RAVE_FACEBOOK_ACCOUNT_LINKAGE_STATUS_CHANGE
			-- events if the Actor is setup to respond to Facebook account
			-- linkage.
			return
		end
		
		self:TryUpdatingAppearance(self)
	end
}

-- Attach this to an existing button (via part of a 'resources' property) to make it a 'Connect' button
-- EXAMPLE: resources = {"OtherButtonResource", "FacebookConnectDecorator"},
--
Resources.FacebookConnectDecorator = Resources.AuthenticationAwareDecorator:new {
	resource = "AuthenticationAwareDecorator",

	connectingText = STRINGS("Connecting") or "Connecting",
	logoutText = STRINGS("Log out") or "Log Out",
	connectText = STRINGS("Connect") or "Connect",
	visibleWhenConnected = false,
	visibleWhenDisconnected = true,
	decoratorCanChangeText = true,

	InitFacebookConnectDecorator = function (self)
		self:AddObserver(EventMap.GURU_BUTTONCLICK, self)
		Resources.AuthenticationAwareDecorator.InitAuthenticationAwareDecorator(self)
	end,

	-- DEPRECATED: Call InitFacebookConnectDecorator instead of InitFacebookConnect
	-- This name change is to make FacebookConnectDecorator's init method be
	-- named more closely with AuthenticationAwareDecorator's
	InitFacebookConnect = function (self, ...)
		return self:InitFacebookConnectDecorator(...)
	end,
	
	OnButtonClick = function (self, event)
		if self ~= event.button then
			return
		end
		self:Connect()
		return true
	end,
	
	Connect = function (self)
		if RAVE then
			RaveConnect(self.raveLoginType, self.analyticsTag)
		--else
		--	FacebookConnect()
		end
	end,
}
