	-- All breakpoints specific to your app should go in a table called ApplicationSpecificBreakPoints 
	-- ApplicationSpecificBreakPoints should either contain a series of tables in the format
	-- {filename = "luafile.lua", line = x, condition = Conditions.y}, or it should contain
	-- a series of calls to SetBreakPoint specific to the application
	
Conditions = 
{
	always		= {iterations = 1},
	everyTen	= {iterations = 10},
	custom		= function(iterations) return {iterations = iterations} end,	
}

DebugState = 
{
	new = function(self, obj)
		obj = obj or {}
		obj["source"]		= obj["source"]			or self["source"]		or " "
		obj["what"]			= obj["what"]			or self["what"]			or " "
		obj["function"]		= obj["function"]		or self["function"]		or 0
		obj["name"]			= obj["name"]			or self["name"]			or " "
		obj["nups"]			= obj["nups"]			or self["nups"]			or 0
		obj["currentLine"]	= obj["currentLine"]	or self["currentLine"]	or 0
		obj["namewhat"]		= obj["namewhat"]		or self["namewhat"]		or " "
		obj["linedefined"]	= obj["linedefined"]	or self["linedefined"]	or 0
		obj["short_src"]	= obj["short_src"]		or self["short_src"]	or " "	
		return obj
	end
}

Debugger = 
{
	breakPoints		= {},
	enabled			= enabled or false,
	bpHalt			= false,
	currentState	= DebugState:new(),
		 
	ShutDown = function(self)
		self:Enable(false)
		self.enabled = false
		self.breakPoints = nil
	end,

	Enable = function(self)
		logprint("Enabling Debugger")
		self.enabled = true
		debug.sethook(Debugger.Hook, "crl")
		logprint("Debugger Enabled")
	end,

	Disable = function(self)
		logprint("Disabling Debugger")
		self.enabled = false
		debug.sethook()
		logprint("Debugger Disabled")
	end,

	IsEnabled = function(self)
		return self.enabled
	end,

	FindBreakPoint = function(self, breakPoint)
		assert(breakPoint ~= nil)

		local id = MakeId(breakPoint)

		local found = self.breakPoints[id]
		
		if id ~= nil and found ~= nil then
			if AreTheSameBreakPoint(breakPoint, found) then return found end
		end
		return nil
	end,

	IsBreakPointSet = function(self, breakPoint)
		bp = self:FindBreakPoint(breakPoint)
		if bp ~= nil and bp.enabled == true then 
			return true
		else 
			return false
		end
	end,

	Hook = function(self, line)
		if self.enabled == false then return end
		
		newState = DebugState:new(debug.getinfo(2))
		assert(newState ~= nil, "getinfo returned nil")
		
		output = "["..newState["source"] .. " ("..tostring(newState["currentline"]) .. ":" .. tostring(newState["linedefined"]) .. ") ]"

		---[[	
		if newState["namewhat"] ~= nil then 
			output = output .. " (" .. tostring(newState["namewhat"]) .. ") "
		end
		--]]
		---[[	
		if newState["name"] ~= nil then
			output = output .. tostring(newState["name"]);	
		end
		--]]
		logprint(output)

		index = 1
		repeat 
			name, value = debug.getlocal(2, index)
			if name then
				if index == 1 then logprint("\tLocals") end
				logprint("\t\t[\""..tostring(name).."\"]\t=\t"..tostring(value))
			else
				if index == 1 then logprint("\tNo Locals") end
			end
			index = index + 1
		until not name 

		--[[	
		if newState.nups > 0 then
			logprint("\tUpValues")

			index = 1
			repeat 
				name, value = debug.getupvalue(, index)
				if name then
					logprint("\t[\""..tostring(name).."\"]\t=\t"..tostring(value))
					index = index + 1
				end
			until not name 
			logprint("\n")
		end
		--]]
		
		self.currentState = newState
		
		
		--[[
		id = debugInfo.source..":"..tostring(debugInfo.currentline)
		
		bp = self.breakPoints[id]
		if bp == nil then
			if bp.handler ~= nil and type(bp.handler) == "function" then
				bp.handler(debugInfo)
			else 
				logprint("Breakpoint id:"..bp.id.."caught")
				self.breakPointHalt = true
			end
		end
		--]]
		while self.bpHalt == true do
			if IsGamePaused() == false then PauseGame() end
			ScriptBreak()
		end
		if IsGamePaused() then ResumeGame() end
	end,

	Continue = function(self)
		self.bpHalt = false
	end,

	Break = function(self)
		self.bpHalt = true
	end,
	
	SetBreakPoint = function(self, fileName, line, enabled, --[[optional]]handler, --[[optional]]condition)
		assert(fileName ~= nil, "SetBreakPoint() - Cannot set breakpoint. Nil filename passed.")
		assert(line ~= nil and line > 0, "SetBreakPoint() - Invalid line number: ".. tostring(line))
		condition = conditon or Condition.always
		assert(type(enabled) == "boolean", "Invalid type \'"..type(enabled).."\' for enabled parameter")
		
		local id = table.getN()
		table.insert(self.breakPoints({id, filename, line, enabled, handler, condition, }))
		
		return id
	end,

	SetBreakPointHandler = function(self, id, handler)
		assert(self.breakPoint[id] ~= nil, "SetBreakPointHandler() - Nil Id passed")
		assert(handler == nil or type(handler) == "function")
	end,

	AddBreakPoints = function(self, bpList)
		if bpList ~= nil then
			if type(bpList) == "table" then
				for k, v in pairs(bpList) do
					SetBreakPoint(v["fileName"], v["line"], v["enabled"], v["handler"], v["condition"])
				end
			end
		elseif type(bpList) == "function" then
			bpList()
		else
			logprint("bpList table is invalid type: ".. type(bpList)". Should be either a table of " ..
					 "breakpoint tables or a function which either returns a table of breakpoint tables "..
					 "or which contains one or more individual SetBreakPoint calls")			
		end
	end,

	EnableBreakPoint = function(self, id)
		bp = self.breakPoints[id]
		if bp ~= nil then 
			bp.enabled = true	
			logprint("Enabled Breakpoint: ".. id)
		else
			logprint("No breakpoint exists with Id: ".. id)
		end
	end,

	DisableBreakPoint = function(self, id)
		bp = self.breakPoints[id]
		if bp ~= nil then 
			bp.enabled = false 
			logprint("Disabled Breakpoint: ".. id)
		else
			logprint("No breakpoint exists with Id: ".. id)
		end
	end,

	RemoveBreakPoint = function(self, id)
		-- See if we can find the breakpoint
		bp = FindBreakPoint(id)
		
		if bp ~= nil then
			logprint("\nRemoving Breakpoint:\n\tid:"..bp.id..
					 "\n\tfilename:"..bp.filename..
					 "\n\tline:"..bp.line..
					 "\n\tcondition: Break every "..tostring(bp.condition.iterations).."\n\n")
					 
			self.breakPoints[bp.id] = nil
		end
	end,
}

DebugStart = function()
	Debugger:Enable()
end

DebugStop = function()
	Debugger:Disable()
end

function MakeId(breakPoint)
	if breakPoint == nil or breakPoint.filename == nil or breakPoint.line == nil then 
		return nil 
	else
		return "@"..breakPoint.filename..":"..tostring(breakPoint.line)..tostring(breakPoint.handler)
	end
end

function AreTheSameBreakPoint(bp1, bp2)
	if (bp1.id == bp2.id or bp1.id == nil)	and 
	bp1.filename	== bp2.filename			and 
	bp1.line		== bp2.line				and
	bp1.handler		== bp2.handler			and
	bp1.condition	== bp2.condition		then 
		return true
	else 
		return false
	end
end

