--[[
	Dialog: Inherits all properties from Actor
		All Actor Properties plus:
			title
			titleAlignmentOffset
			introScriptName
			dialogExitScriptName
--]]
Resources.Dialog = Resources.Actor:new
{
	class						= "Dialog",
	title						= "Untitled Dialog",
	titleAlignmentOffset		= { 0, 0 },
}

Resources.DialogTitleLabel = 
{
    fgColor			= Black,
    shadowColor		= White,
    font			= Resources.DialogTitleFont:new
    {
        resource = "DialogTitleFont",
    },
    alignmentOffset = {0, 20},
    alignment		= MidTop,
    alignTo			= MidTop,
}

--
-- SlideInDialog script
--
-- SlideIn a dialog from the side of the screen.
--
Scripts.SlideInDialog = function (dialog)
        local originalPosition = dialog.position

        -- Show the dialog
        dialog.visible = true

        -- Place the dialog just above the screen
        local width = dialog.size.x
        local height = dialog.size.y
        dialog.position = {-width, originalPosition.y}

        -- Slide it from left to right
        SlideAndWait(dialog, {originalPosition.x + 40, originalPosition.y}, 250)
        SlideAndWait(dialog, originalPosition, 100)
		
		if IsFunction(dialog.ShowOrHideOnScreenKeyboard) then
			dialog.allowOnScreenKeyboard = true
			dialog:ShowOrHideOnScreenKeyboard()
		end
end


Scripts.StopDialogAndWait = function (target, dialogName)
	local screen = target.screen

	-- Stop the dialog:
	local dialog = screen:GetTopDialog(dialogName)
	if dialog and not dialog:IsStopped() then
		screen:StopDialog(dialog)
	end

	-- Wait for the dialog to finish displaying:
	while screen:IsDialogDisplaying(dialog) do
		ScriptBreak()
	end
end



--
-- SlideOffDialogToTheRight script
--
-- Slides a dialog off to the right of the screen
--
Scripts.SlideOffDialogToTheRight = function (dialog)
        local width = dialog.size.x
        local height = dialog.size.y
        local position = dialog.position
        --PlaySound("SwishUp.ogg")
        --FadeOut(dialog,400)	-- NOTE from davidl: Fading most dialogs is extremely slow, fow now.
        SlideAndWait(dialog, {position.x - 40, position.y}, 100)
        SlideAndWait(dialog, {800, position.y}, 300)
end


Scripts.DefaultDialogIntroScript = Scripts.SlideInDialog
Scripts.DefaultDialogExitScript = Scripts.SlideOffDialogToTheRight
