ObserverType = 
{
	Classic		= -1,
	Standard	= function(id) return id end
}

Resources.Event = 
{
	class		= "Event",
	name		= "Event",
	id			= 0,
	which		= -1,	
	
	new			= function(self, obj)
		obj = obj or {},
		setmetatable(obj, self)
		self.__index = self
		return obj
	end,
}

-- Application visibility event structure 
Resources.ActiveEvent = Resources.Event:new 
{
	class		= "ActiveEvent",
	id			= EventMap.SDL_ACTIVEEVENT,
	gain		= 0,						-- Whether given states were gained or lost (1/0) 
	state		= 0,						-- A mask of the focus states 
}

-- KeyUp event structure 
Resources.KeyUpEvent = Resources.Event:new  
{
	class		= "KeyUpEvent",
	id			= EventMap.SDL_KEYUP, 
	scancode	= 0,						-- hardware specific scancode
	sym			= 0,						-- SDL virtual keysym 
	mod			= 0,						-- current key modifiers
	unicode		= 0,						-- translated character
}

-- KeyDown event structure 
Resources.KeyDownEvent = Resources.Event:new  
{
	class		= "KeyDownEvent",
	id			= EventMap.SDL_KEYDOWN, 
	scancode	= 0,						-- hardware specific scancode
	sym			= 0,						-- SDL virtual keysym 
	mod			= 0,						-- current key modifiers
	unicode		= 0,						-- translated character
}

-- Mouse motion event structure 
Resources.MouseMotionEvent = Resources.Event:new  
{
	class		= "MouseMotionEvent",
	id			= EventMap.SDL_MOUSEMOTION,
	state		= 0,						-- The current button state 
	x			= 0, 						-- The X coordinates of the mouse 
	y			= 0,						-- The Y coordinates of the mouse 
	xrel		= 0,						-- The relative motion in the X direction 
	yrel		= 0,						-- The relative motion in the Y direction 
}

-- MouseButton Down event structure 
Resources.MouseButtonDownEvent = Resources.Event:new 
{
	class		= "MouseButtonDownEvent",
	id			= EventMap.SDL_MOUSEBUTTONDOWN, 
	button		= 0,						-- The mouse button index 
	state		= 0,						-- SDL_PRESSED or SDL_RELEASED 
	x			= 0,						-- The X/Y coordinates of the mouse at press time 
	y			= 0,				
}

-- MouseButton Up event structure 
Resources.MouseButtonUpEvent = Resources.Event:new 
{
	class		= "MouseButtonUpEvent",
	id			= EventMap.SDL_MOUSEBUTTONDOWN, 
	button		= 0,						-- The mouse button index 
	state		= 0,						-- SDL_PRESSED or SDL_RELEASED 
	x			= 0,						-- The X/Y coordinates of the mouse at press time 
	y			= 0,				
}

-- Joystick axis motion event structure 
Resources.JoyAxisMotionEvent = Resources.Event:new 
{
	class		= "JoyAxisEvent",
	id			= EventMap.SDL_JOYAXISMOTION, 
	axis		= 0,						-- The joystick axis index 
	value		= 0,						-- The axis value (range: -32768 to 32767) 
}

-- Joystick trackball motion event structure 
Resources.JoyBallMotionEvent = Resources.Event:new 
{
	class		= "JoyBallEvent",
	id			= EventMap.SDL_JOYBALLMOTION, 
	ball		= 0,						-- The joystick trackball index 
	xrel		= 0,						-- The relative motion in the X direction 
	yrel		= 0,						-- The relative motion in the Y direction 
}

-- Joystick hat position change event structure 
Resources.JoyHatMotionEvent = Resources.Event:new 
{
	class		= "JoyHatEvent",
	id			= EventMap.SDL_JOYHATMOTION, 
	hat			= 0,						-- The joystick hat index 
	value		= 0,						-- The hat position value:
											-- SDL_HAT_LEFTUP   SDL_HAT_UP       SDL_HAT_RIGHTUP
											-- SDL_HAT_LEFT     SDL_HAT_CENTERED SDL_HAT_RIGHT
											-- SDL_HAT_LEFTDOWN SDL_HAT_DOWN     SDL_HAT_RIGHTDOWN
											-- Note that zero means the POV is centered.
}

-- Joystick button event structure 
Resources.JoyButtonEvent = Resources.Event:new 
{
	class		= "JoyButtonEvent",
	id			= EventMap.SDL_JOYBUTTON, 
	button		= -1,						-- The joystick button index 
	id			= -1,						-- SDL_PRESSED or SDL_RELEASED 
}

--[[ 
	The "window resized" event
	When you get this event, you are responsible for setting a new video
	mode with the new width and height.
--]]
Resources.ResizeEvent = Resources.Event:new 
{
	class		= "ResizeEvent",
	id			= EventMap.SDL_VIDEORESIZE, 
	w			= 0,						-- New width 
	h			= 0,						-- New height 
}

-- The "screen redraw" event 
Resources.ExposeEvent = Resources.Event:new
{
	class		= "ExposeEvent",
	id			= EventMap.SDL_VIDEOEXPOSE, 
}

-- The "quit requested" event 
Resources.QuitEvent = Resources.Event:new 
{
	class		= "QuitEvent",
	id			= EventMap.SDL_QUIT,
}

Resources.SysMsgEvent = Resources.Event:new
{
	class		= "SysMsgEvent",
	id			= EventMap.SDL_SYSWMEVENT,
	msg			= 0, 	 					-- The type of message * 
	wParam		= 0, 	 					-- WORD message parameter
	lParam		= 0, 	 					-- LONG message parameter
}
-- A user-defined event type 
Resources.UserEvent = Resources.Event:new 
{
	class		= "UserEvent",
	id			= EventMap.SDL_USEREVENT,
	code		= 0,						-- User defined event code 
	data1		= nil,						-- User defined data pointer 
	data2		= nil,						-- User defined data pointer 
}

function ListEvents() 
	for ek, ev in pairs(EventMap) do
		logprint("Event Name: " .. ek .. "\tId: " .. ev) 
	end
end
