
local function HandleButtonClick_Clear(eventData)
	if eventData.sender ~= eventData.receiver then
		return false
	end

	KillAllParticleEmitters()
end


local function HandleButtonClick_RunHGEEffect(eventData)
	if eventData.sender ~= eventData.receiver then
		return false
	end
	
	if type(eventData.sender.effectName) == "string" then
		ParticleEditor_StartEmitterFromFile(eventData.sender.effectName)
	end
	
	return true
end


local function HandleButtonClick_LaunchHGEEditor(eventData)
	if eventData.sender ~= eventData.receiver then
		return false
	end

	ParticleEditor_LaunchEditorApp()
	
	return true
end


Resources.ParticleEditorScreen = {
	color = "Black",
	
	-- Make OnScreenShow and/or OnScreenHide get called as appropriate.
	GURU_SCREENTRANSITION = CreateBuilderScreenTransitionHelper("ParticleEditorScreen"),
	
	OnScreenShow = function (self, oldScreen)
		self.originalValue_NoPauseOnFocusLoss = APP.NoPauseOnFocusLoss
		APP.NoPauseOnFocusLoss = true

		if oldScreen then
			self.nameOfLastScreen = oldScreen.name
		end
	end,
	
	OnScreenHide = function (self, newScreen)
		self.nameOfLastScreen = nil
		APP.NoPauseOnFocusLoss = self.originalValue_NoPauseOnFocusLoss
	end,
	
	OnKeyDown = function (self, eventData)
		logprint("OnKeyDown: %s", tostring(eventData.keyName))
	    if eventData.keyName == "escape" then
	        if self.nameOfLastScreen then
				ReloadParticleResources()
	            SwitchToScreen(self.nameOfLastScreen)
	            return true
	        end
		elseif eventData.keyName == "r" then
			ReloadParticleResources()
			return true
	    end
	end,
	
	buttons = {
		class = "Actor",
		size = {130, 0},	-- HACK: Set height to zero to get the GridLayout to align to the top left.
		position = {0, 0},

		OnPostSpawn = function (self)
			-- Layout buttons from top to bottom
			local children = self.children
			local h = 0
			for k,v in pairs(children) do
				if IsObject(v) then
					v.y = h
					h = h + v.height
				end
			end
		end,

		children = {
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "Clear All",
				GURU_BUTTONCLICK = HandleButtonClick_Clear,
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 18,
				},
			},

			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "Launch Editor App",
				GURU_BUTTONCLICK = HandleButtonClick_LaunchHGEEditor,
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 10,
				},
			},
			
			Resources.Label:new {
				resource = "Label",
				text = "Sample FX:",
				fgColor = "White",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 16,
				},
			},

			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "Burst_1",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "Burst_1.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},

			EmitHGEParticles2 = Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "Fire_1",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "Fire_1.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},

			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_1",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_1.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_2",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_2.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},

			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_3",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_3.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_4",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_4.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_5",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_5.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_6",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_6.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_7",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_7.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_8",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_8.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "hge_9",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "hge_9.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},

			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "Sparks_1",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "Sparks_1.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
			
			Resources.Button:new {
				resource = "Button",
				image = "Button.small.long.png",
				text = "Sparks_2",
				GURU_BUTTONCLICK = HandleButtonClick_RunHGEEffect,
				effectName = "Sparks_2.guru_particle",
				font = Resources.Font:new{
					resource = "Font",
					fontSize = 14,
				},
			},
		},
	},
}
