
Card_Version = GetBuildNumber()

--Resources.Player = Resources.Object:new
--{
--	class = "Player",
--}


if not PlayerCardFiles then
	PlayerCardFiles = {}
	setmetatable(PlayerCardFiles, { __mode = "k" })
end

function EngineUpdatePlayer(playerTable, upgradeInfo)
end

function DoPlayerUpdate(playerTable)
	-- 'upgradeInfo' is a table that allows the engine-wide player-updating
	-- code send data to the app-specific player-updating function.  This
	-- can be used to have the engine perform common upgrades, with apps
	-- adding custom degrees of functionality on top of it.  For example,
	-- the engine might add an option related to a certain feature, while
	-- some specific apps present a custom dialog mentioning its
	-- availability.
	local upgradeInfo = {}

	local status, err = pcall(function () EngineUpdatePlayer(playerTable, upgradeInfo) end)
	if not status then
		logprint("Error applying engine-specific player card updates: "..tostring(err))
	end
	
	if IsFunction(UpdatePlayer) then
		local status, err = pcall(function () UpdatePlayer(playerTable, upgradeInfo) end)
		if not status then
			logprint("Error applying app-specific player card updates: "..tostring(err))
		end
	end

--	if IsTable(playerTable) and
--	   playerTable.Card_Version ~= Card_Version and
--	   TEST_FLIGHT
--	then
--		local checkpointName = string.format("Card_Version Change, <%s> to <%s>",
--			tostring(playerTable.Card_Version),
--			tostring(Card_Version))
--		TEST_FLIGHT:PassCheckpoint(checkpointName)
--	end

	local oldCardVersion = playerTable.Card_Version
	playerTable.Card_Version = Card_Version
	if not IsTable(playerTable.CardUpdates) then
		playerTable.CardUpdates = {}
	end
	if oldCardVersion ~= Card_Version then
		table.insert(playerTable.CardUpdates, string.format("%s-%s", tostring(oldCardVersion), tostring(Card_Version)))
	end
end

local function GetPlayerCardBackupFile(fileName, playerTable)
	if not IsString(fileName) or
	   not IsTable(playerTable) or
	   not IsNumber(playerTable.Card_Version) or
	   not IsNumber(Card_Version) or
	   playerTable.Card_Version ~= Card_Version
	then
		local newFileName =
			fileName ..
			string.format(".BackupFromUpgrade_%s-%s",
				tostring(playerTable.Card_Version),
				tostring(Card_Version))
		return newFileName
	end
end

function HandlePlayerCardBackup(fileName, playerTable)
	--if not IsFunction(AppGetPlayerCardBackupFile) then
	--	return
	--end

	local newFileName
	local doPlayerCardBackup = function ()
		newFileName = GetPlayerCardBackupFile(fileName, playerTable)
		if not newFileName then
			-- When GetPlayerCardBackupFile returns nil, it means that there is
			-- no need to make a backup.
			return nil
		end

		local didCopy, errorFromCopy = CopyFile(fileName, newFileName)
		if not didCopy then
			error("CopyFile failed: " .. tostring(errorFromCopy))
		end
	end

	local did_copy, result = xpcall(doPlayerCardBackup, GuruLuaErrorHandler)
	if not did_copy then
		logprintf("ERROR BACKING UP PLAYER CARD:\n\told file: %s\n\tnew file: %s\n\terror: %s\n",
			tostring(fileName), tostring(newFileName), tostring(result))
	end
end

local function CreatePlayersTable()
	local players = { current={}, remote={} }
    return players
end

function IsCardEmpty(player)
    if not player or not player.Name or not player.guid then
        return true
    else
        return false
    end
end

function ReplacePlayerTable(old, new)
    --logprint("ReplacePlayerTable: START")
    --logprint("ReplacePlayerTable: OLD: <"..SerializeToString(old)..">")
    --logprint("ReplacePlayerTable: NEW: <"..SerializeToString(new)..">")
    --logprint("ReplacePlayerTable: DIFF? = "..tostring(not DeepEquals(old, new)))

	for k, v in pairs(Players) do
		if v == old then
            --logprint("ReplacePlayerTable: Players, "..tostring(k))
			Players[k] = new
		end
	end

	for k, v in pairs(Players.remote) do
		if v == old then
            --logprint("ReplacePlayerTable: Players.remote, "..tostring(k))
			Players.remote[k] = new
		end
	end

	for k, v in pairs(Players.current) do
		if v == old then
            --logprint("ReplacePlayerTable: Players.current, "..tostring(k))
			Players.current[k] = new
		end
	end

	for playerGuid, multiSitePlayer in pairs(MultiSitePlayerTables) do
		for site, v in pairs(multiSitePlayer) do
			if v == old then
                --logprint("ReplacePlayerTable: MultiSitePlayerTables, "..tostring(playerGuid)..", "..tostring(site))
				MultiSitePlayerTables[playerGuid][site] = new
			end
		end
	end

	for k, v in pairs(DummyPlayers) do
		if v == old then
            --logprint("ReplacePlayerTable: DummyPlayers, "..tostring(k))
			DummyPlayers[k] = new
		end
	end

    --logprintf("ReplacePlayerTable: END\n")
end

function GetPlayerWithGuid(player_guid, disable_error_check)
    for i, player in ipairs(Players) do
        if IsTable(player) and player.guid == player_guid then
            return Players[i]
        end
    end

	for k, player in pairs(Players.remote) do
		if IsTable(player) and player.guid == player_guid then
			return player
		end
	end

    for i, player in ipairs(DummyPlayers) do
        if player.guid == player_guid then
            return DummyPlayers[i]
        end
    end

    if not disable_error_check then
        error("GetPlayerWithGuid failed, invalid guid=" .. player_guid)
    else
        return nil
    end
end

function CreatePlayerGuid()
    local guid = math.random(2000000000)
    guid = guid + 1000 --The first 1000 guids are reserved for engine use
    if(type(GetPlayerWithGuid(guid, true)) == 'nil') then
		return guid
    else
		return CreatePlayerGuid()	
	end
end

function PlayerManager_LoadGameData(self, siteID)
	-- Create a new version of the global table, MultiSitePlayerTables.
	MultiSitePlayerTables = {}
	
	-- Create a new version of the global table, Players.
	Players = CreatePlayersTable()
	
	-- Create a new dummy player table, which houses fake entries for any "indexed" player
	-- (e.g. player "1", player "2", etc.) for which a real, human player does not exist.
	DummyPlayers = CreateDummyPlayersTable()
	
	-- Try to load data for player cards 1 through 4
	for playerIndex = 1, 4 do
		-- Attempt to load data.
		local pathToCardFile = self:GetPlayerCardFullPath(playerIndex, "", 0)
		local loadedMultiSiteData = self:_DoLoadPlayerCardFile(pathToCardFile)
		
		if not loadedMultiSiteData then
			-- The .card did not load.
			
			-- Display an error message.
			local errorMessage = last_player_card_load_error
			logprint(string.format("Couldn't load card file for player %d.", playerIndex))
			
			-- Put a fake entry in the 'Players' table, for now, to indicate
			-- that a player should be created here.
			Players[playerIndex] = 0
		else
			local loadedPlayer = loadedMultiSiteData[siteID]
			if not loadedPlayer then
				-- Put a fake entry in the 'Players' table, for now, to indicate
				-- that a player should be created here.
				Players[playerIndex] = 0
			else
				-- Set the appropriate entry in 'Players'
				Players[playerIndex] = loadedPlayer

				-- Set the appropriate entry in 'MultiSitePlayerTables'
				local playerGuid = loadedPlayer.guid
				if not IsNumber(playerGuid) then
					LogGenericError("Unable to detect loaded player guid!")

					-- This should never happen, but in case...
					-- Fallback to a new guid (slightly error-prone in case of duplicate id):
					playerGuid = CreatePlayerGuid()
				end
				MultiSitePlayerTables[playerGuid] = loadedMultiSiteData
			end

			-- Make app-specific backups of the file.  This might be done after a major
			-- update:
			HandlePlayerCardBackup(pathToCardFile, loadedPlayer)

			-- Perform game-specific player updates, if possible.
			if loadedPlayer then
				DoPlayerUpdate(loadedPlayer)
			end
		end
	end

	-- Create new players as necessary.
	for playerIndex = 1, 4 do
		if Players[playerIndex] == 0 then
			-- Create the new player
			local newPlayer = CreatePlayer()

			-- Assign the player to the proper spot in 'MultiSitePlayerTables':
			local playerGuid = newPlayer.guid
			MultiSitePlayerTables[playerGuid] = {[siteID] = newPlayer}

			-- Make sure the entry is loaded into the 'Players' table as well.
			Players[playerIndex] = newPlayer
		end
	end

    -- Load non-indexed players
    self:_LoadNonIndexedPlayers()
	
	-- Set the current players.
	local playerOneGuid = (Config and Config.current_player_1_guid)
	if not IsNumber(playerOneGuid) or not SetCurrentPlayerFromGuid(playerOneGuid, 1) then
		self:SetCurrentPlayerByPlayerIndex(1, 1)
	end

	if APP then
		AppPostEvent("GURU_PLAYERCHANGE")
	end
end


local function GetCurrentPlayerIndex(playerTable)
	for i,p in ipairs(Players.current) do
		if p == playerTable then
			return i
		end
	end
end

function PlayerManager_ClearPlayerByIndex(self, siteID, playerIndex)
	logprintf(LOG_HIGH, "ClearPlayerByIndex: playerIndex:%s\n", tostring(playerIndex))

	local backupOfCurrentPlayerIndex = GetCurrentPlayerIndex(Players[playerIndex])

	local oldPlayer = Players[playerIndex]
	if oldPlayer then
		local oldPlayerGuid = oldPlayer.guid
		if not IsNil(oldPlayerGuid) then
			if IsTable(MultiSitePlayerTables[oldPlayerGuid]) then
				MultiSitePlayerTables[oldPlayerGuid][siteID] = nil
			end
		end
	end

	local newPlayer = CreatePlayer()
	Players[playerIndex] = newPlayer
	local newPlayerGuid = newPlayer.guid
	if not IsTable(MultiSitePlayerTables[newPlayerGuid]) then
		MultiSitePlayerTables[newPlayerGuid] = {}
	end
	MultiSitePlayerTables[newPlayerGuid][siteID] = newPlayer
	
	if backupOfCurrentPlayerIndex then
		--Players.current[backupOfCurrentPlayerIndex] = newPlayer
		self:SetCurrentPlayerByPlayerIndex(playerIndex, backupOfCurrentPlayerIndex)
	end
	
	return newPlayer
end

