--[[
	Class: Screen
	Parent Class: Actor
	Properties: 
		All Actor Properties
--]]
Resources.Screen = Resources.Actor:new
{
	class = "Screen",
}

OnScreenChangeHandler = function(source, eventData)
	if _G.screenToReload ~= nil then 
		SwitchToScreen(_G.screenToReload)
	end
	return false
end

Resources.ReloadTransitionScreen = Resources.Screen:new
{
	color		= Black,
	size		= {800, 600},
	alignment	= Center,
	alignTo		= Center,
	
	reloading	= Resources.Label:new
	{
		font			= Resources.MainFontMedium:new{resource = "MainFontMedium", fontSize = 70},
		text			= "RELOADING...",
		textColor		= White,
		textAlignment	= CENTER,
		alignment		= Center,
		alignTo			= Center,
	},
	
	GURU_SCREENTRANSITION = OnScreenChangeHandler,
}

function ReloadScreen(screen)
	if type(screen) ~= "userdata" then
		logprint("Cannot reload invalid screen.\n")
		return
	elseif screen.name == nil then
		logprint("Cannot reload unnamed screen.\n")
		return
	end

	-- store the name of the screen to reload
	_G.screenToReload = screen.name

	-- first switch to ReloadTransitionScreen
	SwitchToScreen("ReloadTransitionScreen")
end

-- A button used to minimize a fullscreen app.
Resources.MinimizeAppButton = Resources.Button:new
{
	image				= "btnMin.png",
	name				= "MinimizeButton",
	shortcutKey			= GURUKEY_SPACE,
	useAltShortcutKey	= true,
	clickSound			= "click2.ogg",
	text				= "",
}

-- A button used to switch a fullscreen app to windowed mode.
Resources.RestoreAppButtonNormal = Resources.Button:new
{
	image				= "btnRes.png",
	name				= "RestoreButton",
	shortcutKey			= GURUKEY_RETURN,
	useAltShortcutKey	= true,
	clickSound			= "click2.ogg",
	text				= "",
}

Resources.RestoreAppButtonMac = Resources.Button:new
{
	image				= "btnRes_Mac.png",
	name				= "RestoreButton",
	shortcutKey			= GURUKEY_RETURN,
	useAltShortcutKey	= true,
	clickSound			= "click2.ogg",
	text				= "",
}

-- A button used to close a fullscreen app.
Resources.CloseAppButtonNormal = Resources.Button:new
{
	resource			= "Button",
	image				= "btnClose.png",
	name				= "CloseButton",
	shortcutKey			= GURUKEY_F4,
	useAltShortcutKey	= true,
	clickSound			= "click2.ogg",
	text				= "",
}

Resources.CloseAppButtonMac = Resources.Button:new
{
	resource			= "Button",
	image				= "btnClose_Mac.png",
	name				= "CloseButton",
	shortcutKey			= GURUKEY_F4,
	useAltShortcutKey	= true,
	clickSound			= "click2.ogg",
	text				= "",
}

-- Choose platform-appropriate window buttons
if IsMac() then
	Resources.RestoreAppButton = Resources.RestoreAppButtonMac
	Resources.CloseAppButton = Resources.CloseAppButtonMac
else
	Resources.RestoreAppButton = Resources.RestoreAppButtonNormal
	Resources.CloseAppButton = Resources.CloseAppButtonNormal
end

-- Retrieves a reference to the current Screen.
function CurrentScreen()
	return Screens.currentScreen
end

-- Fades in a dialog underlay
Scripts.FadeInDialogUnderlay = function (target)
	local destAlpha = target.alpha
	target.alpha = 0
	FadeAndWait(target, destAlpha, 250, "DialogTimers")
end

-- Fades out a dialog underlay
Scripts.FadeOutAndDestroyDialogUnderlay = function (target)
	FadeAndWait(target, 0, 250, "DialogTimers")
	target:Destroy()
end

Scripts.DefaultDialogUnderlayIntroScript = Scripts.FadeInDialogUnderlay
Scripts.DefaultDialogUnderlayDestructionScript = Scripts.FadeOutAndDestroyDialogUnderlay

