--
-- SlowDownToStop script
--
-- Slow down the scrollable area until it stops.
--
Scripts.SlowDownToStop = function (scrollableArea, startVelocityX, startVelocityY, stoppingVelocity, slowFraction)
	local currentVelocity = {x=startVelocityX, y=startVelocityY};
	local currentVelocityMagnitude = Point_Magnitude(currentVelocity);
    while (currentVelocityMagnitude > stoppingVelocity) do
		currentVelocityMagnitude = slowFraction * currentVelocityMagnitude;
		currentVelocity = Point_Normalize(currentVelocity, currentVelocityMagnitude);
		scrollableArea:ScrollByAmount(currentVelocity.x, currentVelocity.y);
		ScriptBreak()
	end
end

Scripts.AnimateToScrollPosition = function (scrollableArea, dest, scrollTime)
	local timePerStep = 30

	local steps = math.floor(scrollTime / timePerStep)

	local moveOffset = {}
	local currentPosition = scrollableArea.scrollPosition
	moveOffset.x = dest.x - currentPosition.x
	moveOffset.y = dest.y - currentPosition.y

	--[[logprint("currentPos")
	logprint(currentPosition.x)

	logprint("dest")
	logprint(dest.x)]]
	

	local offsetMagnitude = Point_Magnitude(moveOffset)
	local distancePerStep = offsetMagnitude / steps
	local stepOffset = Point_Normalize(moveOffset, distancePerStep)

	--logprint("offset")
	--logprint(stepOffset.x)

	local reachedDest = false
	
	if(offsetMagnitude == 0) then
		return
	end

	-- HACK: Crude hack to get out of stuck state on new card, and first dig, only on iphone
	-- This fixes problem with bubbles not coming up
	local lastMagnitudeRemaining = -1
    local stuckCount = 0

	while(reachedDest == false) do
		while scrollableArea.IsPaused do
			--logprint("Paused")
			ScriptBreak()
		end
		currentPosition = scrollableArea.scrollPosition
		local offsetRemaining = {}
		offsetRemaining.x = dest.x - currentPosition.x
		offsetRemaining.y = dest.y - currentPosition.y

		--Check to see if the animation has overshot. If so, abort.
		if((moveOffset.x < 0 and offsetRemaining.x > 0) or
			(moveOffset.x > 0 and offsetRemaining.x < 0) or
			(moveOffset.y < 0 and offsetRemaining.y > 0) or
			(moveOffset.y > 0 and offsetRemaining.y < 0)) then
			return
		end

		if (scrollableArea.horizontalScrollLocked) then
			offsetRemaining.x = 0
		end
		
		if (scrollableArea.verticalScrollLocked) then
			offsetRemaining.y = 0
		end

		local magnitudeRemaining = Point_Magnitude(offsetRemaining)
		if (magnitudeRemaining <= distancePerStep) then
			scrollableArea.scrollPosition = dest
			reachedDest = true
		else
			scrollableArea:ScrollByAmount(stepOffset.x, stepOffset.y)
			scrollableArea:RemoveCurrentVelocity()
		end

		-- HACK: Crude hack to get out of stuck state on new card, and first dig, only on iphone
		-- This fixes problem with bubbles not coming up
        if(lastMagnitudeRemaining==magnitudeRemaining)then
            --logprint("stuckCount ".. stuckCount)
            if(stuckCount==10)then
                reachedDest = true
            else
                stuckCount = stuckCount + 1
            end
        else
            lastMagnitudeRemaining=magnitudeRemaining
            stuckCount=0
        end

		ScriptBreak()
	end
end