
--
-- TIMING SETTINGS
--

-- Slow-down factor for successful match animations during demo mode. This global may be modified by C++ code. 1.0 = normal speed. Higher = slower. Lower = faster.

DemoAnimationSlowingFactor = 1.0

DemoSlowingTable = {
    [1] = 2.5,
    [2] = 2.5,
    [3] = 2.0,
    [4] = 1.5,
    [5] = 1.5,
    [6] = 1.2,
    [7] = 1.1,
    [8] = 1.0,
}

-- How long should each individual slide in a false-move animation take?
local FalseMoveSlideTimeMS = 200  -- was 300

-- Speed on Gem Flying Up
SpeedOfGemFlying = 1562.5 -- was 1800.0 -- was 1200

-- Delay Of Match Happening - gives some time for match - not sure if this one does anything
DelayOfMatch = 0

-- Delay Of Master Match Happening - waits before showing all the points
DelayOfMasterMatch = 0

-- Delay of Special being presented - like when match 7 makes diamond. This gives the match some time before the diamond comes out.
DelayOfSpecialPresent = 250

-- Delay of Pieces being dropped - gives some time to let match happen
DelayOfDropPiece = 500

--
-- ANIMATIONS
--
function Scripts.GoodJobWord(rootGameView,goodJobWordImage,goodJobWordText,delay,direction)
	local goodJobWord = rootGameView:FindChild("FlashWord",true)
	goodJobWord.extraScale = {1.0,1.0}
	if (Scripts.ShouldUseArt() or goodJobWordText == "") then
		goodJobWord.image = goodJobWordImage
		goodJobWord.localizedLabel.text = ""
	else
		goodJobWord.localizedLabel.text = goodJobWordText
		goodJobWord.image = ""
		goodJobWord.localizedLabel.widthLimit = 290
	end
	ScaleIn(goodJobWord,{0.0,0.0},250,false)
	ScriptDelay(delay)
	SlideOff(goodJobWord,200,direction,"wait")
end

function Scripts.GoodJobWord_withSqueeze(rootGameView,goodJobWordImage,goodJobWordText,delay,direction)
	local goodJobWord = rootGameView:FindChild("FlashWord",true)
	goodJobWord.extraScale = {1.0,1.0}
	if (Scripts.ShouldUseArt() or goodJobWordText == "") then
		goodJobWord.image = goodJobWordImage
		goodJobWord.localizedLabel.text = ""
	else
		goodJobWord.localizedLabel.text = goodJobWordText
		goodJobWord.image = ""
		goodJobWord.localizedLabel.widthLimit = 250
	end
	ScaleIn(goodJobWord,{0.0,0.0},250,false)
	ScriptDelay(250)
	goodJobWord:AddScript("Squeeze","Squeeze")
	ScriptDelay(delay-250)
	SlideOff(goodJobWord,200,direction,"wait")
end

Scripts.Shiver = function(actor)
	--logprint("Shiver")
	local amount = 1
	local speed = 32
	local slidePosition = actor.position
	local originalPosition = actor.position
	
	while(true)do
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x+(amount*2)
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
	end
	actor.position = originalPosition		
end

Scripts.ShiverFast = function(actor)
	--logprint("ShiverFast")
	local amount = 1
	local speed = 23
	local slidePosition = actor.position
	local originalPosition = actor.position
	
	while(true)do
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x+(amount*2)
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
	end
	actor.position = originalPosition		
end

Scripts.ShiverFastest = function(actor)
	--logprint("ShiverFastest")
	local amount = 1
	local speed = 13
	local slidePosition = actor.position
	local originalPosition = actor.position
	
	while(true)do
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x+(amount*2)
		SlideAndWait(actor,slidePosition, speed)
		slidePosition.x=slidePosition.x-amount
		SlideAndWait(actor,slidePosition, speed)
	end
	actor.position = originalPosition		
end

Scripts.PulseSize = function(self)
	local oldScale = self.extraScale
	local newScale = {x = oldScale.x * 1.25, y = oldScale.x * 1.25}
	local duration = self.pulseDuration
	Scripts.Interpolate(self,"extraScale",newScale,duration/2)
	Scripts.Interpolate(self,"extraScale",oldScale,duration/2)
end

function Scripts.AnimateMove(self, gamePieceView,x,y)
	-- Go no further if neither the main board's piece view or the slot board's
	-- piece view exits.
	if not gamePieceView then
		return
	end
end

function Scripts.SlideUpAndDown(self)
	local backedUpPosition = self.globalPosition
	local goUpPosition = {backedUpPosition.x,backedUpPosition.y+10}

	PlaySound("pop3.ogg")
	FireParticle(self,"Launch1","MidBottom",nil,{0,-20})

	Scripts.Interpolate(self, "globalPosition", goUpPosition, FalseMoveSlideTimeMS/4)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, FalseMoveSlideTimeMS)

	goUpPosition = {backedUpPosition.x,backedUpPosition.y-4}

	Scripts.Interpolate(self, "globalPosition", goUpPosition, FalseMoveSlideTimeMS/4)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, FalseMoveSlideTimeMS/2)

end

--Causes the actor to slide to the destination in a quarter-circle path
function Scripts.SlideToArc(self, destination, duration)
	--Calculate midpoint of line
	local source = self:GetGlobalPoint(TopLeft)
	local segmentMidpoint = {x = source.x / 2 + destination.x /2, y = source.y / 2 + destination.y /2}

	--Calculate center of circle
	--If you take the vector from source to midpoint, and then rotate it left 90 degrees, then add it to midpoint, that's the center of the circle

	local vectorFromSourceToMidpoint = {x = segmentMidpoint.x - source.x, y = segmentMidpoint.y - source.y}
	local rotatedVector = {x = vectorFromSourceToMidpoint.y, y = -vectorFromSourceToMidpoint.x}
	local circleCenter = {x = segmentMidpoint.x + rotatedVector.x, y = segmentMidpoint.y + rotatedVector.y}

	--Calculate initial angle and radius of circle
	local initialVectorToEdge = {x = source.x- circleCenter.x, y = source.y - circleCenter.y}
	local radius = math.sqrt(initialVectorToEdge.x * initialVectorToEdge.x + initialVectorToEdge.y * initialVectorToEdge.y)
	local initialAngle = math.atan2(initialVectorToEdge.x,initialVectorToEdge.y)
	local finalAngle = initialAngle - math.pi/2

	--Interpolate based on axis length
	local startMS = APP:GetElapsedGameTime()
	local endMS = startMS + duration
	local currentMS = startMS
	local elapsedMS = 0
	local anglePerMS = -math.pi/2 / duration

	while true do
		if endMS <= currentMS then
			-- Make sure the destination value gets set, then exit.
			self:SetGlobalPoint(TopLeft,destination)
			return
		end
		
		-- Yield the hosting coroutine/Script.
		ScriptBreak()
		
		-- Advance time, and the property-to-update.
		currentMS = APP:GetElapsedGameTime()
		elapsedMS = currentMS - startMS
		local currentAngle = initialAngle + elapsedMS * anglePerMS
		local positionOffset = {x = math.cos(currentAngle) * radius, y = math.sin(currentAngle) * radius}
		local currentPosition = {circleCenter.x + positionOffset.x, circleCenter.y + positionOffset.y}
		self:SetGlobalPoint(TopLeft,currentPosition)
	end
end

function Scripts.AnimateFalseMove(self, mainBoardPieceView, slotsBoardPieceView, x, y, isBounce)
	-- Go no further if neither the main board's piece view or the slot board's
	-- piece view exits.
	if not mainBoardPieceView or not slotsBoardPieceView then
		return
	end
	
	WaitForTaggedScriptCompletion(self, "Piece_Present")

	local boxName = "Box"..(x+1)
	local box = self.parent:FindChild(boxName,true)
	box:AddScript("SlideUpAndDown","")

	-- Make sure the piece view is above all others for the duration of the
	-- animation.
	local oldPieceViewLayer = slotsBoardPieceView.layer
	slotsBoardPieceView.layer = slotsBoardPieceView.layer + 4
	
	-- Slide the slot board's piece view towards the main board's piece view,
	-- then slide it back.
	
	--HACK: Since rotation and scale throw off global position, I will set them to 0 while measuring this, then restore them later.
	local backedUpRotation = slotsBoardPieceView.rotationAngle
	local backedUpScale = slotsBoardPieceView.extraScale

	slotsBoardPieceView.rotationAngle = {degrees = 0}
	slotsBoardPieceView.extraScale = 1

	local backedUpPosition = slotsBoardPieceView.globalPosition

	slotsBoardPieceView.rotationAngle = backedUpRotation
	slotsBoardPieceView.extraScale = backedUpScale
	
	--HACK: Since rotation and scale throw off global position, I will set them to 0 while measuring this, then restore them later.
	local backedUpMainRotation = mainBoardPieceView.rotationAngle
	local backedUpMainScale = mainBoardPieceView.extraScale

	mainBoardPieceView.rotationAngle = {degrees = 0}
	mainBoardPieceView.extraScale = 1

	local destinationHack = mainBoardPieceView.globalPosition

	mainBoardPieceView.rotationAngle = backedUpMainRotation
	mainBoardPieceView.extraScale = backedUpMainScale
	--End hack.

	PlaySound("Move2_up.ogg")
	PlaySound("swish10.ogg")
	slotsBoardPieceView.pulseDuration = FalseMoveSlideTimeMS - 32
	slotsBoardPieceView:AddScript("PulseSize","PulseSize")
	if(slotsBoardPieceView.isBoomerang) then
		slotsBoardPieceView:AddScript("SpinBoomerang","")
		Scripts.SlideToArc(slotsBoardPieceView,destinationHack, FalseMoveSlideTimeMS)
	else
		Scripts.QuadraticInterpolate(slotsBoardPieceView, "globalPosition", destinationHack, FalseMoveSlideTimeMS)
	end
	
	if(isBounce) then
		PlaySound("Hit_KickPad2.ogg")
		mainBoardPieceView:AddScript("ShakeSlide","")
	end

	PlaySound("Move2_down.ogg")
	slotsBoardPieceView.pulseDuration = FalseMoveSlideTimeMS - 32
	slotsBoardPieceView:AddScript("PulseSize","PulseSize")
	if(slotsBoardPieceView.isBoomerang) then
		slotsBoardPieceView:AddScript("SpinBoomerang","")
		Scripts.SlideToArc(slotsBoardPieceView,backedUpPosition, FalseMoveSlideTimeMS)
	else
		Scripts.QuadraticInterpolate(slotsBoardPieceView, "globalPosition", backedUpPosition, FalseMoveSlideTimeMS)
	end
	PlaySound("CLEEP_B.ogg")	

	-- Restore the piece view's layer:
	slotsBoardPieceView.layer = oldPieceViewLayer
	if oldPieceViewLayer == 0 then
		slotsBoardPieceView:ClearLayer()	-- HACK: make sure layering actually gets disabled, if the layer was zero
	end
end

Scripts.SpinBoomerang = function(pieceView)
	local duration = pieceView.pulseDuration
	RotateAtSpeed(pieceView,  {revolutions=5}, duration, true)
	pieceView.rotationAngle = {degrees = 0}
end

function Scripts.WildGemImageCycle(pieceView)
	-- Specify images
	local imageNames = { "Wildgem1.png", "Wildgem2.png", "Wildgem3.png", "Wildgem4.png"}
	
	-- Randomize the order in which the gems appear in:
	local imageOrder = "random start"
	if GameUITweaks then
		if IsString(GameUITweaks.wildcard_image_order) then
			imageOrder = GameUITweaks.wildcard_image_order
		end
	end
	
	if imageOrder == "random sort" then
		-- Randomize elements in imageNames
		local tempTable = {}
		while #imageNames > 0 do
			local i = math.random(1, #imageNames)
			local imageName = table.remove(imageNames, i)
			table.insert(tempTable, imageName)
		end
		imageNames = tempTable
		tempTable = nil
	elseif imageOrder == "random start" then
		-- Randomize where imageNames starts while keeping the array size the
		-- same and maintaining the same cyclical order.
		local tempTable = {}
		local i = math.random(1, #imageNames)
		while #tempTable < #imageNames do
			table.insert(tempTable, imageNames[i])
			--logprintf("%d, %s, %d, %d\n", i, imageNames[i], #tempTable, #imageNames)
			i = i + 1
			if i > #imageNames then
				i = 1
			end
		end
		imageNames = tempTable
		tempTable = nil
	elseif imageOrder == "uniform" then
		-- Do nothing.
	else
		-- Do nothing except log a warning.
		logprintf("WARNING: image order, <%s>, is of an unknown value.\n", tostring(imageOrder))
	end
	
	-- Create actors for each image, making each effectively invisible for the
	-- time being.
	local imageActors = {}
	for i,imageName in ipairs(imageNames) do
		local imageActor = CreateInstance(Resources.Actor:new {
			resource = "Actor",
			image = imageName,
			alignment = "Center",
			name = "GemCycleImage" .. i,
			alpha = 0,
		})
		table.insert(imageActors, imageActor)
	end
	
	-- If there are no image actors then log a warning and go no further.
	if #imageActors == 0 then
		logprint("WARNING: No images to cycle through for pieceView: "..tostring(pieceView))
		return
	end
	
	-- Make sure the first image actor is visible:
	imageActors[1].alpha = 255
	pieceView:AddChild(imageActors[1])
	
	-- If there is only one image actor, then make sure it gets displayed, but
	-- don't try cycling through other actors.
	if #imageActors == 1 then
		return
	end

	-- Repeatedly cross-fade from one image to the other
	local currentIndex = 1
	local nextIndex = nil
	local doFirstIterationSetup = true
	local randomStartA = tonumber(GameUITweaks and GameUITweaks.wildcard_random_start_A) or 20
	local randomStartB = tonumber(GameUITweaks and GameUITweaks.wildcard_random_start_B) or 230
	local randomStartMin = math.min(randomStartA, randomStartB)
	local randomStartMax = math.max(randomStartA, randomStartB)
	--logprintf("random start range = [%d,%d]\n", randomStartMin, randomStartMax)
	while true do
		-- Pick the next image to display:
		nextIndex = currentIndex + 1
		if nextIndex > #imageActors then
			nextIndex = 1
		end
	
		-- Start fading in the next image:
		imageActors[nextIndex].alpha = 0
		if doFirstIterationSetup then
			-- On the first iteration of this loop, randomize the starting position
			-- of the fade-in by a bit, in order to de-synchronize fade timing:
			local randomStart = math.random(randomStartMin, randomStartMax)
			--logprintf("randomStart = %s\n", tostring(randomStart))
			imageActors[nextIndex].alpha = randomStart
			doFirstIterationSetup = false
		end
		pieceView:AddChild(imageActors[nextIndex])
		local baseFadeTimeMS = tonumber(GameUITweaks and GameUITweaks.wildcard_cycle_time_ms) or 600
		local adjustedFadeTimeMS = baseFadeTimeMS * (1.0 - (imageActors[nextIndex].alpha / 255))
		RunScriptWithParams {
			target = imageActors[nextIndex],
			script = "Interpolate",
			params = { "alpha", 255, adjustedFadeTimeMS },
			tags = { "ImageCycleHelper" },
		}
		
		-- Wait until the fade-in is complete
		WaitForTaggedScriptCompletion(imageActors[nextIndex], "ImageCycleHelper")
		
		-- Remove/hide the now covered actor
		imageActors[currentIndex]:RemoveFromParent()
		
		-- All done with this iteration.  Update indices accordingly:
		currentIndex = nextIndex
	end
end

function Scripts.SuperWildGemImageCycle(pieceView)
	-- Specify images
	local imageNames = { "superWildgem1.png", "superWildgem2.png", "superWildgem4.png", "superWildgem4.png"}
	
	-- Randomize the order in which the gems appear in:
	local imageOrder = "random start"
	if GameUITweaks then
		if IsString(GameUITweaks.superWildcard_image_order) then
			imageOrder = GameUITweaks.superWildcard_image_order
		end
	end
	
	if imageOrder == "random sort" then
		-- Randomize elements in imageNames
		local tempTable = {}
		while #imageNames > 0 do
			local i = math.random(1, #imageNames)
			local imageName = table.remove(imageNames, i)
			table.insert(tempTable, imageName)
		end
		imageNames = tempTable
		tempTable = nil
	elseif imageOrder == "random start" then
		-- Randomize where imageNames starts while keeping the array size the
		-- same and maintaining the same cyclical order.
		local tempTable = {}
		local i = math.random(1, #imageNames)
		while #tempTable < #imageNames do
			table.insert(tempTable, imageNames[i])
			--logprintf("%d, %s, %d, %d\n", i, imageNames[i], #tempTable, #imageNames)
			i = i + 1
			if i > #imageNames then
				i = 1
			end
		end
		imageNames = tempTable
		tempTable = nil
	elseif imageOrder == "uniform" then
		-- Do nothing.
	else
		-- Do nothing except log a warning.
		logprintf("WARNING: image order, <%s>, is of an unknown value.\n", tostring(imageOrder))
	end
	
	-- Create actors for each image, making each effectively invisible for the
	-- time being.
	local imageActors = {}
	for i,imageName in ipairs(imageNames) do
		local imageActor = CreateInstance(Resources.Actor:new {
			resource = "Actor",
			image = imageName,
			alignment = "Center",
			name = "GemCycleImage" .. i,
			alpha = 0,
		})
		table.insert(imageActors, imageActor)
	end
	
	-- If there are no image actors then log a warning and go no further.
	if #imageActors == 0 then
		logprint("WARNING: No images to cycle through for pieceView: "..tostring(pieceView))
		return
	end
	
	-- Make sure the first image actor is visible:
	imageActors[1].alpha = 255
	pieceView:AddChild(imageActors[1])
	
	-- If there is only one image actor, then make sure it gets displayed, but
	-- don't try cycling through other actors.
	if #imageActors == 1 then
		return
	end

	-- Repeatedly cross-fade from one image to the other
	local currentIndex = 1
	local nextIndex = nil
	local doFirstIterationSetup = true
	local randomStartA = tonumber(GameUITweaks and GameUITweaks.wildcard_random_start_A) or 20
	local randomStartB = tonumber(GameUITweaks and GameUITweaks.wildcard_random_start_B) or 230
	local randomStartMin = math.min(randomStartA, randomStartB)
	local randomStartMax = math.max(randomStartA, randomStartB)
	--logprintf("random start range = [%d,%d]\n", randomStartMin, randomStartMax)
	while true do
		-- Pick the next image to display:
		nextIndex = currentIndex + 1
		if nextIndex > #imageActors then
			nextIndex = 1
		end
	
		-- Start fading in the next image:
		imageActors[nextIndex].alpha = 0
		if doFirstIterationSetup then
			-- On the first iteration of this loop, randomize the starting position
			-- of the fade-in by a bit, in order to de-synchronize fade timing:
			local randomStart = math.random(randomStartMin, randomStartMax)
			--logprintf("randomStart = %s\n", tostring(randomStart))
			imageActors[nextIndex].alpha = randomStart
			doFirstIterationSetup = false
		end
		pieceView:AddChild(imageActors[nextIndex])
		local baseFadeTimeMS = tonumber(GameUITweaks and GameUITweaks.wildcard_cycle_time_ms) or 600
		local adjustedFadeTimeMS = baseFadeTimeMS * (1.0 - (imageActors[nextIndex].alpha / 255))
		RunScriptWithParams {
			target = imageActors[nextIndex],
			script = "Interpolate",
			params = { "alpha", 255, adjustedFadeTimeMS },
			tags = { "ImageCycleHelper" },
		}
		
		-- Wait until the fade-in is complete
		WaitForTaggedScriptCompletion(imageActors[nextIndex], "ImageCycleHelper")
		
		-- Remove/hide the now covered actor
		imageActors[currentIndex]:RemoveFromParent()
		
		-- All done with this iteration.  Update indices accordingly:
		currentIndex = nextIndex
	end
end


function Scripts.HintAnimation(rootGameView, centerPointOfMatchableSpace, actorToDim)
	local spread = 20
	local hintActor = CreateInstance { 
		name = "CurrentHintAnimation",
		class = "hgeParticleSystemActor",
		file = "FirstHint",
		alignment = "Center",
		alignTo = "TopLeft",
		alignmentOffset = centerPointOfMatchableSpace,
		ArrowWheel_top = Resources.Actor:new {
			rotationAngle = {degrees = 0},
			alignment = MidBottom,
			alignTo = MidTop,
			alignmentOffset = {0,-spread},
			image = "HintArrow.png",
			zorder = 4,
			scripts = {"PointUp"},
		},
		ArrowWheel_right = Resources.Actor:new {
			rotationAngle = {degrees = 90},
			alignment = MidLeft,
			alignTo = MidRight,
			alignmentOffset = {spread,0},
			image = "HintArrow.png",
			zorder = 4,
			scripts = {"PointRight"},
		},
		ArrowWheel_bottom = Resources.Actor:new {
			rotationAngle = {degrees = 180},
			alignment = MidTop,
			alignTo = MidBottom,
			alignmentOffset = {0,spread},
			image = "HintArrow.png",
			zorder = 4,
			scripts = {"PointDown"},
		},
		ArrowWheel_left = Resources.Actor:new {
			rotationAngle = {degrees = 270},
			alignment = MidRight,
			alignTo = MidLeft,
			alignmentOffset = {-spread,0},
			image = "HintArrow.png",
			zorder = 4,
			scripts = {"PointLeft"},
		},

	}
	rootGameView:AddChild(hintActor)
    if(actorToDim ~= nil) then
        actorToDim.tag = "HintGem"
        Scripts.Interpolate(actorToDim, "alpha", 128, 500)
    end
end

--Config.EnableSmallSpinButton = false

Script.SpinButton_Reset = function(spinButton)
	spinButton.alignmentOffset = ConfigPick{{0,-50},EnableSmallSpinButton={0,0}}
	spinButton.size = ConfigPick{{382/2,411/2},EnableSmallSpinButton={382/7,411/7}}
	if(Config.EnableSmallSpinButton)then
		spinButton.Handle.visible = false
	else
		spinButton.Handle.visible = true
	end
	--spinButton.Handle.visible = ConfigPick{true,EnableSmallSpinButton=false}
end

function Scripts.BadMove_SpinNeeded_Animation(rootGameView)
	PlaySound("metalbeep.ogg")
	local spinButton = rootGameView:FindChild("SpinButton", true)
	if spinButton then
		Scripts.Interpolate(spinButton, "extraScale", ConfigPick{{1.2, 1.2},EnableSmallSpinButton={1.1, 1.1}}, 150)
		Scripts.Interpolate(spinButton, "extraScale", {1.0, 1.0}, 150)
	end
end

function Scripts.BadMove_NoSlotPiece_Animation(columnHighlight)
	PlaySound("metalbeep.ogg")
	columnHighlight.alpha = 128
	ScriptDelay(100)
	Scripts.Interpolate(columnHighlight, "alpha", 0, 150)
	ScriptDelay(5000) --This is used to indicate to the logic that a bad move had been made within the past 20 seconds.
end

Scripts.Blink = function(eyeLids)
	eyeLids.visible = false
	while true do
		ScriptDelay(math.random(1000, 2500), "dialogTimers")
		eyeLids.visible = true
		ScriptDelay(150, timerGroup)
		eyeLids.visible = false
	end
end

Scripts.HideAndSeek = function(actor)
	actor.visible = false
	while true do
		local delay = math.random(1000, 2500)
		ScriptDelay(delay, "dialogTimers")
		actor.visible = true
		ScriptDelay(delay, timerGroup)
		actor.visible = false
	end
end

Scripts.OpenAndClosed = function(actor)
	actor.visible = false
	while true do
		local delay = math.random(1000, 2500)
		ScriptDelay(delay, "dialogTimers")
		actor.visible = true
		ScriptDelay(delay, timerGroup)
		actor.visible = false
	end
end

Scripts.Talkie = function(actor)
	actor.visible = false
	while true do
		local delay = math.random(1000, 2500)
		ScriptDelay(delay, "dialogTimers")
		actor.image = actor.images[math.random(1,#actor.images)]
		actor.visible = true
		ScriptDelay(delay, timerGroup)
		actor.visible = false
	end
end

--Used for characters where talking pieces can't show over the body
Scripts.Talkie2 = function(actor)
    --save initial image
    local initialImage = actor.image
    while true do
        local delay = math.random(1000, 2500)
        ScriptDelay(delay, "dialogTimers")
        actor.image = actor.images[math.random(1,#actor.images)]
        ScriptDelay(delay, timerGroup)
        actor.image = initialImage
    end
end

function Scripts.Preshow_StartingNewGame(screen,gameScreen, stats, goal1,goal2)
	-- HACK: A bit hacky saving in var like this to show later
    gameScreen.LevelGoalText1 = goal1
    gameScreen.LevelGoalText2 = goal2
	gameScreen.LevelName = stats.levelName
	gameScreen.LevelNumber = stats.levelNumber
	gameScreen.IsLevelGame = stats.isLevelGame
	gameScreen.HiddenLevel = stats.hiddenLevel
	gameScreen.Stats = stats
	gameScreen.EventScore = 0
	if (stats.isLevelGame) then
		gameScreen.LandNumber = stats.landNumber
	end

	didJackpot_Cascade = false
	didJackpot_Jackpot = false

	local MenuButton = gameScreen:FindChild("MenuButton",true)
	local SpinButton = gameScreen:FindChild("SpinButton",true)
	local Logo = gameScreen:FindChild("Logo",true)
	local ScoreContainer = gameScreen:FindChild("ScoreContainer",true)
	local MultiplierPanel = gameScreen:FindChild("MultiplierPanel",true)
	local TopScoresContainer = gameScreen:FindChild("TopScoresContainer",true)
	local LabGoalButton = gameScreen:FindChild("LabGoalButton",true)
	local RemainingSpinsContainer = gameScreen:FindChild("RemainingSpinsContainer",true)
	local SpecialGemPanel = gameScreen:FindChild("SpecialGemPanel",true)
	local TrickPanel = gameScreen:FindChild("TrickPanel",true)
	local AllPieceDropCells = gameScreen:FindChild("AllPieceDropCells",true)
	local AllBoardCells = gameScreen:FindChild("AllBoardCells",true)
	local AllSlotsCells = gameScreen:FindChild("AllSlotsCells",true)
	local StarMeter = gameScreen:FindChild("StarMeter",true)
	local GameBoard = gameScreen:FindChild("ReelBackground",true)
	local AllGamePieces = gameScreen:FindChild("AllGamePieces",true)
	local BoardSpaceMatchedView = gameScreen:FindChild("BoardSpaceMatchedView",true)
	
	MenuButton.visible = false
	SpinButton.visible = false
	Logo.visible = false
	ScoreContainer.visible = false
	MultiplierPanel.visible = false
	TopScoresContainer.visible = false
	LabGoalButton.visible = false
	RemainingSpinsContainer.visible = false
	SpecialGemPanel.visible = false
	TrickPanel.visible = false
	AllPieceDropCells.visible = false
	AllBoardCells.visible = false
	GameBoard.visible = false

	AllBoardCells.visible = false
	AllGamePieces.visible = false

	local isChallenge = TopScoresContainer.GoalContainer.InnerScoreLabel.score > 0
	local isLevel = TopScoresContainer.LevelGoalContainer.InnerScoreLabel.text ~= ""

	local boardWidth = AllBoardCells.boardWidth
	local boardHeight = AllBoardCells.boardHeight
	
	-- Turn off pattern until we want to show
	for y=0,boardHeight-1 do
		for x=0,boardWidth-1 do
			local nameOfMatchingActor = "BoardSpaceMatchedView_" .. x .. "_" .. y
			local matchingActor = gameScreen:FindChild(nameOfMatchingActor,true)
			if(matchingActor)then
				matchingActor.visible = false
			end
			
			local nameOfGemEater = "GemEaterView_" .. x .. "_" .. y
			local gemEater = gameScreen:FindChild(nameOfGemEater,true)
			if(gemEater)then
				gemEater.visible = false
			end
			
			local cellName = "BoardCell_"..x.."_"..y
			local cell = AllBoardCells:FindChild(cellName,true)
			if(cell ~= nil) then
				if(cell.SteelGlassDecoration ~= nil) then
					cell.SteelGlassDecoration.visible = true
				end
			end
		end
	end

	local metersize = StarMeter.size.x

	if(stats.goalScore1 ~= nil) then
		local goalScore1_percent	= (stats.goalScore1*100)/stats.goalScore3
		local goalScore2_percent	= (stats.goalScore2*100)/stats.goalScore3
		local goalScore3_percent	= (stats.goalScore3*100)/stats.goalScore3

		local goalScore1_pixels		= (metersize*goalScore1_percent)/100
		local goalScore2_pixels		= (metersize*goalScore2_percent)/100
		local goalScore3_pixels		= (metersize*goalScore3_percent)/100

		local StarMeter = gameScreen:FindChild("StarMeter",true)
		local Star1 = StarMeter:FindChild("Star1",true)
		local Star2 = StarMeter:FindChild("Star2",true)
		local Star3 = StarMeter:FindChild("Star3",true)
		
		Star1.alignmentOffset = {goalScore1_pixels,Star1.alignmentOffset.y}
		Star2.alignmentOffset = {goalScore2_pixels,Star2.alignmentOffset.y}
		Star3.alignmentOffset = {goalScore3_pixels,Star3.alignmentOffset.y}

		Star1.visible = true
		Star2.visible = true
		Star3.visible = true

		if(isChallenge)then
			Star1.image = "Medal_small_off.png"
			Star2.image = "Medal_small_off.png"
			Star3.image = "Medal_small_off.png"
		elseif (stats.isLabEventGame) then
			Star1.image = "LabLevel_MeterFlask_Slot.png"
			Star2.image = "LabLevel_MeterFlask_Slot.png"
			Star3.image = "LabLevel_MeterFlask_Slot.png"
		else
			if(stats.workerImage1 ~= "") then
				Star1.workerImage.image = stats.workerImage1
			else
				Star1.workerImage.visible = false
			end
			if(stats.workerImage2 ~= "") then
				Star2.workerImage.image = stats.workerImage2
			else
				Star2.workerImage.visible = false
			end
			if(stats.workerImage3 ~= "") then
				Star3.workerImage.image = stats.workerImage3
			else
				Star3.workerImage.visible = false
			end
			if(stats.starForQuickQuest ~= nil) then
				local starNameForQuickQuest = "Star" .. stats.starForQuickQuest
				local quickQuestStar = StarMeter:FindChild(starNameForQuickQuest,true)
				quickQuestStar.quickQuestCup.visible = true
			end
		end
	else
		local StarMeter = gameScreen:FindChild("StarMeter",true)
		local Star1 = StarMeter:FindChild("Star1",true)
		local Star2 = StarMeter:FindChild("Star2",true)
		local Star3 = StarMeter:FindChild("Star3",true)
		
		Star1.visible = false
		Star2.visible = false
		Star3.visible = false

		if(isChallenge)then
			Star1.image = "Medal_small_off.png"
			Star2.image = "Medal_small_off.png"
			Star3.image = "Medal_small_off.png"
		elseif (isLabEventGame) then
			Star1.image = "LabLevel_MeterFlask_Slot.png"
			Star2.image = "LabLevel_MeterFlask_Slot.png"
			Star3.image = "LabLevel_MeterFlask_Slot.png"
		end
	end
end

function Scripts.StartingNewGame(screen,gameScreen, isLevelGame)
	--logprint("StartingNewGame")

	local MenuButton = gameScreen:FindChild("MenuButton",true)
	local SpinButton = gameScreen:FindChild("SpinButton",true)
	local Logo = gameScreen:FindChild("Logo",true)
	local ScoreContainer = gameScreen:FindChild("ScoreContainer",true)
	local MultiplierPanel = gameScreen:FindChild("MultiplierPanel",true)
	local TopScoresContainer = gameScreen:FindChild("TopScoresContainer",true)
	local LabGoalButton = gameScreen:FindChild("LabGoalButton",true)
	local RemainingSpinsContainer = gameScreen:FindChild("RemainingSpinsContainer",true)
	local SpecialGemPanel = gameScreen:FindChild("SpecialGemPanel",true)
	local TrickPanel = gameScreen:FindChild("TrickPanel",true)
	local AllPieceDropCells = gameScreen:FindChild("AllPieceDropCells",true)
	local AllBoardCells = gameScreen:FindChild("AllBoardCells",true)
	local AllSlotsCells = gameScreen:FindChild("AllSlotsCells",true)
	local AllGamePieces = gameScreen:FindChild("AllGamePieces",true)
	local StarMeter = gameScreen:FindChild("StarMeter",true)
	local GameBoard = gameScreen:FindChild("ReelBackground",true)
	
	local boardWidth = AllBoardCells.boardWidth
	local boardHeight = AllBoardCells.boardHeight

	ScriptDelay(250)
	SlideOn(AllBoardCells,250,"right","nowait")
	SlideOn(AllGamePieces,250,"right","nowait")
	PlaySound("Win11_60.ogg")

	local showTricks = isLevelGame or Config.PayToWinChallenge

	if(TrickPanel.position.y >300)then
		if(showTricks) then
			SlideOn(TrickPanel,250,"bottom","nowait")
		end
		SlideOn(ScoreContainer,250,"top","nowait")
	else
		if(showTricks) then
			SlideOn(TrickPanel,250,"top","nowait")
		end
		SlideOn(ScoreContainer,250,"bottom","nowait")
	end

	--[[if IsIPhone5() or APP.useIPhone5AppAssets then
		SlideOn(RemainingSpinsContainer,250,"top","nowait")
	elseif IsIPad() or APP.useIPadAppAssets then
		SlideOn(RemainingSpinsContainer,250,"top","nowait")
	else --]]
		SlideOn(RemainingSpinsContainer,250,"bottom","nowait")
	--end

	if (gameScreen.Stats.isLabEventGame) then
		SlideOn(LabGoalButton,250,"top","nowait")
	end

	local subpanelsShowing = false
	subpanelsShowing = subpanelsShowing or SpecialGemPanel.SpecialGem1_1.visible
	subpanelsShowing = subpanelsShowing or SpecialGemPanel.SpecialGem2_1.visible
	subpanelsShowing = subpanelsShowing or SpecialGemPanel.SpecialGem2_2.visible
	subpanelsShowing = subpanelsShowing or SpecialGemPanel.SpecialGem3_1.visible
	subpanelsShowing = subpanelsShowing or SpecialGemPanel.SpecialGem3_2.visible
	subpanelsShowing = subpanelsShowing or SpecialGemPanel.SpecialGem3_3.visible

	if(subpanelsShowing) then
		SlideOn(SpecialGemPanel,250,"top","nowait")
	end
	SlideOn(MenuButton,250,"bottom","nowait")

	SlideOn(TopScoresContainer,250,"top","nowait")
	if IsIPhone5() or APP.useIPhone5AppAssets then
		if(Logo.visible)then
			SlideOn(Logo,250,"top","nowait")
		end
	elseif IsIPad() or APP.useIPadAppAssets then
		if(Logo.visible)then
			SlideOn(Logo,250,"top","nowait")
		end
	end

	ScriptDelay(350)
	local playSoundOnce = false
	local playGemEaterSound = false
	local playSteelGlassSound = false

	--Animate in pattern squares and gem eaters.
	for y=0,boardHeight-1 do
		for x=0,boardWidth-1 do
			local nameOfMatchingActor = "BoardSpaceMatchedView_" .. x .. "_" .. y
			local matchingActor = gameScreen:FindChild(nameOfMatchingActor,true)
			if(matchingActor)then
				playSoundOnce = true
				ScaleIn(matchingActor,{1.25,1.25},250,false)
				FadeIn(matchingActor,250)
			end

			local nameOfGemEater = "GemEaterView_" .. x .. "_" .. y
			local gemEater = gameScreen:FindChild(nameOfGemEater,true)
			if(gemEater)then
				playGemEaterSound = true
				ScaleIn(gemEater,{1.25,1.25},250,false)
				FadeIn(gemEater,250)
			end
			
			local cellName = "BoardCell_"..x.."_"..y
			local cell = AllBoardCells:FindChild(cellName,true)
			if(cell ~= nil) then
				if(cell.SteelGlassDecoration ~= nil) then
					playSteelGlassSound = true
					RotateIn(cell.SteelGlassDecoration,{revolutions=-0.25},250,false)
					--RotateIn(cell.SteelGlassDecoration,{revolutions=0.25},250,false)
					--ScaleIn(cell.SteelGlassDecoration,{1.15,1.15},250,false)
					--FadeIn(cell.SteelGlassDecoration,250)
				end
			end
		end
	end

	if(playSoundOnce)then
		PlaySound("slot_machine_win_21_short3.ogg")
	end
	if(playGemEaterSound) then
		PlaySound("Hit_MetalRobotPunch1.ogg")
	end
	if(playSteelGlassSound) then
		PlaySound("Hit_MetalSpikes_Disengage.ogg")
	end

	gameScreen:StartGameMusic()

	local gameBoard = gameScreen:FindChild("ReelBackground",true)
	if(gameBoard)then
		SlideOn(gameBoard,500,"bottom","nowait")
	end

	StopSound("slot_machine_spin_loop_02.ogg")
end

Scripts.Preshow_FinishGameIntroIntro = function(screen, gameScreen)
end

Scripts.Present_MultiplierBonus = function(rootActor)
	local MultiplierPanel = rootActor:FindChild("MultiplierPanel",true)
	PlaySound("sparkle_little_reverse.ogg")

	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarToss1.bigger.blue",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	rootActor:AddChild(burst)	

	AddScriptWithParams(rootActor,"GoodJobWord","goodjobmultiplierbonus.png",STRINGS("MultiplierBonus"),750,"top")
	PlaySound("slot_machine_bet_18.ogg")
	ScriptDelay(750)

	local multiplierText = 2 .."X"
	PlaySound("slot_machine_bet_01.ogg")
	Scripts.AnnounceMultiplier(rootActor,multiplierText)
	
	FireParticle(MultiplierPanel,"StarBurst_Blue_Small","Center")

	PlaySound("slot_machine_bet_03.ogg")
	ScriptDelay(250) --give time for the good job word to do its thing
end

Scripts.Present_StartBurst = function(rootActor, affectedPieceViews)
	--Hide the new burst actors until it's time to present them
	for unused,pieceView in pairs(affectedPieceViews) do
		if(pieceView.VerticalSplashDecoration ~= nil) then
			pieceView.VerticalSplashDecoration.visible = false
		end
		if(pieceView.HorizontalSplashDecoration ~= nil) then
			pieceView.HorizontalSplashDecoration.visible = false
		end
		if(pieceView.AreaSplashDecoration ~= nil) then
			pieceView.AreaSplashDecoration.visible = false
		end
	end

	PlaySound("sparkle_little_reverse.ogg")

	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarToss1.bigger.blue",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	rootActor:AddChild(burst)	

	AddScriptWithParams(rootActor,"GoodJobWord","goodjobstartburst.png",STRINGS("StartBurst"),750,"top")
	PlaySound("slot_machine_bet_17.ogg")
	ScriptDelay(750)
	local i = 1
	for unused,pieceView in pairs(affectedPieceViews) do
		if(pieceView.VerticalSplashDecoration ~= nil) then
			pieceView.VerticalSplashDecoration.visible = true
		end
		if(pieceView.HorizontalSplashDecoration ~= nil) then
			pieceView.HorizontalSplashDecoration.visible = true
		end
		if(pieceView.AreaSplashDecoration ~= nil) then
			pieceView.AreaSplashDecoration.visible = true
		end

		PlaySound("slot_machine_bet_0"..i..".ogg")
		i = i + 1
		if (i>3) then i = 3 end
		PlaySound("Hit_MetalRobotPunch1.ogg")
		--Can't use Pop here because it will wipe out scaling from 7x8 board on some gem types
		local oldScale = pieceView.extraScale
		FireParticle(pieceView,"StarBurst_Blue_Small","Center")
		ScaleTo(pieceView, {1.3,1.3}, 150)
		ScriptDelay(150)
		ScaleTo(pieceView, oldScale, 150)
		ScriptDelay(150)
	end
	
	ScriptDelay(250) --give time for the good job word to do its thing
end

Scripts.Present_SpinSaver = function(rootActor)
	--Hide spin saver tokens
	local remainingSpinsContainer = rootActor:FindChild("RemainingSpinsContainer",true)
	for i = 1,3 do
		local fairyName = "FairyToken" .. i
		local spinFairy = remainingSpinsContainer[fairyName]
		spinFairy.visible = false
	end

	PlaySound("sparkle_little_reverse.ogg")

	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarToss1.bigger.blue",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	rootActor:AddChild(burst)	

	AddScriptWithParams(rootActor,"GoodJobWord","goodjobspinsaver.png",STRINGS("SpinSaver"),2000,"top")
	PlaySound("slot_machine_bet_18.ogg")
	ScriptDelay(750)
	
	for i = 1,3 do
		local fairyName = "FairyToken" .. i
		local spinFairy = remainingSpinsContainer[fairyName]
		PlaySound("slot_machine_bet_0"..i..".ogg")
		FireParticle(spinFairy,"StarBurst_Blue_Small","Center")

		local destination = spinFairy.position
		local globalDestination = rootActor.center
		globalDestination.x = globalDestination.x-25
		globalDestination.y = globalDestination.y+20
		local localDestination = spinFairy.parent:GlobalToLocalCoord(globalDestination)
	
		spinFairy.position=localDestination
		spinFairy.visible = true
		ScaleTo(spinFairy, {2.0, 2.0}, 100)
		ScriptDelay(250)
		ScaleTo(spinFairy, {1.0, 1.0}, 150,false)
		Scripts.Interpolate(spinFairy, "position", destination, 150)

		--Pop(spinFairy)
	end

	ScriptDelay(250) --give time for the good job word to do its thing
end

Scripts.Present_FinishGameIntroIntro = function(screen, gameScreen, goal1,goal2, isLevelGame, stats)
	ScriptDelay(500)
	local allBoardCells = gameScreen:FindChild("AllBoardCells",true)
	allBoardCells:AddScript("BoardShine","")

	gameScreen.showGoal = true

	screen:AddScript("FinishGameIntroIntro_LetsGo","FinishGameIntroIntro_LetsGo")
	
	ScriptDelay(1000)
	if(goal1 ~= "" and gameScreen.showGoal) then
		local topScoresContainer = screen:FindChild("TopScoresContainer",true)
		Scripts.ShowGoalDialog(screen,gameScreen,goal1,goal2,topScoresContainer,stats,isLevelGame)
	end
	gameScreen.showGoal = true
end

Scripts.FinishGameIntroIntro_LetsGo = function(rootGameView)
	local screen = rootGameView.parent.parent
	local goodJobWord = screen:FindChild("FlashWord",true)
	goodJobWord.extraScale = {1.0,1.0}
	local goodJobWordImage = "letsgo.png"
	goodJobWord.image = goodJobWordImage

	goodJobWord.visible = false
	
	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarBurst3",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	rootGameView:AddChild(burst)	
	AddScriptWithParams(rootGameView,"GoodJobWord","letsgo.png",STRINGS("Let's go!"),1000,"top")
	PlaySound("Cascade_LetsGo.ogg")
end

local CharactersByLand = {
	[1] = "Jasper",
	[2] = "Otter",
	[3] = "Bear",
	[4] = "Bat",
	[5] = "Lemur",
	[6] = "Jaguar",
	[7] = "Marmot",
	[8] = "Fox",
	[9] = "Monkey",
	[10] = "Jaguar",
	[11] = "RedPanda",
	[12] = "StBernard",
	[13] = "Polecat",
	[14] = "Bull",
	[15] = "Monkey",
	[16] = "Jackrabbit",
	[17] = "Caracal",
	[18] = "Lion",
	[19] = "Cat",
	[20] = "Coyote",
	[21] = "GiantPanda",
	[22] = "Wolf",
	[23] = "Mouse",
	[24] = "Narwhal",
	[25] = "SpacePup",
	[26] = "SabertoothCubs",
	[27] = "Griffin",
	[28] = "Penguin",
	[29] = "Goat",
	[30] = "RedFox",
	[31] = "Jackal",
	[32] = "Jackalope",
	[33] = "Raccoon",
	[34] = "Jasper",
}

Scripts.ShowGoalDialog = function(screen,gameScreen,goal1,goal2,self,stats,isLevelGame)
	local title = ""
	local character = "Jasper"
	if(isLevelGame)then
		if(stats.hiddenLevel) then
			title = stats.levelName
		else
			title = stats.levelNumber..": "..stats.levelName
			if (Language == "fr") then
				title = stats.levelNumber.." : "..stats.levelName
			end
		end
		character = CharactersByLand[stats.land]
		if (not character)then
			character = "Jasper"
		end
	else
		title = stats.levelName
		--goal = STRINGS("Score ")..Int2Comma(stats.goalScore1)..STRINGS(" points")
	end

	local dialogImageName = "WoodPanel_Big_"..character..".png"
	if(not HasResourceInfo(dialogImageName)) then
		dialogImageName = ""
	end
		
	--Display main goal in center of screen, then fade out. Dismiss early if clicked.
    --Goal Flash Dialog
	screen.GoalFlash = Resources.Button:new {
		resource = "Button",
		wasClicked = false,
		layer = 5,
		alignment = Center,
		alignmentOffset = {0,-25},
		image = "",
		size = gameScreen.size,
		useFrames = false,
		visible = false,
		GURU_BUTTONCLICK = function (eventData)
			-- Filter out events that weren't sent by the item view to itself.
			if eventData.sender ~= eventData.receiver then
				return false
			end
		
			eventData.sender.wasClicked = true
			return true
		end,
		
        goalText1 = Resources.GameLabelBold:new {
			resource = "GameLabelBold",
			text = goal1,
			alignment = Center,
			alignmentOffset = {0,-12},
			wrapLimit = 200,
			heightLimit = 28,
			font = Resources.DialogFont:new {
				resource = "DialogFont",
				fontSize = 28,
			},
		},

        goalText2 = Resources.GameLabelBold:new {
            resource = "GameLabelBold",
            text = goal2,
            alignment = Center,
            alignmentOffset = {0,12},
            wrapLimit = 200,
            heightLimit = 30,
            font = Resources.DialogFont:new {
                resource = "DialogFont",
                fontSize = 22,      
            },
        },

        levelName = Resources.AppTitleDialogLabel:new {
            resource = "AppTitleDialogLabel",
            text = title,
            font = Resources.DialogBoldFont:new {
            resource = "DialogBoldFont",
            fontSize = 28,      },
            alignment = Center,
            alignTo = Center,
            alignmentOffset = {0,-62},
            visible = true,
            zorder = 1,
            widthLimit = 220,
        },

        backgroundLayer = Resources.Actor:new {
            resource = "Actor",
            image = "WoodPanel_Big_Characters.png",
            alignment = Center,
            alignTo = Center,
            alignmentOffset = {0,-90},
            characterImg = Resources.Actor:new {
                image = dialogImageName,
            },
            --[[
            eyeLids = Resources.Actor:new {
                resource = "Actor",
                image = "WoodPanel_Big_Jasper_Blink3.png",
                visible = false,
                scripts = {"Blink"},
            },
            ]]
        },
	}
	PlaySound("slot_machine_win_23.ogg")
    local originalPosition = screen.GoalFlash.position
		
	SlideOn(screen.GoalFlash,250,"top","wait")

	local delay = 4000
	for i = 1,delay/TICK_INTERVAL do
		ScriptBreak()
		if(screen.GoalFlash.wasClicked) then
			break
		end
	end

	PlaySound("slot_machine_win_23_reverse.ogg")
	local delay = 600
	for i = 1,delay/TICK_INTERVAL do
		ScriptBreak()
		if(screen.GoalFlash.wasClicked) then
			break
		end
	end

    local originalPosition = screen.GoalFlash.position
	SlideAndWait(screen.GoalFlash, {originalPosition.x , originalPosition.y + 10}, 150)

	SlideOff(screen.GoalFlash,250,"top","nowait")
	FadeOutAndDestroy(screen.GoalFlash,250)
end

Scripts.ShowLabEventGoalDialog = function(screen,gameScreen,self,levelName,eventScore)
	local title = levelName
	local character = "Jasper"
	local dialogImageName = "WoodPanel_Big_"..character..".png"
	if(not HasResourceInfo(dialogImageName)) then
		dialogImageName = ""
	end
		
	--Display main goal in center of screen, then fade out. Dismiss early if clicked.
    --Goal Flash Dialog
	screen.LabGoalFlash = Resources.Button:new {
		resource = "Button",
		wasClicked = false,
		layer = 5,
		alignment = Center,
		alignmentOffset = {0,-25},
		image = "",
		size = gameScreen.size,
		useFrames = false,
		visible = false,
		GURU_BUTTONCLICK = function (eventData)
			-- Filter out events that weren't sent by the item view to itself.
			if eventData.sender ~= eventData.receiver then
				return false
			end
		
			eventData.sender.wasClicked = true
			return true
		end,

        metricText = Resources.GameLabelBold:new {
            resource = "GameLabelBold",
            text = LAB_EVENT_MGR:GetMetricDescriptionText(false),
            alignment = Center,
            alignmentOffset = {0,-24},
            wrapLimit = 200,
            heightLimit = 25,
            font = Resources.DialogFont:new {
                resource = "DialogFont",
                fontSize = 22,      
            },
        },

		currentScoreText = Resources.GameLabelBold:new {
            resource = "GameLabelBold",
            text = string.format(STRINGS("CurrentScoreFMT"), Int2Comma(eventScore)),
            alignment = Center,
            alignmentOffset = {0,0},
            wrapLimit = 200,
            heightLimit = 25,
            font = Resources.DialogFont:new {
                resource = "DialogFont",
                fontSize = 22,      
            },
        },

		highScoreText = Resources.GameLabelBold:new {
            resource = "GameLabelBold",
            text = string.format(STRINGS("HighScoreFMT"), LAB_EVENT_MGR:GetCurrentPlayerScore()),
            alignment = Center,
            alignmentOffset = {0,24},
            wrapLimit = 200,
            heightLimit = 25,
            font = Resources.DialogFont:new {
                resource = "DialogFont",
                fontSize = 22,      
            },
        },

        levelName = Resources.AppTitleDialogLabel:new {
            resource = "AppTitleDialogLabel",
            text = title,
            font = Resources.DialogBoldFont:new {
            resource = "DialogBoldFont",
            fontSize = 28,      },
            alignment = Center,
            alignTo = Center,
            alignmentOffset = {0,-62},
            visible = true,
            zorder = 1,
            widthLimit = 220,
        },

        backgroundLayer = Resources.Actor:new {
            resource = "Actor",
            image = "WoodPanel_Big_Characters.png",
            alignment = Center,
            alignTo = Center,
            alignmentOffset = {0,-90},
            characterImg = Resources.Actor:new {
                image = dialogImageName,
            },
            --[[
            eyeLids = Resources.Actor:new {
                resource = "Actor",
                image = "WoodPanel_Big_Jasper_Blink3.png",
                visible = false,
                scripts = {"Blink"},
            },
            ]]
        },
	}
	PlaySound("slot_machine_win_23.ogg")
    local originalPosition = screen.LabGoalFlash.position
		
	SlideOn(screen.LabGoalFlash,250,"top","wait")

	local delay = 4000
	for i = 1,delay/TICK_INTERVAL do
		ScriptBreak()
		if(screen.LabGoalFlash.wasClicked) then
			break
		end
	end

	PlaySound("slot_machine_win_23_reverse.ogg")
	local delay = 600
	for i = 1,delay/TICK_INTERVAL do
		ScriptBreak()
		if(screen.LabGoalFlash.wasClicked) then
			break
		end
	end

    local originalPosition = screen.LabGoalFlash.position
	SlideAndWait(screen.LabGoalFlash, {originalPosition.x , originalPosition.y + 10}, 150)

	SlideOff(screen.LabGoalFlash,250,"top","nowait")
	FadeOutAndDestroy(screen.LabGoalFlash,250)
end

Scripts.ShowStarsDialog = function(screen,gameScreen,self,stats,isLevelGame)
	local title = ""
	local goalScore1Text = Int2Comma(stats.goalScore1)
	local goalScore2Text = Int2Comma(stats.goalScore2)
	local goalScore3Text = Int2Comma(stats.goalScore3)

	local star1Image = "StarBig_achieved.png"
	local star2Image = "StarBig_achieved.png"
	local star3Image = "StarBig_achieved.png"

	logprint(SerializeToString(stats))

	local character = "Jasper"
	if(isLevelGame)then
		if(stats.hiddenLevel) then
			title = stats.levelName
		else
			title = stats.levelNumber..": "..stats.levelName
			if (Language == "fr") then
				title = stats.levelNumber.." : "..stats.levelName
			end
		end
		character = CharactersByLand[stats.land]
		if (not character)then
			character = "Jasper"
		end

		if (stats.isLabEventGame) then
			star1Image = "LabLevel_MeterFlask_1.png"
			star2Image = "LabLevel_MeterFlask_2.png"
			star3Image = "LabLevel_MeterFlask_3.png"
		end
	else
		title = stats.levelName
		star1Image = "Medal_small_bronze.png"
		star2Image = "Medal_small_silver.png"
		star3Image = "Medal_small_gold.png"
	end
	
	local dialogImageName = "WoodPanel_Big_"..character..".png"
	if(not HasResourceInfo(dialogImageName)) then
		dialogImageName = ""
	end

	--Display main goal in center of screen, then fade out. Dismiss early if clicked.
    -- Star Flash Dialog
	screen.GoalFlash = Resources.Button:new {
		resource = "Button",
		wasClicked = false,
		layer = 5,
		alignment = Center,
		alignmentOffset = {0,-40},
		image = "",
		size = gameScreen.size,
		useFrames = false,
		visible = false,
		GURU_BUTTONCLICK = function (eventData)
			-- Filter out events that weren't sent by the item view to itself.
			if eventData.sender ~= eventData.receiver then
				return false
			end
		
			eventData.sender.wasClicked = true
			return true
		end,
		goalText = Resources.GameLabelBold:new {
			resource = "GameLabelBold",
			text = starText,
			alignment = Center,
			alignmentOffset = {0,20},
			wrapLimit = 200,
			textAlignment = "Right",
			font = Resources.DialogFont:new {
				resource = "DialogFont",
				fontSize = 24,
			},
			margins = Resources.Margins:new
			{
				class	= "Margins",
				left	= 10,
				right	= 10,
				top		= 10,
				bottom	= 10,
			},

            levelName = Resources.AppTitleDialogLabel:new {
                resource = "AppTitleDialogLabel",
                text = title,
                font = Resources.DialogBoldFont:new {
                resource = "DialogBoldFont",
                fontSize = 28,      },
                alignment = Center,
                alignTo = Center,
                alignmentOffset = {0,-62},
                visible = true,
                zorder = 1,
                widthLimit = 220,
            },

			Star1 = Resources.Actor:new{
				resource = "Actor",
				image = star1Image,
				zorder = 1,
				size = {20,20},
				alignment = "Center",
				alignmentOffset = {-40,-25},
				Star_text = Resources.GameLabel:new {
					resource = "GameLabel",
					text = goalScore1Text,
					alignment = MidLeft,
					alignTo = MidRight,
					alignmentOffset = {20,0},
					font = Resources.DialogFont:new {
						resource = "DialogFont",
						fontSize = 24,
					},
					widthLimit = 80,
				},
				Star1_Worker = Resources.Actor:new {
					resource = "Actor",
					image = stats.workerImage1 or "",
					size = {20,20},
					alignment = MidRight,
					alignTo = MidLeft,
					alignmentOffset = {-10,0},
				},
			},

			Star2 = Resources.Actor:new{
				resource = "Actor",
				image = star2Image,
				zorder = 1,
				size = {20,20},
				alignment = "Center",
				alignmentOffset = {-40,0},
				Star_text = Resources.GameLabel:new {
					resource = "GameLabel",
					text = goalScore2Text,
					alignment = MidLeft,
					alignTo = MidRight,
					alignmentOffset = {20,0},
					font = Resources.DialogFont:new {
						resource = "DialogFont",
						fontSize = 24,
					},
					widthLimit = 80,
				},
				Star2_Worker = Resources.Actor:new {
					resource = "Actor",
					image = stats.workerImage2 or "",
					size = {20,20},
					alignment = MidRight,
					alignTo = MidLeft,
					alignmentOffset = {-10,0},
				},
			},

			Star3 = Resources.Actor:new{
				resource = "Actor",
				image = star3Image,
				zorder = 1,
				size = {20,20},
				alignment = "Center",
				alignmentOffset = {-40,25},
				Star_text = Resources.GameLabel:new {
					resource = "GameLabel",
					text = goalScore3Text,
					alignment = MidLeft,
					alignTo = MidRight,
					alignmentOffset = {20,0},
					font = Resources.DialogFont:new {
						resource = "DialogFont",
						fontSize = 24,
					},
					widthLimit = 80,
				},
				Star3_Worker = Resources.Actor:new {
					resource = "Actor",
					image = stats.workerImage3 or "",
					size = {20,20},
					alignment = MidRight,
					alignTo = MidLeft,
					alignmentOffset = {-10,0},
				},
			},

			backgroundLayer = Resources.Actor:new {
				resource = "Actor",
				image = "WoodPanel_Big_Characters.png",
				alignment = Center,
				alignTo = Center,
				alignmentOffset = {0,-90},
				characterImg = Resources.Actor:new {
					image = dialogImageName,
				},
				--[[
				eyeLids = Resources.Actor:new {
					resource = "Actor",
					image = "WoodPanel_Big_Jasper_Blink3.png",
					visible = false,
					scripts = {"Blink"},
				},
				]]
			},

            OnPostSpawn = function(self)

                if (QQ_MGR:ShouldCurrentLevelShowIcon()) then

                    local QuickQuestIcon = Resources.Actor:new {
                        resource = "Actor",
                        image = QQ_MGR:GetImagePath(false,0.34),
                        alignment = MidRight,
                        alignTo = MidLeft,
                        alignmentOffset = {-30,0},
                    }

                    local starChild
                    local numStarsToWin = QQ_MGR:GetCurrentLevelStarToWin()

                    if (numStarsToWin == 1) then starChild = self.Star1
                    elseif (numStarsToWin == 2) then starChild = self.Star2
                    elseif (numStarsToWin == 3) then starChild = self.Star3
                    end

                    if (starChild) then
                        starChild:AddChild(CreateInstance(QuickQuestIcon))
                    end

                end
            end,
		},
	}
	PlaySound("slot_machine_win_23.ogg")
    local originalPosition = screen.GoalFlash.position
		
	SlideOn(screen.GoalFlash,250,"top","wait")

	local delay = 4000
	for i = 1,delay/TICK_INTERVAL do
		ScriptBreak()
		if(screen.GoalFlash.wasClicked) then
			break
		end
	end

	PlaySound("slot_machine_win_23_reverse.ogg")
	local delay = 600
	for i = 1,delay/TICK_INTERVAL do
		ScriptBreak()
		if(screen.GoalFlash.wasClicked) then
			break
		end
	end

    local originalPosition = screen.GoalFlash.position
	SlideAndWait(screen.GoalFlash, {originalPosition.x , originalPosition.y + 10}, 150)

	SlideOff(screen.GoalFlash,250,"top","nowait")
	FadeOutAndDestroy(screen.GoalFlash,250)
end

function Scripts.ClearGameScreen(screen,gameScreen)
	local MenuButton = gameScreen:FindChild("MenuButton",true)
	local SpinButton = gameScreen:FindChild("SpinButton",true)
	local Logo = gameScreen:FindChild("Logo",true)
	local ScoreContainer = gameScreen:FindChild("ScoreContainer",true)
	local MultiplierPanel = gameScreen:FindChild("MultiplierPanel",true)
	local TopScoresContainer = gameScreen:FindChild("TopScoresContainer",true)
	local LabGoalButton = gameScreen:FindChild("LabGoalButton", true)
	local RemainingSpinsContainer = gameScreen:FindChild("RemainingSpinsContainer",true)
	local TrickPanel = gameScreen:FindChild("TrickPanel",true)
	local AllPieceDropCells = gameScreen:FindChild("AllPieceDropCells",true)
	local AllBoardCells = gameScreen:FindChild("AllBoardCells",true)
	local AllSlotsCells = gameScreen:FindChild("AllSlotsCells",true)
	local StarMeter = gameScreen:FindChild("StarMeter",true)
	local GameBoard = gameScreen:FindChild("ReelBackground",true)
	local AllGamePieces = gameScreen:FindChild("AllGamePieces",true)
	local BoardSpaceMatchedView = gameScreen:FindChild("BoardSpaceMatchedView",true)
	
	MenuButton.visible = false
	SpinButton.visible = false
	Logo.visible = false
	ScoreContainer.visible = false
	MultiplierPanel.visible = false
	TopScoresContainer.visible = false
	LabGoalButton.visible = false
	RemainingSpinsContainer.visible = false
	TrickPanel.visible = false
	AllPieceDropCells.visible = false
	AllBoardCells.visible = false
	GameBoard.visible = false

	AllBoardCells.visible = false
	AllGamePieces.visible = false

	local boardWidth = AllBoardCells.boardWidth
	local boardHeight = AllBoardCells.boardHeight

	-- Turn off pattern until we want to show
	for y=0,boardHeight-1 do
		for x=0,boardWidth-1 do
			local nameOfMatchingActor = "BoardSpaceMatchedView_" .. x .. "_" .. y
			local matchingActor = gameScreen:FindChild(nameOfMatchingActor,true)
			if(matchingActor)then
				matchingActor.visible = false
			end
		end
	end
end

function Scripts.SlideAndClearGameScreen(screen,gameScreen)
	local MenuButton = gameScreen:FindChild("MenuButton",true)
	local SpinButton = gameScreen:FindChild("SpinButton",true)
	local Logo = gameScreen:FindChild("Logo",true)
	local ScoreContainer = gameScreen:FindChild("ScoreContainer",true)
	local MultiplierPanel = gameScreen:FindChild("MultiplierPanel",true)
	local TopScoresContainer = gameScreen:FindChild("TopScoresContainer",true)
	local LabGoalButton = gameScreen:FindChild("LabGoalButton", true)
	local RemainingSpinsContainer = gameScreen:FindChild("RemainingSpinsContainer",true)
	local TrickPanel = gameScreen:FindChild("TrickPanel",true)
	local SpecialGemPanel = gameScreen:FindChild("SpecialGemPanel",true)
	local AllPieceDropCells = gameScreen:FindChild("AllPieceDropCells",true)
	local AllBoardCells = gameScreen:FindChild("AllBoardCells",true)
	local AllSlotsCells = gameScreen:FindChild("AllSlotsCells",true)
	local AllGemEaters = gameScreen:FindChild("AllGemEaters",true)
	local StarMeter = gameScreen:FindChild("StarMeter",true)
	local GameBoard = gameScreen:FindChild("ReelBackground",true)
	local AllGamePieces = gameScreen:FindChild("AllGamePieces",true)
	local BoardSpaceMatchedView = gameScreen:FindChild("BoardSpaceMatchedView",true)
	
	SpinButton.visible = false

	local boardWidth = AllBoardCells.boardWidth
	local boardHeight = AllBoardCells.boardHeight
	-- Turn off pattern until we want to show
	for y=0,boardHeight-1 do
		for x=0,boardWidth-1 do
			local nameOfMatchingActor = "BoardSpaceMatchedView_" .. x .. "_" .. y
			local matchingActor = gameScreen:FindChild(nameOfMatchingActor,true)
			if(matchingActor)then
				matchingActor.visible = false
				--SlideOff(matchingActor,250,"left","nowait")
			end
		end
	end

	SlideOff(TopScoresContainer,250,"top","nowait")
	if IsIPhone5() or APP.useIPhone5AppAssets then
		if(Logo.visible)then
			SlideOff(Logo,250,"top","nowait")
		end
	elseif IsIPad() or APP.useIPadAppAssets then
		if(Logo.visible)then
			SlideOff(Logo,250,"top","nowait")
		end
	end
	
	--[[if IsIPhone5() or APP.useIPhone5AppAssets then
		SlideOff(RemainingSpinsContainer,250,"top","nowait")
	elseif IsIPad() or APP.useIPadAppAssets then
		SlideOff(RemainingSpinsContainer,250,"top","nowait")
	else--]]
		SlideOff(RemainingSpinsContainer,250,"bottom","nowait")
	--end

	SlideOff(MenuButton,250,"bottom","nowait")

	local showTricks = TrickPanel.visible
	if(TrickPanel.position.y >300)then
		if(showTricks) then
			SlideOff(TrickPanel,250,"bottom","nowait")
		end
		SlideOff(ScoreContainer,250,"top","nowait")
		if(MultiplierPanel.visible)then
			SlideOff(MultiplierPanel,250,"top","nowait")
		end
	else
		if(showTricks) then
			SlideOff(TrickPanel,250,"top","nowait")
		end
		SlideOff(ScoreContainer,250,"bottom","nowait")
		if(MultiplierPanel.visible)then
			SlideOff(MultiplierPanel,250,"bottom","nowait")
		end
	end
	
	if(SpecialGemPanel.visible)then
		SlideOff(SpecialGemPanel,250,"top","nowait")
	end

	if (LabGoalButton.visible) then
		SlideOff(LabGoalButton,250,"top","nowait")
	end
	
	local gameBoard = gameScreen:FindChild("ReelBackground",true)
	if(gameBoard)then
		SlideOff(gameBoard,250,"left","nowait")
		SlideOff(AllGemEaters,250,"left","nowait")
	end

	SlideOff(AllBoardCells,250,"left","nowait")
	SlideOff(AllGamePieces,250,"left","wait")

	Scripts.ClearGameScreen(screen,gameScreen)

	ScriptDelay(50)

end

function Scripts.SlideAndClearGameScreen_level(screen,gameScreen)
	Scripts.SlideAndClearGameScreen(screen,gameScreen)
	ScriptDelay(50)
end

function Scripts.SlideAndClearGameScreen_challenge(screen,gameScreen)
	Scripts.SlideAndClearGameScreen(screen,gameScreen)
	ScriptDelay(50)
end

function Scripts.SlideAndClearGameScreen_LeaveLevel(screen,gameScreen)
	Scripts.SlideAndClearGameScreen(screen,gameScreen)
	ScriptDelay(50)
end

function Scripts.Confetti(screen)
	local confetti_spot = -210

	local shape = "_normal_shape"
	
	local confetti1 = CreateInstance {
		name = "Confetti1",
		class = "hgeParticleSystemActor",
		file = "Confetti1"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti1)	
	
	local confetti2 = CreateInstance {
		name = "Confetti2",
		class = "hgeParticleSystemActor",
		file = "Confetti2"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti2)	
	
	local confetti3 = CreateInstance {
		name = "Confetti3",
		class = "hgeParticleSystemActor",
		file = "Confetti3"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti3)	
	
	local confetti4 = CreateInstance {
		name = "Confetti4",
		class = "hgeParticleSystemActor",
		file = "Confetti4"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti4)	
	
	local confetti5 = CreateInstance {
		name = "Confetti5",
		class = "hgeParticleSystemActor",
		file = "Confetti5"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti5)	
	
	local confetti6 = CreateInstance {
		name = "Confetti6",
		class = "hgeParticleSystemActor",
		file = "Confetti6"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti6)	

	PlaySound("air_cannon_pressure_release_02.ogg");

end

function Scripts.Confetti_short(screen)
    --logprint("Confetti_Short")
	local confetti_spot = -210

	local shape = "_normal_shape_short"
	
	local confetti1 = CreateInstance {
		name = "Confetti1",
		class = "hgeParticleSystemActor",
		file = "Confetti1"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti1)	
	
	local confetti2 = CreateInstance {
		name = "Confetti2",
		class = "hgeParticleSystemActor",
		file = "Confetti2"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti2)	
	
	local confetti3 = CreateInstance {
		name = "Confetti3",
		class = "hgeParticleSystemActor",
		file = "Confetti3"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti3)	
	
	local confetti4 = CreateInstance {
		name = "Confetti4",
		class = "hgeParticleSystemActor",
		file = "Confetti4"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti4)	
	
	local confetti5 = CreateInstance {
		name = "Confetti5",
		class = "hgeParticleSystemActor",
		file = "Confetti5"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti5)	
	
	local confetti6 = CreateInstance {
		name = "Confetti6",
		class = "hgeParticleSystemActor",
		file = "Confetti6"..shape,
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,confetti_spot},
	}	
	screen.RootActor:AddChild(confetti6)	

	PlaySound("air_cannon_pressure_release_02.ogg");
end

function Scripts.SpinButtonIntroduction(spinButton,numRemainingSpins,rootGameView, spinsTaken)
	-- Animate the button in, if it is the sort that should be.
	
	--NOTE: Use this to find out spinbutton position when doing layout
	--      and setting SpinButtonPos
	--logprint("spinButton "..spinButton.position.x.." "..spinButton.position.y)

	--spinButton:AddScript("KillDelayedWiggleLoop","KillDelayedWiggleLoop")

	if IsFunction(spinButton.AnimateIn) then
		--logprint("Spinbutton.AnimateIn is a function in SpinButtonIntroduction")
		local screen = rootGameView.parent.parent
		WaitForTaggedScriptCompletion(screen, "SlideSpinButtonUpThenSlideDownAndAway")
		spinButton.visible = false
		
		if(Config.EnableSmallSpinButton)then
			Script.SpinButton_Reset(spinButton)
			spinButton:AnimateIn()
		end
		
		ScriptDelay(250)
		ScriptDelay(250)
		PlaySound("slot_machine_win_03.ogg")
		if(numRemainingSpins==1)then
			PlaySound("horn_lose_05_sine.ogg")
		end
		ScriptDelay(200)
		spinButton:AnimateIn()
		if(numRemainingSpins==1 and spinsTaken > 0)then
			PlaySound("Cascade_LastSpin2.ogg")
			AddScriptWithParams(rootGameView,"GoodJobWord","lastspin.png", STRINGS("Last Spin!"),750,"top")
		end
	end
	SpinButtonPos[1] = spinButton.position.x
	SpinButtonPos[2] = spinButton.position.y
end

local slideOffset = 10
local slideTime = 100

--SpinButtonPos = {86,(UseIPhone5AppAssets() and 381) or 291}
SpinButtonPos =  Layout{{86, 291}, iphone5={86, 381}, ipad={117, 322} }

Scripts.SlideSpinButtonDown = function(screen)
    --logprint("SlideSpinButtonDown")

	PlaySound("Magic_Absorb1.ogg")

	if(Config.EnableSmallSpinButton)then
		return
	end

	LoopSound("Magical_EnergyLoop01.ogg")
	LoopSound("Mystical_EnergyGlowLoop.ogg")
	
	local spinButton = screen:FindChild("SpinButton",true)
	local imageActor = spinButton.ImageActor
	spinButton.position.x = SpinButtonPos[1]
	imageActor.position = {0,0}

	spinButton.scalingPoint = "MidBottom"

	local slidePosition = {0,slideOffset}
	local offset = slideOffset - imageActor.position.y
	local goTime = slideTime * offset / slideOffset
	if(goTime < 0) then
		goTime = goTime * -1
	end
	if(goTime == 0) then
		goTime = 1
	end
	imageActor:SlideTo(slidePosition, goTime)
	ScaleTo(imageActor, {0.9,0.9}, goTime)
	ScriptDelay(slideTime)
	LoopSound("slot_machine_spin_loop_02.ogg")
end

SpinButtonPos2 = {86,470}

--Config.EnableSmallSpinButton = true

Scripts.SlideSpinButtonUp = function(screen, slideDownAndAwayAfterSlidingUp)
	spinButton = screen:FindChild("SpinButton",true)
	ScriptBreak() --Delays activation of script by one tick so other scripts can kill it if needed.
	if(slideDownAndAwayAfterSlidingUp and spinButton.visible == false) then
		return --If already invisible, don't show
	end
	spinButton.visible = true
	PlaySound("Magic_CollectorStopSpinning.ogg")
	PlaySound("slot_machine_spin_stop_06.ogg")
	StopSound("Magic_Absorb1.ogg")
	StopSound("Magical_EnergyLoop01.ogg")
	StopSound("Mystical_EnergyGlowLoop.ogg")
	StopSound("slot_machine_spin_loop_02.ogg")

	local imageActor = spinButton.ImageActor

	local offset = imageActor.position.y - 0
	local returnTime = slideTime * offset / slideOffset
	if(returnTime < 0) then
		returnTime = returnTime * -1
	end
	imageActor.scalingPoint = MidBottom
	imageActor:SlideTo({0,0}, returnTime)
	ScaleTo(imageActor, {1.0,1.0}, returnTime)
	ScriptDelay(slideTime)
	
	if(Config.EnableSmallSpinButton)then
		spinButton.visible = true
		imageActor.alpha = 100
		spinButton.active = false
		Script.SpinButton_Reset(spinButton)
		return
	end

	if slideDownAndAwayAfterSlidingUp then
		SlideOff(spinButton,150,"bottom","wait") --The wait lets the notebubble know not to pop up until animations are done
	end	
end

Scripts.SlideSpinButtonUpAndStay = function(screen)
	Scripts.SlideSpinButtonUp(screen, false)
end

Scripts.SlideSpinButtonUpThenSlideDownAndAway = function(screen)
	Scripts.SlideSpinButtonUp(screen, true)
end

Scripts.SpinButton_Present = function(spinButton,gameScreen,rootGameView)
	spinButton.position = SpinButtonPos
	spinButton.rotation = { degrees = 0 }
	if(Config.EnableSmallSpinButton)then
		spinButton.alpha = 255
		spinButton.active = true
		Script.SpinButton_Reset(spinButton)
		
		local diamondSparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "SparkleBurst1",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}
		spinButton:AddChild(diamondSparkle)	
    	local sparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "Shine2",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}
		diamondSparkle:AddChild(sparkle)	
		Pop(spinButton, {expandTo={1.15, 1.25},expandTime=150, shrinkTime=250})
	end

	trickPanel = gameScreen:FindChild("TrickPanel",true)
	if(not trickPanel.default_alignmentOffset)then
		trickPanel.default_alignmentOffset = trickPanel.alignmentOffset
		trickPanel.default_position = trickPanel.globalPosition
	end
	trickPanel.globalPosition = trickPanel.default_position
	local slidePosition = {trickPanel.default_position.x,trickPanel.default_position.y-10}

	spinButton.rotationPoint = MidBottom
	--spinButton:AddScript("DelayedWiggleLoop","DelayedWiggleLoop")
end

Scripts.AnnounceMultiplier = function(actor,text)
	--Clean up previous instance
	
	--logprint("AnnounceMultiplier")

	local multiplierPanel = actor:FindChild("MultiplierPanel",true)

	local floatText = actor:FindChild("FloatText",true)
	local originalPos = floatText.position
	floatText.MultiplierLabel.text = text

	multiplierPanel.visible = false
	local savePos = {multiplierPanel.position.x-20,multiplierPanel.position.y-20}
	local destPos = {floatText.position.x,floatText.position.y+120}
	--local destPos = GetGlobalCenterPoint(multiplierPanel)

	floatText.position = savePos
	floatText.visible = true
	
	ScaleTo(floatText, {1.0, 1.0}, 250, false)
	Scripts.Interpolate(floatText, "position", destPos, 250)

	ScriptDelay(250)

	ScaleTo(floatText, {0.5, 0.5}, 250, false)
	Scripts.Interpolate(floatText, "position", savePos, 250)
	multiplierPanel.MultiplierLabel.text = text
	multiplierPanel.visible = true
	multiplierPanel:AddScript("MultiplierPop","")
	floatText.visible = false
	floatText.position = originalPos
end

Spin_Say_Yeah = false

Scripts.Spin_Present = function(rootGameView, isAutomaticSpin, stateName, multiplierCount)
    PlaySound("Magic_Absorb3.ogg")
	if(stateName == "EndGame_ExtraSpins") then --Don't do the autospin animations when you're in endgame state
		isAutomaticSpin = false
	end
	-- DEBUG 
	--multiplierCount = 10
	--logprint("Spin_Present multiplierCount = "..multiplierCount)
	if(multiplierCount > 1)then
		local text = multiplierCount.."X"
		AddScriptWithParams(rootGameView,"AnnounceMultiplier",text)
	end
	if isAutomaticSpin == true then
		if(Spin_Say_Yeah) then
			PlaySound("Cascade_Yeah.ogg")
		else
			PlaySound("Cascade_FreeSpin2.ogg")
			Spin_Say_Yeah = true
		end
		local Sparks = CreateInstance {
			name = "Sparks",
			class = "hgeParticleSystemActor",
			file = "Sparks1",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}	
		AddScriptWithParams(rootGameView,"GoodJobWord","freespin.png", STRINGS("Free Spin!"),750,"bottom")
		rootGameView:AddChild(Sparks)
		local allBoardCells = rootGameView:FindChild("AllBoardCells",true)
		allBoardCells:AddScript("BoardShine","")
		ScriptDelay(500)
	else
		Spin_Say_Yeah = false
	end
end

Scripts.RandomSuperMatchShine = function(allBoardCells,affectedPiecesChart)
	--logprint("RandomSuperMatchShine")
	--logprint("RandomSuperMatchShine, affected chart size " .. #affectedPiecesChart)

	-- Collect Cells to highlight from the gems that can be played
	local affectedCellsChart = {}
	for i=1,#affectedPiecesChart do
		local cellName = "BoardCell_"..affectedPiecesChart[i].spot_x.."_"..affectedPiecesChart[i].spot_y
		local cell = allBoardCells:FindChild(cellName,true)
		table.insert(affectedCellsChart,cell)
	end

	if(allBoardCells)then
		while(true)do
			--logprint("Outer while loop")
			-- Wait for no shining
			if(not allBoardCells.isShining)then
				--logprint("RandomShine")
				allBoardCells.isShining = true
				-- Randomly flash cells
				while(true)do
					local i = math.random(1,#affectedPiecesChart)
					--logprint("i = " .. i)
					local pieceView = affectedPiecesChart[i].pieceView
					local cell_square = affectedCellsChart[i]
					if(cell_square)then
						cell_square:AddScript("CellShine","")
						ScriptDelay(50)
					end
				end
			end
			ScriptBreak()
		end
		allBoardCells.isShining = false
	end
end

Scripts.RandomSuperMatchShine_cleanup = function(allBoardCells)
	if(allBoardCells)then
		local boardWidth = allBoardCells.boardWidth
		local boardHeight = allBoardCells.boardHeight
		-- Turn off all shines
		for y=0,boardHeight-1 do
			for x=0,boardWidth-1 do
				local cellName = "BoardCell_"..x.."_"..y
				local cell = allBoardCells:FindChild(cellName,true)
				if(cell)then
					cell:AddScript("CellShine_cleanup","")
				end
			end
		end
		allBoardCells.isShining = false
	end
end


function CleanUp_TrickLauncherPanel(rootGameView)
	--logprint("CleanUp_TrickLauncherPanel")
	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	if(trickLauncherPanel)then
		local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
		local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)
		if(trickLauncherPanel.savePosition)then
			trickLauncherPanel.position = trickLauncherPanel.savePosition
		end
		trickLauncher:KillTaggedScripts("TrickLauncherScript")
		trickLauncher:StopRotation()
		flyingTrick:KillTaggedScripts("TrickLauncherScript")
		flyingTrick:StopRotation()
		SlideOff(trickLauncherPanel,150,"bottom","wait")

		trickLauncher.rotationAngle = { degrees = 0 }
		flyingTrick.rotationAngle = { degrees = 0 }
		if(flyingTrick.savePosition)then
			flyingTrick.position = flyingTrick.savePosition 
		end
	end
end

function Scripts.WaitForSuperMatch(rootGameView,affectedPiecesChart)
    --logprint("WaitForSuperMatch")

	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_Shifter")
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_SuperMatch")
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_Hammer")

	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
	local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)
	flyingTrick.image = "Trick_SuperMatch.png"
	flyingTrick.savePosition = flyingTrick.position
	flyingTrick.visible = true
	trickLauncherPanel.alignmentOffset = {0,10}
	trickLauncherPanel.savePosition = trickLauncherPanel.position
	
	PlaySound("Cascade_SuperMatch.ogg")
	AddScriptWithParams(rootGameView,"GoodJobWord","goodjobsupermatch.png",STRINGS("Super Match!"),500,"top")

	--ScriptDelay(500)
	SlideOn(trickLauncherPanel,150,"bottom","wait")
	Scripts.Bounce_UpAndBack(trickLauncherPanel,2)
	Scripts.Bounce_DownAndBack(trickLauncherPanel,4)
	
	trickLauncher:AddScript("RotateBackAndForth","TrickLauncherScript")
	flyingTrick:AddScript("RotateBackAndForth","TrickLauncherScript")

	local allBoardCells = rootGameView:FindChild("AllBoardCells",true)
	if(affectedPiecesChart)then
		AddScriptWithParams(allBoardCells,"RandomSuperMatchShine", affectedPiecesChart)
	end
	LoopSound("Magical_EnergyLoop01.ogg")
end

function Scripts.PerformSuperMatch(rootGameView,boardPieceView)
    --logprint("PerformSuperMatch")

	rootGameView:KillTaggedScripts("WaitForSuperMatch")

	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
	local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)

	local boardPieceViewGlobalDestination = boardPieceView.globalPosition
	local boardPieceViewLocalDestination = flyingTrick.parent:GlobalToLocalCoord(boardPieceViewGlobalDestination)

	boardPieceViewLocalDestination.y = boardPieceViewLocalDestination.y
	local xDiff = boardPieceViewLocalDestination.x - flyingTrick.position.x
    local yDiff = boardPieceViewLocalDestination.y - flyingTrick.position.y
    local flyingAngle = math.atan2(yDiff, xDiff) * (180 /math.pi) 
	local flyingAngle = flyingAngle + 90 
	--logprint("PerformSuperMatch flyingAngle "..flyingAngle)
	
	flyingTrick:KillTaggedScripts("TrickLauncherScript")
	flyingTrick:StopRotation()
	flyingTrick.rotationAngle = { degrees = flyingAngle }

	trickLauncher:KillTaggedScripts("TrickLauncherScript")
	trickLauncher:StopRotation()
	trickLauncher.rotationAngle = { degrees = flyingAngle }

	StopSound("Magical_EnergyLoop01.ogg")

	PlaySound("catapult_release_small_01.ogg")
	FireParticle(trickLauncherPanel,"Launch1","MidBottom",nil,{0,-20})
	trickLauncherPanel:AddScript("Animate_DownAndUp","")
	ScriptDelay(150)
	Scripts.Interpolate(flyingTrick, "position", boardPieceViewLocalDestination, 150)
	ScriptDelay(100)
	PlaySound("Magic_WandZap6.ogg")

	flyingTrick.visible = false
end

function Scripts.CleanUp_SuperMatch(rootGameView)
	--logprint("CleanUp_SuperMatch")
	
	WaitForTaggedScriptCompletion(rootGameView, "PerformSuperMatch")

	rootGameView:KillTaggedScripts("WaitForSuperMatch")
	StopSound("Magical_EnergyLoop01.ogg")
	local allBoardCells = rootGameView:FindChild("AllBoardCells",true)
	if(allBoardCells)then
		allBoardCells:KillTaggedScripts("RandomSuperMatchShine")
		allBoardCells:AddScript("RandomSuperMatchShine_cleanup","")
	end

	CleanUp_TrickLauncherPanel(rootGameView)

end

function Scripts.WaitForHammer(rootGameView,affectedPiecesChart)
    --logprint("WaitForHammer")
	
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_Shifter")
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_SuperMatch")
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_Hammer")

	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
	local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)
	flyingTrick.image = "Trick_Hammer.png"
	flyingTrick.savePosition = flyingTrick.position
	flyingTrick.visible = true
	trickLauncherPanel.alignmentOffset = {0,10}
	trickLauncherPanel.savePosition = trickLauncherPanel.position

	PlaySound("Cascade_Hammer.ogg")
	AddScriptWithParams(rootGameView,"GoodJobWord","Hammer.png",STRINGS("Hammer!"),500,"top")
	
	--ScriptDelay(500)
	SlideOn(trickLauncherPanel,150,"bottom","wait")
	Scripts.Bounce_UpAndBack(trickLauncherPanel,2)
	Scripts.Bounce_DownAndBack(trickLauncherPanel,4)

	trickLauncher:AddScript("RotateBackAndForth","TrickLauncherScript")
	flyingTrick:AddScript("RotateBackAndForth","TrickLauncherScript")

	local allBoardCells = rootGameView:FindChild("AllBoardCells",true)
	if(affectedPiecesChart)then
		AddScriptWithParams(allBoardCells,"RandomSuperMatchShine", affectedPiecesChart)
	end
	--PlaySound("Magic_Respawn38_short.ogg")
	LoopSound("Magical_EnergyLoop01.ogg")
end

function Scripts.PerformHammer(rootGameView,aligner,boardPieceView)
	rootGameView:KillTaggedScripts("WaitForHammer")

	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
	local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)

	local boardPieceViewGlobalDestination = aligner.globalPosition
	local boardPieceViewLocalDestination = flyingTrick.parent:GlobalToLocalCoord(boardPieceViewGlobalDestination)

	local xDiff = boardPieceViewLocalDestination.x - flyingTrick.position.x
    local yDiff = boardPieceViewLocalDestination.y - flyingTrick.position.y
    local flyingAngle = math.atan2(yDiff, xDiff) * (180 /math.pi) 
	local flyingAngle = flyingAngle + 90 
	--logprint("PerformSuperMatch flyingAngle "..flyingAngle)
	
	flyingTrick:KillTaggedScripts("TrickLauncherScript")
	flyingTrick:StopRotation()
	flyingTrick.rotationAngle = { degrees = flyingAngle }

	trickLauncher:KillTaggedScripts("TrickLauncherScript")
	trickLauncher:StopRotation()
	trickLauncher.rotationAngle = { degrees = flyingAngle }

	StopSound("Magical_EnergyLoop01.ogg")

	PlaySound("catapult_release_small_01.ogg")
	FireParticle(trickLauncherPanel,"Launch1","MidBottom",nil,{0,-20})
	trickLauncherPanel:AddScript("Animate_DownAndUp","")
	ScriptDelay(150)
	Scripts.Interpolate(flyingTrick, "position", boardPieceViewLocalDestination, 150)
	ScriptDelay(100)

	PlaySound("Hit_MetalRobotPunch1.ogg")
	
	flyingTrick.visible = false
	FireParticle(aligner,"BigBrightTrail_oneHit","Center")
	FireParticle(aligner,"Boom1","Center")
	FireParticle(aligner,"SparkBoom1","Center")

	--PlaySound("Cascade_Boom.ogg")
end

function Scripts.CleanUp_Hammer(rootGameView)
    --logprint("CleanUp_Hammer")

	WaitForTaggedScriptCompletion(rootGameView, "PerformHammer")
	
	rootGameView:KillTaggedScripts("WaitForHammer")
	StopSound("Magical_EnergyLoop01.ogg")
	local allBoardCells = rootGameView:FindChild("AllBoardCells",true)
	if(allBoardCells)then
		allBoardCells:KillTaggedScripts("RandomSuperMatchShine")
		allBoardCells:AddScript("RandomSuperMatchShine_cleanup","")
	end

	CleanUp_TrickLauncherPanel(rootGameView)
end


function Scripts.WaitForMakeWild(rootGameView)
    --logprint("WaitForSuperMatch")
	LoopSound("Magical_EnergyLoop01.ogg")
	AddScriptWithParams(rootGameView,"GoodJobWord","goodjobmakewild.png",STRINGS("Make Wild!"),750,"top")
end

function Scripts.CleanUp_MakeWild(rootGameView)
    --logprint("CleanUp_MakeWild")
	StopSound("Magical_EnergyLoop01.ogg")
end

function Scripts.MegaMatch(rootGameView)
	AddScriptWithParams(rootGameView,"GoodJobWord","goodjobmegamatch.png",STRINGS("Mega Match!"),750,"top")
	PlaySound("Magic_SpiritBlast03.ogg")
end

function Scripts.SpinNow(rootGameView)
	PlaySound("Cascade_SpinNow.ogg")
	AddScriptWithParams(rootGameView,"GoodJobWord","goodjobspinnow.png",STRINGS("Spin Now!"),750,"top")
	PlaySound("slot_machine_jackpot_lose_reverse.ogg")
end

function Scripts.WaitForGemSwipe(rootGameView)
    --logprint("WaitForGemSwipe")
	LoopSound("Magical_EnergyLoop01.ogg")
	AddScriptWithParams(rootGameView,"GoodJobWord","goodjobgemswipe.png",STRINGS("Gem Swipe!"),750,"top")
end

function Scripts.PerformGemSwipe(rootGameView)
    --logprint("PerformGemSwipe")
	StopSound("Magical_EnergyLoop01.ogg")
	PlaySound("Magic_WandZap6.ogg")
end

function Scripts.CleanUp_GemSwipe(rootGameView)
    --logprint("CleanUp_GemSwipe")
	StopSound("Magical_EnergyLoop01.ogg")
end

-- SHIFTER

Scripts.Cell_ShifterSelect = function(cell,colorName,rootGameView,slotsBoardPieceView)
    --logprint("Cell_ShifterSelect")
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_Shifter")
	local highlight = cell:FindChild("highlight",true)
	StopFade(highlight)
	highlight.image = "slot_highlight_"..colorName..".png"
	highlight.size = cell.size
	highlight.visible = true
	highlight.alpha = 255
	highlight.layer = 1
	highlight.extraScale = 1.0
	cell.SuperMatchGemDecoration = Resources.SuperMatchGemDecoration

	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
	local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)
	local flyingTrick_savePosition = flyingTrick.position

	local cellGlobalDestination = cell.globalPosition
	local cellLocalDestination = flyingTrick.parent:GlobalToLocalCoord(cellGlobalDestination)

	local flyingTrickInitialGlobalDestination = flyingTrick.globalPosition

	local xDiff = cellLocalDestination.x - flyingTrick.position.x
    local yDiff = cellLocalDestination.y - flyingTrick.position.y
    local flyingAngle = math.atan2(yDiff, xDiff) * (180 /math.pi) 
	local flyingAngle = flyingAngle + 90 

	flyingTrick:KillTaggedScripts("TrickLauncherScript")
	flyingTrick:StopRotation()
	flyingTrick.rotationAngle = { degrees = flyingAngle }

	trickLauncher:KillTaggedScripts("TrickLauncherScript")
	trickLauncher:StopRotation()
	trickLauncher.rotationAngle = { degrees = flyingAngle }

	PlaySound("catapult_release_small_01.ogg")
	FireParticle(trickLauncherPanel,"Launch1","MidBottom",nil,{0,-20})
	trickLauncherPanel:AddScript("Animate_DownAndUp","")
	ScriptDelay(150)
	Scripts.Interpolate(flyingTrick, "position", cellLocalDestination, 200)

	--flyingTrick.visible = false
	FireParticle(cell,"BigBrightTrail_oneHit","Center")
	FireParticle(cell,"Boom1","Center")

	--logprint("slotsBoardPieceView.image "..slotsBoardPieceView.image)
	--logprint("cell.name "..cell.name)
	flyingTrick.image = slotsBoardPieceView.image
	flyingTrick.position = cellLocalDestination

	--flyingTrick.visible = true
	Scripts.Interpolate(flyingTrick, "position", flyingTrick_savePosition, 150)

	slotsBoardPieceView.visible = false
	cell.highlight.visible = false
	if(cell.SuperMatchGemDecoration)then
		cell.SuperMatchGemDecoration.visible = false
	end

	ScriptDelay(150)

	--trickLauncher:AddScript("RotateBackAndForth_wide","TrickLauncherScipt")
	--flyingTrick:AddScript("RotateBackAndForth_wide","TrickLauncherScript")

	ScriptDelay(200)

end

Scripts.Cell_UnShifterSelect = function(cell,colorName,rootGameView,slotsBoardPieceView)
    --logprint("Cell_UnShifterSelect")
	local highlight = cell:FindChild("highlight",true)
	StopFade(highlight)
	highlight.visible = false
	if(cell.SuperMatchGemDecoration ~= nil) then
		cell.SuperMatchGemDecoration:Destroy()
	end
	if(slotsBoardPieceView)then
		slotsBoardPieceView.visible = true
	end
end

function Scripts.WaitForShifter(rootGameView)
    --logprint("WaitForShifter")

	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_Shifter")
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_SuperMatch")
	WaitForTaggedScriptCompletion(rootGameView, "CleanUp_Hammer")

	local gameScreen = rootGameView.parent
	local backgroundImage = gameScreen.backgroundImage
	local reelBackground = backgroundImage.ReelBackground

	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
	local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)
	flyingTrick.image = "Trick_Shifter.png"
	flyingTrick.savePosition = flyingTrick.position
	flyingTrick.visible = true
	trickLauncherPanel.alignmentOffset = {0,30}
	trickLauncherPanel.savePosition = trickLauncherPanel.position

	PlaySound("Cascade_Shifter.ogg")
	AddScriptWithParams(rootGameView,"GoodJobWord","goodjobshifter.png",STRINGS("Shifter!"),500,"top")

	--ScriptDelay(500)
	LoopSound("Magical_EnergyLoop01.ogg")

	SlideOn(trickLauncherPanel,150,"bottom","wait")
	Scripts.Bounce_UpAndBack(trickLauncherPanel,2)
	Scripts.Bounce_DownAndBack(trickLauncherPanel,4)

	trickLauncher:AddScript("RotateBackAndForth_wide","TrickLauncherScript")
	flyingTrick:AddScript("RotateBackAndForth_wide","TrickLauncherScript")

	local AllSlotsCells = rootGameView:FindChild("AllSlotsCells",true)
	
	local boardWidth = AllSlotsCells.boardWidth
	local boardHeight = AllSlotsCells.boardHeight

	while(true)do
		for x = 0,boardWidth-1 do
			local backgroundName = "Box" .. (x + 1)
			local boxActor = reelBackground[backgroundName]
			if(boxActor.visible == false) then
			else
				local cellName = "BoardCell_"..x.."_0"
				--logprint(cellName)
				local cell = AllSlotsCells:FindChild(cellName,true)
				if(cell and cell.visible)then
					cell:AddScript("CellShine","")
				end
				ScriptDelay(100)
			end
		end
		ScriptDelay(100)
	end
end

function Scripts.PerformShifter(rootGameView,cell_source,cell_dest,slotsBoardPieceView_dest,slotsBoardPieceView_source)
    --logprint("PerformShifter")
	rootGameView:KillTaggedScripts("WaitForShifter")
	StopSound("Magical_EnergyLoop01.ogg")

	if(slotsBoardPieceView_source and slotsBoardPieceView_dest)then
		slotsBoardPieceView_dest.visible = true
		slotsBoardPieceView_source.visible = false
	elseif(slotsBoardPieceView_dest)then
		slotsBoardPieceView_dest.visible = false
	end

	local trickLauncherPanel = rootGameView:FindChild("TrickLauncherPanel",true)
	local trickLauncher = trickLauncherPanel:FindChild("TrickLauncher",true)
	local flyingTrick = trickLauncherPanel:FindChild("FlyingTrick",true)
	local flyingTrick_savePosition = flyingTrick.position

	local cellGlobalDestination = cell_dest.globalPosition
	local cellLocalDestination = flyingTrick.parent:GlobalToLocalCoord(cellGlobalDestination)

	local flyingTrickInitialGlobalDestination = flyingTrick.globalPosition

	local xDiff = cellLocalDestination.x - flyingTrick.position.x
    local yDiff = cellLocalDestination.y - flyingTrick.position.y
    local flyingAngle = math.atan2(yDiff, xDiff) * (180 /math.pi) 
	local flyingAngle = flyingAngle + 90 

	flyingTrick:KillTaggedScripts("TrickLauncherScript")
	flyingTrick:StopRotation()
	flyingTrick.rotationAngle = { degrees = flyingAngle }

	trickLauncher:KillTaggedScripts("TrickLauncherScript")
	trickLauncher:StopRotation()
	trickLauncher.rotationAngle = { degrees = flyingAngle }

	PlaySound("catapult_release_small_01.ogg")
	FireParticle(trickLauncher,"Launch1","MidBottom",nil,{0,-20})
	trickLauncherPanel:AddScript("Animate_DownAndUp","")
	ScriptDelay(150)
	RotateTo(flyingTrick, {degrees=0}, 150, false)
	Scripts.Interpolate(flyingTrick, "position", cellLocalDestination, 150)

	PlaySound("Magic_WandZap6.ogg")

	ScriptDelay(150)
	flyingTrick.visible = false
	FireParticle(cell_dest,"BigBrightTrail_oneHit","Center")
	FireParticle(cell_dest,"SparkBoom1","Center")

	if(slotsBoardPieceView_source)then
		slotsBoardPieceView_source.visible = true
		FireParticle(cell_source,"BigBrightTrail_oneHit","Center")
		FireParticle(cell_source,"SparkBoom1","Center")
	end

	flyingTrick:KillTaggedScripts("TrickLauncherScript")
	flyingTrick:StopRotation()
	--flyingTrick.rotationAngle = { degrees = 0.0 }

	trickLauncher:KillTaggedScripts("TrickLauncherScript")
	trickLauncher:StopRotation()
	--trickLauncher.rotationAngle = { degrees = 0.0 }

	ScriptDelay(200)

end

function Scripts.CleanUp_Shifter(rootGameView,slotsBoardPieceView1,slotsBoardPieceView2)
    --logprint("CleanUp_Shifter")
	WaitForTaggedScriptCompletion(rootGameView, "PerformShifter")

	StopSound("Magical_EnergyLoop01.ogg")
	rootGameView:KillTaggedScripts("WaitForShifter")

	--Iterate over each reel cell, clearing highlights from being progress
	local slotCells = rootGameView:FindChild("AllSlotsCells",true)
	if(slotCells)then
		for x = 0,7 do
			local slotCellName = "BoardCell_" .. x .. "_0"
			local slotCell = slotCells[slotCellName]
			if(slotCell ~= nil) then
				Scripts.Cell_UnShifterSelect(slotCell)
			end
		end
	end

	if(slotsBoardPieceView1)then
		slotsBoardPieceView1.visible = true
	end
	if(slotsBoardPieceView2)then
		slotsBoardPieceView2.visible = true
	end
	
	CleanUp_TrickLauncherPanel(rootGameView)

end

Scripts.Spin_Fairy = function(rootGameView, numSpinFairies, SpinSaverView, multiplierCount)
	local SpinSaverName = "FairyToken" .. numSpinFairies + 1
	local expiredSpinSaver = rootGameView:FindChild(SpinSaverName,true)
	expiredSpinSaver.visible = true --Was made false in C++ code.
	local positionSave = expiredSpinSaver.position

	AddScriptWithParams(rootGameView,"GoodJobWord","goodjobspinsaver.png",STRINGS("SpinSaver"),750,"top")
	PlaySound("slot_machine_bet_18.ogg")

	--Fly spin saver to center of screen
	expiredSpinSaver.layer = 10
	
	local SpinSaverGlobalDestination = SpinSaverView.globalPosition
	local SpinSaverLocalDestination = expiredSpinSaver.parent:GlobalToLocalCoord(SpinSaverGlobalDestination)

	Scripts.Interpolate(expiredSpinSaver, "position", SpinSaverLocalDestination, 500)
	PlaySound("Win10.ogg")

	--Pop spin saver
	Pop(expiredSpinSaver)

	--Fly spin saver off the screen
	SpinSaverLocalDestination.y = SpinSaverLocalDestination.y + 400
	local pieceDestination = SpinSaverView.position
	pieceDestination.y = pieceDestination.y + 400
	SpinSaverView:SlideTo(pieceDestination,500)
	Scripts.Interpolate(expiredSpinSaver, "position", SpinSaverLocalDestination, 500)

	--Return spin saver to original location, hidden
	expiredSpinSaver.visible = false
	expiredSpinSaver.position = positionSave

	--Make free spin presentation
	
	Scripts.Spin_Present(rootGameView, true, "Playing", multiplierCount)
end

Scripts.SlideOff_SpinAgainButton = function(screen)
	-- SpinAgainButton DISABLED FOR NOW
	--[[
	local SpinAgainButton =  screen:FindChild("SpinAgainButton",true)
	local SpinButton =  screen:FindChild("SpinButton",true)
	--PlaySound("swish10.ogg")
	--SpinAgainButton.visible = false
	SlideOff(SpinAgainButton,100,"bottom","wait")
	--Scripts.Spin_Present(screen,false)
	--Scripts.SpinButton_Present(SpinButton)
	--SpinButton.visible = true
	--Scripts.SlideSpinButtonUp(screen, false)
	--]]
end

local soundPopSound = {
	"Click1.ogg",
	"Click2.ogg",
	"Click3.ogg",
	"Click4.ogg",
	"Click5.ogg",
	"Click4.ogg",
	"Click3.ogg",
	"Click2.ogg",
	"Click1.ogg",
	"Click2.ogg",
	"Click3.ogg",
	"Click4.ogg",
	"Click5.ogg",
	"Click4.ogg",
	"Click3.ogg",
	"Click2.ogg",
	"Click1.ogg",
}

Scripts.Cell_Touched = function(cell,colorName)
	--logprint("Cell_Touched "..colorName)
	local highlight = cell:FindChild("highlight",true)
	highlight.image = "slot_highlight_"..colorName..".png"
	highlight.size = cell.size
	highlight.visible = true
	highlight.alpha = 255
	highlight.layer = 1
	highlight.extraScale = 1.0
	local delay = 500
	StopFade(highlight)
	--ScriptDelay(delay)
	FadeOut(highlight,delay)
	--ScaleOut(highlight,{1.2,1.2},delay,"wait")
	highlight.visible = false
end

Scripts.DontShowUntilDrop = function(matchStartPieceView, delay)
	matchStartPieceView.visible = false
	ScriptBreak() --Allows master match animation a tick to define GlobalDropDelay
    ScriptDelay(50) --More of the same
	if(GlobalDropDelay == nil) then
		GlobalDropDelay = 0
	end
	ScriptDelay(GlobalDropDelay)
	ScriptDelay(delay * DemoAnimationSlowingFactor)
	--logprint("Dont show until drop: visibility is...")
	if(matchStartPieceView.visible) then
		--logprint("true")
	else
		--logprint("false")
	end
	matchStartPieceView.visible = true
end

Scripts.ShakeMaker = function(maker, delay)
	--Dont modify this stuff; it's to sync with dont show until drop and piece drop.
	ScriptBreak() --Allows master match animation a tick to define GlobalDropDelay
    ScriptDelay(50) --More of the same
	if(GlobalDropDelay == nil) then
		GlobalDropDelay = 0
	end
	ScriptDelay(GlobalDropDelay)
	ScriptDelay(delay * DemoAnimationSlowingFactor)
	
	if(maker.dropping ~= true) then
		--Add the actual shaking down here.
		if(maker:GetTaggedScriptCount("Shake") == 0) then
			maker:AddScript("MakerOutput","Shake")
		end
	end
end

placeHolderIndex = 0

Scripts.Piece_Drop = function(rootGameView,matchStartPieceView, startingBoardCoord, endingBoardCoord, slideDest, slideVelocity, delay, portalEntrance, portalExit, fallUp, fakePieceView)
	--logprint("Piece_drop queued")
	if(fakePieceView ~= nil) then
		fakePieceView.visible = false
	else
	end
	matchStartPieceView.dropping = true
	local pieceViewName = matchStartPieceView.name
	local pieceViewParent = matchStartPieceView.parent
	ScriptBreak() --Allows master match animation a tick to define GlobalDropDelay
    ScriptDelay(50) --More of the same
	if(GlobalDropDelay == nil) then
		GlobalDropDelay = 0
	end
	ScriptDelay(GlobalDropDelay)
	ScriptDelay(delay * DemoAnimationSlowingFactor)
	--logprint("Piece_drop starting")
	--Kill script early if piece view has been destroyed to avoid warnings about accessing deleted C++ objects
	if(pieceViewParent:FindChild(pieceViewName,true) ~= matchStartPieceView) then
		--logprint("Cant find piece view. Killing early.")
		return
	end
	if(type (matchStartPieceView.dropDestinations) == "table" and #matchStartPieceView.dropDestinations > 0) then
		--logprint("Queuing slide for piece " .. matchStartPieceView.name)
		--add this point as the next place to drop to and exit
		table.insert(matchStartPieceView.dropDestinations,{slideDest,slideVelocity, startingBoardCoord, endingBoardCoord, portalEntrance, portalExit, fallUp, fakePieceView,delay})
		return
	else
		--logprint("Setting slide for piece " .. matchStartPieceView.name)
		--add this point as the first place to drop to and exit
		matchStartPieceView.dropDestinations = { [1] = { slideDest,slideVelocity, startingBoardCoord, endingBoardCoord, portalEntrance, portalExit, fallUp, fakePieceView,delay,} }
	end
	
	while(#matchStartPieceView.dropDestinations > 0) do
		--logprint("Dequeuing slide for piece " .. matchStartPieceView.name)
		local firstQueuedLocation = matchStartPieceView.dropDestinations [1]
		local queuedDest = firstQueuedLocation[1]
		local queuedVelocity = firstQueuedLocation[2]
		local queuedStartCoord = firstQueuedLocation[3]
		local queuedEndCoord = firstQueuedLocation[4]
		local queuedPortalEntrance = firstQueuedLocation[5]
		local queuedPortalExit = firstQueuedLocation[6]
		local queuedFallUp = firstQueuedLocation[7]
		local queuedFakePieceView = firstQueuedLocation[8]
		local queuedDelay = firstQueuedLocation[9]
		local slideSrc = GetGlobalCenterPoint(matchStartPieceView)
		--logprint("Dequeuing slide for piece from " .. slideSrc.x .. "," .. slideSrc.y .. " to " .. queuedDest.x .. "," .. queuedDest.y)
		--logprint("Portal locations " .. queuedPortalEntrance.x .. "," .. queuedPortalEntrance.y .. " to " .. queuedPortalExit.x .. "," .. queuedPortalExit.y)
		--logprint("queuedDelay of " .. queuedDelay)

		--Determine if the piece is moving into or out of steel glass
		local startingCell = rootGameView.AllBoardCells:FindChild("BoardCell_" .. queuedStartCoord.x .. "_" .. queuedStartCoord.y, true)
		local startingDecoration = nil
		if(startingCell ~= nil) then
			startingDecoration = startingCell.SteelGlassDecoration
		end
		local startHasSteelGlass = (startingDecoration ~= nil)
		
		local endingCell = rootGameView.AllBoardCells:FindChild("BoardCell_" .. queuedEndCoord.x .. "_" .. queuedEndCoord.y, true)
		local endingDecoration = nil
		if(endingCell ~= nil) then
			endingDecoration = endingCell.SteelGlassDecoration
		end
		local endHasSteelGlass = (endingDecoration ~= nil)

		if(startHasSteelGlass and not endHasSteelGlass) then
			while(IsScaling(matchStartPieceView)) do
				ScriptBreak()
			end
			--Leaving steel glass. Remove scaling.
			local newScale = matchStartPieceView.extraScale
			newScale.x = newScale.x / SteelGlassGemScaling
			newScale.y = newScale.y / SteelGlassGemScaling
			ScaleTo(matchStartPieceView,newScale,50)
		end

		if(endHasSteelGlass and not startHasSteelGlass) then
			while(IsScaling(matchStartPieceView)) do
				ScriptBreak()
			end
			--Entering steel glass. Add scaling.
			local newScale = matchStartPieceView.extraScale
			newScale.x = newScale.x * SteelGlassGemScaling
			newScale.y = newScale.y * SteelGlassGemScaling
			ScaleTo(matchStartPieceView,newScale,50)
		end
		
		if(queuedPortalEntrance.x < 0) then
			--logprint("In no portal code "  .. matchStartPieceView.name)
			--No portal involved. Slide directly to new destination
			local deltaX = queuedDest.x - slideSrc.x
			local deltaY = queuedDest.y - slideSrc.y 
			local distance = math.sqrt(deltaX * deltaX + deltaY * deltaY)
			local slideDuration = distance / queuedVelocity * 1000.0 * DemoAnimationSlowingFactor
		
			--Make the piece slide, so that if it is deleted, it will not crash the script.
			RunScriptWithParams {
				target = matchStartPieceView,
				script = "InterpolateGlobalPoint",
				params = { "Center", queuedDest, slideDuration },
				tags = {},
			}
			ScriptDelay(slideDuration)
			--Kill script early if piece view has been destroyed to avoid warnings about accessing deleted C++ objects
			if(pieceViewParent[pieceViewName] ~= matchStartPieceView) then
				return
			end
		else
			--logprint("In portal code")
			--A portal is involved. Break the motion into preportal, during portal, and post-portal.

			--Pre-portal
			--logprint("Pre-portal " .. matchStartPieceView.name)
			local deltaXInitial = queuedPortalEntrance.x - slideSrc.x
			local deltaYInitial = queuedPortalEntrance.y - slideSrc.y

			local distanceInitial = math.sqrt(deltaXInitial * deltaXInitial + deltaYInitial * deltaYInitial)
			local slideDurationInitial = distanceInitial / queuedVelocity * 1000.0 * DemoAnimationSlowingFactor

			--Make the piece slide, so that if it is deleted, it will not crash the script.
			RunScriptWithParams {
				target = matchStartPieceView,
				script = "InterpolateGlobalPoint",
				params = { "Center", queuedPortalEntrance, slideDurationInitial },
				tags = {},
			}
			ScriptDelay(slideDurationInitial)
			--Kill script early if piece view has been destroyed to avoid warnings about accessing deleted C++ objects
			if(pieceViewParent[pieceViewName] ~= matchStartPieceView) then
				return
			end

			--During portal
			--Calculate gem size
			--logprint("In-portal " .. matchStartPieceView.name)
			local gemSize = matchStartPieceView.size.x
			gemSize = gemSize * matchStartPieceView.extraScale.x
			if(matchStartPieceView.isBalloons) then
				gemSize = gemSize / 0.7
			end

			--Move the copy of the gem view to go into the entrance portal
			queuedFakePieceView:SetGlobalPoint("Center",queuedPortalEntrance)
			queuedFakePieceView.visible = true
			
			--Move main gem to go out of the exit portal
			local preExitLocation = {x = queuedPortalExit.x, y = queuedPortalExit.y - gemSize}
			local postEntranceLocation = {x = queuedPortalEntrance.x, y = queuedPortalEntrance.y + gemSize}
			if(queuedFallUp) then
				preExitLocation = {x = queuedPortalExit.x, y = queuedPortalExit.y + gemSize}
				postEntranceLocation = {x = queuedPortalEntrance.x, y = queuedPortalEntrance.y - gemSize}
			end
			matchStartPieceView:SetGlobalPoint("Center",preExitLocation)

			--Move main gem and copy to be the children of placeholders, so that the placeholders can clip them as they slide
			local mainPlaceHolderName = "PlaceHolder_" .. placeHolderIndex
			placeHolderIndex = placeHolderIndex + 1
			matchStartPieceView.parent[mainPlaceHolderName] = {
				class = "Actor",
				size = {gemSize,gemSize},
				alpha = 0,
				color = "Red",
				image = "",
			}
			local mainPlaceHolder = matchStartPieceView.parent[mainPlaceHolderName]

			local fakePlaceHolderName = "PlaceHolder_" .. placeHolderIndex
			placeHolderIndex = placeHolderIndex + 1
			queuedFakePieceView.parent[fakePlaceHolderName] = {
				class = "Actor",
				size = {gemSize,gemSize},
				alpha = 0,
				color = "Blue",
				image = "",
			}
			local fakePlaceHolder = queuedFakePieceView.parent[fakePlaceHolderName]
			
			mainPlaceHolder:SetGlobalPoint("Center",queuedPortalExit)
			fakePlaceHolder:SetGlobalPoint("Center",queuedFakePieceView:GetGlobalPoint("Center"))

			matchStartPieceView.parent:RemoveChild(matchStartPieceView)
			mainPlaceHolder:AddChild(matchStartPieceView)
			matchStartPieceView:SetGlobalPoint("Center",preExitLocation)
			
			queuedFakePieceView.parent:RemoveChild(queuedFakePieceView)
			fakePlaceHolder:AddChild(queuedFakePieceView)
			queuedFakePieceView:SetGlobalPoint("Center",fakePlaceHolder:GetGlobalPoint("Center"))

			mainPlaceHolder:ClipToSelf()
			fakePlaceHolder:ClipToSelf()

			--Slide main gem and copy at the same time

			local slideDurationTransition = gemSize / queuedVelocity * 1000.0 * DemoAnimationSlowingFactor
			
			RunScriptWithParams {
				target = matchStartPieceView,
				script = "InterpolateGlobalPoint",
				params = { "Center", queuedPortalExit, slideDurationTransition },
				tags = {},
			}
			
			RunScriptWithParams {
				target = queuedFakePieceView,
				script = "InterpolateGlobalPoint",
				params = { "Center", postEntranceLocation, slideDurationTransition },
				tags = {},
			}

			ScriptDelay(slideDurationTransition)

			--Clean up the copy and return gems back to their normal actor heirarchy, removing the placeholders
			fakePlaceHolder:Destroy()
			mainPlaceHolder:RemoveChild(matchStartPieceView)
			mainPlaceHolder.parent:AddChild(matchStartPieceView)
			matchStartPieceView:SetGlobalPoint("Center",mainPlaceHolder:GetGlobalPoint("Center"))
			mainPlaceHolder:Destroy()

			--After portal. Slide from post portal location to end destination
			--logprint("Post-portal " .. matchStartPieceView.name)
			local deltaXFinal = queuedDest.x - queuedPortalExit.x
			local deltaYFinal = queuedDest.y - queuedPortalExit.y

			local distanceFinal = math.sqrt(deltaXFinal * deltaXFinal + deltaYFinal * deltaYFinal)
			local slideDurationFinal = distanceFinal / queuedVelocity * 1000.0 * DemoAnimationSlowingFactor

			--Make the piece slide, so that if it is deleted, it will not crash the script.
			RunScriptWithParams {
				target = matchStartPieceView,
				script = "InterpolateGlobalPoint",
				params = { "Center", queuedDest, slideDurationFinal },
				tags = {},
			}
			ScriptDelay(slideDurationFinal)
			--Kill script early if piece view has been destroyed to avoid warnings about accessing deleted C++ objects
			if(pieceViewParent[pieceViewName] ~= matchStartPieceView) then
				return
			end
			--logprint("Slide complete for " .. matchStartPieceView.name)
		end

		table.remove(matchStartPieceView.dropDestinations,1)
	end
	matchStartPieceView.dropping = false
	--logprint("Piece_drop done")
end

Scripts.LandingBounce = function(matchStartPieceView,align,queuedDest,slideDuration,firstQueuedLocation)

	--Scripts.JelloLanding_squash(matchStartPieceView)
	-----------------------------
	-- Added Bounce
	local bounceAmount = 2
	local bounceDest = queuedDest
		
	local queuedDest = firstQueuedLocation[1]
	local queuedVelocity = firstQueuedLocation[2]

	bounceDest.y = bounceDest.y + -bounceAmount
	local slideSrc = GetGlobalCenterPoint(matchStartPieceView)
	local deltaX = bounceDest.x - slideSrc.x
	local deltaY = bounceDest.y - slideSrc.y 
	local distance = math.sqrt(deltaX * deltaX + deltaY * deltaY)
	slideDuration = distance / queuedVelocity * 1000.0 * DemoAnimationSlowingFactor

	Scripts.InterpolateGlobalPoint(matchStartPieceView, align, bounceDest, slideDuration*20)
		
	bounceDest.y = bounceDest.y + bounceAmount

	slideSrc = GetGlobalCenterPoint(matchStartPieceView)
	deltaX = bounceDest.x - slideSrc.x
	deltaY = bounceDest.y - slideSrc.y 
	distance = math.sqrt(deltaX * deltaX + deltaY * deltaY)
	slideDuration = distance / queuedVelocity * 1000.0 * DemoAnimationSlowingFactor

	Scripts.InterpolateGlobalPoint(matchStartPieceView, align, bounceDest, slideDuration*12)
	-----------------------------
end

Scripts.PieceChangedScript = function(rootGameView, pieceView, oldSpecials, specials,newPieceTable, fromIce)
	ScriptBreak()
		
	WaitForTaggedScriptCompletion(rootGameView, "Piece_Drop")
	
	local hadVines = HasBit(oldSpecials,CascadePieceSpecial_SpreadCancelSpecials)
	local hasVines = HasBit(specials,CascadePieceSpecial_SpreadCancelSpecials)
	if(not hadVines and hasVines) then
		pieceView.VinesGemDecoration = Resources.VinesGemDecoration
		pieceView.VinesGemDecoration.Glow.visible = true
		pieceView.isVines = true
		pieceView.VinesGemDecoration:AddScript("Wiggle","wiggled")
		PlaySound("medium_wood_break_02.ogg")
		ScaleIn(pieceView.VinesGemDecoration,{0.8,0.8},150,true)
		FadeOut(pieceView.VinesGemDecoration.Glow,500)
	end
	local hadLava = HasBit(oldSpecials,CascadePieceSpecial_SpreadAndConsume)
	local hasLava = HasBit(specials,CascadePieceSpecial_SpreadAndConsume)
	if(not hadLava and hasLava) then
		if (pieceView.VerticalSplashDecoration) then
			pieceView.VerticalSplashDecoration:Destroy()
		elseif (pieceView.HorizontalSplashDecoration) then
			pieceView.HorizontalSplashDecoration:Destroy()
		elseif (pieceView.AreaSplashDecoration) then
			pieceView.AreaSplashDecoration:Destroy()
		elseif (pieceView.PlusSplashDecoration) then
			pieceView.PlusSplashDecoration:Destroy()
		elseif (pieceView.WidePlusSplashDecoration) then
			pieceView.WidePlusSplashDecoration:Destroy()
		elseif (pieceView.WideVerticalSplashDecoration) then
			pieceView.WideVerticalSplashDecoration:Destroy()
		elseif (pieceView.WideHorizontalSplashDecoration) then
			pieceView.WideHorizontalSplashDecoration:Destroy()
		end
		if (pieceView.VerticalSplashAura) then
			pieceView.VerticalSplashAura:Destroy()
		elseif (pieceView.HorizontalSplashAura) then
			pieceView.HorizontalSplashAura:Destroy()
		elseif (pieceView.AreaSplashAura) then
			pieceView.AreaSplashAura:Destroy()
		elseif (pieceView.SuperMatchGemDecoration) then
			pieceView.SuperMatchGemDecoration:Destroy()
		end
		if(pieceView.IconDecoration) then
			pieceView.IconDecoration:Destroy()
		end
		local pieceViewSize = pieceView.size
		pieceView:ApplyProperties(Resources.Gems_LavaGamePieceView)
		if(newPieceTable.icon ~= nil and newPieceTable.icon ~= "") then
			pieceView.IconDecoration = Resources[newPieceTable.icon]
		end
		pieceView.Glow.visible = true
		pieceView.isLava = true
		pieceView.size = pieceViewSize
        if(newPieceTable.turnsToLoss ~= nil and newPieceTable.turnsToLoss > -1) then
			local bombExtraScale = 1.0
			if(pieceView.size.x < 50 or pieceView.extraScale.x < 1.0) then
				--Hack: Apply 7x8 piece view scaling for big board
				bombExtraScale = 44.0/50.0
			end
            pieceView.BombDecoration = Resources.BombDecoration:new {
                resource = "BombDecoration",
				extraScale = bombExtraScale,
				TurnsLabel = {
					class = "Label",
					text = Int2Comma(newPieceTable.turnsToLoss)
				},
            }
        end
		PlaySound("volcano_lava_flow_short1.ogg")
		FireParticle(pieceView, "Steam1", "Center")
		ScaleIn(pieceView,{0.75,0.75},400,true)
		FadeOut(pieceView.Glow,500)
	end
	if(fromIce) then
		pieceView.IceGemDecoration = Resources.IceGemDecoration
		pieceView.isIce = true
		PlaySound("ice_freeze_crack_slow_01_01.ogg")
		--pieceView.IceGemDecoration:AddScript("Wiggle","wiggled")
		FadeIn(pieceView.IceGemDecoration,600)
		ScaleIn(pieceView.IceGemDecoration,{1.25,1.25},600,true)
	end
end

Scripts.BombCountDown = function(rootGameView, pieceView, newTurns)
	ScriptBreak()
		
	WaitForTaggedScriptCompletion(rootGameView, "MasterMatchAnimation")
	
	local bombDecoration = pieceView.BombDecoration
	local turnCountLabel = bombDecoration.TurnsLabel

	if(newTurns > 0) then
		local currentQuantity = tonumber(turnCountLabel.text)
		if(newTurns > currentQuantity)then
			turnCountLabel:AddScript("Animate_GettingTicks","")
		end
	end

	turnCountLabel.text = tostring(newTurns)

	if(newTurns > 0) then
		if(bombDecoration.explosion ~= nil) then
			bombDecoration:DestroyChild(bombDecoration.explosion)
			turnCountLabel.visible = true
			turnCountLabel.fgColor = "White"
		end
		PlaySound("PHAT_A_short.ogg")
		if(bombDecoration:GetTaggedScriptCount("Shake") == 0) then
			bombDecoration:AddScript("Shake","Shake")
		end
		local popScale = bombDecoration.extraScale
		local oldScale = bombDecoration.extraScale
		popScale.x = popScale.x * 1.2
		popScale.y = popScale.y * 1.2
		PopKeepScale(bombDecoration,{expandTo = popScale,finalScale = oldScale})
		if(newTurns == 3)then
			bombDecoration.image = "Bomb_red3.png"
			bombDecoration:AddScript("Shiver","Shiver")
		end
		if(newTurns == 2)then
			bombDecoration.image = "Bomb_red2.png"
			bombDecoration:KillTaggedScripts("Shiver")
			bombDecoration:AddScript("ShiverFast","Shiver")
		end
		if(newTurns == 1)then
			bombDecoration.image = "Bomb_red1.png"
			bombDecoration:KillTaggedScripts("Shiver")
			bombDecoration:AddScript("ShiverFastest","Shiver")
		end
		if(newTurns > 3) then
			bombDecoration.image = "Bomb.png"
			bombDecoration:KillTaggedScripts("Shiver")
		end
	else
		bombDecoration:KillTaggedScripts("Shiver")
		bombDecoration.explosion = Resources.Actor:new {
			resource = "Actor",
			image = "Explosion.png",
			alignment = Center,
			alignmentOffset = {-3,2},
			visible = false,
		}
		turnCountLabel.fgColor = "Black"
		turnCountLabel.visible = false
		PlaySound("Explo_Small_03_crop.ogg")
		ScaleIn(bombDecoration.explosion,{0.0,0.0},50,true)
		ScriptDelay(250)
		PlaySound("horn_lose_05.ogg")
	end
end

Scripts.Piece_Flyup = function(rootGameView, matchStartPieceView, align, slideDest, boardCoordinate, slideVelocity, delay, board_x, board_y)
	ScriptDelay(delay * DemoAnimationSlowingFactor)

	WaitForTaggedScriptCompletion(rootGameView, "Piece_Present")

	local boxName = "Box"..(boardCoordinate.x+1)
	local box = rootGameView.parent:FindChild(boxName,true)
	box:AddScript("SlideUpAndDown","")
		
	--HACK: Since rotation and scale throw off global position, I will set them to 0 while measuring this, then restore them later.
	local backedUpRotation = matchStartPieceView.rotationAngle
	local backedUpScale = matchStartPieceView.extraScale
	matchStartPieceView.rotationAngle = {degrees = 0}
	matchStartPieceView.extraScale = 0
	local slideSrc = GetGlobalCenterPoint(matchStartPieceView)
	matchStartPieceView.rotationAngle = backedUpRotation
	matchStartPieceView.extraScale = backedUpScale

	local deltaX = slideDest.x - slideSrc.x
	local deltaY = slideDest.y - slideSrc.y
	local slideDestTopLeft = {x = slideDest.x - matchStartPieceView.size.x/2,y = slideDest.y - matchStartPieceView.size.y/2}
	local distance = math.sqrt(deltaX * deltaX + deltaY * deltaY)
	if(matchStartPieceView.isBoomerang) then
		slideVelocity = slideVelocity * 0.5 --Animate boomerangs slower so we can see them go
	end
	local slideDuration = distance / slideVelocity * 1000.0 * DemoAnimationSlowingFactor

	PlaySound("swish10.ogg")
	matchStartPieceView.layer = 4
	matchStartPieceView.pulseDuration = slideDuration
	matchStartPieceView:AddScript("PulseSize","PulseSize")
	if(matchStartPieceView.isBoomerang) then
		matchStartPieceView:AddScript("SpinBoomerang","")
		Scripts.SlideToArc(matchStartPieceView,slideDestTopLeft, slideDuration)
	else
		Scripts.QuadraticInterpolateGlobalPoint(matchStartPieceView, align, slideDest, slideDuration)
	end
	matchStartPieceView:ClearLayer()
	PlaySound("CLEEP_B.ogg")
end

Scripts.Piece_Present = function(rootGameView, pieceView, matchFlags, isSpecial, isArea, isFromReelSpin, x, y, isLevelGame)
	-- logprintf("Scripts.Piece_Present "..matchFlags)
	local pieceViewName = pieceView.name
	local pieceViewParent = pieceView.parent

	if isFromReelSpin == true then
		pieceView.visible = false
		local spinTime = 60
		local spinChanges = 4
        
        ScriptDelay(spinTime * x)
		--Kill script early if piece view has been destroyed to avoid warnings about accessing deleted C++ objects
		if(pieceViewParent[pieceViewName] ~= pieceView) then
			return
		end

		--Prevent help content from spinning in. This must be after script delay to work.
        if(pieceView.DisabledOverlay ~= nil) then
            pieceView.DisabledOverlay.visible = false
        end

		local waitForCompletion = false
		RotateIn(pieceView, {revolutions=-2.0}, spinTime*spinChanges, waitForCompletion)
		ScaleIn(pieceView,{0.50,0.50},spinTime*spinChanges,false)

		-- Spin Gems
		local saveImage = pieceView.image
		local imageTable = {"GemBlue.png","GemPurple.png","GemRed.png","GemGreen.png"}
        if(saveImage == "GemBlue_88.png" or saveImage == "GemPurple_88.png" or saveImage == "GemGreen_88.png" or saveImage == "GemRed_88.png" or saveImage == "GemYellow_88.png" or saveImage == "GemOrange_88.png" or saveImage == "Wildgem_88.png") then
            imageTable = {"GemBlue_88.png","GemPurple_88.png","GemRed_88.png","GemGreen_88.png"}
        end
		for i = 1,spinChanges do
			local keepOn = true
			while(keepOn) do
				local currentImage = pieceView.image
				pieceView.image = imageTable[math.random(1,#imageTable)]
				keepOn = (pieceView.image == currentImage)
			end
			ScriptDelay(spinTime)
		end
		
		local soundPick = x+1
		if soundPick > #soundPopSound then
			soundPick = 1
		end
		PlaySound(soundPopSound[soundPick])
		pieceView.image = saveImage
		pieceView.visible = true

        --Help content can now be shown
        if(pieceView.DisabledOverlay ~= nil) then
            pieceView.DisabledOverlay.visible = true
        end
	else -- show on board 
		if((matchFlags==0x1000) or --if special
			(isSpecial) or
			(matchFlags==0x2000))then
			pieceView.visible = false
			ScriptDelay(DelayOfSpecialPresent)
			pieceView.visible = true
		end
	end

	-- CascadeMatchFlags_Diamond
	if(matchFlags==0x1000)then
		pieceView.visible = true
		PlaySound("Cascade_Chimes.ogg")
		local sparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "Shine2",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}
		pieceView:AddChild(sparkle)	
		Pop(pieceView, {expandTo={1.15, 1.25},expandTime=150, shrinkTime=250})
	end
	-- CascadeMatchFlags_Key
	if(matchFlags==0x2000)then
		ScriptDelay(300) -- extra delay to let super wild work
		pieceView.visible = true
		PlaySound("slot_machine_bet_01.ogg")
		local sparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "Shine2",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}
		pieceView:AddChild(sparkle)	
		Pop(pieceView, {expandTo={1.15, 1.25},expandTime=150, shrinkTime=250})
	end
	-- CascadeMatchFlags_Wildcard
	if(isSpecial)then
		ScriptDelay(50) -- extra delay to let super wild work
		pieceView.visible = true
		PlaySound("Magic_EnergyRingPass1.ogg")
		local sparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "Shine2",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}
		pieceView:AddChild(sparkle)	
		Pop(pieceView, {expandTo={1.15, 1.25},expandTime=150, shrinkTime=250})
	elseif(isArea)then 	-- CascadePieceSpecials_AllAreaGems
		pieceView.visible = false
		ScriptDelay(50) 
		pieceView.visible = true
		PlaySound("BlingBling.ogg")
		local sparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "Shine2",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}
		pieceView:AddChild(sparkle)	
		local oldExtraScale = pieceView.extraScale
		PopKeepScale(pieceView, {expandTo={1.15, 1.25},expandTime=150, shrinkTime=250})
		pieceView.extraScale = oldExtraScale
	end	
end

function Scripts.Scored(rootGameView,totalScoreOfMove,totalMatchesOfMove,isDiamondMatch,isSuperWildGemMatch,isKeyGemMatch,gameScreen,score,stats,isLevelGame,isLabEventGame)
    --logprint("Scored: totalScoreOfMove = "..totalScoreOfMove.." totalMatchesOfMove = "..totalMatchesOfMove)
    
	--ScriptDelay(150)

	gameScreen.EventScore = stats.eventScore

	local MultiplierLabel = rootGameView:FindChild("MultiplierLabel",true)
	if(MultiplierLabel.visible)then
		MultiplierLabel:AddScript("MultiplierWork","")
	end

	StopSound("swish05.ogg")
	PlaySound("Pickup_Coins06.ogg")
    PlaySound("slot_machine_wheel_stop_01.ogg")
	
	if(totalMatchesOfMove>=18)then
		PlaySound("slot_machine_jackpot_win_loop.ogg")
    elseif(totalMatchesOfMove>=14)then
        PlaySound("slot_machine_bet_16.ogg")
    elseif(totalMatchesOfMove>=12)then
        PlaySound("slot_machine_bet_15.ogg")
    elseif(totalMatchesOfMove>=10)then
        PlaySound("slot_machine_bet_14.ogg")
    elseif(totalMatchesOfMove>=8)then
        PlaySound("slot_machine_bet_13.ogg")
    end
	
    local scoreSparkle = CreateInstance {
        name = "DiamondSparkle",
        class = "hgeParticleSystemActor",
        file = "SparkleBurst1",
        alignment = "Center",
        alignTo = "Center",
        alignmentOffset = {0,0},
    }
    local ScoreLabel = rootGameView:FindChild("ScoreLabel",true)
    ScoreLabel:AddChild(scoreSparkle)
	
    CorrectedPop(ScoreLabel, {expandTo={1.15, 1.25},expandTime=150, shrinkTime=250})

	if(stats.goalScore1 ~= nil)then
		--logprint("Scored: score = "..score.." stats.goalScore1 = "..stats.goalScore1.." stats.goalScore2 = "..stats.goalScore2.." stats.goalScore3 = "..stats.goalScore3)
		

		local StarMeter = rootGameView:FindChild("StarMeter",true)
		local Star1 = StarMeter:FindChild("Star1",true)
		local Star2 = StarMeter:FindChild("Star2",true)
		local Star3 = StarMeter:FindChild("Star3",true)
		
		local metersize = StarMeter.size.x --8 is blank margin space
		--logprint("Scored: metersize = "..metersize)

		local score_percent			= (score*100)/stats.goalScore3
		local goalScore1_percent	= (stats.goalScore1*100)/stats.goalScore3
		local goalScore2_percent	= (stats.goalScore2*100)/stats.goalScore3
		local goalScore3_percent	= (stats.goalScore3*100)/stats.goalScore3

		local score_pixels			= (metersize*score_percent)/100
		local goalScore1_pixels		= (metersize*goalScore1_percent)/100
		local goalScore2_pixels		= (metersize*goalScore2_percent)/100
		local goalScore3_pixels		= (metersize*goalScore3_percent)/100
		
		--logprint("Scored: score_percent = "..score_percent.." goalScore1_percent = "..goalScore1_percent.." goalScore2_percent = "..goalScore2_percent.." goalScore3_percent = "..goalScore3_percent)
		--logprint("Scored: score_pixels = "..score_pixels.." goalScore1_pixels = "..goalScore1_pixels.." goalScore2_pixels = "..goalScore2_pixels.." goalScore3_pixels = "..goalScore3_pixels)

		if(score_pixels>metersize)then
			score_pixels=metersize
		end

		StarMeter.meterFrame.meterBar.size = Layout{{score_pixels,5},iphone4={score_pixels,4},iphone5={score_pixels,5},ipad={score_pixels,5}}

		if(score>=stats.goalScore1)then
			if(Star1.image~="GamePanel_ScoreMeter_Star_Gold.png" and Star1.image~="Medal_small_bronze.png" and Star1.image~="LabLevel_MeterFlask_1.png")then
				PlaySound("pop_sparkle_01.ogg")
				if(isLevelGame)then
					if (isLabEventGame) then
						Star1.image = "LabLevel_MeterFlask_1.png"
					else
						Star1.image = "GamePanel_ScoreMeter_Star_Gold.png"
					end
				else
					Star1.image = "Medal_small_bronze.png"
				end
   				local starSparkle = CreateInstance {
					name = "DiamondSparkle",
					class = "hgeParticleSystemActor",
					file = "SparkleBurst1",
					alignment = "Center",
					alignTo = "Center",
					alignmentOffset = {0,0},
				}
				Star1:AddChild(starSparkle)	
				CorrectedPop(Star1, {expandTo={1.15, 1.15},expandTime=50, shrinkTime=250})
			end
		end
		if(score>=stats.goalScore2)then
			if(Star2.image~="GamePanel_ScoreMeter_Star_Gold.png" and Star2.image~="Medal_small_silver.png" and Star2.image~="LabLevel_MeterFlask_2.png")then
				PlaySound("pop_sparkle_01.ogg")
				if(isLevelGame)then
					if (isLabEventGame) then
						Star2.image = "LabLevel_MeterFlask_2.png"
					else
						Star2.image = "GamePanel_ScoreMeter_Star_Gold.png"
					end
				else
					Star2.image = "Medal_small_silver.png"
				end
   				local starSparkle = CreateInstance {
					name = "DiamondSparkle",
					class = "hgeParticleSystemActor",
					file = "SparkleBurst1",
					alignment = "Center",
					alignTo = "Center",
					alignmentOffset = {0,0},
				}
				Star2:AddChild(starSparkle)	
				CorrectedPop(Star2, {expandTo={1.15, 1.15},expandTime=50, shrinkTime=250})
			end
		end
		if(score>=stats.goalScore3)then
			if(Star3.image~="GamePanel_ScoreMeter_Star_Gold.png" and Star3.image~="Medal_small_gold.png" and Star3.image~="LabLevel_MeterFlask_3.png")then
				PlaySound("pop_sparkle_01.ogg")
				if(isLevelGame)then
					if (isLabEventGame) then
						Star3.image = "LabLevel_MeterFlask_3.png"
					else
						Star3.image = "GamePanel_ScoreMeter_Star_Gold.png"
					end
				else
					Star3.image = "Medal_small_gold.png"
				end
   				local starSparkle = CreateInstance {
					name = "DiamondSparkle",
					class = "hgeParticleSystemActor",
					file = "SparkleBurst1",
					alignment = "Center",
					alignTo = "Center",
					alignmentOffset = {0,0},
				}
				Star3:AddChild(starSparkle)	
				CorrectedPop(Star3, {expandTo={1.15, 1.15},expandTime=50, shrinkTime=250})
			end
		end
	end

	if(isKeyGemMatch)then
		local spinSparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "SparkleBurst1",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}	   
	    local RemainingSpinsLabel = rootGameView:FindChild("RemainingSpinsLabel",true)
		RemainingSpinsLabel:AddChild(spinSparkle)	
		PopKeepScale(RemainingSpinsLabel, {expandTo={1.15, 1.25},expandTime=150, shrinkTime=250})
	end
    
	local discoBall

	--Global cooldowns are used to prevent over-play of sound effects
	
	if(Cooldown_Outrageous == nil) then
		Cooldown_Outrageous = 0
	end
	if(Cooldown_Fantastic == nil) then
		Cooldown_Fantastic = 0
	end
	if(Cooldown_Wow == nil) then
		Cooldown_Wow = 0
	end
	if(Cooldown_RightOn == nil) then
		Cooldown_RightOn = 0
	end
	
	if(Cooldown_Outrageous > 0) then
		Cooldown_Outrageous = Cooldown_Outrageous - 1
	end
	if(Cooldown_Fantastic > 0) then
		Cooldown_Fantastic = Cooldown_Fantastic - 1
	end
	if(Cooldown_Wow > 0) then
		Cooldown_Wow = Cooldown_Wow - 1
	end
	if(Cooldown_RightOn > 0) then
		Cooldown_RightOn = Cooldown_RightOn - 1
	end

    -- Audience reaction -- By Score
    if(totalScoreOfMove>=100000 and Cooldown_Outrageous <= 0)then
        PlaySound("Cascade_Outrageous.ogg")
		Cooldown_Outrageous = 16
    elseif(totalScoreOfMove>=45000 and Cooldown_Fantastic <= 0)then
        PlaySound("Cascade_Fantastic.ogg")
		Cooldown_Fantastic = 16
    elseif(totalScoreOfMove>=30000 and Cooldown_Wow <= 0)then
        PlaySound("Cascade_Wow2.ogg")        
		Cooldown_Wow = 16
    elseif(totalScoreOfMove>=15000 and Cooldown_RightOn <= 0)then
		PlaySound("Cascade_RightOn.ogg")
		Cooldown_RightOn = 16
    end

	if(isKeyGemMatch)then
		ScriptDelay(1500) --Let Music Play a bit
		gameScreen:StartGameMusic()
	end

end

function Scripts.CalloutSpinsChanging(rootGameView, remainingSpins)
	ScriptDelay(100)
	local RemainingSpinsLabel = rootGameView:FindChild("RemainingSpinsLabel",true)
	PopKeepScale(RemainingSpinsLabel, {expandTo={1.25, 1.25},expandTime=200, shrinkTime=150})
	if(remainingSpins <= 2) then
		Scripts.RemainingSpins_Pulsate(RemainingSpinsLabel,remainingSpins)
	else
		Scripts.RemainingSpins_Cleanup(RemainingSpinsLabel)
	end
end

function Scripts.CalloutSpinsChanging_bought(rootGameView, remainingSpins)
	local sparkle = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "SparkleBurst1",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,0},
	}	   
	PlaySound("sparkle_little.ogg")
	ScriptDelay(100)
	PlaySound("slot_machine_win_22.ogg")
	local RemainingSpinsLabel = rootGameView:FindChild("RemainingSpinsLabel",true)
	RemainingSpinsLabel:AddChild(sparkle)	
	PopKeepScale(RemainingSpinsLabel, {expandTo={1.25, 1.25},expandTime=200, shrinkTime=150})
	if(remainingSpins <= 2) then
		Scripts.RemainingSpins_Pulsate(RemainingSpinsLabel,remainingSpins)
	else
		Scripts.RemainingSpins_Cleanup(RemainingSpinsLabel)
	end
end

Scripts.RemainingSpins_Pulsate = function(label,remainingSpins)
	label:KillTaggedScripts("HeartBeatLoop")
	if(remainingSpins == 2) then
		label:AddScript("HeartBeatLoop","HeartBeatLoop")
	end
	if(remainingSpins == 1)then
		label:AddScript("HeartBeatLoopFast","HeartBeatLoop")
	end
	if(remainingSpins == 0)then
		label:AddScript("HeartBeatLoopFastest","HeartBeatLoop")
	end
end

Scripts.PulsateViaPercent = function(label,percent)
	--logprint("PulsateViaPercent "..percent)
	label:KillTaggedScripts("HeartBeatLoop")
	if(percent ~= 100)then
		if(percent > 90) then
			label:AddScript("HeartBeatLoopFastest","HeartBeatLoop")
		elseif(percent > 80) then
			label:AddScript("HeartBeatLoopFast","HeartBeatLoop")
		elseif(percent > 70) then
			label:AddScript("HeartBeatLoop","HeartBeatLoop")
		end
	end
end

Scripts.RemainingSpins_Cleanup = function(label)
	label:KillTaggedScripts("HeartBeatLoop")
end

Scripts.MultiplierUp = function(self)
	--logprint("MultiplierUp")
	local sparkle = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "SparkleBurst1",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,0},
	}	

	if(self.visible)then -- already on, just pop
		ScriptDelay(1000)
		Pop(self, {expandTo={1.5, 1.5},expandTime=50, shrinkTime=250})
		self:AddChild(sparkle)	
	else	-- wait a little bit then bring on
		ScriptDelay(1400)
		self:AddChild(sparkle)	
		self.layer = 1
		PlaySound("soft_lose_02_quick_reverse.ogg")
		self.visible = true
		--SlideOn(self,250,"top","wait")
		Pop(self, {expandTo={1.5, 1.5},expandTime=50, shrinkTime=250})
		self:ClearLayer()
	end
end

Scripts.MultiplierPop = function(self)
	--if(not self.visible) then 
		--logprint("MultiplierPop")
		local sparkle = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "SparkleBurst1",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}	

		--logprint("MultiplierPop")
		self:AddChild(sparkle)	
		self.layer = 1
		PlaySound("soft_lose_02_quick_reverse.ogg")
		--SlideOn(self,250,"top","wait")
		CorrectedPop(self, {expandTo={1.5, 1.5},expandTime=50, shrinkTime=250})
		self:ClearLayer()
	--end
end

Scripts.MultiplierWork = function(self)
	if(self.visible) then 
		--logprint("MultiplierWork")

		CorrectedPop(self, {expandTo={1.25, 1.25},expandTime=50, shrinkTime=250})
	end
end

local MultiplierOn = false

Scripts.MultiplierOn = function(gameScreen,currentMultiplier,multiplierLabel,labelText)
	--logprint("MultiplierOn "..currentMultiplier)
	local ScoreContainer = gameScreen:FindChild("ScoreContainer",true)
	local MultiplierPanel = gameScreen:FindChild("MultiplierPanel",true)
	ScoreContainer.image=Layout{"GamePanel_ScoreMeter_Banner_alt.png",iphone4="GamePanel_ScoreMeter_Banner_alt_medium.png",ipad="GamePanel_ScoreMeter_Banner_alt.png"}
	
	MultiplierPanel.image=Layout{"GamePanel_Circle_alt.png",iphone4="GamePanel_Circle_alt_medium.png",ipad="GamePanel_Circle_alt_medium.png"}

	--multiplierLabel.text = labelText
	
	-- Some work on giving the screen an effect on high multipliers
	-- Todo: Does this effect perf on low end machiens
	-- Note: Defaulted to off right now
	if(Config.Multiplier_Effect == true) then
		local BoardFrame = gameScreen:FindChild("BoardFrame",true)
		if(not MultiplierOn)then 
			MultiplierOn = true
			local hgeParticleLayer = BoardFrame:FindChild("hgeParticleLayer",true)
			if(not BoardFrame.ScreenParticle1)then 
				BoardFrame.ScreenParticle1 = FireParticle(BoardFrame,"Stream1_toBottom","MidBottom",hgeParticleLayer)
			else
				BoardFrame.ScreenParticle1:Fire()
			end
			if(not  BoardFrame.ScreenParticle2)then 
				BoardFrame.ScreenParticle2 = FireParticle(BoardFrame,"Stream1_toTop","MidTop",hgeParticleLayer)
			else
				BoardFrame.ScreenParticle2:Fire()
			end
			if(not BoardFrame.ScreenParticle3)then 
				BoardFrame.ScreenParticle3 = FireParticle(BoardFrame,"Stream1_toLeft","MidLeft",hgeParticleLayer)
			else
				BoardFrame.ScreenParticle3:Fire()
			end
			if(not BoardFrame.ScreenParticle4)then 
				BoardFrame.ScreenParticle4 = FireParticle(BoardFrame,"Stream1_toRight","MidRight",hgeParticleLayer)
			else
				BoardFrame.ScreenParticle4:Fire()
			end
		end

		--DEBUG
		--currentMultiplier = 20

		local emissionFactor = currentMultiplier
		if(emissionFactor>20)then	
			emissionFactor = 20
		end
		local emissionRate = 1*emissionFactor

		if(BoardFrame.ScreenParticle1)then
			BoardFrame.ScreenParticle1.info.nEmission=emissionRate
		end
		if(BoardFrame.ScreenParticle2)then
			BoardFrame.ScreenParticle2.info.nEmission=emissionRate
		end
		if(BoardFrame.ScreenParticle3)then
			BoardFrame.ScreenParticle3.info.nEmission=emissionRate
		end
		if(BoardFrame.ScreenParticle4)then
			BoardFrame.ScreenParticle4.info.nEmission=emissionRate
		end
	end
end

Scripts.MultiplierOff = function(gameScreen)
	--logprint("MultiplierOff")
	local ScoreContainer = gameScreen:FindChild("ScoreContainer",true)
	local MultiplierPanel = gameScreen:FindChild("MultiplierPanel",true)
	ScoreContainer.image=Layout{"GamePanel_ScoreMeter_Banner.png",iphone4="GamePanel_ScoreMeter_Banner_medium.png",ipad="GamePanel_ScoreMeter_Banner.png"}
		
	if(MultiplierPanel.visible) then
		PlaySound("soft_lose_02_quick.ogg")
		MultiplierPanel.layer = 1
		SlideOff(MultiplierPanel,250,"top","wait")
		MultiplierPanel.image=Layout{"GamePanel_Circle.png",iphone4="GamePanel_Circle_medium.png"}
		MultiplierPanel:ClearLayer()
	end

	if(Config.Multiplier_Effect == true) then
		local BoardFrame = gameScreen:FindChild("BoardFrame",true)
		local hgeParticleLayer = BoardFrame:FindChild("hgeParticleLayer",true)
		--hgeParticleLayer:KillAllEmitters() --Instead just stop
		if(MultiplierOn)then
			MultiplierOn = false
			if(BoardFrame.ScreenParticle1) then
				BoardFrame.ScreenParticle1:Stop(false)
			end
			if(BoardFrame.ScreenParticle2) then
				BoardFrame.ScreenParticle2:Stop(false)
			end
			if(BoardFrame.ScreenParticle3) then
				BoardFrame.ScreenParticle3:Stop(false)
			end
			if(BoardFrame.ScreenParticle4) then
				BoardFrame.ScreenParticle4:Stop(false)
			end
		end
	end
end

Scripts.Preshow_HandleMiniGame = function(screen)
end

Scripts.Present_HandleMiniGame_Start = function(screen)
	--local discoBallFX = screen:FindChild("discoBallFX",true)
	--discoBallFX.visible = true
	--StopSoundAsMusic(Logic_GameMusic)
	--LoopSoundAsMusic("Trinklit_MiniGameMusicLoop.ogg",true)
end

Scripts.Present_HandleMiniGame_End = function(screen,winType,addedSpins,percentCost,score)
	--logprint("Present_HandleMiniGame_End "..Logic_GameMusic)
	--LoopSoundAsMusic(Logic_GameMusic,true)

	--local discoBallFX = screen:FindChild("discoBallFX",true)
	--discoBallFX.visible = false
		
	if(winType=="KARMA")then
		PlaySound("slot_machine_win_03.ogg")
	elseif(winType=="1SPIN")then
		PlaySound("slot_machine_win_02.ogg")
		PlaySound("sci_fi_lose_04_reverse.ogg")
	elseif(winType=="PAIDSPIN")then
		PlaySound("sci_fi_lose_02.ogg")
		PlaySound("slot_machine_win_02.ogg")
		PlaySound("sci_fi_lose_04_reverse.ogg")
	elseif(winType=="2SPINS")then
		PlaySound("slot_machine_win_02.ogg")
		PlaySound("sci_fi_lose_04_reverse.ogg")
	elseif(winType=="3SPINS")then
		PlaySound("slot_machine_win_02.ogg")
		PlaySound("sci_fi_lose_04_reverse.ogg")
	elseif(winType=="FORTUNE")then
		PlaySound("slot_machine_win_03.ogg")
	elseif(winType=="10TAX")then
		PlaySound("sci_fi_lose_02.ogg")
	end
end

Scripts.Leave_HandleMiniGame = function(screen)
	--logprint("Leave_HandleMiniGame "..Logic_GameMusic)
	--LoopSoundAsMusic(Logic_GameMusic)
end

Scripts.RestartGame = function(screen)
	local Sparks = CreateInstance {
		name = "Sparks",
		class = "hgeParticleSystemActor",
		file = "Sparks1",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,0},
	}	
	screen.RootActor:AddChild(Sparks)	
end

Resources.RandomSparkle = Resources.Actor:new {
	resource = "Actor",
	image = "sparkle.png",
	zorder = 10,
}

function PixelHasAlpha(actor,position)
	local oldNonrectangular = actor.isNonrectangular
	actor.isNonrectangular = true
	local globalPoint = actor:LocalToGlobalCoord(position)
	local retVal = actor:CollidesWithGlobalPoint(globalPoint)
	actor.isNonrectangular = oldNonrectangular
	return retVal
end

Scripts.SparkleRandomly = function(sparkler)
	while(true) do
		ScriptDelay(math.random(200, 4000))
		local sparklePosition = {}
		for i = 1,1 do -- was 1,200 - why?
			sparklePosition.x = math.random(0,sparkler.size.x)
			sparklePosition.y = math.random(0,sparkler.size.y)

			--[[	-- DavidL, 2013-03-08: I took this part out, as it currently requires a fair amount of CPU and memory resources for some iOS devices
			if(PixelHasAlpha(sparkler,sparklePosition)) then
				break
			end
			--]]
		end
		sparkler.sparkle = Resources.RandomSparkle
		local sparkleSpeed = math.random(500, 1500)
		local sparkle = sparkler.sparkle
		sparkle.center = sparklePosition
		sparkle.extraScale = math.random(0.8,1.2)
		sparkle.rotationAngle = {degrees = 0}
		FadeIn(sparkle,sparkleSpeed)
		ScaleIn(sparkle,{0.0, 0.0},sparkleSpeed)
		RotateIn(sparkle,{revolutions=-0.25},sparkleSpeed,true)
		FadeOutAndDestroy(sparkle,sparkleSpeed)
		ScaleOut(sparkle,{0.0, 0.0},sparkleSpeed)
		RotateOut(sparkle,{revolutions=0.25},sparkleSpeed,true)
		ScriptBreak()
	end
end

Scripts.SparkleRandomly_gold = function(sparkler)
	--FireParticle(sparkler,"GoldSparkle1","Center")
	while(true) do
		ScriptDelay(math.random(50, 100),"DialogTimers")
		local sparklePosition = {}
		local offset = 5
		sparklePosition.x = math.random(0+offset ,sparkler.size.x-offset )
		sparklePosition.y = math.random(0+offset ,sparkler.size.y-offset )
		sparkler.sparkle = Resources.RandomSparkle
		local sparkleSpeed = math.random(500/2, 1500/2)
		local sparkle = sparkler.sparkle
		sparkle.center = sparklePosition
		sparkle.extraScale = math.random(0.8,1.2)
		sparkle.rotationAngle = {degrees = 0}
		FadeIn(sparkle,sparkleSpeed)
		ScaleIn(sparkle,{0.0, 0.0},sparkleSpeed)
		RotateIn(sparkle,{revolutions=-0.25},sparkleSpeed,true)
		FadeOutAndDestroy(sparkle,sparkleSpeed)
		ScaleOut(sparkle,{0.0, 0.0},sparkleSpeed)
		RotateOut(sparkle,{revolutions=0.25},sparkleSpeed,true)
		ScriptBreak()
	end
end

Scripts.SparkleLoop = function(sparkler)
	while(true) do
		ScriptDelay(math.random(50, 100))
		local sparklePosition = {}
		for i = 1,1 do
			sparklePosition.x = math.random(0,sparkler.size.x)
			sparklePosition.y = math.random(0,sparkler.size.y)

			--[[	-- DavidL, 2013-03-08: I took this part out, as it currently requires a fair amount of CPU and memory resources for some iOS devices
			if(PixelHasAlpha(sparkler,sparklePosition)) then
				break
			end
			--]]
		end
		sparkler.sparkle = Resources.RandomSparkle
		local sparkleSpeed = math.random(500, 600)
		local sparkle = sparkler.sparkle
		sparkle.center = sparklePosition
		sparkle.extraScale = math.random(0.8,1.2)
		sparkle.rotationAngle = {degrees = 0}
		FadeIn(sparkle,sparkleSpeed)
		ScaleIn(sparkle,{0.0, 0.0},sparkleSpeed)
		RotateIn(sparkle,{revolutions=-0.25},sparkleSpeed,true)
		FadeOutAndDestroy(sparkle,sparkleSpeed)
		ScaleOut(sparkle,{0.0, 0.0},sparkleSpeed)
		RotateOut(sparkle,{revolutions=0.25},sparkleSpeed,true)
		ScriptBreak()
	end
end


Scripts.SparkRandomly = function(sparkler)
	while(true) do
		ScriptDelay(math.random(6000, 12000))
		local Sparks = CreateInstance {
			name = "Sparks",
			class = "hgeParticleSystemActor",
			file = "Sparks1",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,0},
		}	
		sparkler:AddChild(Sparks)	
		ScriptDelay(2000)
		sparkler:DestroyChild(Sparks)
	end
end

Scripts.Present_GameOverAnimation = function(gameScreen,numRemainingSpins)
end

didJackpot_Cascade = false
didJackpot_Jackpot = false

Scripts.Present_Begin_EndGame_BurstGems_Initial = function(rootGameView)
	didJackpot_Jackpot = true

	local goodJobWord = rootGameView:FindChild("FlashWord",true)

	goodJobWord.image = "jackpot.png"
	LoopSound("slot_machine_jackpot_win_loop.ogg")
	FireParticle(rootGameView, "Sparks1", "Center")
	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarBurst3",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	rootGameView:AddChild(burst)	
	PlaySound("slot_machine_win_23.ogg")
	
	--ScaleIn(goodJobWord,{0.0,0.0},150,true)
	goodJobWord:AddScript("Jello2","Jello2")
	AddScriptWithParams(rootGameView,"GoodJobWord",goodJobWord.image, STRINGS("Jackpot!"),750,"bottom")
	PlaySound("Cascade_Jackpot.ogg")
	ScriptDelay(1000)
	
	--ScaleOut(goodJobWord,{0.0,0.0},150,false)
	--FadeOut(goodJobWord,150)
	ScriptDelay(150)
end

Scripts.Present_Begin_EndGame_ExtraSpins = function(rootGameView, spinsToProcess, oldMultiplier, updatedPieces, pieceViewScaling)
	didJackpot_Cascade = true

	local goodJobWord = rootGameView:FindChild("FlashWord",true)
	
	if(not didJackpot_Jackpot) then
		didJackpot_Jackpot = true
		goodJobWord.image = "jackpot.png"
		LoopSound("slot_machine_jackpot_win_loop.ogg")
		FireParticle(rootGameView, "Sparks1", "Center")
		local burst = CreateInstance {
			name = "DiamondSparkle",
			class = "hgeParticleSystemActor",
			file = "StarBurst3",
			alignment = "Center",
			alignTo = "Center",
			alignmentOffset = {0,-30},
		}	
		rootGameView:AddChild(burst)	
		PlaySound("slot_machine_win_23.ogg")
		--ScaleIn(goodJobWord,{0.0,0.0},150,true)
		goodJobWord:AddScript("Jello2","Jello2")
		AddScriptWithParams(rootGameView,"GoodJobWord",goodJobWord.image, STRINGS("Jackpot!"),750,"bottom")
		PlaySound("Cascade_Jackpot.ogg")
		ScriptDelay(650)
		
		--ScaleOut(goodJobWord,{0.0,0.0},150,false)
		--FadeOut(goodJobWord,150)
		ScriptDelay(150)
	end

	--Tick up the multiplier while ticking down remaining spins
	local remainingSpinsContainer = rootGameView:FindChild("RemainingSpinsContainer",true)
	local spinsField = remainingSpinsContainer.RemainingSpinsLabel
	local bonusField = rootGameView:FindChild("MultiplierPanel",true):FindChild("MultiplierLabel",true) --jsupnik todo
	local setupDuration = 200
	if(spinsToProcess > 0) then
		for i = 1,spinsToProcess do
			spinsField.text = Int2Comma(spinsToProcess - i)
			PlaySound("slot_machine_win_04.ogg")
			FireParticle(spinsField,"SparkleBurst1","Center")
			CorrectedPop(remainingSpinsContainer)
			local updatePieceTable = updatedPieces[i]
			local pieceView = updatePieceTable["pieceView"] 
			PlaySound("laserShot1.ogg")

			--Fly a +1 to the multiplier

			local origin =  GetGlobalCenterPoint(spinsField)
			local destination = GetGlobalCenterPoint(bonusField)

			rootGameView["PlusOne" .. i .. "A"] = {
				class = "Actor",
				image = "PlusOne.png",
				layer = 11,
				center = GetGlobalCenterPoint(spinsField),
				visible = false,
			}
			local plusOneForSpins = rootGameView["PlusOne" .. i .. "A"]

			destination.x = destination.x -- plusOneForSpins.size.x/2
			destination.y = destination.y -- plusOneForSpins.size.y/2

			plusOneForSpins:SlideTo(destination,setupDuration * DemoAnimationSlowingFactor)
			
			local sparkle = CreateInstance {
				name = "",
				class = "hgeParticleSystemActor",
				file = "SuperWildTrail2",
				position = origin,
			}
			sparkle.destination  = destination
			sparkle.duration = setupDuration * DemoAnimationSlowingFactor
			rootGameView:AddChild(sparkle)
			sparkle:AddScript("FlyToDestination","")
			sparkle:AddScript("BoomAtDestination","")

			--Also fly a +1 to the piece that got affected

			if(pieceView ~= nil) then
				local origin =  GetGlobalCenterPoint(spinsField)
				local destination = GetGlobalCenterPoint(updatePieceTable["pieceView"])
				
				rootGameView["PlusOne" .. i .. "B"] = {
					class = "Actor",
					image = "PlusOne.png",
					layer = 11,
					center = GetGlobalCenterPoint(spinsField),
					visible = false,
				}

				local plusOneForPieces = rootGameView["PlusOne" .. i .. "B"]
				
				destination.x = destination.x -- plusOneForPieces.size.x/2
				destination.y = destination.y -- plusOneForPieces.size.y/2
				
				local sparkle_piece = CreateInstance {
					name = "",
					class = "hgeParticleSystemActor",
					file = "SuperWildTrail2",
					position = origin,
				}
				sparkle_piece.destination  = destination
				sparkle_piece.duration = setupDuration * DemoAnimationSlowingFactor
				rootGameView:AddChild(sparkle_piece)
				sparkle_piece:AddScript("FlyToDestination","")
				sparkle_piece:AddScript("BoomAtDestination","")

				plusOneForPieces:SlideTo(destination,setupDuration * DemoAnimationSlowingFactor)
			end
			ScriptDelay(setupDuration+50)
			rootGameView["PlusOne" .. i .. "A"]:Destroy()
			if(pieceView ~= nil) then
				rootGameView["PlusOne" .. i .. "B"]:Destroy()
			end
			--Update multiplier
			bonusField.text = Int2Comma(oldMultiplier + i) .. "x"
			--Update piece
			if(pieceView ~= nil) then
				if(updatePieceTable["specials"] == CascadePieceSpecial_HorizontalSplash) then
					pieceView.HorizontalSplashDecoration = Resources.HorizontalSplashDecoration
					pieceView.HorizontalSplashAura = Resources.HorizontalSplashAura
				end
				if(updatePieceTable["specials"] == CascadePieceSpecial_VerticalSplash) then
					pieceView.VerticalSplashDecoration = Resources.VerticalSplashDecoration
					pieceView.VerticalSplashAura = Resources.VerticalSplashAura
				end
				if(updatePieceTable["specials"] == CascadePieceSpecial_AreaSplash) then
					pieceView.AreaSplashDecoration = Resources.AreaSplashDecoration
					pieceView.AreaSplashAura = Resources.AreaSplashAura
				end
				if(updatePieceTable["specials"] == CascadePieceSpecial_ClearKeyColorOnMatch) then
					--Save position from its old size
					local oldGlobalCenter = pieceView:GetGlobalPoint(Center)
					local oldExtraScale = pieceView.extraScale
					pieceView:ApplyProperties(Resources.Gems_WildcardSuperGemGamePieceView)
					pieceView.SuperMatchGemDecoration = Resources.SuperMatchGemDecoration
					oldExtraScale.x = oldExtraScale.x * pieceViewScaling
					oldExtraScale.y = oldExtraScale.y * pieceViewScaling
					pieceView.extraScale = oldExtraScale
					--Reposition the piece according to its new size
					pieceView:SetGlobalPoint(Center,oldGlobalCenter)
				end
			end
			Scripts.MultiplierOn(rootGameView,oldMultiplier+i)
			PlaySound("pop_sparkle_01.ogg")
			bonusField.parent.visible = true
			CorrectedPop(bonusField, {expandTo={1.15, 1.15},expandTime=50, shrinkTime=50})
		end
	end
	goodJobWord.image = "cascade.png"
	--LoopSound("slot_machine_jackpot_win_loop.ogg")
	FireParticle(rootGameView, "Sparks1", "Center")
	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarBurst3",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	rootGameView:AddChild(burst)	
	PlaySound("slot_machine_win_23.ogg")
	AddScriptWithParams(rootGameView,"GoodJobWord","cascade.png",STRINGS("Cascade!"),750,"top")
	--ScaleIn(goodJobWord,{0.0,0.0},150,true)
	goodJobWord:AddScript("Jello2","Jello2")
	
	PlaySound("Cascade_Cascade.ogg")

	if(spinsToProcess==0)then
		ScriptDelay(2000)
	else
		ScriptDelay(1000)
	end
	--ScaleOut(goodJobWord,{0.0,0.0},150,false)
	--FadeOut(goodJobWord,150)
	ScriptDelay(150)
end

Scripts.Present_StarCardRankBonus = function(rootGameView,score,StarCardRankBonusScore,bonusScore)

	logprint("Present_StarCardRankBonus: ".."Score: "..score.."StarCardRankBonusScore: "..StarCardRankBonusScore.."BonusScore: "..bonusScore)

	local goodJobWord = rootGameView:FindChild("FlashWord",true)
	goodJobWord.image = "vipbonus.png"
	--LoopSound("slot_machine_jackpot_win_loop.ogg")
	FireParticle(rootGameView, "Sparks1", "Center")
	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarBurst3",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	rootGameView:AddChild(burst)	
	--PlaySound("slot_machine_win_13.ogg")
	--ScaleIn(goodJobWord,{0.0,0.0},150,true)
	goodJobWord:AddScript("Jello2","Jello2")
	AddScriptWithParams(rootGameView,"GoodJobWord",goodJobWord.image,STRINGS("VIP Bonus!"),750,"top")
	PlaySound("Cascade_Bonus.ogg")
	

	ScriptDelay(1000)
	PlaySound("slot_machine_win_13.ogg")
	
	--ScaleOut(goodJobWord,{0.0,0.0},150,false)
	--FadeOut(goodJobWord,150)
	ScriptDelay(150)

	--Make big score flash
	PlaySound("slot_machine_win_22.ogg")
	local allUnmatchedSpaces = rootGameView:FindChild("AllUnmatchedSpaces",true)
	FireParticle( rootGameView.AllScoreFlashes,"SparkleBurst1","Center")
	local position = GetGlobalCenterPoint(allUnmatchedSpaces)
	position.y = position.y - 30
	RunScriptWithParams {
		target = rootGameView.AllScoreFlashes,
		script = "CascadeScoreFlash",
		params = {position, StarCardRankBonusScore, "White", "AppScoreFlash42" },
	}
	
	PlaySound("Cascade_Boom.ogg")
	ScriptDelay(500)
	PlaySound("slot_machine_win_07.ogg")
	ScriptDelay(200) --Allows score flash to get off the screen before transitioning to endgame state
end

Scripts.Present_OutOfSpins = function(rootGameView, soonestBomb)
	WaitForTaggedScriptCompletion(rootGameView, "GoodJobWord")
	local goodJobWord = rootGameView:FindChild("FlashWord",true)
	PlaySound("Magic_Absorb4.ogg")
	if(soonestBomb == 0) then
		goodJobWord.image = "outofticks.png"
		AddScriptWithParams(rootGameView,"GoodJobWord_withSqueeze",goodJobWord.image, STRINGS("Out Of Ticks"),750,"bottom")
	else
		goodJobWord.image = "nospinsleft.png"
		AddScriptWithParams(rootGameView,"GoodJobWord_withSqueeze",goodJobWord.image, STRINGS("Out Of Spins"),750,"bottom")
	end
	--ScaleIn(goodJobWord,{0.0,0.0},150,true)
	--goodJobWord:AddScript("Squeeze","Squeeze")
	ScriptDelay(1000)
	--FadeOut(goodJobWord,300)
	--ScaleOut(goodJobWord,{2.0,0.0},300,"wait")
	PlaySound("sci_fi_lose_04.ogg");
end

Scripts.ShowOutOfSpinsPanel = function(rootGameView,stats)
	--logprint("Entering ShowOutOfSpinsPanel")
	--logprint("Script count: ShowOutOfSpinsPanel " .. rootGameView:GetTaggedScriptCount("ShowOutOfSpinsPanel"))
	--logprint("Script count: HideOutOfSpinsPanel " .. rootGameView:GetTaggedScriptCount("HideOutOfSpinsPanel"))
	WaitForTaggedScriptCompletion(rootGameView, "HideOutOfSpinsPanel")
	
	if(rootGameView.dimmer== nil) then
		local dialog = rootGameView.NoMoreSpinsPanel
	
		local titlePanel = dialog:FindChild("titlePanel",true)
		if(stats.soonestBomb == 0) then
			titlePanel.text = STRINGS("BombGoingOff")
		else
			titlePanel.text = STRINGS("Out Of Spins")
		end

		if(stats.isNearMiss) then
			titlePanel.nearMissText.text = stats.nearMissText
			titlePanel.nearMissPanel.visible = true
			if (Language == "ru" or Language == "ja") then
				titlePanel.nearMissText.alignment = "MidBottom"
			else
				titlePanel.nearMissText.alignment = "Center"
			end
			
			rootGameView.SpecialGemPanel.SpecialGem1_1:AddScript("Shake","Shake")
			rootGameView.SpecialGemPanel.SpecialGem2_1:AddScript("Shake","Shake")
			rootGameView.SpecialGemPanel.SpecialGem2_2:AddScript("Shake","Shake")
			rootGameView.SpecialGemPanel.SpecialGem3_1:AddScript("Shake","Shake")
			rootGameView.SpecialGemPanel.SpecialGem3_2:AddScript("Shake","Shake")
			rootGameView.SpecialGemPanel.SpecialGem3_3:AddScript("Shake","Shake")
		end

        --If there are no tricks, hide the UseTricksSign
        local useTricks = dialog:FindChild("UseTrickSign",true)
        local trickPanel = rootGameView:FindChild("TrickPanel", true)
        useTricks.visible = false
        if(trickPanel.visible and trickPanel:HasOneTrick()) then
            useTricks.visible = true
        end
		
		dialog.TopTenPanel.ButtonPanel.PlayMoreButton.active = true
		dialog.TopTenPanel.ButtonPanel.EndGameButton.active = true
	
		local MainPanel_originalPosition = dialog.MainPanel.position
		local TopTenPanel_originalPosition = dialog.TopTenPanel.position
		dialog.visible = true
	
		local height = dialog.size.y
		dialog.MainPanel.position = {MainPanel_originalPosition.x, -height}
		dialog.TopTenPanel.position = {TopTenPanel_originalPosition.x, height*2}

		dialog:UploadImagesToVidmem()
		--ScriptDelay(150) -- Give some time to see if this fixes hitch

		local duration = 200
		dialog.MainPanel:SlideTo(MainPanel_originalPosition, duration)
		dialog.TopTenPanel:SlideTo(TopTenPanel_originalPosition, duration)

		rootGameView.dimmer = Resources.Actor:new {
			resource = "Actor",
			size = (APP and APP.screenSize) or {320, 480},
			layer = 4,
			alpha = 64,
			color = "Black",
		}
		FadeIn(rootGameView.dimmer,duration)
		ScriptDelay(duration,"DialogTimers")
		LoopSound("slot_machine_spin_loop_02.ogg")
	end
end

Scripts.HideOutOfSpinsPanel = function(rootGameView)
	--logprint("Entering HideOutOfSpinsPanel")
	--logprint("Script count: ShowOutOfSpinsPanel " .. rootGameView:GetTaggedScriptCount("ShowOutOfSpinsPanel"))
	--logprint("Script count: HideOutOfSpinsPanel " .. rootGameView:GetTaggedScriptCount("HideOutOfSpinsPanel"))
	WaitForTaggedScriptCompletion(rootGameView, "ShowOutOfSpinsPanel")
	if(rootGameView.dimmer) then
		local dialog = rootGameView.NoMoreSpinsPanel

		local height = dialog.size.y
		local MainPanel_originalPosition = dialog.MainPanel.position
		local TopTenPanel_originalPosition = dialog.TopTenPanel.position

		dialog.TopTenPanel.ButtonPanel.PlayMoreButton.active = false
		dialog.TopTenPanel.ButtonPanel.EndGameButton.active = false
	
		local duration = 200
		dialog.MainPanel:SlideTo( {MainPanel_originalPosition.x, -height}, duration)
		dialog.TopTenPanel:SlideTo( {TopTenPanel_originalPosition.x, height*2}, duration)
		FadeOutAndDestroy(rootGameView.dimmer,duration)
		ScriptDelay(duration)

		dialog.visible = false
		dialog.MainPanel.position = MainPanel_originalPosition
		dialog.TopTenPanel.position = TopTenPanel_originalPosition
		StopSound("slot_machine_spin_loop_02.ogg")
	end
end

Scripts.HideOutOfSpinsPanel_ContinueToken = function(rootGameView,quantity)
	--logprint("Entering HideOutOfSpinsPanel_ContinueToken")
	--logprint("Script count: ShowOutOfSpinsPanel " .. rootGameView:GetTaggedScriptCount("ShowOutOfSpinsPanel"))
	--logprint("Script count: HideOutOfSpinsPanel_ContinueToken " .. rootGameView:GetTaggedScriptCount("HideOutOfSpinsPanel_ContinueToken"))
	WaitForTaggedScriptCompletion(rootGameView, "ShowOutOfSpinsPanel")
	if(rootGameView.dimmer) then
		local dialog = rootGameView.NoMoreSpinsPanel

		dialog.TopTenPanel.ButtonPanel.PlayMoreButton.active = false
		dialog.TopTenPanel.ButtonPanel.EndGameButton.active = false

		local continueToken = dialog:FindChild("ContinueToken",true)
		local quantityLabel = continueToken:FindChild("QuantityLabel",true)
		local currentScore = quantityLabel.score
		currentScore = currentScore-1
		quantityLabel:SetScore(currentScore,true)
		PlaySound("slot_machine_win_19.ogg")
		FireParticle(continueToken,"StarBurst_White_Small","Center")
		Pop(quantityLabel)

		if(quantity==0)then
			ScaleOut(continueToken,{0.0,0.0},150,true)
		end

		local height = dialog.size.y
		local MainPanel_originalPosition = dialog.MainPanel.position
		local TopTenPanel_originalPosition = dialog.TopTenPanel.position
	
		local duration = 200
		dialog.MainPanel:SlideTo( {MainPanel_originalPosition.x, -height}, duration)
		dialog.TopTenPanel:SlideTo( {TopTenPanel_originalPosition.x, height*2}, duration)
		FadeOutAndDestroy(rootGameView.dimmer,duration)
		ScriptDelay(duration)

		dialog.visible = false
		dialog.MainPanel.position = MainPanel_originalPosition
		dialog.TopTenPanel.position = TopTenPanel_originalPosition
		StopSound("slot_machine_spin_loop_02.ogg")
	end
end

Scripts.ShowNoMoreSpinsDialog_SpinsAdded = function(screen)
	--logprint("Present_ShowNoMoreSpinsDialog_End "..Logic_GameMusic)
	local sparkle = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "Sparks1",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,0},
	}	 
	screen.RootActor:AddChild(sparkle)

	StopSound("slot_machine_spin_loop_02.ogg");

	PlaySound("slot_machine_win_02.ogg")
	PlaySound("sci_fi_lose_04_reverse.ogg")

	ScriptDelay(1000)
	screen:StartGameMusic()
end

Scripts.ShowNoMoreSpinsDialog_GameOver = function(screen)
	StopSound("slot_machine_spin_loop_02.ogg");
end


Scripts.Present_Begin_EndGame_BurstGems_Secondary = function(rootGameView)
end

Scripts.Leave_GameOverAnimation = function(gameScreen)
	StopSound("slot_machine_jackpot_win_loop.ogg")
	PlaySound("slot_machine_win_19.ogg")
	FireParticle(gameScreen, "Sparks1", "Center")
end

Scripts.Present_GameOver = function(gameScreen,victorious,goalKey,rootGameView)
	local goodJobWord = gameScreen:FindChild("FlashWord",true)
	--logprint("Game over with goal of: " .. goalKey)

	--DEBUG:
	--victorious = true
	--goalKey = finish

	-- Slide Off Boards ------------------------------------------------------------------------
	local AllBoardCells = gameScreen:FindChild("AllBoardCells",true)
	local AllSlotsCells = gameScreen:FindChild("AllSlotsCells",true)
	local AllGamePieces = gameScreen:FindChild("AllGamePieces",true)
	local StarMeter = gameScreen:FindChild("StarMeter",true)
	local GameBoard = gameScreen:FindChild("ReelBackground",true)

	local BoardFrame = gameScreen:FindChild("BoardFrame",true)
	if(BoardFrame)then
		local hgeParticleLayer = BoardFrame:FindChild("hgeParticleLayer",true)
		if(hgeParticleLayer)then
			hgeParticleLayer:KillAllEmitters()
		end
	end

	-- TODO: Turned off pending bug fix
	--[[
	SlideOff(AllBoardCells,250,"right","nowait")
	SlideOff(AllGamePieces,250,"right","nowait")
	
	local gameBoard = gameScreen:FindChild("ReelBackground",true)
	if(gameBoard)then
		--FadeIn(gameBoard,250)
		SlideOff(gameBoard,500,"bottom","nowait")
	end

	local boardWidth = AllBoardCells.boardWidth
	local boardHeight = AllBoardCells.boardHeight
	
	-- Turn off pattern until we want to show
	for y=0,boardHeight-1 do
		for x=0,boardWidth-1 do
			local nameOfMatchingActor = "BoardSpaceMatchedView_" .. x .. "_" .. y
			local matchingActor = gameScreen:FindChild(nameOfMatchingActor,true)
			if(matchingActor)then
				SlideOff(matchingActor,250,"right","nowait")
			end
		end
	end
	--]]
	------------------------------------------------------------------------

	--Wait for previous good job words
	WaitForTaggedScriptCompletion(rootGameView, "GoodJobWord")

	gameScreen.RootActor.Heart = Resources.Actor:new {
		resource = "Actor",
		image = "Icon_Lives.png",
		alignment = "Center",
		alignmentOffset = {0,-100},
		alpha = 0,
		visible = false,
		layer = 5,
		Full = Resources.Actor:new {
			resource = "Actor",
			image = "Icon_Lives.png",
			alignment = "Center",
		},
		BrokeLeft = Resources.Actor:new {
			resource = "Actor",
			image = "Icon_Lives_crackLeft.png",
			alignment = "Center",
			visible = false,
			rotationAngle = { degrees = -10 },
			alignmentOffset = {-50,0},
		},
		BrokeRight = Resources.Actor:new {
			resource = "Actor",
			image = "Icon_Lives_crackRight.png",
			alignment = "Center",
			visible = false,
			rotationAngle = { degrees = 10 },
			alignmentOffset = {50,0},
		},
	}

	goodJobWord.extraScale = {1.0,1.0}
	if(didJackpot_Cascade) then
		goodJobWord.image = "fantastic.png"
		AddScriptWithParams(rootGameView,"GoodJobWord",goodJobWord.image, STRINGS("Fantastic!"),750,"bottom")
	elseif(victorious) then
		goodJobWord.image = "goalcomplete.png"
		AddScriptWithParams(rootGameView,"GoodJobWord",goodJobWord.image, STRINGS("Goal Complete!"),750,"bottom")
	else
		goodJobWord.image = "gameover.png"
		AddScriptWithParams(rootGameView,"GoodJobWord",goodJobWord.image, STRINGS("Game Over"),2000,"bottom")
	end
	local burst = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarBurst3",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	gameScreen.RootActor:AddChild(burst)	
	goodJobWord.extraScale = 1.0 --Clean up incorrect old extraScale value from previous scaleOuts.
	--ScaleIn(goodJobWord,{0.0,0.0},150,true)
	if(victorious)then
		goodJobWord:AddScript("Jello","Jello")
	else
		goodJobWord:AddScript("Shake","Shake")
	end
	if(didJackpot_Cascade)then
		PlaySound("Cascade_Fantastic.ogg")
		ScriptDelay(500)
		StopSound("slot_machine_jackpot_win_loop.ogg")
		Scripts.Confetti(gameScreen)
		ScriptDelay(500)
		PlaySound("crowd_indoor_cheer_02.ogg")
		--ScriptDelay(2000)
		ScriptDelay(1000)
		ScriptDelay(500)
	else
		if(victorious) then
			PlaySound("Cascade_Awesome.ogg")
			ScriptDelay(500)
			StopSound("slot_machine_jackpot_win_loop.ogg")
			Scripts.Confetti(gameScreen)
			--ScriptDelay(2500)
			ScriptDelay(1000)
			ScriptDelay(500)
		else
			PlaySound("Cascade_GameOver.ogg")
			ScriptDelay(250)
			if(not victorious)then
				gameScreen.RootActor.Heart.visible = true
				--PlaySound("soft_lose_02_quick.ogg")
				SlideOn(gameScreen.RootActor.Heart,150,"top","wait")
				--ScriptDelay(250)
				--ScaleTo(gameScreen.RootActor.Heart, {1.25, 1.25}, 250, true)
				--ScaleTo(gameScreen.RootActor.Heart, {1.0, 1.0}, 100, true)
				--ScaleTo(gameScreen.RootActor.Heart, {1.25, 1.25}, 100, true)
				ScriptDelay(250)
				PlaySound("Baseball_Bat_Crack13.ogg")
				FireParticle(gameScreen.RootActor.Heart,"StarBurst1.smaller.white.fall","Center")
				FireParticle(gameScreen.RootActor.Heart,"Heart_boom1","Center")
				gameScreen.RootActor.Heart.Full.visible = false
				gameScreen.RootActor.Heart.BrokeLeft.visible = true 
				gameScreen.RootActor.Heart.BrokeRight.visible = true
				RotateTo(gameScreen.RootActor.Heart.BrokeLeft, {degrees=-60}, 1000)
				RotateTo(gameScreen.RootActor.Heart.BrokeRight, {degrees=60}, 1000)
				ScriptDelay(1000)
				gameScreen.RootActor.Heart.BrokeLeft:AddScript("SpinAndSlideOff_left","SpinAndSlideOff")
				gameScreen.RootActor.Heart.BrokeRight:AddScript("SpinAndSlideOff_right","SpinAndSlideOff")
				PlaySound("slot_machine_jackpot_lose.ogg")
				ScriptDelay(1000)
			end
		end
	end

	--PlaySound("Move2_down.ogg")
	CorrectedPop(goodJobWord,  {expandTime=200, shrinkTime=50, expandTo={1.5, 1.0}})
	--FadeOut(goodJobWord,100)
	--ScaleOut(goodJobWord,{0.0,0.0},100,true)
	--[[
	local burst2 = CreateInstance {
		name = "DiamondSparkle",
		class = "hgeParticleSystemActor",
		file = "StarBurst1.smaller",
		alignment = "Center",
		alignTo = "Center",
		alignmentOffset = {0,-30},
	}	
	PlaySound("OING_A.ogg")
	gameScreen.RootActor:AddChild(burst2)	
	ScriptDelay(50)
	--]]
	--ScriptDelay(100)
	--ScriptDelay(500)
	if(not victorious)then
		gameScreen.RootActor.Heart:Destroy()
	end


end

Scripts.Game_Present = function(gameScreen)
end

Scripts.PointOutTap  = function (gameScreen,actorName)
	local tapHerePointer = gameScreen:FindChild("TapHerePointer",true)
	local pointAtActor = gameScreen:FindChild(actorName,true)
	local globalPoint = gameScreen:LocalToGlobalCoord(pointAtActor.center)
	globalPoint.y = globalPoint.y - (pointAtActor.height/2)
	tapHerePointer:KillAllScripts()
	tapHerePointer.position = globalPoint
	AddScriptWithParams(tapHerePointer,"TapHereAction")
	tapHerePointer.visible = true
end

Scripts.PointOutTap_stop  = function (gameScreen)
	local tapHerePointer = gameScreen:FindChild("TapHerePointer",true)
	
	tapHerePointer:KillAllScripts()
	tapHerePointer.visible = false
end

Scripts.CellShine = function(cell)
	StopFade(cell.shine)
	if(not cell.missingSquare)then
		cell.shine.alpha = 200
		cell.shine.size = cell.size
		cell.shine.visible = true
		--cell.highlight.layer = 1
		--FadeIn(cell.shine,200)
		ScriptDelay(100)
		FadeOut(cell.shine,200)
		ScriptDelay(200)
	end
end

Scripts.CellShine_cleanup = function(cell)
	StopFade(cell.shine)
	cell.shine.visible = false
end

Scripts.BoardShine = function(allBoardCells)
	if(not allBoardCells.isShining)then
		allBoardCells.isShining = true
		local boardWidth = allBoardCells.boardWidth
		local boardHeight = allBoardCells.boardHeight
		for y=0,boardHeight-1 do
			--logprint("---")
			for x=0,boardWidth-1 do
				local cellName = "BoardCell_"..x.."_"..y-x
				--logprint(cellName)
				local cell = allBoardCells:FindChild(cellName,true)
				if(cell)then
					cell:AddScript("CellShine","")
				end
			end
					ScriptDelay(40)
		end
		--logprint("NEXT")
		for i=1,boardWidth-1 do
			--logprint("---")
			local count = 0
			for x=i,boardWidth-1 do
				y=boardHeight-1
				local cellName = "BoardCell_"..x.."_"..y-count
				--logprint(cellName)
				local cell = allBoardCells:FindChild(cellName,true)
				if(cell)then
					cell:AddScript("CellShine","")
				end
				count = count + 1
			end
					ScriptDelay(40)
		end
	--[[
		for y=0,boardHeight do
			for x=0,boardWidth-1 do
				local cellName = "BoardCell_"..x.."_"..y
				--logprint(cellName)
				cell = allBoardCells:FindChild(cellName,true)
				if(cell)then
					cell.highlight.alpha = 100
					cell.highlight.visible = true
				end
			end
			for x=0,boardWidth-1 do
				local cellName = "BoardCell_"..x.."_"..y-1
				--logprint(cellName)
				cell = allBoardCells:FindChild(cellName,true)
				if(cell)then
					cell.highlight.visible = false
					cell.highlight.alpha = 255
				end
			end
			ScriptDelay(60)
		end
	--]]
	--[[
		for y=0,boardHeight-1 do
			for x=0,boardWidth-1 do
				local cellName = "BoardCell_"..x.."_"..y
				logprint(cellName)
				cell = allBoardCells:FindChild(cellName,true)
				cell.highlight.alpha = 100
				cell.highlight.visible = true
			end
			ScriptDelay(30)
		end
		for y=0,boardHeight-1 do
			for x=0,boardWidth-1 do
				local cellName = "BoardCell_"..x.."_"..y
				logprint(cellName)
				cell = allBoardCells:FindChild(cellName,true)
				cell.highlight.visible = false
				cell.highlight.alpha = 255
			end
			ScriptDelay(30)
		end
	--]]
	--[[
		for x=0,boardWidth-1 do
			for y=0,boardHeight-1 do
				local cellName = "BoardCell_"..x.."_"..y
				logprint(cellName)
				cell = allBoardCells:FindChild(cellName,true)
				cell.highlight.alpha = 100
				cell.highlight.visible = true
			end
			ScriptDelay(30)
		end
		for x=0,boardWidth-1 do
			for y=0,boardHeight-1 do
				local cellName = "BoardCell_"..x.."_"..y
				logprint(cellName)
				cell = allBoardCells:FindChild(cellName,true)
				cell.highlight.visible = false
				cell.highlight.alpha = 255
			end
			ScriptDelay(30)
		end
	--]]
		allBoardCells.isShining = false
	end
end

Scripts.Playing = function(rootGameView)
	--logprint("Playing ")
	local allBoardCells = rootGameView:FindChild("AllBoardCells",true)
	playing = true
	while(playing)do
		ScriptDelay(60000,120000)
		--ScriptDelay(2000)
		allBoardCells:AddScript("BoardShine","")
	end
end

Scripts.BeatChallenge = function(check)
	if(not check.showedScript)then
		check.showedScript = true
		PlaySound("slot_machine_win_19.ogg")
		FireParticle(check,"StarBurst_Blue_Small","Center")
		PopKeepScale(check)	
	end
end

Resources.CutsceneNextButton = Resources.DialogButton:new {
	resource = "DialogButton",
	text = STRINGS("Next"),
	alignment = BottomRight,
	--alignmentOffset = {0,-2},
	shortcutKey = "N",
	enterKey = true,
	spaceKey = true,
	zorder = 401,
	active = false,
	keyTipPosition = {-200,-10},
	textWidthLimit = 65,
	
	-- Set up a button click handler.
	GURU_BUTTONCLICK = function (eventData)
		-- Filter out events that weren't sent by the item view to itself.
		if eventData.sender ~= eventData.receiver then
			return false
		end
		
		eventData.sender.parent.parent.doNextLine = true
		return true
	end
}


Resources.CutsceneFullScreenNextButton = Resources.Button:new {
	resource = "Button",
	image = "",
	text = STRINGS(""),
	useFrames = false,
	layer = 900,
	tag = "Clickable",

	-- Set up a button click handler.
	GURU_BUTTONCLICK = function (eventData)
		-- Filter out events that weren't sent by the item view to itself.
		if eventData.sender ~= eventData.receiver then
			return false
		end
		
		eventData.sender.parent.parent.doNextLine = true
		return true
	end,
}

Resources.CutsceneSkipButton= Resources.DialogButton:new {
	resource = "DialogButton",
	text = STRINGS("Skip"),
	alignment = BottomLeft,
	--alignmentOffset = {0,-2},
	shortcutKey = "S",
	escapeKey = true,
	enterKey = false,
	spaceKey = false,
	zorder = 401,
	keyTipPosition = {-90,-10},
	textWidthLimit = 65,
}

Scripts.Preshow_ShowIntroScript = function(screen)
	--logprint("Preshow_ShowIntroScript")
	if(screen.RootActor)then
		if(screen.RootActor.dimmer) then
			screen.RootActor.dimmer:Destroy()
		end
		if(screen.RootActor.NextButton) then
			screen.RootActor.NextButton:Destroy()
		end
		if(screen.RootActor.SkipButton)then
			screen.RootActor.SkipButton:Destroy()
		end
		if(screen.RootActor.CutsceneBackdrop)then
			screen.RootActor.CutsceneBackdrop:Destroy()
		end
	end

	LoopSoundAsMusic("Cascade_GoodJob_loop.mp3",true)

	screen.RootActor.dimmer = Resources.Actor:new {
		resource = "Actor",
		size = APP.screenSize,
		zorder = 300,
		layer = 100,
		color = "Black",
		alpha = 0,
	}

	screen.RootActor.NextButton = Resources.CutsceneNextButton:new {
		resource = "CutsceneNextButton",
		visible = false,
		layer = 101,
	}

	screen.RootActor.FullScreenNextButton = Resources.CutsceneFullScreenNextButton:new {
		resource = "CutsceneFullScreenNextButton",
		visible = false,
		layer = 101,
		size = {screen.size.x,screen.size.y},
	}

	screen.RootActor.SkipButton = Resources.CutsceneSkipButton:new {
		resource = "CutsceneSkipButton",
		visible = false,
		layer = 101,
	}

	local imageName = screen.RootActor.backgroundImage.image
	if(screen.RootActor.cutsceneImageName)then
		imageName = screen.RootActor.cutsceneImageName
	end

	if(screen.RootActor.backgroundImage.ReelBackground)then
		screen.RootActor.backgroundImage.ReelBackground.visible = false
	end
		
	screen.RootActor.CutsceneBackdrop = Resources.BackgroundImage:new {
		resource = "BackgroundImage",
		image = imageName,
		layer = 100,
		zorder = 1,
		ReelBackground = {class = "Actor",visible = false,	},
		visible = false,
	}
end

Scripts.Present_ShowIntroScript = function(screen)
	--logprint("Present_ShowIntroScript")
	
	screen.RootActor.CutsceneBackdrop:FadeIn(250,false)

	SlideOn(screen.RootActor.NextButton,250,"bottom","nowait")
	SlideOn(screen.RootActor.FullScreenNextButton,250,"bottom","nowait")
	SlideOn(screen.RootActor.SkipButton,250,"bottom","nowait")
	ScriptDelay(450)
end

Scripts.Leave_ShowIntroScript = function(screen,dimmerChildren)
	--logprint("Leave_ShowIntroScript")

	local message = screen:FindChild("message",true)
	if(message)then
		message:Destroy()
	end

	--Slide off any children of dimmer.
	if(dimmerChildren)then
		for unused,actor in ipairs(dimmerChildren) do
			--FadeOut(actor,250)
			SlideOff(actor,250,"bottom","nowait")
		end
	end
	
	SlideOff(screen.RootActor.NextButton,250,"bottom","nowait")
	SlideOff(screen.RootActor.FullScreenNextButton,250,"bottom","nowait")
	SlideOff(screen.RootActor.SkipButton,250,"bottom","nowait")
	screen.RootActor.dimmer:FadeOut(250,false,false)
	screen.RootActor.CutsceneBackdrop:FadeOut(250,false,false)
	ScriptDelay(350)
	screen.RootActor.NextButton:Destroy()
	screen.RootActor.FullScreenNextButton:Destroy()
	screen.RootActor.SkipButton:Destroy()
	screen.RootActor.CutsceneBackdrop:Destroy()
	screen.RootActor.dimmer:Destroy()
end


Scripts.Preshow_ShowOuttroScript = Scripts.Preshow_ShowIntroScript 

Scripts.Present_ShowOuttroScript  = function(screen)
		
	--local discoBallFX = screen:FindChild("discoBallFX",true)
	--discoBallFX.visible = false -- turned off for now

	local spinButton = screen.RootActor:FindChild("SpinButton",true)
	spinButton.visible = false

	PlaySound("slot_machine_win_22.ogg")

	Scripts.Present_ShowIntroScript(screen) -- Set up cutscene

end

Scripts.Leave_ShowOuttroScript = function(screen,dimmerChildren)
	PlaySound("slot_machine_win_19.ogg")

	--logprint("Leave_ShowOuttroScript")

	local message = screen:FindChild("message",true)
	if(message)then
		message:Destroy()
	end

	--Slide off any children of dimmer.
	if(dimmerChildren)then
		for unused,actor in ipairs(dimmerChildren) do
			--FadeOut(actor,250)
			SlideOff(actor,250,"bottom","nowait")
		end
	end
	
	SlideOff(screen.RootActor.NextButton,250,"bottom","nowait")
	SlideOff(screen.RootActor.SkipButton,250,"bottom","nowait")
	screen.RootActor.dimmer:FadeOut(250,false,false)
	ScriptDelay(300)
	screen.RootActor.NextButton:Destroy()
	screen.RootActor.SkipButton:Destroy()
	screen.RootActor.CutsceneBackdrop:Destroy()
	screen.RootActor.dimmer:Destroy()
end

Scripts.Cell_GemSwipe = function(cell,colorName)
	local highlight = cell:FindChild("highlight",true)
	StopFade(highlight)
	highlight.image = "slot_highlight_"..colorName..".png"
	highlight.size = cell.size
	highlight.visible = true
	highlight.alpha = 255
	highlight.layer = 1
	highlight.extraScale = 1.0
end

Scripts.Cell_UnGemSwipe = function(cell,colorName)
	local highlight = cell:FindChild("highlight",true)
	StopFade(highlight)
	highlight.visible = false
end

Scripts.PurchaseWasMade = function(parentScreen,name)
	logprint("PurchaseWasMade: "..name.." parentScreen "..parentScreen.name)
end

Scripts.Animate_GettingTricks = function(self)
	logprint("Animate_GettingTricks: "..self.name)
	PlaySound("coin_magic_06.ogg")
	FireParticle(self,"StarBurst2","Center")
	FireParticle(self,"StarBurst1.smaller.white","Center")
	CorrectedPop(self)
end

Scripts.Animate_GettingSpins = function(self)
	logprint("Animate_GettingSpins: "..self.name)
	PlaySound("coin_magic_06.ogg")
	FireParticle(self,"StarBurst2","Center")
	FireParticle(self,"StarBurst1.smaller.white","Center")
	CorrectedPop(self)
end

Scripts.Animate_GettingTicks = function(self)
	--logprint("Animate_GettingTicks: "..self.name)
	PlaySound("coin_magic_06.ogg")
	FireParticle(self,"StarBurst2","Center")
	FireParticle(self,"StarBurst1.smaller.white","Center")
	CorrectedPop(self)
end

Scripts.Animate_GettingBoosts_dialog = function(self,dialog)
	--logprint("Animate_GettingBoosts_dialog: "..self.name.." "..dialog.name)
	PlaySound("coin_magic_06.ogg")
	FireParticle(self,"StarBurst2","Center",dialog.hgeParticles)
	FireParticle(self,"StarBurst1.smaller.white","Center",dialog.hgeParticles)
	CorrectedPop(self)
end

Scripts.Animate_GettingShopItem_dialog = function(self,dialog)
	--logprint("Animate_GettingTicks: "..self.name)
	PlaySound("coin_magic_06.ogg")
	FireParticle(self,"StarBurst2","Center",dialog.hgeParticles)
	FireParticle(self,"StarBurst1.smaller.white","Center",dialog.hgeParticles)
	CorrectedPop(self)
end

Scripts.Animate_GettingShopItem = function(self) --self == ShopButton
	logprint("Animate_GettingShopItem_MapScreen: "..self.name)
	PlaySound("coin_magic_06.ogg")
	FireParticle(self,"Shine2","Center",self.hgeParticles, {18, 0})
	FireParticle(self,"StarToss1.smaller","Center",self.hgeParticles, {18, 0})
	CorrectedPop(self.label)
	ScriptDelay(250)
	PlaySound("Cascade_Yeah.ogg")
end

Scripts.RefreshContinueToken = function(self,continueToken,quantityLabel,oldQuantity,quantity)
	--logprint("RefreshContinueToken: "..self.name)
	if(oldQuantity==0)then
		PlaySound("catapult_release_small_01.ogg")
		SlideOn(continueToken,150,"top","wait")
	end
	quantityLabel.visible = true
	continueToken.visible = true
	PlaySound("coin_magic_06.ogg")
	FireParticle(continueToken,"StarBurst2","Center")
	FireParticle(continueToken,"StarBurst1.smaller.white","Center")
	CorrectedPop(continueToken)
end

function Scripts.Animate_UpAndDown(self)
	local backedUpPosition = self.globalPosition
	local goUpPosition = {backedUpPosition.x,backedUpPosition.y-10}

	Scripts.Interpolate(self, "globalPosition", goUpPosition, FalseMoveSlideTimeMS/4)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, FalseMoveSlideTimeMS)

	goUpPosition = {backedUpPosition.x,backedUpPosition.y+4}

	Scripts.Interpolate(self, "globalPosition", goUpPosition, FalseMoveSlideTimeMS/4)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, FalseMoveSlideTimeMS/2)

end

function Scripts.Animate_DownAndUp(self)
	local backedUpPosition = self.globalPosition
	local goUpPosition = {backedUpPosition.x,backedUpPosition.y+10}

	Scripts.Interpolate(self, "globalPosition", goUpPosition, FalseMoveSlideTimeMS/4)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, FalseMoveSlideTimeMS)

	goUpPosition = {backedUpPosition.x,backedUpPosition.y-4}

	Scripts.Interpolate(self, "globalPosition", goUpPosition, FalseMoveSlideTimeMS/4)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, FalseMoveSlideTimeMS/2)

end

function Scripts.Bounce_UpAndBack(self,amount)
	local amount = amount or 6
	local backedUpPosition = self.globalPosition
	local goUpPosition = {backedUpPosition.x,backedUpPosition.y-amount}

	Scripts.Interpolate(self, "globalPosition", goUpPosition, 40)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, 60)
end

function Scripts.Bounce_DownAndBack(self,amount)
	local amount = amount or 6
	local backedUpPosition = self.globalPosition
	local goUpPosition = {backedUpPosition.x,backedUpPosition.y+amount}

	Scripts.Interpolate(self, "globalPosition", goUpPosition, 60)
	Scripts.Interpolate(self, "globalPosition", backedUpPosition, 100)
end

function Scripts.TrickClicked(root,gameScreen, trickName)
--[[ OFF FOR NOW
	logprint("Scripts.TrickClicked: "..trickName)

	local trickPanel = root:FindChild("TrickPanel", true)
	local trick1Container = trickPanel:FindChild("Trick1Container", true)
	local trick2Container = trickPanel:FindChild("Trick2Container", true)
	local trick3Container = trickPanel:FindChild("Trick3Container", true)

	local trickActivePanel = root:FindChild("TrickActivePanel", true)
	local TrickActiveContainer = trickActivePanel:FindChild("TrickActiveContainer", true)

	if(trickName=="SuperMatch")then	
		trick1Container.visible = false
		trick2Container.visible = false
		trick3Container.visible = true
		--TrickActiveContainer.Trick.image = trick3Container.Trick.image
		--TrickActiveContainer.visible = true

		trick3Container.savedGlobalPosition = trick3Container.globalPosition
		trick3Container.globalPosition = TrickActiveContainer.globalPosition
	end
	if(trickName=="Hammer")then	
		trick1Container.visible = false
		trick2Container.visible = true
		trick3Container.visible = false
		--TrickActiveContainer.Trick.image = trick2Container.Trick.image
		--TrickActiveContainer.visible = true

		trick2Container.savedGlobalPosition = trick2Container.globalPosition
		trick2Container.globalPosition = TrickActiveContainer.globalPosition
	end--
	if(trickName=="Shifter")then	
		trick1Container.visible = true
		trick2Container.visible = false
		trick3Container.visible = false
		--TrickActiveContainer.Trick.image = trick1Container.Trick.image
		--TrickActiveContainer.visible = true

		trick1Container.savedGlobalPosition = trick1Container.globalPosition
		trick1Container.globalPosition = TrickActiveContainer.globalPosition
	end
--]]
end

function Scripts.TrickUnClicked(root,gameScreen, trickName)
--[[ OFF FOR NOW
	logprint("Scripts.TrickClicked: "..trickName)

	local trickPanel = root:FindChild("TrickPanel", true)
	local trick1Container = trickPanel:FindChild("Trick1Container", true)
	local trick2Container = trickPanel:FindChild("Trick2Container", true)
	local trick3Container = trickPanel:FindChild("Trick3Container", true)

	local trickActivePanel = root:FindChild("TrickActivePanel", true)
	local TrickActiveContainer = trickActivePanel:FindChild("TrickActiveContainer", true)

	if(trickName=="SuperMatch")then	
		trick1Container.visible = true
		trick2Container.visible = true
		trick3Container.visible = true
		trick3Container.globalPosition = trick3Container.savedGlobalPosition
	end
	if(trickName=="Hammer")then	
		trick1Container.visible = true
		trick2Container.visible = true
		trick3Container.visible = true
		trick2Container.globalPosition = trick2Container.savedGlobalPosition
	end
	if(trickName=="Shifter")then	
		trick1Container.visible = true
		trick2Container.visible = true
		trick3Container.visible = true
		trick1Container.globalPosition = trick1Container.savedGlobalPosition
	end

	TrickActiveContainer.visible = false
--]]
end
